//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files ("the
//     Programs") are Copyright (c) 1999 - 2000 Nortel Networks Corporation, 
//     8200 Dixie Road, Suite 100, Brampton, Ontario, Canada L6R 5P6.  
//     All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out in
//     the Nortel Networks FIPA-OS Public License, a copy of which can be found
//     in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version can
//     also be found at http://fipa-os.sourceforge.net/. By downloading the
//     files containing the Programs you accept the terms and  conditions of the
//     Public License. You do not have to accept these terms and conditions, but
//     unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.agent.conversation;

//----------------------------------------------------------------------

import java.util.Enumeration;
import fipaos.agent.conversation.*;
import fipaos.mts.MessageSender;
import fipaos.mts.Message;

  /**
   * <br> Interface that objects may implement to be notified of changes in the state of 
   * <br> Conversations by the Planner Scheduler.
   * <br>
   */

public interface ConversationListener
{
   /**
   * May be called to notify of a single conversation that is in need of intervention.
   */
   public void notify(Conversation conversation); 
     
   /**
   * May be called to notify of a number of conversations that are in need of intervention.
   */
   public void notify(Enumeration conversations);
  
	/**
	 * Sets the MessageSender that the ConversationListener should use to forward messages
	 */
	public void setMessageSender( MessageSender ms );
	
	/**
	 * Indicates that the given Message arrived for a completed Conversation
	 */
	public void notifyMessageInEndedConversation( Message msg );
	
	/**
	 * Indicates that the given Message was not deliverable
	 */
	public void notifyMessageUndeliverable( Message msg );
} // ConversationListener
