//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.agent.conversation;

//----------------------------------------------------------------------

import fipaos.mts.*;

/**
 * This interface should be implemented by all CM implementations.
 */
public interface ConversationManager extends MessageSender, MessageReceiver
{
	/**
	 * Method to create a new conversation given a protcol, and add it to the active list
	 */
	Conversation addConversation(String protocol) throws UnknownProtocolException;
	
	
	/**
	 * Retrieves an active/previous Conversation that an Agent has had
	 */
	Conversation getConversation(String conversation_id) throws InvalidConversationIDException;
	
	
	/**
	 * Removes the given conversation from the ConversationManager - this may be
	 * necessary in order to end conversations that have no defined end-point
	 *
	 * @param id The conversation-id of the conversation to remove
	 */
	void removeConversation( String id ) throws InvalidConversationIDException;
	
	/**
	 * Adds a new protocol to the CM - the CM will attempt to learn the protocol and will
	 * cache it for future use.
	 *
	 * @param protocol_name The name of the protocol to store (e.g. fipa-new-protocol)
	 * @param protocol_defn The RDF definition of the protocol
	 * @return True if the protocol was learned and stored correctly, false otherwise
	 **/
	boolean addProtocol( String protocol_name, String protocol_defn );
	
	
	/**
	 * Indicates whether the specified protocol is already in the cache, i.e. has it already been learned.
	 *
	 * @param protocol The protocol to check for
	 * @return True if the protocol has already been learned, false otherwise
	 **/
	boolean hasProtocol( String protocol );	
}

