//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.agent.conversation.standard.protocol;

//----------------------------------------------------------------------

import fipaos.ont.fipa.FIPACONSTANTS;
import fipaos.agent.conversation.*;
import fipaos.agent.conversation.standard.*;

//----------------------------------------------------------------------

/**
 * Specialisation of the Conversation class to deal with the fipa-iterated-
 * contract-net protocol
 * <p> Implements XC00030E
 */
public class FIPAIteratedContractNet extends ConversationImpl implements java.io.Serializable
{
	
	//----------------------------------------------------------------------
	// STATIC VARIABLES
	//----------------------------------------------------------------------
	
	/**
	 * Definition of protocol model
	 */
	private static Object[] __after_accept =
	{	FIPACONSTANTS.FAILURE, new Integer( CONVERSATION_END ), new Integer( 1 ),
	FIPACONSTANTS.INFORM, new Integer( CONVERSATION_END ), new Integer( 1 ) };
	
	private static Object[] __after_propose =
	{	FIPACONSTANTS.ACCEPT_PROPOSAL, new Integer( AGENT_ACTION_REQ ), new Integer( 0 ), __after_accept,
	FIPACONSTANTS.REJECT_PROPOSAL, new Integer( AGENT_ACTION_REQ ), new Integer( 0 ), null };
	
	private static Object[] __after_cfp =
	{ 	FIPACONSTANTS.REFUSE, new Integer( CONVERSATION_END ), new Integer( 1 ),
	FIPACONSTANTS.PROPOSE, new Integer( AGENT_ACTION_REQ ), new Integer( 1 ), __after_propose };
	
	public static Object[] __protocol =
	{ FIPACONSTANTS.CFP, new Integer( AGENT_ACTION_REQ ), new Integer( 0 ), __after_cfp };
	
	
	/**
	 * Static method to setup superclass __protocol variable
	 */
	static
	{
		// Setup loop in protocol
		__after_propose[7] = __protocol;
	}
	
	//----------------------------------------------------------------------
	// TEST METHOD
	//----------------------------------------------------------------------
	
	public static void main( String[] args ) throws Throwable
	{
		String msgs[] =
		{ "cfp", "test@iiop://195.8.93.20:50/acc",
		"propose", "test2@iiop://195.8.93.20:50/acc",
		"reject-proposal", "test@iiop://195.8.93.20:50/acc",
		"cfp", "test@iiop://195.8.93.20:50/acc"};
		String msgs2[] =
		{"propose", "test2@iiop://195.8.93.20:50/acc",
		"reject-proposal", "test@iiop://195.8.93.20:50/acc",
		"cfp", "test@iiop://195.8.93.20:50/acc",
		"propose", "test2@iiop://195.8.93.20:50/acc",
		"accept-proposal", "test@iiop://195.8.93.20:50/acc",
		"inform", "test2@iiop://195.8.93.20:50/acc" };
		String msgs3[] =
		{"propose", "test2@iiop://195.8.93.20:50/acc",
		"accept-proposal", "test@iiop://195.8.93.20:50/acc",
		"inform", "test2@iiop://195.8.93.20:50/acc" };
		
		ConversationImpl conv1 = ConversationImpl.getConversation( "fipa-iterated-contract-net", msgs[1] );
		
		testAddMessages(conv1, msgs, false);
		
		ConversationImpl conv2 = (ConversationImpl) conv1.clone();
		
		testAddMessages(conv1, msgs2, false);
		testAddMessages(conv2, msgs3, false);
	}
}
