//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files ("the
//     Programs") are Copyright (c) 1999 - 2000 Nortel Networks Corporation, 
//     8200 Dixie Road, Suite 100, Brampton, Ontario, Canada L6R 5P6.  
//     All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out in
//     the Nortel Networks FIPA-OS Public License, a copy of which can be found
//     in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version can
//     also be found at http://fipa-os.sourceforge.net/. By downloading the
//     files containing the Programs you accept the terms and  conditions of the
//     Public License. You do not have to accept these terms and conditions, but
//     unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.agent.task;

/** This task waits for a specified amount of time before invoking the done()
 * method (and hence notifying its parent-task)
 */
public class WaitTask extends Task
{
	private long _wait;
	
  /** Creates a new WaitTask with the given timeout.
	 * @param time The time to wait after initialisation before informing
	 * the parent task (from initialisation - i.e. by using
	 * the TaskManager or Task newTask() method)
	 */
	public WaitTask(long time)
	{
		_wait = time;
	}
	
	/** Invoked after this Task has been initialised - starts the countdown!
	 */
	protected void startTask()
	{
		newTask( new DummyTask(), _wait );
	}
	
	/**
	 * Invoked when timeout occurs
	 */
	public void timeoutWaitTask_DummyTask( Task t )
	{
		done();
	}
	
	private class DummyTask extends Task
	{
	}
}
