//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved.
//
//******************************************************************************

package fipaos.mts;

//----------------------------------------------------------------------

import fipaos.util.*;

//----------------------------------------------------------------------

/**
 * TODO: Write a description of the class
 * <br>
 * <br>
 */
public class DefaultTransportProvider extends TransportProvider
{
	/**
	 * Package only constructor
	 */
	DefaultTransportProvider()
	{
	}
	
	/** Creates an InternalMTP instance based upon the URL given for a naming service
	 * @param ns URL of the naming service for the transport
	 * @param name Name of the Agent this belongs to
	 * @return Either an initialised InternalMTP instance, or null
	 */
	public InternalMTP getInternalMTP( String name, URL url )
	{
		// Create an InternalMTP instance based upon the protocol required
		InternalMTP ic = null;
		
		try
		{
			// FIPAOS proprietory RMI transport?
			if( url.getProtocol().equalsIgnoreCase( "fipaos-rmi" ) )
			{
				// Dynamically instantiate the transport
				Class arg_types[] = new Class[]
				{ String.class, URL.class };
				Object args[] = new Object[]
				{ name, url };
				DynamicClass dc = new DynamicClass( "fipaos.mts.rmi.internal.RMIComms",
				arg_types, args );
				
				ic = (InternalMTP) dc.getObject();
			}
			// FIPAOS proprietory SSL-RMI transport?
			else if( url.getProtocol().equalsIgnoreCase( "fipaos-ssl-rmi" ) )
			{
				// Dynamically instantiate the transport
				Class arg_types[] = new Class[]
				{ String.class, URL.class };
				Object args[] = new Object[]
				{ name, url };
				DynamicClass dc = new DynamicClass( "fipaos.mts.rmi.internal.SSLRMIComms",
				arg_types, args );
				
				ic = (InternalMTP) dc.getObject();
			}
			// FIPAOS HTTP transport
			// TODO: Doesn't currently work
/*			else if( url.getProtocol().equalsIgnoreCase( "fipaos-http" ) )
			{
				// Dynamically instantiate the transport
 
				Class arg_types[] = new Class[] { String.class, URL.class };
				Object args[] = new Object[] { name, url };
				DynamicClass dc = new DynamicClass( "fipaos.mts.http.internal.HTTPComms", arg_types, args );
 
				ic = (InternalMTP) dc.getObject();
			} */
			// FIPA-OS Test MTP
			else if( url.getProtocol().equalsIgnoreCase( "test" ) )
			{
				// Dynamically instantiate the transport
				Class arg_types[] = new Class[]
				{ String.class, URL.class };
				Object args[] = new Object[]
				{ name, url };
				DynamicClass dc = new DynamicClass( "fipaos.test.agent.TestingMTP",
				arg_types, args );
				
				ic = (InternalMTP) dc.getObject();
			}
		}
		catch( DynamicClassException e )
		{
			DIAGNOSTICS.println( "Error when locating InternalMTP for " + url + ": ",
			e.getThrowable(), DIAGNOSTICS.LEVEL_MAX );
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "Error when locating InternalMTP for " + url + ": ",
			t, DIAGNOSTICS.LEVEL_MAX );
		}
		
		// Return initialised InternalMTP if necessary
		return ic;
	}
	
	/** Creates an ExternalMTP instance based upon the URL given for a naming service
	 * @param ns Naming service URL for the transport
	 * @param name Name of the Agent this belongs to
	 * @return An initialised ExternalMTP instance, or null
	 */
	public ExternalMTP getExternalMTP( String name, URL url )
	{
		// Create an InternalMTP instance based upon the protocol required
		ExternalMTP ic = null;
		
		try
		{
			// FIPAOS proprietory RMI transport?
			if( url.getProtocol().equalsIgnoreCase( "iiop" ) ||
			url.getProtocol().equalsIgnoreCase( "corbaname" ) )
			{
				// Dynamically instantiate the transport
				Class arg_types[] = new Class[]
				{ String.class, URL.class };
				Object args[] = new Object[]
				{ name, url };
				DynamicClass dc = new DynamicClass( "fipaos.mts.iiop.sun.external.SUNIIOPComms",
				arg_types, args );
				
				ic = (ExternalMTP) dc.getObject();
			}
			else if( url.getProtocol().equalsIgnoreCase( "http" ))
			{
				// Dynamically instantiate the transport
				Class arg_types[] = new Class[]
				{ String.class, URL.class };
				Object args[] = new Object[]
				{ name, url };
				DynamicClass dc = new DynamicClass( "fipaos.mts.httpMTP.HTTPComms",
				arg_types, args );
				
				ic = (ExternalMTP) dc.getObject();
			}
		}
		catch( DynamicClassException e )
		{
			DIAGNOSTICS.println( "Error when locating ExternalMTP for " + url + ": ",
			e.getThrowable(), DIAGNOSTICS.LEVEL_MAX );
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "Error when locating ExternalMTP for " + url + ": ",
			t, DIAGNOSTICS.LEVEL_MAX );
		}
		
		// Return initialised ExternalMTP if necessary
		return ic;
	}
	
	
	/**
	 * Gets the FIPA-defined MTP profile-name for the given protocol type
	 */
	public String getMTPProfileName( String protocol )
	{
		String name = null;
		
		// IIOP transport?
		if( protocol.equalsIgnoreCase( "iiop" ) ||
		protocol.equalsIgnoreCase( "corbaname" ) ||
		protocol.equalsIgnoreCase( "ior" ) )
		{
			name = "fipa.mts.mtp.iiop.std";
		}
		
		return name;
	}
} // End class



