//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.mts;

//----------------------------------------------------------------------

import fipaos.util.URL;
import fipaos.mts.service.PreParserService;
import java.util.List;

/** <p>Common interface to a specific transport type - generally a proprietory
 * transport (hence provides an intra-platform communication protocol)
 *
 * <p>Semantics of <CODE>outgoing()</CODE> (inehrited from <CODE>PostParserService</CODE>)
 * are as follows:
 * <ul><li>If <CODE>Envelope.getViaACC()</CODE> is true, the message should be routed
 * to the ACC, rather than the intended-receiver
 * <li>Otherwise, the message is sent to the first address specified within the AID of the
 * intended-receiver field of the envelope only.
 * <li>If no addresses are specified, the Agent's name should be used to locate
 * the Agent, provided its HAP address is the same as the Agent the transport represents,
 * or is ommited.
 * </ul>
 *
 * <p>In the event that a failure occurs, <CODE>Envelope.setErrorCode()</CODE>
 * should be used to set the appropriate error condition, and the message
 * propogated back up the stack (via the <CODE>incoming()</CODE> method of
 * the previous <CODE>Service</CODE> in the comms stack)
 * <br>
 * <br>
 */
public interface InternalMTP extends PreParserService
{
	/**
	 * Method to retreive the protocol dependant addresses of an object that can be used
	 * to receive messages.
	 *
	 * @return A List containing the required addresses, or null if this Comms
	 *					doesn't publish such an object
	 */
	public List getAddresses();
	
	/**
	 * Retreives the protocols known by this comms implementation
	 *
	 * @return A List of String's 
	 */
	public List getProtocols();
}
