//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.mts;

//----------------------------------------------------------------------

import fipaos.ont.fipa.fipaman.*;
import fipaos.ont.fipa.ACL;
import fipaos.mts.event.*;

import java.io.Serializable;

//----------------------------------------------------------------------

/** Provides an encapsulated entity for the parts which compose an Agent message
 */
public class Message implements Serializable, Cloneable
{

//----------------------------------------------------------------------
// INSTANCE VARIABLES
//----------------------------------------------------------------------

	private Envelope _envelope;
	
	private ACL _acl;
	
	private Object _content;

//----------------------------------------------------------------------
// CONSTRUCTOR(S)
//----------------------------------------------------------------------

	/** Create a new Message based upon the given Message
	 * @deprecated Use the clone() method instead
	 */
	public Message( Message msg )
	{
		// Duplicate Envelope, ACL & re-retreive content
		_envelope = (Envelope) msg._envelope.clone();
		_acl = (ACL) msg._acl.clone();
		_content = _acl.getContentObject();
	}

	
	/** Create a new Message based upon the given Envelope (which is assumed to contain
	 * an ACL object).
	 * @param env The Envelope to base the Message upon
	 */
	public Message( Envelope env, ACL acl )
	{
		// Initialisation
		_envelope = (Envelope) env.clone();
		_acl = (ACL) acl.clone();
		_content = _acl.getContentObject();
	}

	
	/** Creates a Message based upon the given ACL object (automatically generates an
	 * Envelope based upon the ACL object given)
	 * @param msg ACL object to create the Message from
	 */
	public Message( ACL msg )
	{
		// Initialisation
		_envelope = new Envelope( msg );
		_acl = (ACL) msg.clone();
		_content = _acl.getContentObject();
	}
	
	/**
	 * Add a MessageListener to this
	 */
	public void addMessageListener( MessageListener ml )
	{
		_envelope.addMessageListener( ml );
	}
	
	/**
	 * Remove a MessageListener from this
	 */
	public void removeMessageListener( MessageListener ml )
	{
		_envelope.removeMessageListener( ml );
	}
	
	public Object clone()
	{
		try
		{
			Message msg = (Message) super.clone();
			
			// Duplicate Envelope, ACL & re-retreive content
			msg._envelope = (Envelope) _envelope.clone();
			msg._acl = (ACL) _acl.clone();
			msg._content = msg._acl.getContentObject();
			
			return msg;
		}
		catch( CloneNotSupportedException e )
		{
			return null;
		}
	}
	
//----------------------------------------------------------------------
// PUBLIC METHODS
//----------------------------------------------------------------------	
	
	/** Gets the Envelope that is part of this Message
	 * @return The Envelope belonging to this message
	 */
	public Envelope getEnvelope()
	{
		return _envelope;
	}

	/** Gets the ACL that is part of this Message
	 * @return The ACL part of this message
	 */
	public ACL getACL()
	{
		return _acl;
	}

	/** Gets the content of this Message
	 * @return The content of this Message
	 */
	public Object getContent()
	{
		return _content;
	}
	
	/**
	 * Convert Message to human-readable form
	 */
	public String toString()
	{
		return "[Message: env=" + _envelope.toString() + ";  acl=" + _acl + "]";
	}
} // End class

	
