//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.mts;

//----------------------------------------------------------------------

import fipaos.util.URL;

//----------------------------------------------------------------------

/** <p>This class should be extended by classes providing factory-like services
 * for the creation of MTP classes.
 * <p>To change the default TransportProvider, the
 * <CODE>setDefaultTransportProvider()</CODE> method should be used.
 * <br>
 * <br>
 */
public abstract class TransportProvider
{
	/**
	 * Reference to the default provider
	 */
	private static TransportProvider __default_provider = new DefaultTransportProvider();
	
	/**
	 * Sets the default TransportProvider currently in use
	 */
	public void setDefaultTransportProvider( TransportProvider tp )
	{
		if( tp != null )
		{
			__default_provider = tp;
		}
	}
	
	/**
	 * Retreives the default TransportProvider currently in use
	 */
	public TransportProvider getDefaultTransportProvider()
	{
		return __default_provider;
	}
	
	/** Creates an InternalMTP instance based upon the URL given for a naming service
	 * @param ns URL of the naming service for the transport
	 * @param name Name of the Agent this belongs to
	 * @return Either an initialised InternalMTP instance, or null
	 */
	public abstract InternalMTP getInternalMTP( String agent_name, URL ns );
	
	/** Creates an ExternalMTP instance based upon the URL given for a naming service
	 * @param ns Naming service URL for the transport
	 * @param name Name of the Agent this belongs to
	 * @return An initialised ExternalMTP instance, or null
	 */
	public abstract ExternalMTP getExternalMTP( String agent_name, URL ns );
	
	/**
	 * Gets the FIPA-defined MTP profile-name for the given protocol type
	 */
	public abstract String getMTPProfileName( String protocol );
	
	/** Creates an InternalMTP instance based upon the URL given for a naming service
	 * using the default TransportProvider instance
	 * @param ns URL of the naming service for the transport
	 * @param name Name of the Agent this belongs to
	 * @return Either an initialised InternalMTP instance, or null
	 */
	public static InternalMTP getDefaultInternalMTP( String agent_name, URL ns )
	{
		return __default_provider.getInternalMTP( agent_name, ns );
	}
	
	/** Creates an ExternalMTP instance based upon the URL given for a naming service
	 * using the default TransportProvider instance
	 * @param ns Naming service URL for the transport
	 * @param name Name of the Agent this belongs to
	 * @return An initialised ExternalMTP instance, or null
	 */
	public static ExternalMTP getDefaultExternalMTP( String agent_name, URL ns )
	{
		return __default_provider.getExternalMTP( agent_name, ns );
	}
	
	/**
	 * Gets the FIPA-defined MTP profile-name for the given protocol type
	 */
	public static String getDefaultMTPProfileName( String protocol )
	{
		return __default_provider.getMTPProfileName( protocol );
	}
} // End class
