//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Queen Mary, University of London.
//
//  Portions created by Queen Mary University of London are Copyright (C) 2001
//  Queen Mary, University of London licensed under the Emorphia Public License,
//  available at http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (Stefan Channing)
//
//******************************************************************************
package fipaos.mts.httpMTP;

import java.net.*;
import java.io.*;

import fipaos.util.DIAGNOSTICS;

public class HTTPTransportService extends ThreadingAbility
{
    
//----------------------------------------------------------------------
// INSTANCE VARIABLES
//----------------------------------------------------------------------
    
    private ServerSocket serverSocket;
    private volatile boolean active;
    private CallBack httpCommsRef;

//----------------------------------------------------------------------
// CONSTRUCTOR
//----------------------------------------------------------------------
    
   /**
    * Sets up the server socket based upon the profile information
    * @param port The port that the server should listen on.
    * @exception IOException
    */
    public HTTPTransportService(int port, CallBack httpCommsRef) throws IOException
    {
		super();
        serverSocket=new ServerSocket(port);
        this.httpCommsRef=httpCommsRef;
        active=true;
		internal.start();
    }

//----------------------------------------------------------------------
// PUBLIC METHODS
//----------------------------------------------------------------------

   /**
    * Sets state of whether the service should be running
    * @param state <code>boolean</code> true - service should run<BR>false - service is closed.
    */
    public void setActive(boolean state)
    {
        active=state;
    }

//----------------------------------------------------------------------
// PROTECTED METHODS - ThreadingAbility implementation
//----------------------------------------------------------------------

   /**
    * Implementation of Server's functionality. Listens on the port specified until a connection is made.
    * It then starts a <code>ServerHandler</code> to deal with the communicating client.
    */
    protected void runImplementation()
    {
        DIAGNOSTICS.println("HTTP Transport Running",this,DIAGNOSTICS.LEVEL_MAX);
        while (active)
        {
            try
            {
    		new ConnectionThread(serverSocket.accept(),httpCommsRef);
            }
            catch(IOException IOEx){
                DIAGNOSTICS.println("A client connection has failed: "+IOEx.toString(),this,DIAGNOSTICS.LEVEL_MAX);
            }
        }
        try {
            if (serverSocket!=null) serverSocket.close();
            serverSocket=null;
            DIAGNOSTICS.println("HTTPTransportService Shutdown",this,DIAGNOSTICS.LEVEL_MAX);
        }
        catch(IOException IOex){}        
    }
	
//----------------------------------------------------------------------
// MAIN/TESTING METHOD
//----------------------------------------------------------------------

    public static void main(String[] args)
    {
		try
		{
            System.err.println("Server Started on http://localhost:4000");
		    new HTTPTransportService(4000, null);
		}
		catch (IOException IOEx) {}
    }
} // End class