//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis,
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Queen Mary, University of London.
//
//  Portions created by Queen Mary University of London are Copyright (C) 2001
//  Queen Mary, University of London licensed under the Emorphia Public License,
//  available at http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (Stefan Channing)
//
//******************************************************************************
package fipaos.mts.httpMTP;

import java.util.Random;
import java.io.*;
import java.net.HttpURLConnection;

import fipaos.ont.fipa.fipaman.Envelope;
import fipaos.parser.envelope.xml.*;
import fipaos.util.DIAGNOSTICS;
import fipaos.parser.ParserException;

/**
 * SenderThread is responsible for sending the Envelope and message
 * to the specified host:port connection. Encodes the output as
 * specified at http://www.fipa.org/specs/fipa00084/
 */
public class SenderThread extends ThreadingAbility
{
	
	//----------------------------------------------------------------------
	// CONSTANTS
	//----------------------------------------------------------------------
	
	private static final int BOUNDARY_LENGTH=24,PASS=-1,UNTRIED=0;
	private static final byte NUM=(byte)'0',CAP=(byte)'A',SMALLCAP=(byte)'a';
	private static final Random r=new Random();
	private static final String STANDARD_TYPE="application/text; charset=US-ASCII";
	
	//----------------------------------------------------------------------
	// INSTANCE VARIABLES
	//----------------------------------------------------------------------
	
	private String envelope,body,boundary,envMimeType,msgMimeType;
	private fipaos.util.URL _target;
	
	//----------------------------------------------------------------------
	// CONSTRUCTOR
	//----------------------------------------------------------------------
	
	public SenderThread(fipaos.util.URL target, Envelope env, byte[] msg)
	{
		super();
		try
		{
			envelope=new String(Parser.deparse(env));
			body=new String(msg);
			_target=target;
			internal.start();
		}
		catch(ParserException PEx)
		{
			DIAGNOSTICS.println("Failed to parse Envelope",this,DIAGNOSTICS.LEVEL_MAX);
		}
	}
	
	//----------------------------------------------------------------------
	// PROTECTED METHODS - ThreadingAbility implementation
	//----------------------------------------------------------------------
	
	protected void runImplementation()
	{
		try
		{
			DIAGNOSTICS.println("HTTP Header and MIME content being constructed",this,DIAGNOSTICS.LEVEL_MAX);
			
			java.net.URL java_url = new java.net.URL(_target.toString());
			
			DIAGNOSTICS.println("Target URL: " + java_url, this, DIAGNOSTICS.LEVEL_2);
			
			HttpURLConnection  uc=(HttpURLConnection) java_url.openConnection();//set up the protocol and content handlers but not the socket
			String message=constructMessage();
			uc.setRequestMethod("POST");
			uc.setRequestProperty("Connection", "close");
			uc.setRequestProperty("Cache-Control", "no-cache");
			uc.setRequestProperty("Content-Type", "multipart/mixed ;\n\tboundary=\"--"+boundary+"\"");
			uc.setRequestProperty("Host", java_url.getHost() + ":" + java_url.getPort());
			uc.setRequestProperty("MIME-Version", "1.0");
			//the request property that specifies the content length is automatically produced by the HttpURLConnection class
			uc.setAllowUserInteraction(false);
			uc.setDefaultUseCaches(false);
			uc.setDoOutput(true);//Allows POST
			uc.setDoInput(true);//default
			PrintWriter out = new PrintWriter(uc.getOutputStream(),true);
			DIAGNOSTICS.println("**TEST**\n"+message, this, DIAGNOSTICS.LEVEL_2);
			out.print(message);
			out.close();
			// Get the response code
			int responseCode=uc.getResponseCode();
			String responseMessage=uc.getResponseMessage();
			DIAGNOSTICS.println("Response Code: "+responseCode, this, DIAGNOSTICS.LEVEL_2);
			uc.disconnect();
		}
		catch (IOException IOEx)
		{
			IOEx.printStackTrace();
			throw new RuntimeException(IOEx.toString());
		}
	}
	
	//----------------------------------------------------------------------
	// PRIVATE METHODS
	//----------------------------------------------------------------------
	
	private String constructMessage()
	{
		StringBuffer buf=new StringBuffer();
		//ensure that the boundary does not appear in either the envelope or the message
		int boundaryTest=UNTRIED;
		do
		{
			boundary=generateBoundary();
		} while(envelope.indexOf(boundary)!=PASS || body.indexOf(boundary)!=PASS);
		//by this stage boundary is valid.
		//combine envelope and message with boundary and content type descriptors.
		buf.append("This is not part of the MIME multipart encoded message.\n--");
		buf.append(boundary);
		buf.append("\n");
		buf.append("Content-Type: ");
		buf.append((envMimeType!=null)?envMimeType:STANDARD_TYPE);
		buf.append("\n\n");
		buf.append(envelope);
		buf.append("\n\n--");
		buf.append(boundary);
		buf.append("\n");
		buf.append("Content-Type: ");
		buf.append((msgMimeType!=null)?msgMimeType:STANDARD_TYPE);
		buf.append("\n\n");
		buf.append(body);
		buf.append("\n--");
		buf.append(boundary);
		buf.append("--\n");
		return buf.toString();
	}
	
    /**
     * Generates the boundary used in the HTTP message i.e nG86KIR9bCMlUm6lTsQwupEj
     * @return <code>String</code>
     */
	private String generateBoundary()
	{
		DIAGNOSTICS.println("Constructing the boundary",this,DIAGNOSTICS.LEVEL_MAX);
		StringBuffer buf=new StringBuffer();
		int n;
		for (int i=0; i<BOUNDARY_LENGTH; i++)
			buf.append((char)(((n=r.nextInt(3))==0)?r.nextInt(10)+NUM:((n==1)?r.nextInt(26)+CAP:r.nextInt(26)+SMALLCAP)));
			return buf.toString();
	}
} // End class