//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Queen Mary, University of London.
//
//  Portions created by Queen Mary University of London are Copyright (C) 2001
//  Queen Mary, University of London licensed under the Emorphia Public License,
//  available at http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (Stefan Channing)
//
//******************************************************************************
package fipaos.mts.httpMTP;

/**
 * ThreadingAbility provides classes with the ability to be self runnable objects.
 * Calling the constructor sets up an inner class that implements <code>runnable</code>.
 * This has the advantage that the <code>run()</code> method is not available from outside of
 * outer class.
 */
public abstract class ThreadingAbility  
{
    
//----------------------------------------------------------------------
// INSTANCE VARIABLES
//----------------------------------------------------------------------
    
    protected Thread internal;
    
//----------------------------------------------------------------------
// CONSTRUCTOR
//----------------------------------------------------------------------
    
    protected ThreadingAbility() 
    {
        internal=new Thread(new Threading());
    }

//----------------------------------------------------------------------
// ABSTRACT PROTECTED METHODS
//----------------------------------------------------------------------
    
    protected abstract void runImplementation();

//----------------------------------------------------------------------
// INNER CLASS - PROVIDES THREADING ABILITY TO OUTTER CLASS
//----------------------------------------------------------------------
    
    private class Threading implements Runnable 
    {				
        public void run() 
        {
	    runImplementation();
	}
    }
} // End class