//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.mts.rmi.internal;

import fipaos.mts.*;
import fipaos.util.*;

import java.io.*;
import java.rmi.*;
import java.rmi.server.*;
import java.net.Socket;
import java.net.ServerSocket;
import java.util.*;

import javax.net.*;
import javax.net.ssl.*;
import java.security.*;
import com.sun.net.ssl.*;

/**
 * This class is totally dependant upon Java 2 & JSSE!!!!!!!! You have been warned
 */
public class SSLRMIComms extends fipaos.mts.rmi.internal.RMIComms
{	
//----------------------------------------------------------------------
// CONSTANTS
//----------------------------------------------------------------------

  /**
   * Transport specific protocol type
   */
  private static final String TRANSPORT_PROTOCOL = "fipaos-ssl-rmi";
  
	public SSLRMIComms( String name, URL ns ) throws InitialisationException, RemoteException
	{
		super( name, ns );
	}
	
	/**
	 * Retreives the protocols known by this comms implementation
	 *
	 * @return A List of String's 
	 */
	public List getProtocols()
	{
		return Arrays.asList( new String[] { TRANSPORT_PROTOCOL } );
	}
	
	/**
	 * Method to get an instance of MTSListener
	 */
	protected MTSListener getMTSListener( RMIComms parent ) throws RemoteException
	{
		// Get MTSListener
		MTSListener mts_listener = new SSLMTSListener( parent );
		
		// Un-export object
		UnicastRemoteObject.unexportObject( mts_listener, true );
		
		// Export MTSListener using custom socket factories
		UnicastRemoteObject.exportObject( mts_listener, 0, new FIPAOS_SSL_Factory(),
																			new FIPAOS_SSL_Server_Factory() );
		
		// Return MTSListener
		return mts_listener;
	}
	
	/**
	 * Class that is actually bound to the NS
	 */
	public class SSLMTSListener extends MTSListener
	{
		public SSLMTSListener( RMIComms parent )  throws RemoteException
		{
			super( parent );
		}
	}
	
	/**
	 * Custom RMISocketFactory to produce SSL sockets :)
	 */
	public static class FIPAOS_SSL_Factory implements RMIClientSocketFactory, Serializable
	{
		private static SocketFactory __factory;
		
		static
		{
			try
			{
				if( java.security.Security.getProvider( "SunJSSE" ) == null )
				{
					java.security.Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());
				}

				// Get Agent key-store and trusted key-store passwords
				char[] agent_keys_pass = System.getProperty( "fipaos.agent_keys.pass", "changeme" ).toCharArray();
				char[] trusted_keys_pass = System.getProperty( "fipaos.trusted_keys.pass", "changeme" ).toCharArray();

				// Get location of key-stores
				String location = System.getProperty( "fipaos.home", ".." );
				File f = new File( location, "certificates" );
				File agent_keys = new File( f, "agent_keys" );
				File trusted_keys = new File( f, "trusted_keys" );
				
				// Set-up SSL context
				SSLContext ctx = SSLContext.getInstance("TLS");
				KeyManagerFactory kmf = KeyManagerFactory.getInstance("SunX509");
				TrustManagerFactory tmf = TrustManagerFactory.getInstance( "SunX509" );
				KeyStore agent_ks = KeyStore.getInstance("JKS");
				KeyStore trusted_ks = KeyStore.getInstance("JKS");

				// Load key-stores
				agent_ks.load( new FileInputStream( agent_keys ), agent_keys_pass );
				trusted_ks.load( new FileInputStream( trusted_keys ), trusted_keys_pass );
				kmf.init( agent_ks, agent_keys_pass );
				tmf.init( trusted_ks );
				ctx.init( kmf.getKeyManagers(), tmf.getTrustManagers(), null );
				
				__factory = ctx.getSocketFactory();
			}
			catch( Throwable t )
			{
				DIAGNOSTICS.println( "Exception when initialising SSL", t, DIAGNOSTICS.LEVEL_MAX );
			}
		}

		public Socket createSocket(String p1,int p2) throws java.io.IOException
		{
			SSLSocket s = (SSLSocket) __factory.createSocket( p1, p2 );
			return (Socket)s;			
		}
	}
		
	/**
	 * Custom RMIServerSocketFactory to produce SSL sockets :)
	 */
	public static class FIPAOS_SSL_Server_Factory implements RMIServerSocketFactory, Serializable
	{
		private static ServerSocketFactory __factory;
		
		static
		{
			try
			{
				if( java.security.Security.getProvider( "SunJSSE" ) == null )
				{
					java.security.Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());
				}

				// Get Agent key-store and trusted key-store passwords
				char[] agent_keys_pass = System.getProperty( "fipaos.agent_keys.pass", "changeme" ).toCharArray();
				char[] trusted_keys_pass = System.getProperty( "fipaos.trusted_keys.pass", "changeme" ).toCharArray();

				// Get location of key-stores
				String location = System.getProperty( "fipaos.home", ".." );
				File f = new File( location, "certificates" );
				File agent_keys = new File( f, "agent_keys" );
				File trusted_keys = new File( f, "trusted_keys" );
				
				// Set-up SSL context
				SSLContext ctx = SSLContext.getInstance("TLS");
				KeyManagerFactory kmf = KeyManagerFactory.getInstance("SunX509");
				TrustManagerFactory tmf = TrustManagerFactory.getInstance( "SunX509" );
				KeyStore agent_ks = KeyStore.getInstance("JKS");
				KeyStore trusted_ks = KeyStore.getInstance("JKS");

				// Load key-stores
				agent_ks.load( new FileInputStream( agent_keys ), agent_keys_pass );
				trusted_ks.load( new FileInputStream( trusted_keys ), trusted_keys_pass );
				kmf.init( agent_ks, agent_keys_pass );
				tmf.init( trusted_ks );
				ctx.init(kmf.getKeyManagers(), tmf.getTrustManagers(), null);
				
				__factory = ctx.getServerSocketFactory();
			}
			catch( Throwable t )
			{
				DIAGNOSTICS.println( "Exception when initialising SSL", t, DIAGNOSTICS.LEVEL_MAX );
			}
		}

		public ServerSocket createServerSocket(int port) throws IOException
		{
			ServerSocket s = __factory.createServerSocket( port );
			return s;			
		}
	}
}