//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.mts.service;

//----------------------------------------------------------------------

import fipaos.ont.fipa.fipaman.*;
import fipaos.ont.fipa.*;
import fipaos.mts.*;
import fipaos.util.*;
//import fipaos.agent.monitor.*;

//----------------------------------------------------------------------

/**
 * TODO: Write a description of the class
 * <br>
 * <br>
 */
public class BufferService implements PreParserService, PostParserService//, BufferMonitor
{
	private static final int INCOMING = 0;
	private static final int OUTGOING = 1;
	private static final int PREPARSER = 0;
	private static final int POSTPARSER = 2;
	
//----------------------------------------------------------------------
// INSTANCE VARIABLES
//----------------------------------------------------------------------

	/**
	 * <p>Queues for all possible combinations of incoming/outgoing pre/post
	 * <p>Only instantiated on demand for efficiency
	 */
	private Queue _queues[] = new Queue[4];
	
	/**
	 * Monitors for each of the queues
	 */
	private Monitor _monitors[] = new Monitor[4];

	/**
	 * Targets for our buffers
	 */
	private Service _targets[] = new Service[4];

	/**
	 * Indicates if this Service has been shutdown
	 */
	private boolean _shutdown = false;
	
//----------------------------------------------------------------------
// PUBLIC METHODS - Service
//----------------------------------------------------------------------

   /**
   * <br> Method required by the BufferMonitor interface.
   * <br>
   * <br>
   */	
   public int getBufferSize()
	 {
	    return -1;
	 }
	 
	 /**
   * <br> Method required by the BufferMonitor interface.
   * <br>
   * <br>
   */
   public void setLimit(int i)
	 {
	 }
	 
	 /**
   * Method required by the BufferMonitor interface.
   */
   public String getIdentifier()
	 {
	    return "bufferservice";
	 }
	 
	 /**
   * Method required by the BufferMonitor interface.
   */
   public void halt()
	 {
	 }
   
   /**
   * Method required by the BufferMonitor interface.
   */
   public void restart()
	 {
	 }
	 
	
	/**
	 * Shutdown this Service, and those below
	 */
	public synchronized void shutdown()
	{
		// Indicate we're now shutdown
		_shutdown = true;
		
		// Kill off threads
		for( int i = 0; i < _monitors.length; i++ )
		{
			// Check if current index has a monitor!
			if( _monitors[ i ] != null )
			{
				// Shutdown monitor
				_monitors[ i ].halt();
				_monitors[ i ] = null;
			}
		}
		_monitors = null;
		
		// Flush queues
		for( int i = 0; i < _queues.length; i++ )
		{
			// Check if current index has a monitor!
			if( _queues[ i ] != null )
			{
				// Shutdown monitor
				_queues[ i ].flush();
				_queues[ i ] = null;
			}
		}
		_queues = null;
		
		// Propogate shutdown()
		for( int i = 0; i < _targets.length; i++ )
		{
			// Check if current index has a target!
			if( _targets[ i ] != null && (i%POSTPARSER == OUTGOING) )
			{
				// Shutdown target
				_targets[ i ].shutdown();
				_targets[ i ] = null;
			}
		}
		_targets = null;
	}
	
//----------------------------------------------------------------------
// PUBLIC METHODS - PostParserService
//----------------------------------------------------------------------	
	
	public synchronized void outgoing( Envelope p1, byte[] p2 )
	{
		DIAGNOSTICS.println( "Dealing with outgoing message, msg id = " + p1.getSlotValue( FIPAOS_MSG_ID_SLOT ), this );
		
		if( _shutdown )
		{
			DIAGNOSTICS.println( "Couldn't deal with outgoing message, shutdown() has been invoked", this );
			return;
		}
		
		// Add message to appropriate queue
		getQueue( OUTGOING + POSTPARSER ).addItem( new Object[] { p1, p2 } );
	}
	
	public synchronized void incoming( Envelope p1, byte[] p2 )
	{
		DIAGNOSTICS.println( "Dealing with incoming message, msg id = " + p1.getSlotValue( FIPAOS_MSG_ID_SLOT ), this );
		
		if( _shutdown )
		{
			DIAGNOSTICS.println( "Couldn't deal with incoming message, shutdown() has been invoked", this );
			return;
		}
		
		// Add message to appropriate queue
		getQueue( INCOMING + POSTPARSER ).addItem( new Object[] { p1, p2 } );
	}
	
	public void initialise( PostParserService p1, PostParserService p2 )
	{
		// Save references
		_targets[ INCOMING + POSTPARSER ] = p1;
		_targets[ OUTGOING + POSTPARSER ] = p2;
		
		DIAGNOSTICS.println( "Buffer service joined to PostParserService's: " +
													p1 + ", " + p2, this );
	}
	

//----------------------------------------------------------------------
// PUBLIC METHODS - PreParserService
//----------------------------------------------------------------------	
	
	public synchronized void incoming( Message p1 )
	{
		DIAGNOSTICS.println( "Dealing with incoming message, msg id = " + p1.getEnvelope().getSlotValue( FIPAOS_MSG_ID_SLOT ), this );
		
		if( _shutdown )
		{
			DIAGNOSTICS.println( "Couldn't deal with incoming message, shutdown() has been invoked", this );
			return;
		}
		
		// Add message to appropriate queue
		getQueue( INCOMING + PREPARSER ).addItem( p1 );
	}
	
	public synchronized void outgoing( Message p1 )
	{
		DIAGNOSTICS.println( "Dealing with outgoing message, msg id = " + p1.getEnvelope().getSlotValue( FIPAOS_MSG_ID_SLOT ), this );
		
		if( _shutdown )
		{
			DIAGNOSTICS.println( "Couldn't deal with outgoing message, shutdown() has been invoked", this );
			return;
		}
		
		// Add message to appropriate queue
		getQueue( OUTGOING + PREPARSER ).addItem( p1 );
	}
	
	public void initialise( PreParserService p1 , PreParserService p2 )
	{
		// Save references
		_targets[ INCOMING + PREPARSER ] = p1;
		_targets[ OUTGOING + PREPARSER ] = p2;
	
		DIAGNOSTICS.println( "Buffer service joined to PreParserService's: " +
													p1 + ", " + p2, this );
	}


//----------------------------------------------------------------------
// PRIVATE METHODS
//----------------------------------------------------------------------

	/**
	 * Instantiate (if necessary) and return a queue for the given target
	 */
	private synchronized Queue getQueue( int queue )
	{
		// Check if Queue and monitor have been instantiated
		if( _queues[ queue ] == null )
		{
			// Instantiate a new Queue & MonitorListener
			_queues[ queue ] = new FIFOQueue();
			
			// Get target for this queue
			Object target = _targets[ queue ];
			
			if( ( queue % 2 ) == INCOMING )
			{
				_monitors[ queue ] = new Monitor( _queues[ queue ], new IncomingMonitor( target ) );
			}
			else
			{
				_monitors[ queue ] = new Monitor( _queues[ queue ], new OutgoingMonitor( target ) );
			}
		}
		
		return _queues[ queue ];
	}
	
	
	/**
	 * MonitorListener (for Queue) to deal with passing incoming messages
	 * to previous in stack
	 */
	private class IncomingMonitor implements MonitorListener
	{
		Object _target;
		
		public IncomingMonitor( Object target )
		{
			// Store target object
			_target = target;
		}
		
		public void monitorUpdate( Object p1 )
		{
			// Pass object to target
			if( _target instanceof PreParserService )
			{
				// Cast to correct type
				PreParserService pps = (PreParserService) _target;
				
				// Pass Message to target
				try
				{
					pps.incoming( (Message) p1 );
				}
				catch( Throwable t )
				{
					DIAGNOSTICS.println( "Exception occured: ", t, DIAGNOSTICS.LEVEL_3 );
				}
			}
			else if( _target instanceof PostParserService )
			{
				// Cast to correct type
				PostParserService pps = (PostParserService) _target;
				
				// Pass Message to target
				try
				{
					Object o[] = (Object[]) p1;
					pps.incoming( (Envelope) o[0], (byte[]) o[1] );
				}
				catch( Throwable t )
				{
					DIAGNOSTICS.println( "Exception occured: ", t, DIAGNOSTICS.LEVEL_3 );
				}
			}
			else
			{
				DIAGNOSTICS.println( "Couldn't pass message though stack - _target = " + _target +
																	", msg = " + p1, this, DIAGNOSTICS.LEVEL_MAX );
			}
		}
	}
	

	/**
	 * MonitorListener (for Queue) to deal with passing outgoing messages
	 * to next in stack
	 */
	private class OutgoingMonitor implements MonitorListener
	{
		Object _target;
		
		public OutgoingMonitor( Object target )
		{
			// Store target object
			_target = target;
		}
		
		public void monitorUpdate( Object p1 )
		{
			// Pass object to target
			if( _target instanceof PreParserService )
			{
				// Cast to correct type
				PreParserService pps = (PreParserService) _target;
				
				// Pass Message to target
				try
				{
					pps.outgoing( (Message) p1 );
				}
				catch( Throwable t )
				{
					DIAGNOSTICS.println( "Exception occured: ", t, DIAGNOSTICS.LEVEL_3 );
				}
			}
			else if( _target instanceof PostParserService )
			{
				// Cast to correct type
				PostParserService pps = (PostParserService) _target;
				
				// Pass Message to target
				try
				{
					Object o[] = (Object[]) p1;
					pps.outgoing( (Envelope) o[0], (byte[]) o[1] );
				}
				catch( Throwable t )
				{
					DIAGNOSTICS.println( "Exception occured: ", t, DIAGNOSTICS.LEVEL_3 );
				}
			}
		}
	}

//----------------------------------------------------------------------
// MAIN/TESTING METHOD
//----------------------------------------------------------------------

} // End class
