//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files ("the
//     Programs") are Copyright (c) 1999 - 2000 Nortel Networks Corporation, 
//     8200 Dixie Road, Suite 100, Brampton, Ontario, Canada L6R 5P6.  
//     All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out in
//     the Nortel Networks FIPA-OS Public License, a copy of which can be found
//     in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version can
//     also be found at http://fipa-os.sourceforge.net/. By downloading the
//     files containing the Programs you accept the terms and  conditions of the
//     Public License. You do not have to accept these terms and conditions, but
//     unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.mts.service;

//----------------------------------------------------------------------

import fipaos.mts.*;


//----------------------------------------------------------------------

/** This interface should be implemented by classes which provide pre-parsing (for
 * outbound messages) / post-parsing (for incoming messages) processing
 * <br>
 * <br>
 */
public interface PreParserService extends Service
{
//----------------------------------------------------------------------
// PUBLIC METHODS
//----------------------------------------------------------------------	

	/** Invoked when the undelying implementation should initialise itself.  Allows
	 * references to other PreParserService's before and after it in the stack
	 * to be passed
	 * @param previous Previous PreParserService instance (Incoming messages should be passed to this)
	 * @param next Next PreParserService instance (Outgoing messages should be passed to this)
	 */
	public void initialise( PreParserService previous, PreParserService next );

	/** Outgoing messages are passed to <CODE>this</CODE> via this method
	 * @param msg Outgoing message
	 */
	public void outgoing( Message msg );
	
	/** Incoming messages are passed to <CODE>this</CODE> via this method
	 * @param msg Incoming message
	 */
	public void incoming( Message msg );
} // End class

	
