//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.mts.service;

//----------------------------------------------------------------------

//----------------------------------------------------------------------

/**
 * Common service interface - provides error code constants
 */
public interface Service
{
//----------------------------------------------------------------------
// CONSTANTS
//----------------------------------------------------------------------

	/**
	 * Array containing brief error meanings for error codes below
	 */
	public static final String ERROR_STRING[] = new String[] { "No error",
																								"Couldn't locate target",
																								"MTP('s) unknown",
																								"Unknown error",
																								"ACL encoding unknown",
																								"Content encoding unknown",
																								"Malformed envelope" };
	
	/**
	 * Error code - indicates no error
	 */
	public static final int NONE = 0;

	/**
	 * Error code - indicates target is unknown
	 */
	public static final int TARGET_UNKNOWN = 1;

	/**
	 * Error code - indicates protocol is unknown
	 */
	public static final int PROTOCOL_UNKNOWN = 2;

	/**
	 * Error code - indicates error is unknown
	 */
	public static final int ERROR_UNKNOWN = 3;

	/**
	 * Error code - indicates acl language is unknown
	 */
	public static final int ACL_LANGUAGE_UNKNOWN = 4;

	/**
	 * Error code - indicates content language is unknown
	 */
	public static final int CONTENT_LANGUAGE_UNKNOWN = 5;

	/**
	 * Error code - indicates envelope is malformed
	 */
	public static final int MALFORMED_ENVELOPE = 6;

	/**
	 * Slot name for message id from sender
	 */
	public static final String FIPAOS_MSG_ID_SLOT = "X-fipaos-msg-id";
	
	/**
	 * Slot name for fipa-os debug stuff
	 */
	public static final String FIPAOS_DEBUG = "X-fipaos-debug-";
	
	/**
	 * Slot name for message delivery-failure-reason
	 */
	public static final String MSG_DELIVERY_FAILURE_REASON = "X-msg-delivery-failure-reason";
	
//----------------------------------------------------------------------
// METHODS
//----------------------------------------------------------------------

	/**
	 * This should be invoked when the service should be safely shutdown
	 */
	public void shutdown();	 	 	 
} // End interface

	
