//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.mts.service.preparse;

//----------------------------------------------------------------------

import fipaos.ont.fipa.fipaman.*;
import fipaos.mts.service.*;
import fipaos.mts.*;
import fipaos.util.*;
import fipaos.ont.fipa.FIPACONSTANTS;
import fipaos.ont.BASETOKENS;

import java.util.*;

//----------------------------------------------------------------------

/**
 * TODO: Write a description of the class
 * <br>
 * <br>
 */
public class ACCRouterService implements PreParserService, MonitorListener
{
//----------------------------------------------------------------------
// CONSTANTS
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// STATIC VARIABLES
//----------------------------------------------------------------------
	
//----------------------------------------------------------------------
// INSTANCE VARIABLES
//----------------------------------------------------------------------

	private boolean _dyn_load_ext_transports;

	private PreParserService _next;

	private PreParserService _prev;

	private Queue _awaiting_work = new FIFOQueue();
	
	private Monitor _monitor = new Monitor( _awaiting_work, this );
	
	/**
	 * Indicates if this Service has been shutdown
	 */
	private boolean _shutdown = false;
	
///----------------------------------------------------------------------
// CONSTRUCTOR(S)
//----------------------------------------------------------------------

	public ACCRouterService( boolean dyn_load_ext_transports )
	{
		_dyn_load_ext_transports = dyn_load_ext_transports;
	}
	
//----------------------------------------------------------------------
// PUBLIC METHODS - PreParserService interface
//----------------------------------------------------------------------	
	
	public synchronized void outgoing( Message p1 )
	{
		if( _shutdown )
		{
			DIAGNOSTICS.println( "Couldn't deal with outgoing message, shutdown() has been invoked", this );
			return;
		}
		
		// Just pass the message onwards down the stack
		_next.outgoing( p1 );
	}
	
	public synchronized void incoming( Message p1 )
	{
		if( _shutdown )
		{
			DIAGNOSTICS.println( "Couldn't deal with incoming message, shutdown() has been invoked", this );
			return;
		}
		
		// Check if the message is a "real" incoming message, or an error
		// has occured whilst sending
		
		boolean can_deal = false;
		
		Envelope env = p1.getEnvelope();
		int error_code = env.getErrorCode();
		
		// Check if there was an error!
		if( error_code != NONE && !env.getViaACC() )
		{
			// Determine if this service can deal with the error
			if( error_code == PROTOCOL_UNKNOWN || error_code == TARGET_UNKNOWN )
			{
				DIAGNOSTICS.println( "Outbound message has \"bounced\" - may be able to recover by forwarding to ACC..." );
			
				// We can do something
				can_deal = true;
			}
		}
		
		// Can we deal with this message?
		if( can_deal )
		{
			DIAGNOSTICS.println( "ACCRouterService: Attempting to pass outbound message to ACC" );
			
			// Pass message on to be dealt with
			env.setErrorCode( NONE );
			dealWith( p1 );
		}
		else
		{
			DIAGNOSTICS.println( "ACCRouterService: Passing inbound message back up stack" );
			
			// Pass message back up stack if it is a "real" incoming message
			_prev.incoming( p1 );
		}
	}
	
	public void initialise( PreParserService p1, PreParserService p2 )
	{
		// Save references to other services in stack
		_prev = p1;
		_next = p2;
	}
	
	public synchronized void shutdown()
	{
		_shutdown = true;
		
		// Invoke shutdown on next in stack
		_next.shutdown();
		
		// Remove references to other objects to improve garbage collection
		_monitor.halt();
		_monitor = null;
		_next = null;
		_prev = null;
		_awaiting_work.flush();
		_awaiting_work = null;
	}


//----------------------------------------------------------------------
// PUBLIC METHODS - MonitorListener interface
//----------------------------------------------------------------------

	public void monitorUpdate( Object p1 )
	{
		// Deal with the next message in the queue
		Message msg = (Message) p1;
		
		// Check if we're sending external messages
		if( _dyn_load_ext_transports )
		{
			// TODO: Add support for dynamically loading external transports
		}
		else
		{
			Envelope env = msg.getEnvelope();
			
			// Send message to ACC
			env.setViaACC( true );
			env.getTransportsTried().clear();
			
			// Retrieve failed addresses.....
			AgentID aid = (AgentID) env.getIntendedReceiver().get( 0 );
			aid.getAddresses().addAll( 0, env.getUnknownURLsTried() );
			env.getUnknownURLsTried().clear();
			
			// Forward back out
			_next.outgoing( msg );
		}
	}

//----------------------------------------------------------------------
// PROTECTED METHODS
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// PRIVATE METHODS
//----------------------------------------------------------------------

	private void dealWith( Message msg )
	{
		// Add to worklist
		_awaiting_work.addItem( msg );
	}
	
//----------------------------------------------------------------------
// MAIN/TESTING METHOD
//----------------------------------------------------------------------
  
} // End class
