//******************************************************************************
//	Open Source Copyright Notice and License: FIPA-OS
//
//	1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//	2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.ont.fipa.fipaman;

//----------------------------------------------------------------------

import fipaos.ont.fipa.FIPACONSTANTS;
import fipaos.ont.BASETOKENS;
import fipaos.agent.conversation.content.Content;
import fipaos.parser.sl.*;
import fipaos.parser.*;
import fipaos.util.*;

import java.util.*;

//----------------------------------------------------------------------

/**
 * Provides parsing/deparsing methods for AMA messages with exceptions
 * <br>
 * <br>
 */
public class AgentManagementException implements java.io.Serializable
{

	//----------------------------------------------------------------------
	// CONSTANTS
	//----------------------------------------------------------------------




	//----------------------------------------------------------------------
	// STATIC VARIABLES
	//----------------------------------------------------------------------




	//----------------------------------------------------------------------
	// INSTANCE VARIABLES
	//----------------------------------------------------------------------

	/**
	 * AgentManagement contained within the content that this parses
	 */
	private AgentManagementAction _ama;

	/**
	 * Exception contained within the content that this parses
	 */
	private String _exception;

	//----------------------------------------------------------------------
	// CONSTRUCTOR(S)
	//----------------------------------------------------------------------

	/**
	 * Creates an AgentManagementException based upon the given AMA + exception
	 */
	public AgentManagementException( AgentManagementAction ama, String exception )
	{
		_ama = ama;
		_exception = exception;
	}

	/**
	 * Parses the given content into AgentManagementAction objects + extracts the
	 * associated exception
	 */
	public AgentManagementException( String acl_content ) throws ParserException
	{
		try {
			// Parse content
			Content content = SLParser.parseContent(acl_content);

			// Get the Content representing the main terms of the content-expression
			Object items=content.getAttribute("contentExpression");

			// Discover if an exception is included!
			if (items instanceof List)
			{
				List exps=(List) items;

				// Extract AMA
				_ama = new AgentManagementAction( (Content) ((Content) exps.get(0)).getAttribute("actionExpression") );

				// Extract exception
				_exception = (String) ((Content) exps.get(1)).getAttribute("formattedString");

				// "Pull-off" brackets
				_exception = StringParser.trimBrackets( _exception );
			}
			else
			{

				// Just extract AMA
				_ama = new AgentManagementAction( (Content) ((Content) items).getAttribute("actionExpression") );

			}

		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "Error when parsing content: ", t, this, DIAGNOSTICS.LEVEL_MAX );
			throw new ParserException( "Error when parsing content: " + t );
		}
	}

	//----------------------------------------------------------------------
	// PUBLIC METHODS
	//----------------------------------------------------------------------

	public void setAgentManagementAction( AgentManagementAction ama )
	{
		_ama = ama;
	}


	public void setException( String exception )
	{
		_exception = exception;
	}

	public AgentManagementAction getAgentManagementAction()
	{
		return _ama;
	}

	public String getException()
	{
		return _exception;
	}

	public String toString()
	{
		// Check if an exception has been specified
		if( _exception == null )
		{
			return _ama.toString();
		}
		else
		{
			StringBuffer buff = new StringBuffer();
			buff.append( BASETOKENS.OPENBRACKET );
			buff.append( _ama.toString() );
			buff.append( BASETOKENS.OPENBRACKET );
			buff.append( _exception );
			buff.append( BASETOKENS.CLOSEBRACKET );
			buff.append( BASETOKENS.CLOSEBRACKET );

			return buff.toString();
		}
	}

	//----------------------------------------------------------------------
	// MAIN/TESTING METHOD
	//----------------------------------------------------------------------

	public static void main( String args[] ) throws ParserException
	{
		String test = "((\n" +
		"action \n" +
		"(agent-identifier :name df@phard1f0.europe.nortel.com ) \n" +
		"(register \n" +
		"(ams-agent-description \n" +
		":name (agent-identifier :name df@phard1f0.europe.nortel.com :addresses (sequence \n" +
		" fipaos-rmi://localhost:3000/df ) ) \n" +
		":ownership fipaos \n" +
		":state active \n" +
		") \n" +
		//									")))\n";
		")) (unauthorised))\n";

		//		DIAGNOSTICS.setMemoryLevel( DIAGNOSTICS.LEVEL_MIN );

		AgentManagementException ame = new AgentManagementException( test );

		System.out.println( "AMA = " + ame.getAgentManagementAction() );
		System.out.println( "Exception = " +ame.getException() );

		System.out.println( "\n\n" + ame );
	}


} // End class

