//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.ont.fipa.fipaman;

import fipaos.ont.fipa.FIPACONSTANTS;
import fipaos.ont.BASETOKENS;
import fipaos.agent.conversation.content.Content;
import fipaos.parser.sl.*;
import fipaos.parser.*;
import fipaos.util.*;

import java.util.*;


/** This class represents a result content sent between agents.
 * This corresponds to the second branch of the SL Atomic Formula.
 *
 */
public class AgentManagementResult implements java.io.Serializable
{

	//----------------------------------------------------------------------
	// CONSTANTS
	//----------------------------------------------------------------------




	//----------------------------------------------------------------------
	// STATIC VARIABLES
	//----------------------------------------------------------------------




	//----------------------------------------------------------------------
	// INSTANCE VARIABLES
	//----------------------------------------------------------------------

	private AgentManagementAction _action;
	private Set _results;

	/** This is the empty constructor. Please use set and get to populate fields.
	 */
	public AgentManagementResult()
	{
		_results=Collections.synchronizedSet( new TreeSet( fipaos.util.Comparators.OBJECT_COMPARE ) );
	}

	/** This is a constructor which takes a string as the parameter
	 * @param result_string string representation of the result
	 * @throws ParserException if the string is invalid
	 */
	public AgentManagementResult(String result_string) throws ParserException
	{
		try
		{
			_results=Collections.synchronizedSet(new TreeSet( fipaos.util.Comparators.OBJECT_COMPARE ));
			parse(SLParser.parseAtomicFormula(result_string));
		}
		catch (Throwable e)
		{
			e.printStackTrace();
			throw new ParserException(e.getMessage());
		}
	}

	/** This constructor takes a content object returned by the SL Parser as the parameter.
	 * @param result_content content object returned by the SL Parser
	 * @throws ParserException if the string is invalid
	 */
	public AgentManagementResult(Content result_content) throws ParserException
	{
		try
		{
			_results=Collections.synchronizedSet(new TreeSet( fipaos.util.Comparators.OBJECT_COMPARE ));
			parse(result_content);
		}
		catch (Throwable e)
		{
			throw new ParserException(e.getMessage());
		}
	}
	
	/** This method sets the AgentManagementAction
	 * @param action the AgentManagementAction
	 */
	public void setAgentManagementAction(AgentManagementAction action) 
	{
		_action=action;
	}
	
	/** This method sets the result set.
	 * @param results set of results
	 */
	public void setResultSet(Set results) 
	{
		_results=results;
	}
	
	
	/** This method returns the agent management action.
	 * @return the agent management action
	 */
	public AgentManagementAction getAgentManagementAction()
	{
		return _action;
	}
	
	/** This method returns the result set.
	 * @return result set
	 */
	public Set getResultSet() 
	{
		return _results;
	}
	
	/** This method returns its string representation
	 * @return string representing the agent management result
	 */
	public String toString()
	{
		StringBuffer buff = new StringBuffer();
		buff.append( BASETOKENS.OPENBRACKET );
		buff.append( BASETOKENS.SPACE );
		buff.append( FIPAMANCONSTANTS.RESULT );
		buff.append( BASETOKENS.SPACE );
		buff.append( _action.toString() );
		buff.append( BASETOKENS.NEWLINE );
		buff.append( convertResultSetIntoString(_results) );
		buff.append( BASETOKENS.SPACE);
		buff.append( BASETOKENS.CLOSEBRACKET );
			
		return buff.toString();
	}
	
	private String convertResultSetIntoString(Set set) 
	{
		StringBuffer buffer=new StringBuffer();
		
		buffer.append(BASETOKENS.OPENBRACKET);
		buffer.append(FIPAMANCONSTANTS.SET);
		buffer.append(BASETOKENS.SPACE);
			
		Iterator iterator=set.iterator();
		
		while (iterator.hasNext())  
		{
			String desc=iterator.next().toString();
			buffer.append(desc.toString());
			buffer.append(BASETOKENS.SPACE);
		}
		
		buffer.append(BASETOKENS.CLOSEBRACKET);
		
		return buffer.toString();
	}

	private void parse(Content result_content) throws ParserException {

		if (result_content!=null) {	
			
			String keyword=(String) result_content.getAttribute("binaryTermOp");

			if (keyword==null || !keyword.equals(FIPAMANCONSTANTS.RESULT)) 
			{
				throw new ParserException("Result keyword expected");
			}
			
			Object terms=result_content.getAttribute("term");

			if (! (terms instanceof List)) 
			{
				throw new ParserException("2 items expected");
			}
			
			List items=(List) terms;
			
			Content t1=(Content) items.get(0);
			Content t2=(Content) items.get(1);
			
			Content action=(Content) t1.getAttribute("actionExpression");
			Content results=(Content) t2.getAttribute("set");
			
			_action=new AgentManagementAction(action);
			
			if (results.getAttribute("term") instanceof List) 
			{
				List l=(List) results.getAttribute("term");
				
				for (int i=0;i<l.size();i++) 
				{
					Content r=(Content) l.get(i);
					
					
					_results.add( convertToObject( r ) );
				}
				
			}
			else if (results.getAttribute("term") instanceof Content)
			{
				Content c=(Content) results.getAttribute("term");
				
				_results.add( convertToObject( c ) );
			}
			else if( results.getAttribute("term") == null )
			{
				// Just ignore if there are no results!
			}
			else {
				throw new ParserException("Invalid result set");
			}
		}

	}
	
	
	public Object convertToObject( Content content ) throws ParserException {
		
		Content functionalTerm=(Content) content.getAttribute("functionalTerm");
		
		String functionSymbol=(String) functionalTerm.getAttribute("functionalOperator");
		
		if (functionSymbol.equals(FIPAMANCONSTANTS.DF_AGENT_DESCRIPTION)) 
		{
			return new DFAgentDescription(content);
		}
		else if (functionSymbol.equals(FIPAMANCONSTANTS.AMS_AGENT_DESCRIPTION)) 
		{
			return new AMSAgentDescription(content);
		}
		else
		{
			throw new ParserException( "Unknown description type " + functionSymbol );
		}
	}
	
	
	/** This method is for testing
	 * @param args arguments
	 */
	public static void main(String[] args) 
	{
		String result_string="(result "+
		"(action "+
		"(agent-identifier :name df@foo.com :addresses (sequence iiop://foo.com/acc)) "+
		"(search (df-agent-description :ontology (set meeting-scheduler) :language (set FIPA-SL0 KIF) "+
		":services (set (service-description :name profiling :type meeting-scheduler-service)) "+
		"(search-constraint :max-depth 2)))) "+
		"(set (df-agent-description :name (agent-identifier :name scheduler-agent@foo.com "+
		":addresses (sequence iiop://foo.com/acc)) :ontology (set meeting-scheduler FIPA-Agent-Management) "+
		":languages (set FIPA-SL0 FIPA-SL1 KIF) "+
		":services (set (service-description :name profiling :type meeting-scheduler-service) "+
		"(service-description :name profiling :type user-profiling-service))))) ";
		
		//DIAGNOSTICS.setMemoryLevel( DIAGNOSTICS.LEVEL_MIN );
		
		try
		{
			AgentManagementResult result=new AgentManagementResult(result_string);
			System.out.println(result.getAgentManagementAction());
			System.out.println(result.getResultSet());
			System.out.println(result.toString());
		}
		catch (Throwable e) {
			e.printStackTrace();
		}
	}
}
