//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved.
//
//******************************************************************************

package fipaos.ont.fipa.fipaman;

// -------------------------------------------------------------------------

import java.util.Vector;
import java.util.List;
import java.util.TreeSet;
import java.util.Collections;
import java.util.Iterator;
import java.util.Set;
import fipaos.ont.BASETOKENS;
import fipaos.util.DIAGNOSTICS;
import fipaos.parser.ParserException;
import fipaos.parser.sl.SLParser;
import fipaos.agent.conversation.content.Content;

// -------------------------------------------------------------------------

/**
 * FIPA defined DF-description class for the FIPA agent management ontology.
 * See FIPA 1997 Part 2 Section 9.
 * <br>
 * <br>
 **/
public class DFAgentDescription implements fipaos.skill.db.DatabaseObject, java.io.Serializable
{
	
	//----------------------------------------------------------------------
	// CONSTANTS
	//----------------------------------------------------------------------
	
	//----------------------------------------------------------------------
	// INSTANCE VARIABLES
	//----------------------------------------------------------------------
	
	private AgentID               _agent_name;
	private Set 			_agent_services;
	private Set             	_ontologies;
	private Set             	_languages;
	private Set             	_protocols;
	
	//----------------------------------------------------------------------
	// CONSTRUCTOR
	//----------------------------------------------------------------------
	
	/** This is an empty constructor. Fields can be populated by using the set method.
	 */
	public DFAgentDescription()
	{}
	
	/** This constructor parses the string passed to it to
	 * populate its fields.
	 * @param df_agent_desc the string representing the DF-Agent Description
	 * @throws ParserException if the string is invalid
	 */
	
	public DFAgentDescription( String df_agent_desc ) throws ParserException
	{
		
		try
		{
			//converts it into a content object
			parse(SLParser.parseTerm(df_agent_desc));
		}
		catch (fipaos.parser.sl.ParseException e)
		{
			throw new ParserException(e.getMessage());
		}
	}
	
	/** THis constructor parses the content passed to it to
	 * populate its fields.
	 * @param df_agent_content the string representing the DF-Agent-Description
	 * @throws ParserException if the string is not valid
	 */
	
	public DFAgentDescription( Content df_agent_content ) throws ParserException
	{
		
		parse(df_agent_content);
	}
	
	private void parse(Content df_agent_content) throws IllegalArgumentException, ParserException
	{
		
		Content functional_content=(Content) df_agent_content.getAttribute("functionalTerm");
		
		if (functional_content==null || !functional_content.getAttribute("functionalOperator").equals(FIPAMANCONSTANTS.DF_AGENT_DESCRIPTION))
		{
			throw new IllegalArgumentException("Content is not a DF-AGENT-DESCRIPTION");
		}
		
		if (functional_content.getAttribute("parameter") instanceof List)
		{
			
			List parameters=(List) functional_content.getAttribute("parameter");
			
			for (int i=0;i<parameters.size();i++)
			{
				
				Content content=(Content) parameters.get(i);
				
				if (content.getName()!="parameter")
				{
					throw new IllegalArgumentException("Illegal DF-AGENT-DESCRIPTION FORMAT");
				}
				
				String name=((String) content.getAttribute("parameterName"));
				
				if (name.equals(FIPAMANCONSTANTS.NAME))
				{
					
					String value=(String) ((Content) content.getAttribute("parameterValue")).getAttribute("formattedString");
					
					_agent_name=new AgentID(value);
					
				}
				else if (name.equals(FIPAMANCONSTANTS.AGENT_SERVICES))
				{
					
					_agent_services=createServiceDescriptionSet(content);
					
				}
				else if (name.equals(FIPAMANCONSTANTS.PROTOCOL))
				{
					
					_protocols=createSet(content);
				}
				else if (name.equals(FIPAMANCONSTANTS.ONTOLOGY))
				{
					
					_ontologies=createSet(content);
				}
				else if (name.equals(FIPAMANCONSTANTS.LANGUAGE))
				{
					
					_languages=createSet(content);
				}
			}
		}
		else
		{
			
			Content parameter=(Content) functional_content.getAttribute("parameter");
			
			if( parameter != null )
			{
				String name=((String) parameter.getAttribute("parameterName"));
				
				if (name.equals(FIPAMANCONSTANTS.NAME))
				{
					String value=(String) ((Content) parameter.getAttribute("parameterValue")).getAttribute("formattedString");
					
					_agent_name=new AgentID(value);
					
				}
				else if (name.equals(FIPAMANCONSTANTS.AGENT_SERVICES))
				{
					
					_agent_services=createServiceDescriptionSet(parameter);
					
				}
				else if (name.equals(FIPAMANCONSTANTS.PROTOCOL))
				{
					
					_protocols=createSet(parameter);
				}
				else if (name.equals(FIPAMANCONSTANTS.ONTOLOGY))
				{
					
					_ontologies=createSet(parameter);
				}
				else if (name.equals(FIPAMANCONSTANTS.LANGUAGE))
				{
					
					_languages=createSet(parameter);
				}
			}
		}
	}
	
	private void addKeywordValuePair(StringBuffer buffer, String keyword, String value)
	{
		buffer.append(keyword);
		buffer.append(BASETOKENS.SPACE);
		buffer.append(value);
		buffer.append(BASETOKENS.NEWLINE);
	}
	
	private String convertSetIntoString(Set set)
	{
		StringBuffer buffer=new StringBuffer();
		
		buffer.append(BASETOKENS.OPENBRACKET);
		buffer.append(FIPAMANCONSTANTS.SET);
		buffer.append(BASETOKENS.SPACE);
		
		String value;
		
		Iterator iterator=set.iterator();
		
		while (iterator.hasNext())
		{
			value=(String) iterator.next();
			buffer.append(value);
			
			if (iterator.hasNext())
			{
				buffer.append(BASETOKENS.SPACE);
			}
		}
		
		buffer.append(BASETOKENS.CLOSEBRACKET);
		
		return buffer.toString();
	}
	
	private String convertSDSetIntoString(Set set)
	{
		StringBuffer buffer=new StringBuffer();
		
		buffer.append(BASETOKENS.OPENBRACKET);
		buffer.append(FIPAMANCONSTANTS.SET);
		buffer.append(BASETOKENS.SPACE);
		
		ServiceDescription sd;
		
		Iterator iterator=set.iterator();
		
		while (iterator.hasNext())
		{
			sd=(ServiceDescription) iterator.next();
			buffer.append(sd.toString());
			
			if (iterator.hasNext())
			{
				buffer.append(BASETOKENS.SPACE);
			}
		}
		
		buffer.append(BASETOKENS.CLOSEBRACKET);
		
		return buffer.toString();
	}
	
	/**
	 * Creates a Java set from a SL Set content
	 */
	
	private Set createSet(Content content)
	{
		
		Set set= Collections.synchronizedSet( new TreeSet( fipaos.util.Comparators.OBJECT_COMPARE ) );
		
		Content con=(Content) content.getAttribute("parameterValue");
		
		con=(Content) con.getAttribute("term");
		
		con=(Content) con.getAttribute("set");
		
		if (con==null)
		{
			throw new IllegalArgumentException("Illegal DF-AGENT-DESCRIPTION FORMAT: SET");
		}
		
		if (con.getAttribute("term") instanceof List)
		{
			
			List list=(List) con.getAttribute("term");
			
			int size=list.size();
			
			for (int i=0;i<size;i++)
			{
				
				Content term=(Content) list.get(i);
				
				set.add((String) term.getAttribute("formattedString"));
				
			}
		}
		else
		{
			set.add((String) ((Content) con.getAttribute("term")).getAttribute("formattedString"));
		}
		
		return set;
	}
	
	
	/**
	 * Creates a Java set from a SL Set content
	 */
	
	private Set createServiceDescriptionSet(Content content) throws ParserException
	{
		
		Set set= Collections.synchronizedSet( new TreeSet( fipaos.util.Comparators.OBJECT_COMPARE ) );
		
		Content con=(Content) content.getAttribute("parameterValue");
		
		// Ensure con isn't null
		if( con != null )
		{
			con=(Content) con.getAttribute("term");
			
			// ...and again
			if( con != null )
			{
				con=(Content) con.getAttribute("set");
				
				if (con==null)
				{
					throw new IllegalArgumentException("Illegal DF-AGENT-DESCRIPTION FORMAT: SET");
				}
				
				if (con.getAttribute("term") instanceof List)
				{
					
					List list=(List) con.getAttribute("term");
					
					int size=list.size();
					
					for (int i=0;i<size;i++)
					{
						
						Content term=(Content) list.get(i);
						
						set.add(new ServiceDescription((String) term.getAttribute("formattedString")));
						
					}
				}
				else
				{
					Content con2 = ((Content) con.getAttribute("term"));
					
					// Check con2 actually references something useful!
					if( con2 != null && 
							con2.getAttribute("formattedString") != null )
					{
						ServiceDescription sd = new ServiceDescription((String) con2.getAttribute("formattedString"));
						set.add( sd );
					}
				}
			}
		}
		
		return set;
	}
	
	//----------------------------------------------------------------------
	// GET/SET METHODS
	//----------------------------------------------------------------------
	
	//method in the DatabaseObject interface
	
	/** This method is required by the DatabaseObject interface.
	 * @return String the key for object storage
	 */
	public String getObjectID()
	{
		return _agent_name.getName();
	}
	
	/** This method returns the AgentID.
	 * @return AgentID of the DFAgentDescription
	 */
	public AgentID getAgentID()
	{
		return _agent_name;
	}
	
	/** This method returns the agent id of the DFAgentDescription.
	 * @return AgentID of the DFAgentDescription
	 */
	public String getAgentName()
	{
		return _agent_name.getName();
	}
	
	/** This method returns the set of agent services.
	 * @return Set of agent services
	 */
	public Set getAgentServices()
	{
		return _agent_services;
	}
	
	/** This method returns the number of elements of data in this DFAgentDescription
	 * @return int no.of elements in the description
	 */
	
	public int getNumberofDataElements()
	{
		
		int i = 0;
		
		if(_agent_name != null)
			i++;
		
		if(_agent_services != null)
			i++;
		
		if (_languages != null)
			i++;
		
		if(_protocols != null)
			i++;
		
		if(_ontologies != null)
			i++;
		
		return i;
	}
	
	/** This method returns the ontologies of the DFAgentDescription.
	 * @return Set of ontologies
	 */
	public Set getOntologies()
	{
		return _ontologies;
	}
	
	/** This method returns the set of protocols.
	 * @return set of protocols
	 */
	public Set getProtocols()
	{
		return _protocols;
	}
	
	/** This method returns the set of languages.
	 * @return set of languages
	 */
	public Set getLanguages()
	{
		return _languages;
	}
	
	/** This method sets the agent id of the description.
	 * @param df_desc_item AgentID of the description
	 */
	public void setAgentName(AgentID df_desc_item)
	{
		_agent_name = df_desc_item;
	}
	
	/** This method sets the agent services.
	 * @param df_desc_item set of agent services
	 */
	public void setAgentServices(Set df_desc_item)
	{
		_agent_services = df_desc_item;
	}
	
	/** This method sets the array of service descriptions.
	 * @param df_desc_item array of service descriptions
	 */
	public void setAgentServices(ServiceDescription[] df_desc_item)
	{
		
		Set set=new TreeSet( fipaos.util.Comparators.OBJECT_COMPARE );
		
		for (int i=0;i< df_desc_item.length;i++)
		{
			set.add(df_desc_item[i]);
		}
		
		_agent_services = set;
	}
	
	/** This method sets the protocols .
	 * @param df_desc_item set of protocols
	 */
	public void setProtocols(Set df_desc_item)
	{
		_protocols = df_desc_item;
	}
	
	/** This method sets the protocols
	 * @param df_desc_item String array of protocols
	 */
	public void setProtocols(String[] df_desc_item)
	{
		
		Set set=new TreeSet( fipaos.util.Comparators.OBJECT_COMPARE );
		
		for (int i=0;i< df_desc_item.length;i++)
		{
			set.add(df_desc_item[i]);
		}
		
		_protocols = set;
	}
	
	/** This method sets the ontologies.
	 * @param df_desc_item set of ontologies
	 */
	public void setOntologies(Set df_desc_item)
	{
		_ontologies = df_desc_item;
	}
	
	/** This method sets the ontologies.
	 * @param df_desc_item string array of ontologies
	 */
	public void setOntologies(String[] df_desc_item)
	{
		
		Set set=new TreeSet( fipaos.util.Comparators.OBJECT_COMPARE );
		
		for (int i=0;i< df_desc_item.length;i++)
		{
			set.add(df_desc_item[i]);
		}
		
		_ontologies = set;
	}
	
	/** This method sets the languages.
	 * @param df_desc_item set of languages
	 */
	public void setLanguages(Set df_desc_item)
	{
		_languages = df_desc_item;
	}
	
	/** This method sets the languages
	 * @param df_desc_item string array of languages
	 */
	public void setLanguages(String[] df_desc_item)
	{
		
		Set set=new TreeSet( fipaos.util.Comparators.OBJECT_COMPARE );
		
		for (int i=0;i< df_desc_item.length;i++)
		{
			set.add(df_desc_item[i]);
		}
		
		_languages = set;
	}
	
	/** This method returns an DF-Agent-description as a string
	 * @return string representation of DFAgentDescription
	 */
	
	public String toString()
	{
		
		StringBuffer buffer=new StringBuffer();
		buffer.append(BASETOKENS.OPENBRACKET);
		buffer.append(FIPAMANCONSTANTS.DF_AGENT_DESCRIPTION);
		buffer.append(BASETOKENS.NEWLINE);
		
		if (_agent_name !=null )
		{
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.NAME,_agent_name.toString());
		}
		
		if (_agent_services !=null )
		{
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.AGENT_SERVICES,convertSDSetIntoString(_agent_services));
		}
		
		if (_protocols !=null )
		{
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.PROTOCOL,convertSetIntoString(_protocols));
		}
		
		if (_ontologies !=null )
		{
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.ONTOLOGY,convertSetIntoString(_ontologies));
		}
		
		if (_languages !=null )
		{
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.LANGUAGE,convertSetIntoString(_languages));
		}
		
		buffer.append(BASETOKENS.CLOSEBRACKET);
		
		return buffer.toString();
		//}
	}
	
	//----------------------------------------------------------------------
	// TEST METHOD
	//----------------------------------------------------------------------
	
	/** This is used for testing purposes.
	 * @param args Arguments
	 */
	public static void main (String[] args)
	{
		
		// test out the parsing of a full df description
		//String df_agent_desc = "(df-agent-description "+
		//                 ":name (agent-identifier :name dummy@foo.com :addresses (sequence iiop://foo.com/acc )) )";
		/*               ":services "+ "(set (service-description :name profiling :type user-profiling)) "+
		":ontology (set fixed-properties fxprops) "+
		":protocol (set FIPA-Request Application-Protocol) "+
		":language (set KIF FIPA))";*/
		String df_agent_desc="( df-agent-description :services ( set ( service-description :type ping-agent  )  )  )";
		
		try
		{
			DFAgentDescription desc=new DFAgentDescription(df_agent_desc);
			
			System.out.println(desc.getAgentServices());/*
			System.out.println(desc.getAgentServices().iterator().next());
			System.out.println(desc.getOntologies().iterator().next());
			System.out.println(desc.getProtocols().iterator().next());
			System.out.println(desc.getLanguages().iterator().next());*/
			
			desc=new DFAgentDescription();
			desc.setAgentName(new AgentID("(agent-identifier :name dummy@foo.com :addresses (sequence iiop://foo.com/acc ))"));
			
			ServiceDescription[] sd=new ServiceDescription[1];
			sd[0]=new ServiceDescription("(service-description :name profiling :type user-profiling)");
			
			desc.setAgentServices(sd);
			
			String[] pts=new String[2];
			pts[0]="KIF";
			pts[1]="SL";
			
			desc.setLanguages(pts);
			
			System.out.println(desc);
			System.out.println(desc.getAgentServices());
		}
		catch (ParserException e)
		{
			e.printStackTrace();
		}
		
		//fipaos.util.DIAGNOSTICS.println("\nConstructing next DFDescription with String: \n" + s1);
		//fipaos.util.DIAGNOSTICS.println((new DFDescription(s1)).toString());
		
	}
	//----------------------------------------------------------------------
}
