//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.parser.acl.bitefficient;

import fipaos.ont.fipa.ACL;
import fipaos.ont.fipa.fipaman.*;

import java.io.*;
import fipaos.util.DIAGNOSTICS;

/**
 * Class to provide simplified API to make use of BitEfficient ACL encoding
 */
public class Parser
{
	/**
	 * Convert the given ACL message into bit-efficient format
	 * @param msg The message to convert
	 * @return The byte[] representing the ACL msg, or null
	 */
	public static byte[] deparse( ACL msg )
	{
		try
		{
			// Create BAOS to hold output of ACLOutputStream
			ByteArrayOutputStream baos = new ByteArrayOutputStream();

			// Attempt to deparse
			ACLOutputStream out = new ACLOutputStream( baos );
			out.write( msg );
			out.flush();
			baos.flush();
			baos.close();
			
			// Return deparsed ACL
			return baos.toByteArray();
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "Problem occured when deparsing message", t );
			return null;
		}
	}
	
	
	/**
	 * Convert the given byte[] message into an ACL
	 * @param msg byte[] representing the ACL msg, or null
	 * @return The message to convert 
	 */
	public static ACL parse( byte[] msg )
	{
		ACL acl = null;
		
		try
		{
			// Create BAOS to hold input of ACLInputStream
			ByteArrayInputStream bais = new ByteArrayInputStream( msg );

			// Attempt to parse
			ACLInputStream in = new ACLInputStream( bais );
			acl = in.readMsg();
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "Problem occured when deparsing message", t );
		}
		
		return acl;
	}
	
	
	/**
	 * Test harness - allows byte[] representation of an ACL message to be
	 * produced given an ACL message (assumed to be taken from STD-IN) 
	 */
	public static void main( String args[] ) throws Exception
	{
		System.err.println( "Reading ACL from stdin..." );
		ACL acl = new ACL( ( new BufferedReader( new InputStreamReader( System.in ) ) ).readLine() );
		
		byte acl2[] = deparse( acl );

		System.err.println( "Writing byte[] to stdout..." );
		System.out.println( new String( acl2 ) );
	}
}