//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.parser.rdf;

//----------------------------------------------------------------------

import fipaos.util.DIAGNOSTICS;
import fipaos.ont.fipa.FIPACONSTANTS;
import fipaos.ont.BASETOKENS;
import fipaos.ont.fipa.fipaman.FIPAMANCONSTANTS;
import fipaos.parser.xml.XMLContentHandler;
import fipaos.agent.conversation.content.Content;
import org.xml.sax.Parser;
import org.xml.sax.helpers.ParserFactory;
import org.xml.sax.SAXException;
import java.io.StringReader;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Map;
import java.util.TreeMap;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import org.w3c.rdf.*;
import org.xml.sax.*;
import java.util.Set;
import java.util.Iterator;
import java.util.ListIterator;
import java.util.List;
import java.util.Collections;
import java.util.Hashtable;
import java.util.ArrayList;
import java.util.LinkedList;
import java.lang.Integer;
import java.util.StringTokenizer;

//import fipaos.util.neg.negEngine.DebugLogFile;



//----------------------------------------------------------------------

/**
 * This class provides mechanisms for parsing/deparsing RDF to/from
 * Content object containers, using SiRPACHandler
 *
 * If the system property fipaos.parser.xmlparser is set, the classname
 * it contains will be used as an XML parser (providing it uses the SAX
 * 1.0 interface).
 * <br>
 * <br>
 * @deprecated Content objects should be avoided - please use XML Databinding instead
 * @see fipaos.util.XMLDataBinding*/
public class RDFContentHandler implements fipaos.parser.Parser
{
//----------------------------------------------------------------------
// CONSTANTS
//----------------------------------------------------------------------
	public static final String  RDF_HEADER	= "rdf:RDF";
	public static final String  STARTTAG 		= "<";
	public static final String 	ENDTAG 			= ">";
	public static final String 	HEADER 			= XMLContentHandler.XML_HEADER + "\n<" + RDF_HEADER + " xml:lang=\"en\" ";
	public static final String 	FOOTER 			= "</rdf:RDF>";
	public static final String  RDFSYNTAX 	=  "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
	public static final String  RDFSCHEMA 	=  "http://www.w3.org/1999/02/PR-rdf-schema-19990303#";
	public static final String  RDFABOUT		= "rdf:about";
	public static final String  RDFRESOURCE	= "rdf:resource";	
	public static final String	RDF					= "rdf";
	public static final String	RDFS				=	"rdfs";
	public static final String	RDFNAME			=	"xmlns:rdf=";
	public static final String	RDFSNAME		=	"xmlns:rdfs=";
	public static final String	XMLNS				= "xmlns:";
	public static final String PARSE_TYPE_LITERAL = " parseType=\"Literal\"";

//----------------------------------------------------------------------
// STATIC VARIABLES
//----------------------------------------------------------------------

 
//----------------------------------------------------------------------
// INSTANCE VARIABLES
//----------------------------------------------------------------------

	/**
	 * FIXME: This should be removed one the Parser class has been written
	 *
	 * Reference to the DocumentRepository to use
	 */
	private RDFDocument _document;


//----------------------------------------------------------------------
// CONSTRUCTOR
//----------------------------------------------------------------------

	/**
	 * Initialises the Parser, ready to parse and deparse RDF documents
	 *
	 */
	public RDFContentHandler()
	{
		RDFDocument document = new RDFDocument();
	
	}
	static
	{
			Hashtable array_of_namespaces = new Hashtable();
	}


//----------------------------------------------------------------------
// PUBLIC METHODS
//----------------------------------------------------------------------	

	/**
	 * Attempts to parse the given content, and produce a Content object
	 * to represent it.
	 *
	 * @param content The RDF document to be parsed
	 *
	 * @return A Content object representing the given RDF document, null if
	 *         parsing failed.
	 */
	public Content parse( String content )
	{
		SiRPAC handler = null;
		Enumeration triples = null;

		// Find out which XML parser to use
		String xml_parser = System.getProperty( "fipaos.parser.xmlparser", "org.apache.xerces.parsers.SAXParser" );
		
		try
		{
			// Create the SiRPAC handler
			handler = new SiRPAC( xml_parser );
			
			// Disable fetching of schemas (TEMPORARY!)
			handler.fetchSchemas( false );
			
			// Set the source for parsing
			handler.setRDFSource( new RDFSource( content ) );
			
			// Start parsing
			handler.fetchRDF();
		}
		catch( Throwable e )
		{
			// Check if SiRPAC was instantiated correctly
			if( handler == null )
			{
				DIAGNOSTICS.println( "An error occured whilst creating the SiRPAC RDF handler: ",
																								e, this, DIAGNOSTICS.LEVEL_MAX );
				return null;
			}
		}

		// Get the errors SiRPAC has produced, and display them
		String errors = handler.errors();
		if( errors != null && errors.length() > 0 )
		{
			DIAGNOSTICS.println( "The following errors ocurred during parsing:\n" +
														handler.errors(), this, DIAGNOSTICS.LEVEL_MAX );
			return null;
		}
		
		return createContent( handler.triples() );
	}
	
	
	/**
	 * Attempts to deparse the given content, and produce a RDF document
	 * to represent it.
	 * Works by using a vector of attributes objects. It keeps on getting 	 
	 * more attributenames to get more attributes untill it find primitives
	 * Then repackages according to rdf SYNTAX
	 *
	 * @param content The content to be deparsed
	 *
	 * @return A RDF document representing the given Content object, null if
	 *         parsing failed.
	 */
	public String deparse( Content content )
	{		
		LinkedList array_of_triples = new LinkedList();
		LinkedList array_of_tags = 	new LinkedList();		
	
		RDFDocument document = new RDFDocument();
		
		//make a list of triples		!!!fix it for only content  unpack and ignore primitives.
		array_of_triples = getTriples( array_of_triples, content );
		
		
		//Iterator it2 = array_of_triples.iterator();
		
		
		//System.out.println("FOUND " + array_of_triples.size() + " Content objects");		 
			
		//make name space	
		Hashtable namespace_list = new Hashtable();
		namespace_list.put(BASETOKENS.QUOTES + RDFSYNTAX + BASETOKENS.QUOTES,RDF);
		namespace_list.put(BASETOKENS.QUOTES + RDFSCHEMA + BASETOKENS.QUOTES,RDFS);	
		
		String tags = null;		
		String end	= null;
		
		ListIterator p 				= array_of_triples.listIterator();
		String tag_namespace 	= null;	
		String object 				= null;
		String predicate  		= null;
		String subject 				= null;
		int count 						= 0;
		
		while(p.hasNext()) {				 				
			
			Content xt = ( Content )p.next();
			
			subject = xt.getName();
			
			Set st = xt.getAttributeNames();
			
			Iterator i = st.iterator();
			
			int tag_size = st.size()+1;
			
			LinkedList interim = new LinkedList();	
			LinkedList bubble = new LinkedList();
			
			//for each attribute name get the name of the object
			// object could be a content object part of a list or a primitive 
			while(i.hasNext()) {
				
				count++;		
				
				//run through the set of predicates and get predicate and object for the content object			
				predicate = (String)i.next();
				
				Object obj = xt.getAttribute(predicate);
				
				String predicateURI = document.getNameSpaceURI(predicate);
				
				//Having got the object what type of object is it?
				if( obj instanceof Content ) {					
					
					//if the object is a content object determine what is its domain
					Content new_cont 	= (Content)obj;
					object 						= new_cont.getName();
					
					//System.out.println("OBJECT=" + object);
					//System.out.println("predicateURI=" + predicateURI);
					
					String objectValue 	= null;
					String objectURI 		= null;					
										
					//is object URI
					if(isURI(object)) {
						
						objectValue = document.getSubString(object);
						objectURI = document.getNameSpaceURI(object);
												
					}		
					//an outer tag has RDFSYNTAX as predicate
					if(isOuterTag(predicateURI)) {
						
							//get namespace					
						namespace_list = document.makeNameSpace(namespace_list,objectURI,count);
					
						tag_namespace = document.getNameSpace(namespace_list,objectURI);	
						
						tags = createOuterTag(tag_namespace,subject,predicate,objectValue);
						
						end = document.getEndTag(tags);	

						interim.add(0, tags);

					}
					else if(isMiddleTag(subject, object)) {
						

						namespace_list = document.makeNameSpace(namespace_list,predicateURI,count);
						tag_namespace = document.getNameSpace(namespace_list,predicateURI);

						tags = createMiddleTag(tag_namespace,subject,predicate,object);

						bubble.addLast(tags);

					}
					else {
						
						namespace_list = document.makeNameSpace(namespace_list,predicateURI,count);
						
						tag_namespace = document.getNameSpace(namespace_list,predicateURI);
						
						tags = createInnerTag(tag_namespace,subject,predicate,object);
				
						bubble.addFirst(tags);	
						
					}	
					
				}
				else if ( obj instanceof List )	{
					
					List list = ( List )obj;
					Iterator j = list.iterator();
					
					int elem = 0;
										
					while( j.hasNext() ) {

						elem++;

						Object list_item = ( Object )j.next();
						
						if (list_item instanceof Content) {
												
							Content new_cont = ( Content )list_item;
							
							object = new_cont.getName();	
							
							String objectValue = null;
							
							String objectURI = null;		
							
							if(isURI(object))	{
								
								objectValue = document.getSubString(object);
								objectURI = document.getNameSpaceURI(object);
								
							}
								
							if(isOuterTag(predicateURI)) {
															
								//get namespace					
								namespace_list = document.makeNameSpace(namespace_list,objectURI,count);
								
								tag_namespace = document.getNameSpace(namespace_list,objectURI);	
								
								tags = createOuterTag(tag_namespace,subject,predicate,objectValue);
								
								end = document.getEndTag(tags);		
								
								interim.add(0, tags);

							}
							else if(isMiddleTag(subject,object)) {
								
								namespace_list = document.makeNameSpace(namespace_list,predicateURI,count);
																
								tag_namespace = document.getNameSpace(namespace_list,predicateURI);
								
								tags = createMiddleTag(tag_namespace,subject,predicate,object);
								
								bubble.addLast(tags);
								
							}
																
						} // end if content in a list
						else if (isObjectPrimitive(list_item)) {
														
							namespace_list = document.makeNameSpace(namespace_list,predicateURI,count);
							tag_namespace = document.getNameSpace(namespace_list,predicateURI);
							object = primitiveToString( list_item );
							tags = createInnerTag(tag_namespace,subject,predicate,object);

							bubble.addFirst(tags);
							
						}
						else {
							
							//System.out.println("ITEM " + elem + " is an instance of ??????");
														
						}
								
					} // end while j list
					
				}
				else {
										
					namespace_list = document.makeNameSpace(namespace_list,predicateURI,count);
					
					tag_namespace = document.getNameSpace(namespace_list,predicateURI);
					
					object = primitiveToString( obj );
					
					tags = createInnerTag(tag_namespace,subject,predicate,object);

					//add just after outer tag				
					bubble.addFirst(tags);	
				}	
				
			} // end inner while
				
				interim.addAll(bubble);
				
				if (end != null) {
				
					interim.addLast(end);
				
				}
													
				array_of_tags.addLast(interim);		
				
		}	// end outer while
			
		StringBuffer result_buffer = new StringBuffer();
		String result;
		
		Iterator k = array_of_tags.iterator();
	
		Object next_item;
		
		while(k.hasNext()) {
			
			next_item = k.next();
			
			if(next_item instanceof String) {
				
				result_buffer.append((String)next_item);
				
			}
			else if (next_item instanceof LinkedList) {
				
				Iterator it = ((LinkedList)next_item).iterator();
				
				while (it.hasNext()) {
					
					result_buffer.append((String)(it.next()));
					result_buffer.append(BASETOKENS.NEWLINE);					
					
				}
				
			}
			//debug ****************************************
			else if( next_item instanceof Content )
			{
				
				//System.out.println("Content Objects in the Linked List !!!");
				result_buffer.append(((Content)next_item).toString());

			}
			else {
				
				//System.out.println("Object instance not of type String, LinkedList");
				
			}
			
			result_buffer.append(BASETOKENS.NEWLINE);				
		}		
		
		StringBuffer final_result = new StringBuffer();
		
		final_result.append(HEADER);
		final_result.append(document.makeNameSpaceHeader(namespace_list));
		final_result.append(BASETOKENS.NEWLINE);
		final_result.append(result_buffer);
		final_result.append(FOOTER);		

		return final_result.toString();
	
	}
	/* This method unpacks the content object
	 * Makes sure the that the subject of each object is unique
	 * Adds them to a list
	 *
	 */	
		
	public LinkedList getTriples(LinkedList array_of_triples, Content content)
	{	
				
		String content_name = content.getName();		
		Set st = content.getAttributeNames();
		Iterator i = st.iterator();
		
		Content new_cont = new Content();
		while( i.hasNext () )
		{	
			String next_content_attr = (String) i.next();
			Object new_object = content.getAttribute(next_content_attr);
			if(new_object instanceof Content)
			{
				new_cont = ( Content ) new_object;	
				if(	!array_of_triples.contains(	new_cont ))
				{
					if (!((new_cont.getAttributeNames()).isEmpty())) {			
					
						array_of_triples.addLast(new_cont);			
						
						getTriples(array_of_triples,new_cont);		
					
					} // end if this content has some attributes
						
				}
				else {
					
					//System.out.println("NOT ADDING DUPLICATE CONTENT " + new_cont);
										
				} // end if Content already in array_of_triples
				
			} // end if instanceof Content
			else if (new_object instanceof List)
			{
				
				List new_list = ( List )new_object;
				Iterator j = new_list.iterator();
				
				//System.out.println("TRIPLE LIST SIZE=" + new_list.size());
				
				while( j.hasNext() )
				{
					Object list_item = j.next();
					if (list_item instanceof Content)
					{	
						
						new_cont = ( Content ) list_item;	
						
						if(	!array_of_triples.contains(	new_cont ))
						{						
							
							//System.out.println("ADDING LIST ELEMENT=" + new_cont);
							
							
							array_of_triples.addLast(new_cont);
							
							getTriples(array_of_triples,new_cont);							
						
						}
						else {
							
							//System.out.println("NOT ADDING DUPLICATE LIST CONTENT " + new_cont);
							
						} // end if object already in array_of_triples
							
					} // end if instanceof Content
						
				} // end while j.hasNext()
					
			} // end else if instance of list
			
			//stop condition
			else
			{
				//fipaos.util.DIAGNOSTICS.println("object is not of correct type " + new_object.toString());
			}
					
		}
		return array_of_triples;
	}


//----------------------------------------------------------------------
// PARSER INTERFACE METHODS
//----------------------------------------------------------------------

	/**
	 * Determines if the Parser implementation can parse the given content
	 *
	 * @param A content String.
	 *
	 * @return True if the Parser implementation can parse the given String
	 */
	public boolean canParse( String content )
	{
		// Return result
		return XMLContentHandler.isXML( content ) && isRDF( content );
	}


	/**
	 * Returns the type of content language that the Parser implementation
	 * handles.
	 *
	 * @return A String representation of the content format (e.g. RDF, SL)
	 */
	public String getParserType()
	{
		return FIPACONSTANTS.RDF;	
	}


//----------------------------------------------------------------------
// PUBLIC STATIC METHODS
//----------------------------------------------------------------------	
	
	public static boolean isRDF( String content )
	{
		boolean found_rdf_header = false;
		boolean not_rdf_header = false;
		boolean found_xml_header = false;
		final String ob = "<";
		final String cb = ">";
		final String comment = "--";

		// Find the first valid tag and check if this is an RDF document
		StringTokenizer st = new StringTokenizer( content, "<>", true );

		// Tokenise the document and check if the first valid tag (i.e. not a commment)
		// is the RDF header
		while( ! ( found_rdf_header || not_rdf_header ) && st.hasMoreTokens() )
		{
			String token = st.nextToken();

			if( token.equals( ob ) )
			{
				// Open bracket!  Get the tag content
				String tag = st.nextToken();

				// Ignore this if its just the XML header
				if( !found_xml_header )
				{
					// Skip XML header
					st.nextToken();
					st.nextToken();
					found_xml_header = true;
				}
				else if( ! (tag.startsWith( comment ) && tag.endsWith( comment ) ) )
				{
					// Ignore if this is a comment
					if( tag.startsWith( RDF_HEADER ) )
					{
						// Found the RDF header!
						found_rdf_header = true;
					}
					else
					{
						// Found some other tag!  This isn't an RDF document
						not_rdf_header = true;
					}
				}
				else
				{
					// Skip close bracket
					st.nextToken();
				}
			}
		}
		
		return found_rdf_header;
	}
	
	
//----------------------------------------------------------------------
// PRIVATE METHODS
//----------------------------------------------------------------------

	/**
	 * Method to convert RDF Triples into Content objects
	 *
	 * @param enum An Enumeration of the triples to be converted into
	 *             a Content object
	 *
	 * @return A Content object representing the Triples given
	 */
	private static Content createContent( Enumeration enum )
	{
		// Create a map of all of the Content objects we have
		Map content_table = new TreeMap( fipaos.util.Comparators.STRING_COMPARE );
		Vector refered = new Vector();
		        
    // System.out.println("Triples =========================");
            
		// Iterate through all of the Triples
		while( enum.hasMoreElements() )
		{
			// Get the current Triple
			Triple current = (Triple) enum.nextElement();
			
/*      System.out.println("=================================");
      
      System.out.println( "SUBJECT: " + current.subject() );
      System.out.println( "OBJECT : " + current.object() + " " + 
      										current.object().getClass().getName() ) ;
      System.out.println( "PRED   : " + current.predicate() );
 */
			dealWithTriple( current, content_table, refered );
		}
    
		// Remove unecessary Content objects from the list
		remove( content_table, refered );
		
		return new Content( content_table );
	}


	/**
	 * Method to Create and connect Content objects given the
	 * Triple passed, and connect them together
	 */
	private static void dealWithTriple( Triple current, Map contents, Vector refered ) {
		
		// Get the object, predicate and subject of the Triple
		Object sub = current.subject();
		Object obj = current.object();
		Object pre = current.predicate();
			
		// Reference to the subject Content object
		Content content_sub = null;

		// Reference to the object of this triple (if one is required)
		Object refered_obj = null;
		
		// Get a Content object for the given subject
		content_sub = getContent( sub.toString(), contents );
		
		// Decide if the object is actually a Content object
		// or a primitive type
		if( isPrimitive( obj ) )
		{
			// Get the primitive type
			refered_obj = getPrimitive( obj );	
		}
		else
		{
			// Get/create the Content type
			refered_obj = getContent( obj.toString(), contents );
			
			// Add to refered list if it isn't already present
			if( !refered.contains( obj.toString() ) )
			{
				refered.addElement( obj.toString() );
			}
			
		}
		
		// Add the link into the Content object
		content_sub.addAttribute( pre.toString(), refered_obj );
		
	}


	/**
	 * Method to determine if the given object is a primitive type
	 */
	private static boolean isPrimitive( Object o )
	{
		return ( getPrimitive( o ) != null );
	}


	/**
	 * Method to convert the given object into a primitive type
	 */
	private static Object getPrimitive( Object o )
	{
		// Check if we're dealing with a Literal
		if( o instanceof Literal )
		{
			// Attempt to parse the value into a primitive type

			try
			{
				Long l = new Long( o.toString() );
				return l;	
			}
			catch( Exception e )
			{
			}			

			try
			{
				Double d = new Double( o.toString() );
				return d;	
			}
			catch( Exception e )
			{
			}

			return o.toString();
		}
		// Deal with special case where XML parser churns out a resource reference to
		// "genid.." when an attribute isn't asigned a value!
		// We don't want to do this!
/*		else if( o instanceof Resource && ( o.toString().startsWith("genid") ) )
		{
			return "";
		}
*/		else
		{
			return null;	
		}		
	}


	/**
	 * Method to get/create a Content object from the given Map
	 */
	private static Content getContent( String name, Map all )
	{
		// Find if there is already a content object for the
		// given subject
		if( all.containsKey( name ) )
		{
			return (Content) all.get( name );
		}
		else
		{
			Content sub = new Content( name );
			all.put( name, sub );
			return sub;
		}		
	}


	/**
	 * Method to remove the elements from the given Vector
	 * from the given map
	 */
	private static void remove( Map map, Vector to_remove )
	{
		// Iterate through the elements of the Vector,
		// and remove them from the Map
		Enumeration enum = to_remove.elements();
		while( enum.hasMoreElements() )
		{
			// Get next element
			String next = (String) enum.nextElement();
			
			// Remove it from the Map	
			map.remove( next );
		}
	
	}
	
	/**
	 * Method to to test if object is a primitive	
	 */

	private boolean isObjectPrimitive( Object obj)
	{
		boolean result = false;
		if(primitiveToString( obj ) != null)
		{
			 result = true;
		}
		
		return result;
	}
	
	/**
	 * Method to convert primtive object to string
	 * 
	 */

	private String primitiveToString( Object obj )
	{
		String primitive_string = null;
		Object local_object = null;
		
		if( obj instanceof Long)
		{
			local_object = new Long( ( (Long) obj ).longValue() );
			primitive_string = local_object.toString();
		}
		else if( obj instanceof Double )
		{
			local_object = new Double( ( (Double) obj ).doubleValue() );
			primitive_string = local_object.toString();
		}
		else if( obj instanceof String )
		{
			local_object = new String( (String) obj );
			primitive_string = (String) local_object;
		}
		else
		{
			primitive_string = null;				
		}
		return primitive_string;
	}
	
	/**
	 * Method to check object is inner tag and create tag 
	 */
	
	private String createInnerTag(String namespace, String subject, String predicate, String object)
	{
		String tag_level = "inner";
		RDFDocument document = new RDFDocument();
		String tag = document.makeTag(tag_level, namespace, subject,predicate,object);
		return tag;
	}
	
	/**
	 * Methods to check object is outer tag and create tag 
	 */
	private boolean isOuterTag(String predicate)
	{
		boolean result = false;
		if(predicate.equals(BASETOKENS.QUOTES+RDFSYNTAX+BASETOKENS.QUOTES))
		{
			result = true;
		}
	return result;
	}	
	
	private String createOuterTag(String namespace, String subject, String predicate, String object)
	{
		String tag_level = "outer";
		RDFDocument document = new RDFDocument();						
		String tag = document.makeTag(tag_level, namespace, subject,predicate,object);
		return tag;
	}
	
	/**
	 * Methods to check object is middle tag and create tag 
	 */
	 
	private boolean isMiddleTag(String subject, String object)
	{
		boolean result = false;
		
		if(!subject.equals(object))
		{
			result = true;
		}
		else if (object.indexOf(BASETOKENS.QUOTES+RDFSCHEMA+BASETOKENS.QUOTES) != -1) {
			
			result = true;
			
		}
		
	return result;
	}	


	private String createMiddleTag(String namespace, String subject, String predicate, String object)
	{
		RDFDocument document = new RDFDocument();	
		String tag_level = "middle";
		String tags = document.makeTag(tag_level, namespace, subject,predicate,object);
		return tags;
	}
	
	/**
	 * Method to check whether string is a URI 
	 */
	
	private boolean isURI(String str)
	{
		boolean result = false;
		if(str.indexOf("http://") != -1)
		{
			result = true;
		}
		return result;
	}
		

//----------------------------------------------------------------------
// INNER CLASSES
//----------------------------------------------------------------------
public class RDFDocument
{
		public RDFDocument ()	{}			
		
		public String getDocument( String str, Vector subsections )
		{
			return null;
		}		
	
		private String makeTag(String tagtype, String namespace, String subject, String predicate, String object)
		{	
			String obj_value = null;
			String tag = null;		
			String end_bit = null;	
		
			end_bit = getSubString(predicate);		
							
			if(tagtype.equals("outer"))
			{					
				
				//if the object is a URI
				if(object.startsWith("http://"))
				{
					obj_value = getSubString(object);			
				}
				else
				{
					obj_value = object;
				}
								
				tag = getOuterTag(namespace, obj_value, subject, RDFABOUT);				
			} 		
			else if(tagtype.equals("middle"))
			{
				tag = getMiddleTag(namespace, end_bit, object, RDFRESOURCE);	
			}
			else
			{				
				tag = getInnerTag(namespace, end_bit, object);
			}				
			return tag;
		}
		
		private String getMiddleTag(String numerate, String end_bit,String obj_value, String about)
		{
			String tag = BASETOKENS.TAGOPEN + numerate + BASETOKENS.COLON + end_bit + BASETOKENS.SPACE + about + "=" + BASETOKENS.QUOTES + obj_value + BASETOKENS.QUOTES + BASETOKENS.SLASH + BASETOKENS.TAGCLOSE;
			return tag;
		}

		
		private String getInnerTag(String numerate, String end_bit,String obj_value)
		{
			// Construct the "open" tag
			StringBuffer inner_tag = new StringBuffer();
			inner_tag.append( BASETOKENS.TAGOPEN );
			inner_tag.append( numerate );
			inner_tag.append( BASETOKENS.COLON );
			inner_tag.append( end_bit );
			inner_tag.append( BASETOKENS.TAGCLOSE ); 


			// Construct whole tag + content
			String inner_tag_string = inner_tag.toString();
			StringBuffer tag = new StringBuffer();
			tag.append( inner_tag_string );
			tag.append( XMLContentHandler.getNormalisedForm( obj_value ) );
			tag.append( getEndTag( inner_tag_string ) );
			
			return tag.toString();
		}
		
		
		private String getOuterTag(String numerate, String end_bit,String obj_value, String about)
		{
			String tag = BASETOKENS.TAGOPEN + numerate + BASETOKENS.COLON + end_bit + BASETOKENS.SPACE + about + "=" + BASETOKENS.QUOTES + obj_value + BASETOKENS.QUOTES+BASETOKENS.TAGCLOSE;
			return tag;
		}


		private String getSubString(String att_name)
		{	
			String end_bit = att_name.substring(att_name.indexOf("#")+1,att_name.length());
			return end_bit;
		}
		
		private String getNameSpaceURI(String predicate)
		{
			
			String sub = BASETOKENS.QUOTES+ predicate.substring(0, predicate.indexOf("#")+1)+ BASETOKENS.QUOTES;
			return sub;
		}

		
		private String getEndTag(String tag)
		{
			
			int endpoint = tag.length() - 1;
			
			int about_pos = tag.indexOf(RDFABOUT);
			
			int space_pos = tag.indexOf(" ");
			
			if ( about_pos != -1) {
				
				endpoint = space_pos;
				
			}
			
			String end_tag = BASETOKENS.TAGOPEN + BASETOKENS.FORWARDSLASH + tag.substring(1, endpoint) + BASETOKENS.TAGCLOSE;			
			
			//System.out.println("end tag = '" + end_tag + "'");
			
			return end_tag;
		}
		
		private String getNameSpace(Hashtable hash, String str)
		{			
			String namespace = (String) hash.get(str);
			return namespace;
		}
		
		//iterate throgh list of triples make a list of prediates
		//compare the predicates with the one in prediacte list
		//if predicate has not occured before it must be a new URI
		//add it to output variable  and assign a unique name space
		private Hashtable makeNameSpace(Hashtable namepace_list, String string, int count)
		{
		 //System.out.println(string);		
		
				if(!namepace_list.containsKey(string))				
				{										
					if(string != null)
					{										
						String namespace_name = "ns" + count;							
						namepace_list.put(string,namespace_name);						
					}
					else
					{
						//System.out.println("uri is null ");	
					}
				
			}	
			else
			{
				//System.out.println("key  is already in ");
			}	
			return namepace_list;
		}
			
		private String makeNameSpaceHeader(Hashtable namespace_list) {
			
			StringBuffer ns_header_buffer = new StringBuffer();
			
			Set keysset = namespace_list.keySet();
			
			Iterator i = keysset.iterator();
			
			while(i.hasNext()) {
				
				String key  = (String)i.next();
				
				String value = (String)namespace_list.get(key);
				
				ns_header_buffer.append(XMLNS);
				ns_header_buffer.append(value);
				ns_header_buffer.append(BASETOKENS.EQUALS);
				ns_header_buffer.append(key);
							
				if (i.hasNext()) {
					
					ns_header_buffer.append(BASETOKENS.NEWLINE);
					 
				}
				
			}	
			ns_header_buffer.append(BASETOKENS.TAGCLOSE);		
			
			return ns_header_buffer.toString();
			
		}

	}
	
				

//----------------------------------------------------------------------
// MAIN/TESTING METHOD
//----------------------------------------------------------------------

	public static void main( String args[] ) throws IOException
	{
		if( args.length <= 0 )
		{
			fipaos.util.DIAGNOSTICS.println("Usage: java fipaos.parser.rdf.RDFContentHandler [<filename>]");
			System.exit(-10);
		}
			
		// Read in the given file
		BufferedReader fin = new BufferedReader( new FileReader( args[0] ) );
		
		// String to build up input
		StringBuffer buffer = new StringBuffer();
			
		String next = fin.readLine();
		
		while( next != null )	{
			
			buffer.append(next);
			buffer.append("\n");
			
			next = fin.readLine();	
			
		}
		
		// Close the File
		fin.close();
		
		String in = buffer.toString();		
		
		System.out.println("isXML() == " + XMLContentHandler.isXML( in ) );
		System.out.println("isRDF() == " + isRDF( in ) );
		
		System.out.println("File read == " + in );
		
		// Test the parser
		RDFContentHandler p = new RDFContentHandler();
		
		Content orig_cont = p.parse(in);
		
		System.out.println( "Parsed Content == " + orig_cont );
		
		String orig_deparse = p.deparse(orig_cont);	
		
		System.out.println( "Dearsed Content == " + orig_deparse );
	} // end main()
	
} // end class RDFContentHandler
