//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files ("the
//     Programs") are Copyright (c) 1999 - 2000 Nortel Networks Corporation, 
//     8200 Dixie Road, Suite 100, Brampton, Ontario, Canada L6R 5P6.  
//     All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out in
//     the Nortel Networks FIPA-OS Public License, a copy of which can be found
//     in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version can
//     also be found at http://fipa-os.sourceforge.net/. By downloading the
//     files containing the Programs you accept the terms and  conditions of the
//     Public License. You do not have to accept these terms and conditions, but
//     unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.parser.sl0;

//----------------------------------------------------------------------

import fipaos.ont.fipa.fipaman.AgentManagementAction;
import fipaos.ont.BASETOKENS;
import fipaos.util.DIAGNOSTICS;

//----------------------------------------------------------------------

/**
 * Holds tests for the SLParser.
 * Note that the way the SL Grammar is defined,
 * spaces are not necessary between terms.
 *  
 * <br>
 * <br>
 */
public class SL0Util 
{ 
	
  /**
   * Accessor method to parse an AgentManagementAction from an 
   * SL0ContentExpression.
   * 
   * @param content The SL0ContentExpression containing the action
   * @return AgentManagementAction the action parsed
   * @throws ParseException
   **/
  public static AgentManagementAction getAgentManagementAction(
                                        String content) throws ParseException 
  { 
  	AgentManagementAction agent_mgt_action = null;
  	
  	// remove the "(action guidname" prefix and the ")" suffix.
  	int action_string_start = content.indexOf( BASETOKENS.OPENBRACKET, 1 );
  	int action_string_end = content.indexOf( BASETOKENS.CLOSEBRACKET, content.length() - 1 );
  	String action_string = content.substring( action_string_start, action_string_end );
  	
  	try
  	{
  		
  		agent_mgt_action = new AgentManagementAction( action_string );
  		
  	}
  	catch( Throwable iae )
  	{
  		
  		DIAGNOSTICS.println( "Parses as SL0 but not as an agent mgt action: ", iae,
														DIAGNOSTICS.LEVEL_MAX );
  		throw new ParseException( "Parses as SL0 but not as an agent mgt action" );
  		
  	}
  	
  	return agent_mgt_action;
  	
  } // end getAgentManagementAction
   
} // end class SL0Util
