//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.parser.xml;

//----------------------------------------------------------------------

// Not needed (yet)
//import fipaos.ont.BASETOKENS;
//import fipaos.ont.fipaman.FIPAMANCONSTANTS;

// FIPA-OS Classes
import fipaos.ont.fipa.FIPACONSTANTS;
import fipaos.util.DIAGNOSTICS;
import fipaos.parser.Parser;
import fipaos.agent.conversation.content.Content;
import fipaos.parser.rdf.RDFContentHandler;
import fipaos.util.DocumentRetriever;

// JDK1.2 Classes
import java.io.File;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.StringReader;
import java.io.IOException;
import java.util.Stack;
import java.util.Iterator;
import java.util.List;

// XML4J v1.14 classes
//import com.ibm.xml.parser.StreamProducer;
//import com.ibm.xml.parser.TXDocument;
//import com.ibm.xml.parser.Source;

// SAX specific classes
import org.xml.sax.EntityResolver;
import org.xml.sax.DTDHandler;
import org.xml.sax.ErrorHandler;
import org.xml.sax.DocumentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.ParserFactory;
import org.xml.sax.AttributeList;
import org.xml.sax.Locator;


//----------------------------------------------------------------------

/** <p>Converts between XML messages and Content objects.
 * <p>XML particular Content object implementation details:
 * <ul>
 * <li><CODE>Content</CODE> object attrbiutes are converted to XML TAG attributes
 * unless.
 * <li><CODE>Content</CODE> object sub-nodes becomes inner-tag of parent
 * <li>Attribute "xml_tag_content" becomes the "content" of the tags belonging to
 * the <CODE>Content</CODE> is belongs to.
 * <li>The root <CODE>Content</CODE> can contain attributes "dtd-doctype" and "dtd-system"
 * which will be used to construct a DTD tag at the start of the document if they are specified
 * </ul>
 * <br>
 * <br>
 * @deprecated Content objects should be avoided - please use XML Databinding instead
 * @see fipaos.util.XMLDataBinding*/
public class XMLContentHandler implements Parser, EntityResolver, DTDHandler, ErrorHandler, DocumentHandler
{

//----------------------------------------------------------------------
// CONSTANTS
//----------------------------------------------------------------------

	/** The tag found at the start of every XML document
	 */
	public static final String XML_HEADER ="<?xml version=\"1.0\" encoding=\"UTF-16\" ?>\n";

	public static final String XML_TAG_OPEN = "<";
	public static final String XML_TAG_OPEN_END = "</";	
	public static final String XML_TAG_CLOSE = ">\n";
	public static final String XML_TAG_CLOSE_END = "/>\n";
	public static final String XML_ATTR_OPEN = "=\"";
	public static final String XML_ATTR_CLOSE = "\"";
	public static final String XML_CDATA_START = "<![CDATA[";
	public static final String XML_CDATA_CLOSE = "]]>";
	public static final String XML_CDATA_CLOSE_REPLACE = "]]&gt;";
	public static final String XML_DTD_START = "<!DOCTYPE ";
	public static final String XML_DTD_MIDDLE = " SYSTEM \"";
	public static final String XML_DTD_END = "\">\n";	
	public static final String XML_SPACE = " ";

//----------------------------------------------------------------------
// STATIC VARIABLES
//----------------------------------------------------------------------
		
//----------------------------------------------------------------------
// INSTANCE VARIABLES
//----------------------------------------------------------------------
	
	/** During parsing (which places a mutex lock on this) this contains the Root
	 * content object into which the XML document is being parsed
	 */
	private Content _root;

	/** A Stack of Content objects currently being produced (the top-most being the most
	 * recently created, the next being its parent etc..)
	 */
	private Stack _content_stack;
	
//----------------------------------------------------------------------
// CONSTRUCTOR
//----------------------------------------------------------------------

	/**
	 * Default constructor for this Parser.  Used buy the ParserFactory
	 */
	public XMLContentHandler()
	{	
	}
	
	
//----------------------------------------------------------------------
// PUBLIC METHODS
//----------------------------------------------------------------------	
	
//----------------------------------------------------------------------
// PARSER INTERFACE METHODS
//----------------------------------------------------------------------	
	
	/**
	 * This method converts a Content object into a document
	 * which is of a format determined by the Parser implementation.
	 *
	 * @param content The Content object to deparse
	 *
	 * @return The Stringified representation of the Content object
	 */
	public String deparse( Content content )
	{
		StringBuffer deparsed_msg = new StringBuffer();
		
		try
		{
			// Attempt to deparse this content object
			deparsed_msg.append( XML_HEADER );
			
			// Check if a DTD has been specified
			String doctype = (String) content.getAttribute( "dtd-doctype" );
			String system = (String) content.getAttribute( "dtd-system" );
			if( doctype != null && system != null )
			{
				// Construct XML header
				deparsed_msg.append( XML_DTD_START );
				deparsed_msg.append( doctype );
				deparsed_msg.append( XML_DTD_MIDDLE );
				deparsed_msg.append( system );
				deparsed_msg.append( XML_DTD_END );
			}
			
			// Recursively deparse the Content object
			deparsed_msg.append( deparseContent( content, "" ) );
		}
		catch( Throwable e )
		{
			DIAGNOSTICS.println( "Error occured during parsing: " + e );
			e.printStackTrace();
		}
		
		return deparsed_msg.toString();
	}
		

	/**
	 * This method converts a content String into a set of Content objects
	 *
	 * @param content The String to parse into Content objects
	 *
	 * @return A Content object representing the content String given.
	 */
	public Content parse( String content )
	{
		Content result = null;
		
		try
		{
			// Mutex lock here - no more than one thread should enter here at a time!
			synchronized( this )
			{
				// Find out which XML parser to use & create it
				String xml_parser = System.getProperty( "fipaos.parser.xmlparser", "org.apache.xerces.parsers.SAXParser" );
				org.xml.sax.Parser parser = ParserFactory.makeParser( xml_parser );
				
				// Associate this with the parser
				parser.setDocumentHandler( this );
				parser.setEntityResolver( this );
				parser.setErrorHandler( this );
				parser.setDTDHandler( this );
				
				// Start the parsing process
				InputSource is = new InputSource( new StringReader( content ) );
				is.setSystemId( "" );
				parser.parse( is );
				
				// Return the complete Content object
				result = _root;
			}
		}
		catch( Throwable e )
		{
			DIAGNOSTICS.println( "Error occured during parsing: " + e );
		}
		
		return result;
	}


	/** Determines if the Parser implementation can parse the given content
	 * @return True if the Parser implementation can parse the given String
	 * @param content The content to be tested
	 */
	public boolean canParse( String content )
	{
		boolean result = isXML( content ) && !RDFContentHandler.isRDF( content );
		
		// Return result
		return result;
	}


	/**
	 * Returns the type of content language that the Parser implementation
	 * handles.
	 *
	 * @return A String representation of the content format (e.g. RDF, SL)
	 */
	public String getParserType()
	{
		return FIPACONSTANTS.XML;
	}
	
	
//----------------------------------------------------------------------
// PUBLIC STATIC METHODS
//----------------------------------------------------------------------	
	
	/** Indicates if the given content is an XML document
	 * @param content The content to check
	 * @return <CODE>True</CODE> if the content is XML
	 */
	public static boolean isXML(String content) {
		// Trim the content
		String trimmed_content = content.trim();

		// Attempt to find the XML header
		int xml_header = trimmed_content.indexOf( XML_HEADER );

		// XML header must be the first thing in an XML document
		return (xml_header==0);
	}
	

//----------------------------------------------------------------------
// PRIVATE METHODS
//----------------------------------------------------------------------	
	
	/** Method to convert the given object into a primitive type
	 * @param in The <CODE>Object</CODE> to convert
	 * @return The object as a "primitive" type (in our case a String,
	 * Long or Double).
	 */
	private static Object getPrimitive( String in )
	{
		// Default value in == out
		Object out = in;
		
		// Attempt to parse the value into a Long
		try
		{
			out = new Long( in );
		}
		catch( Exception e )
		{
		
			// Attempt to parse the value into a Double
			try
			{
				out = new Double( in );
			}
			catch( Exception k )
			{
			}
		}

		// Return the appropriate primitive type
		return out;
	}
	
	
	/** Deparses the given <CODE>Content</CODE> object into a String
	 * @param cnt The Content object to be deparsed
	 * @param prepend Anything to prepend to XML tags being produced (i.e. tabs)
	 * @return A String XML representation of the given Content object
	 */
	private static String deparseContent(Content cnt,String prepend)
	{
		// Tag attributes and content
		StringBuffer tag_attributes = new StringBuffer();
		StringBuffer tag_content = new StringBuffer();
		String prepend_next = prepend + "\t";

		// Iterate through the attributes in the given content object
		Iterator iter = cnt.getAttributeNames().iterator();
		while( iter.hasNext() )
		{
			// Get the name of the next attribute
			String name = (String) iter.next();

			// Get the attribute
			Object attribute = cnt.getAttribute( name );

			// See what type attribute is
			if( attribute instanceof Content )
			{
				// Add the deparsed content object into the content of the current tag
				tag_content.append( deparseContent( (Content) attribute, prepend_next ) );
			}
			else if( attribute instanceof List )
			{
				// Iterate through the list of items with the same attribute name
				Iterator i = ( (List) attribute ).iterator();
				while( i.hasNext() )
				{
					// Get the next object
					Object obj = i.next();

					// See what it is
					if( obj instanceof Content )
					{
						// Add the deparsed content object into the content of the current tag
						tag_content.append( deparseContent( (Content) obj , prepend_next ) );
					}
					else
					{
						// Add attribute as tag attribute
						tag_attributes.append( XML_SPACE );
						tag_attributes.append( name );
						tag_attributes.append( XML_ATTR_OPEN );
						tag_attributes.append( obj.toString() );
						tag_attributes.append( XML_ATTR_CLOSE );
					}
				}
			}
			// Check if "content" for the tag is specified & there is some content!
			else if( name.equals( "xml_tag_content" ) && 
						attribute != null && 
						attribute.toString().length() != 0 )
			{
				tag_content.append( getNormalisedForm( attribute.toString() ) );
				tag_content.append( "\n" );
			}
			else
			{
				// Add attribute as tag attribute
				tag_attributes.append( XML_SPACE );
				tag_attributes.append( name );
				tag_attributes.append( XML_ATTR_OPEN );
				tag_attributes.append( attribute.toString() );
				tag_attributes.append( XML_ATTR_CLOSE );
			}
		}

		StringBuffer out = new StringBuffer();

		// Check that we're not dealing with a Root node
		if( !cnt.getName().equalsIgnoreCase( "root" ) )
		{
			// Open the tag and ad its name
			out.append( prepend );
			out.append( XML_TAG_OPEN );
			out.append( cnt.getName() );
			out.append( tag_attributes );

			if( tag_content.length() == 0 )
			{
				// Just close the tag, it has no contents
				out.append( XML_TAG_CLOSE_END );
			}
			else
			{
				// Close the tag, add the contents and close the block
				out.append( XML_TAG_CLOSE );
				out.append( tag_content );
				out.append( prepend );
				out.append( XML_TAG_OPEN_END );
				out.append( cnt.getName() );
				out.append( XML_TAG_CLOSE );
			}
		}
		else
		{
			out.append( tag_content );
		}

		return out.toString();
	}
	
	
	/** If necessary converts the given String into a format suitable for placing
	 * within an XML tags (i.e. appropriate characters extracted and replaced to
	 * ensure unambiguous parsing)
	 * @param non_normalised String to be correctly formatted
	 * @return A String representing the same information as the original
	 * String, but in an XML-friendly format
	 */
	public static String getNormalisedForm(String non_normalised)
	{
	
		StringBuffer normalised = new StringBuffer();
		
		char current;
		int char_val;
		
		int len  = non_normalised.length();
		
		for (int i = 0; i < len; i++)
		{
			current = non_normalised.charAt(i);
			
			char_val = (int)current;
		
			if (char_val == 0x0009)
			{
				normalised.append("&#9;");
			}
			else if (char_val == 0x000A)
			{
				normalised.append("&#10;");
			}
			else if (char_val == 0x000D)
			{
				normalised.append("&#13;");
			}
			else if (char_val == 0x0022)
			{
				normalised.append("&quot;");
			}
			else if (char_val == 0x003C)
			{
				normalised.append("&lt;");
			}
			else if (char_val == 0x003E)
			{
				normalised.append("&gt;");
			}
			else if (char_val == 0x0026)
			{
				normalised.append("&amp;");
			}
			else if (char_val >= 0x0020 && char_val <= 0xD7FF)
			{
				normalised.append(current);
			}
			else if (char_val >= 0xE000 && char_val <= 0xFFFD)
			{
				normalised.append(current);
			}
			else if (char_val >= 0x10000 && char_val <= 0x10FFFF)
			{
				normalised.append(current);
			}
			else // not an allowed character so escape it
			{
				normalised.append("&#" + char_val + ';');				
			}		
			
		} // end for

  	return normalised.toString();
	
	}
	
//----------------------------------------------------------------------
// SAX HANDLER METHODS
//----------------------------------------------------------------------	
	
  /**
   * Resolve an external entity.
   *
   * @param publicId The public identifer, or null if none is
   *                 available.
   * @param systemId The system identifier provided in the XML
   *                 document.
   * @return The new input source, or null to require the
   *         default behaviour.
   * @exception org.xml.sax.SAXException Any SAX exception, possibly
   *            wrapping another exception.
   * @see org.xml.sax.EntityResolver#resolveEntity
   */
  public InputSource resolveEntity (String publicId, String systemId) throws SAXException
	{
		try
		{
			// Get the required document
			String doc = DocumentRetriever.getDoc( systemId );
			
			// Check that it was got OK
			if( doc == null )
			{
				return null;
			}
			else
			{
				// Setup the InputSource and return
				InputSource is = new InputSource( new StringReader( doc ) );
				is.setPublicId( publicId );
				is.setSystemId( systemId );
				return is;
			}
		}
		catch( Exception e )
		{
			throw new SAXException( e );
		}
	}

  
  /**
   * Receive notification of an unparsed entity declaration.
   *
   * @param name The entity name.
   * @param publicId The entity public identifier, or null if not
   *                 available.
   * @param systemId The entity system identifier.
   * @param notationName The name of the associated notation.
   * @see org.xml.sax.DTDHandler#unparsedEntityDecl
   */
	public void unparsedEntityDecl( String p1, String p2, String p3, String p4 ) throws SAXException
	{
	}

  
  /**
   * Receive notification of a notation declaration.
   *
   * @param name The notation name.
   * @param publicId The notation public identifier, or null if not
   *                 available.
   * @param systemId The notation system identifier.
   * @see org.xml.sax.DTDHandler#notationDecl
   */
	public void notationDecl( String p1, String p2, String p3 ) throws SAXException
	{
	}
	
  
  /**
   * Receive a Locator object for document events.
   *
   * @param locator A locator for all SAX document events.
   * @see org.xml.sax.DocumentHandler#setDocumentLocator
   * @see org.xml.sax.Locator
   */
  public void setDocumentLocator (Locator locator)
  {
  }


  /**
   * Receive notification of the beginning of the document.
   *
   * @exception org.xml.sax.SAXException Any SAX exception, possibly
   *            wrapping another exception.
   * @see org.xml.sax.DocumentHandler#startDocument
   */
  public void startDocument () throws SAXException
	{
		// Initialise all instance variables
		_root = new Content("Root");
		_content_stack = new Stack();
		
		_content_stack.push( _root );
  }


  /**
   * Receive notification of the end of the document.
   *
   * @exception org.xml.sax.SAXException Any SAX exception, possibly
   *            wrapping another exception.
   * @see org.xml.sax.DocumentHandler#endDocument
   */
  public void endDocument () throws SAXException
  {
		// Dereference stacks
		_content_stack = null;
  }


  /**
   * Receive notification of the start of an element.
   *
   * @param name The element type name.
   * @param attributes The specified or defaulted attributes.
   * @exception org.xml.sax.SAXException Any SAX exception, possibly
   *            wrapping another exception.
   * @see org.xml.sax.DocumentHandler#startElement
   */
  public void startElement (String name, AttributeList attributes) throws SAXException
  {
		// Create a new Content object for this element
		Content element = new Content( name );
		
		// Bind this new element into the parent Content object
		Content parent = ( (Content) _content_stack.peek() );
		parent.addAttribute( name, element );
		
		// Add the new Content object to the top of the stack
		_content_stack.push( element );
		
		// Deal with the attributes of this tag
		for( int i = 0; i < attributes.getLength(); i++ )
		{
			// Get the values associated with the attribute
			String att_name = attributes.getName( i );
			String att_value = attributes.getValue( i );
			
			// Add attribute to the content object
			element.addAttribute( att_name, getPrimitive( att_value ) );
		}
  }


  /**
   * Receive notification of the end of an element.
   *
   * @param name The element type name.
   * @param attributes The specified or defaulted attributes.
   * @exception org.xml.sax.SAXException Any SAX exception, possibly
   *            wrapping another exception.
   * @see org.xml.sax.DocumentHandler#endElement
   */
  public void endElement (String name) throws SAXException
  {
		// Check if the Content object at the top of the stack
		// maches this tag
		Content curr = (Content) _content_stack.peek();
		
		if( curr.getName().equals( name ) )
		{
			// Remove the Content object at the top of the stack
			_content_stack.pop();
		}
  }


  /**
   * Receive notification of character data inside an element.
   *
   * @param ch The characters.
   * @param start The start position in the character array.
   * @param length The number of characters to use from the
   *               character array.
   * @exception org.xml.sax.SAXException Any SAX exception, possibly
   *            wrapping another exception.
   * @see org.xml.sax.DocumentHandler#characters
   */
  public void characters (char ch[], int start, int length) throws SAXException
  {
		// Put this into the appropriate content object
		Content curr = (Content) _content_stack.peek();
		String name = "xml_tag_content";
		String value = new String( ch, start, length ).trim();
		
		// Check that the content is something worth adding!
		if( value.length() > 0 )
		{
			// Add the characters to the current content object
			curr.setAttribute( name, getPrimitive( value ) );
		}
	}


  /**
   * Receive notification of ignorable whitespace in element content.
   *
   * @param ch The whitespace characters.
   * @param start The start position in the character array.
   * @param length The number of characters to use from the
   *               character array.
   * @exception org.xml.sax.SAXException Any SAX exception, possibly
   *            wrapping another exception.
   * @see org.xml.sax.DocumentHandler#ignorableWhitespace
   */
  public void ignorableWhitespace (char ch[], int start, int length) throws SAXException
  {
  }


  /**
   * Receive notification of a processing instruction.
   *
   * @param target The processing instruction target.
   * @param data The processing instruction data, or null if
   *             none is supplied.
   * @exception org.xml.sax.SAXException Any SAX exception, possibly
   *            wrapping another exception.
   * @see org.xml.sax.DocumentHandler#processingInstruction
   */
  public void processingInstruction (String target, String data) throws SAXException
  {
  }


  /**
   * Receive notification of a parser warning.
   *
   * @param e The warning information encoded as an exception.
   * @exception org.xml.sax.SAXException Any SAX exception, possibly
   *            wrapping another exception.
   * @see org.xml.sax.ErrorHandler#warning
   * @see org.xml.sax.SAXParseException
   */
  public void warning (SAXParseException e) throws SAXException
  {
		DIAGNOSTICS.println("WARNING: Exception occured during XML parse (non-fatal):\n" + e);
  }


  /**
   * Receive notification of a recoverable parser error.
   *
   * @param e The warning information encoded as an exception.
   * @exception org.xml.sax.SAXException Any SAX exception, possibly
   *            wrapping another exception.
   * @see org.xml.sax.ErrorHandler#warning
   * @see org.xml.sax.SAXParseException
   */
  public void error (SAXParseException e) throws SAXException
  {
		DIAGNOSTICS.println("ERROR: Exception occured during XML parse (non-fatal):\n" + e);
  }


  /**
   * Report a fatal XML parsing error.
   *
   *
   * @param e The error information encoded as an exception.
   * @exception org.xml.sax.SAXException Any SAX exception, possibly
   *            wrapping another exception.
   * @see org.xml.sax.ErrorHandler#fatalError
   * @see org.xml.sax.SAXParseException
   */
  public void fatalError (SAXParseException e) throws SAXException
  {
  	DIAGNOSTICS.println("FATAL: Exception occured during XML parse (fatal):\n" + e);
    throw e;
  }

//----------------------------------------------------------------------
// INNER-CLASSES
//----------------------------------------------------------------------	
	
//----------------------------------------------------------------------
// TEST HARNESS
//----------------------------------------------------------------------	
	
	public static void main( String args[] ) throws Exception
	{
		String in = "select clip, price, year, region, name, summary, comments, pid, food, special, grape from Product where (((type = 'red') and (price <= 31) and (country Like '%France%')and(region Like '%%%') and (lighttofull = 3) and (drytosweet = 3))) ORDER BY price DESC";
		
		System.out.println( "Size of String == " + in.length() );
		System.out.println( "\"" + getNormalisedForm( in ) + "\"" );
		
/*		if( args.length <= 0 )
		{
			fipaos.util.DIAGNOSTICS.println("Usage: java fipaos.parser.xml.XMLContentHandler [<filename>]");
			System.exit(-10);
		}
		
		File file = new File( args[0] );
		
		// Read in the given file
		BufferedReader fin = new BufferedReader( new FileReader( file ) );
		
		// String to build up input
		String in = new String();
		
		String next = fin.readLine();
		while( next != null )
		{
			in += next + "\n";
			next = fin.readLine();	
		}
		
		// Close the File
		fin.close();

		// Add our default filehandler
		String path = file.getAbsolutePath();
		path = path.substring( 0, path.lastIndexOf( File.separator ) );
		DocumentRepository.getJVMDocumentRepository().addDocumentRetriever( new FileRetriever( path ) );
		
		// Run the test!
		System.out.println( "Input: " + in );
		
		Parser p = new XMLContentHandler();
		
		Content out = p.parse( in );
		
		System.out.println( "\nOutput: " + out );
		
		String out2 = p.deparse( out );
		
		System.out.println( "\nOutput2: " + out2 );
*/
	}
}
