//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.platform;

//----------------------------------------------------------------------

import fipaos.agent.*;
import fipaos.agent.profile.*;
import fipaos.agent.conversation.*;
import fipaos.agent.task.*;
import fipaos.ont.fipa.fipaman.*;
import fipaos.ont.fipa.*;
import fipaos.ont.proprietary.subscription.*;
import fipaos.util.*;
import fipaos.platform.ams.AMSRegistrationException;
import fipaos.platform.df.*;
import fipaos.skill.db.*;
import fipaos.ont.BASETOKENS;

import javax.swing.*;

//----------------------------------------------------------------------

/**
 * Directory Facilitator implementation
 */
public class DirectoryFacilitator extends FIPAOSAgent
{
	
	//----------------------------------------------------------------------
	// CONSTANTS
	//----------------------------------------------------------------------
	
	
	
	
	//----------------------------------------------------------------------
	// STATIC VARIABLES
	//----------------------------------------------------------------------
	
	
	
	
	//----------------------------------------------------------------------
	// INSTANCE VARIABLES
	//----------------------------------------------------------------------
	
	/**
	 * Reference to Database containing registered Agents
	 */
	private Database _df_database;
	
	/**
	 * Reference to GUI
	 */
	private DFCrossPlatformRegistrationFrame _frame;
	
	/**
	 * SubscriptionManager currently in use
	 */
	public SubscriptionManagerTask _subs_manager;
	
	//----------------------------------------------------------------------
	// CONSTRUCTOR(S)
	//----------------------------------------------------------------------
	
	/**
	 * Default constructor - to be used by AgentLoader etc....
	 */
	public DirectoryFacilitator(String platform_profile,String name,String owner)
	{
		super( platform_profile, name, owner );
		init( name );
	}
	
	/**
	 * Alternative constructor to be used by sub-classes (e.g. MicroFIPAOS DF)
	 */
	protected DirectoryFacilitator( PlatformProfile pp, AgentProfile ap, String name,String owner, 
																	FIPAOSAgentComponentFactory factory )
	{
		super( pp, ap, name, owner, true, factory );
		init( name );
	}
	
	/**
	 * General initialisation of the Agent
	 */
	private void init( String name )
	{
		try
		{
			// Construct Database
			_df_database = DatabaseFactory.getDatabase( name + "_db", getProfile().getDatabaseProfile() );
			
			// Open Database
			_df_database.openDatabase();
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "Error when initialising Database: ", t, this,
			DIAGNOSTICS.LEVEL_MAX );
			return;
		}
		
		// Construct listener task + instantiate
		setListenerTask( new IdleTask() );
		
		try
		{
			// Register with AMS
			registerWithAMS();
		}
		catch (AMSRegistrationException amsre)
		{
			String reason = amsre.getExceptionReason();
			
			if (reason != null && reason.equals(FIPAMANCONSTANTS.AGENT_ALREADY_REGISTERED))
			{
				DIAGNOSTICS.println( "Error registering DF: " + reason, this, DIAGNOSTICS.LEVEL_MAX );	
			}
			else
			{
				DIAGNOSTICS.println( "Error registering DF: " + reason, this, DIAGNOSTICS.LEVEL_MAX );
				return;				
			}
		}
		
		// Check if this is the default DF for the platform
		if( !name.equals( "df" ) )
		{
			
			try
			{
				// Register with DF
				registerWithDF( FIPAMANCONSTANTS.FIPA_DF );
			}
			catch (DFRegistrationException dfre)
			{
				String reason = dfre.getExceptionReason();

				if (reason != null && reason.equals(FIPAMANCONSTANTS.AGENT_ALREADY_REGISTERED))
				{
					DIAGNOSTICS.println( "Error registering DF: " + reason, this, DIAGNOSTICS.LEVEL_MAX );	
				}
				else
				{
					DIAGNOSTICS.println( "Error registering DF: " + reason, this, DIAGNOSTICS.LEVEL_MAX );
					return;				
				}

			} // end catch
			
		} // end if
	
		// Attempt to set the default L&F for the platform we're running on
		try
		{
			UIManager.setLookAndFeel ( UIManager.getSystemLookAndFeelClassName() );
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "Couldn't set the default Look & Feel for this OS", this,
			DIAGNOSTICS.LEVEL_3 );
		}

		try
		{
			_frame = new DFCrossPlatformRegistrationFrame( this );
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "Error starting GUI, ignoring: " + t.toString(), DIAGNOSTICS.LEVEL_MAX );
		}
	} // end DirectoryFacilitator()
	
	
	public void shutdown()
	{
		// Kill GUI, if possible
		if( _frame != null )
		{
			try
			{
				_frame.dispose();
			}
			catch( Throwable t )
			{
			}
		}
		
		// Check if we're registered with the AMS
		if (registeredWithAMS() == true)
		{
			try
			{
				// Deregister with AMS
				deregisterWithAMS();
				DIAGNOSTICS.println("Deregistered with AMS", this, DIAGNOSTICS.LEVEL_MAX );
			}
			catch (AMSRegistrationException amsre)
			{
				// A problem deregistering occured..... we can obtain the reason though!
				String reason = amsre.getExceptionReason();
				
				DIAGNOSTICS.println( "Couldn't deregister from AMS: " + reason, this, DIAGNOSTICS.LEVEL_MAX );
			}
		}
		
		// Invoked FIPAOSAgent.shutdown()
		super.shutdown();
		
		// Close down DB
		if( _df_database != null )
		{
			try
			{
				_df_database.closeDatabase();
			}
			catch( Throwable t )
			{
			}
		}
	}
	
	
	//----------------------------------------------------------------------
	// INNER-CLASSES
	//----------------------------------------------------------------------
	
	public class IdleTask extends Task
	{
		public IdleTask()
		{
		}
		
		public void startTask()
		{
			// Start subscription manager
			_subs_manager = new SubscriptionManagerTask();
			newTask( _subs_manager );
		}
		
		public void handleRequest( Conversation conv )
		{
			// Get the type of request
			ACL acl = conv.getACL( conv.getLatestMessageIndex() );
			
			// Check ontology
			if( acl.getOntology() == null || 
					!acl.getOntology().equalsIgnoreCase( "fipa-agent-management" ) )
			{
				sendNotUnderstood( acl );
				return;
			}
			
			// Check language
			if( acl.getLanguage() == null || 
					!acl.getLanguage().equalsIgnoreCase( FIPACONSTANTS.SL0 ) )
			{
				sendNotUnderstood( acl );
				return;
			}
			
			try
			{
				// Get content of message
				Object content_obj = acl.getContentObject();
				AgentManagementAction ama = null;
				
				// Decide if we need to parse content
				if( content_obj instanceof AgentManagementAction )
				{
					// Object from another FIPA-OS Agent - no parsing required!
					ama = (AgentManagementAction) content_obj;
				}
				else if( content_obj instanceof String )
				{
					// Assume standard SL0 encoding?
					
					// Parse the content to decide what to do
					String content = (String) content_obj;
					ama = new AgentManagementAction( content );						
				}
				else
				{
					// Unknown content ?!?!?!
					DIAGNOSTICS.println( "Couldn't extract AgentManagementAction - sending not-understood",
																this, DIAGNOSTICS.LEVEL_MAX);
					sendNotUnderstood( acl );
					return;
				}
				
				// Get the action of this
				String function = ama.getAction();
				
				// Decide which sub-tasks to spawn
				if( function.equals( FIPAMANCONSTANTS.REGISTER ) )
				{
					newTask( new RegisterTask( conv, ama, _df_database ) );
				}
				else if( function.equals( FIPAMANCONSTANTS.DEREGISTER ) )
				{
					newTask( new DeregisterTask( conv, ama, _df_database ) );
				}
				else if( function.equals( FIPAMANCONSTANTS.MODIFY ) )
				{
					newTask( new ModifyTask( conv, ama, _df_database ) );
				}
				else if( function.equals( FIPAMANCONSTANTS.SEARCH ) )
				{
					newTask( new SearchTask( conv, ama, _df_database ) );
				}
				else
				{
					sendNotUnderstood( acl );
				}
			}
			catch( Throwable t )
			{
				DIAGNOSTICS.println( "Exception occured - sending not-understood", t,
				this, DIAGNOSTICS.LEVEL_MAX);
				sendNotUnderstood( acl );
			}
		}
		
		public void handleSubscribe( Conversation conv )
		{
			// Check if SubscriptionAction has been passed
			ACL acl = conv.getACL( 0 );
			if( acl.getContentObject() instanceof SubscriptionAction )
			{
				// Pass subscription off to SubscriptionManagerTask
				_subs_manager.newSubscriber( conv );
			}
		}
		
		public void doneRegisterTask( Task t )
		{
			DIAGNOSTICS.println( "RegisterTask completed OK" );
		}
		
		public void doneDeregisterTask( Task t )
		{
			DIAGNOSTICS.println( "DeregisterTask completed OK" );
		}
		
		public void doneModifyTask( Task t )
		{
			DIAGNOSTICS.println( "ModifyTask completed OK" );
		}
		
		public void doneSearchTask( Task t )
		{
			DIAGNOSTICS.println( "SearchTask completed OK" );
		}
		
		public void errorRegisterTask( Task task, Throwable t )
		{
			DIAGNOSTICS.println( "RegisterTask didn't completed OK", t, this,
			DIAGNOSTICS.LEVEL_MAX );
		}
		
		public void errorDeregisterTask( Task task, Throwable t )
		{
			DIAGNOSTICS.println( "DeregisterrTask didn't completed OK", t, this,
			DIAGNOSTICS.LEVEL_MAX );
		}
		
		public void errorModifyTask( Task task, Throwable t )
		{
			DIAGNOSTICS.println( "ModifyTask didn't completed OK", t, this,
			DIAGNOSTICS.LEVEL_MAX );
		}
		
		public void errorSearchTask( Task task, Throwable t )
		{
			DIAGNOSTICS.println( "SearchTask didn't completed OK", t, this,
			DIAGNOSTICS.LEVEL_MAX );
		}
	}
	
	//----------------------------------------------------------------------
	// METHODS TO ALLOW CROSS-DF REGISTRATION
	//----------------------------------------------------------------------
	
	/**
	 * When invoked, displays the Agents' GUI
	 */
	public void activate()
	{
		if( _frame != null )
		{
			_frame.setVisible( true );
		}
	}
	
	/**
	 * Tells the DF to register with the remote DF indicated by the given AID
	 */
	public boolean registerWithRemoteDF( AgentID aid ) throws DFRegistrationException
	{
		// Register with DF
		registerWithDF( aid, FIPAMANCONSTANTS.FIPA_DF );
		return true;
	}
	
	/**
	 * Tells the DF to register the given Agent as a DF within its database
	 */
	public boolean registerRemoteDF( AgentID aid )
	{
		boolean ok = false;
		
		// Setup appropriate DFAD
		DFAgentDescription dfad = new DFAgentDescription();
		ServiceDescription sd = new ServiceDescription();
		sd.setServiceType( FIPAMANCONSTANTS.FIPA_DF );
		sd.setServiceName( FIPAMANCONSTANTS.FIPA_DF );
		dfad.setAgentName( aid );
		dfad.setAgentServices( new ServiceDescription[] { sd } );
				
		// Add DF if possible
		try
		{
			_df_database.startTransaction( false );
			
			// Set DFAD for given Agent
			_df_database.addObject( dfad );
			
			ok = true;
			
			// Propogate update to listeners
			AgentManagementAction ama = new AgentManagementAction();
			ama.setAction( FIPAMANCONSTANTS.REGISTER );
			ama.setAgentID( getAID() );
			ama.setDFAgentDescription( dfad );
			
			_subs_manager.DFUpdate( ama );
		}
		catch( Throwable t )
		{	
		}
		finally
		{
			try
			{
				_df_database.endTransaction();
			}
			catch( Throwable t )
			{
				ok = false;
			}
		}
		
		return ok;
	}
	
	//----------------------------------------------------------------------
	// MAIN/TESTING METHOD
	//----------------------------------------------------------------------
	
} // End class
