//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.platform;

import fipaos.FIPAOSException;

/** 
 * RegistrationException is an umbrella exception type that covers all exceptions relating to
 * actions concerning the registration information of agents. Subclasses of this exception can
 * occur during initial registration, modification of registration details, or agent deregistration actions.
 */
public class RegistrationException extends FIPAOSException
{

	
	//----------------------------------------------------------------------
	// CONSTRUCTOR
	//----------------------------------------------------------------------
	
/** The reason that caused the exception
 */
	private String _reason = null;
	
	/**
	 * Creates an exception with no reason
	 */
	public RegistrationException()
	{
	}
	
	/**
	 * Creates an exception with the given reason
	 *
	 * @param error The human readable reason for the exception
	 */
	public RegistrationException( String error )
	{
		super( error );
	}
	
	/**
	 * Creates an exception with the given reason
	 *
	 * @param error The human readable reason for the exception
	 * @param reason The reason for the exception
	 */
	public RegistrationException( String error, String reason )
	{
		super( error );
		_reason = reason;
	}
	
	/**
	 * Returns the exception reason
	 *
	 * @return The reason for the exception
	 */
	public String getExceptionReason()
	{
		return _reason;
	}
	
}