//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.platform.agentcities;

//----------------------------------------------------------------------

import fipaos.ont.fipa.*;
import fipaos.ont.fipa.fipaman.*;
import fipaos.agent.*;
import fipaos.agent.task.*;
import fipaos.agent.conversation.*;
import fipaos.platform.ams.AMSRegistrationException;
import fipaos.platform.df.DFRegistrationException;
import fipaos.util.DIAGNOSTICS;
import fipaos.mts.Message;


//----------------------------------------------------------------------


/**
 * AgentCities ping-agent implementation
 */
public class PingAgent extends FIPAOSAgent
{	
	/**
	 * An agent constructor takes three parameters - the location of the platform
	 * profile file (usually fipa-os\profiles\platform.profile), the agent name
	 * (e.g. df, ams), and the owner of the agent (e.g. Nortel, Bob).
	 *
	 * @param platform_profile The location of the platform profile
	 * @param name The agent name
	 * @param owner The owner of this agent
	 */
	public PingAgent( String platform_profile, String name, String owner )
	{
		super( platform_profile, name, owner, true );
					
		setListenerTask( new IdleTask() );
		
		// Attempt to register with AMS
		try
		{
			registerWithAMS();
			DIAGNOSTICS.println("Registered with AMS", this, DIAGNOSTICS.LEVEL_MAX );
		}
		catch (AMSRegistrationException amsre)
		{
			// An exception has occured - this indicates that the AMS registration failed for some reason
			// Display the exception
			DIAGNOSTICS.println(amsre, this, DIAGNOSTICS.LEVEL_MAX );
			
			String reason = amsre.getExceptionReason();
			if ( reason == null || !reason.equals(FIPAMANCONSTANTS.AGENT_ALREADY_REGISTERED))
			{
				// Shutdown nicely :)
				shutdown();
				return;
			}
		}
	}
	
	//----------------------------------------------------------------------
	// PUBLIC METHODS
	//----------------------------------------------------------------------
	
	public class IdleTask extends Task
	{
		public void startTask()
		{			
		}
		
		public void handleQueryRef( Conversation conv )
		{
			DIAGNOSTICS.println( "Recieved ping..." );
			ACL acl = (ACL) conv.getACL( 0 ).clone();
			acl.setReceiverAID( acl.getSenderAID() );
			acl.setSenderAID( _owner.getAID() );
			acl.setPerformative( FIPACONSTANTS.INFORM );
			acl.setInReplyTo( acl.getReplyWith() );
			acl.setReplyWith( null );
			acl.setContentObject( "alive" );
			
			Message msg = new Message( acl );
			msg.getEnvelope().setACLRepresentation(conv.getEnvelope(0).getACLRepresentation());
						
			forward( msg );
		}
	}
	
	public synchronized void shutdown()
	{
		// Check if we're registered with the AMS
		if (registeredWithAMS() == true)
		{
			try
			{
				// Deregister with AMS
				deregisterWithAMS();
				DIAGNOSTICS.println("Deregistered with AMS", this, DIAGNOSTICS.LEVEL_MAX );
			}
			catch (AMSRegistrationException amsre)
			{
				// A problem deregistering occured..... we can obtain the reason though!
				String reason = amsre.getExceptionReason();
				
				DIAGNOSTICS.println(amsre + ":" + reason, this, DIAGNOSTICS.LEVEL_MAX );
			}
		}
		
		super.shutdown();
	}
	
	/**
	 * Main method for launching from a debugger
	 */
	public static void main( String args[] )
	{
		new PingAgent( args[0], args[1], "fipaos" );
	}
}
