//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.platform.ams;

import fipaos.agent.task.*;
import fipaos.agent.*;
import fipaos.agent.conversation.*;
import fipaos.ont.fipa.*;
import fipaos.ont.fipa.fipaman.*;
import fipaos.ont.*;

public class AMSDeregistrationTask extends Task 
{
   
//----------------------------------------------------------------------
// INSTANCE VARIABLES
//----------------------------------------------------------------------
	
	/**
	 * AID of AMS to deregister from
	 */
	private AgentID _ams;

	/**
	 * AMS description of agent to deregister
	 */
	private AMSAgentDescription _amsad;	
	
//----------------------------------------------------------------------
// CONSTRUCTOR(S)
//----------------------------------------------------------------------

	/**
	 * Create a Task to deregister with the AMS
	 */
	public AMSDeregistrationTask()
	{
		
	}
		
//----------------------------------------------------------------------
// PUBLIC METHODS
//----------------------------------------------------------------------	

	/**
	 * Invoked when task starts
	 */
	public void startTask()
	{

		_ams= _owner.getLocalAMSAID();
		
		// Get AID of Agent to deregister
		AgentID id = _owner.getAID();
		
		_amsad = new AMSAgentDescription();
		_amsad.setAgentID( id );
		_amsad.setState( FIPAMANCONSTANTS.ACTIVE);
		
		// Create Conversation for interaction with the AMS
		ACL to_ams = getNewConversation( FIPACONSTANTS.FIPA_REQUEST );

		// Set-up message parameters
		to_ams.setReceiverAID( _ams );
		to_ams.setOntology( FIPAMANCONSTANTS.FIPA_AGENT_MANAGEMENT );
		to_ams.setLanguage( FIPACONSTANTS.SL0 );
		to_ams.setPerformative( FIPACONSTANTS.REQUEST );

		// Create AgentManagementAction
		AgentManagementAction ama = new AgentManagementAction();
		ama.setAction( FIPAMANCONSTANTS.DEREGISTER );
		ama.setAgentID( _ams );
		ama.setAMSAgentDescription( _amsad );

		to_ams.setContentObject( ama );

		// Send message
		forward( to_ams );
	}
	
	
	/**
	 * Deal with not-understood messages
	 */
	public void handleNotUnderstood( Conversation conv )
	{
		ACL in = conv.getACL( conv.getLatestMessageIndex() );
		AgentID sender = in.getSenderAID();
		
		// Indicate we're done and pass-back results
		done( new Object[] { sender, FIPACONSTANTS.NOT_UNDERSTOOD } );
	}
	
	
	/**
	 * Deal with refuse messages
	 */
	public void handleRefuse( Conversation conv )
	{
		ACL in = conv.getACL( conv.getLatestMessageIndex() );
		AgentID sender = in.getSenderAID();
		String reason = new String();
		Object content = in.getContentObject();
		
		try
		{
			if( content instanceof String )
			{
				AgentManagementException ame = new AgentManagementException( (String)content );

				if( ame.getException() != null )
				{
					reason = ame.getException();
				}
			}
			else if( content instanceof AgentManagementException )
			{
				reason = ( ( AgentManagementException) content ).getException();
			}
		}
		catch( Throwable t )
		{
		}
		
		// Indicate we're done and pass-back results
		done( new Object[] { sender, reason } );
	}
	
	
	/**
	 * Deal with agree messages
	 */
	public void handleAgree( Conversation conv )
	{
		
		// Do nothing
		
	}
	
	
	/**
	 * Deal with failure messages
	 */
	public void handleFailure( Conversation conv )
	{
		ACL in = conv.getACL( conv.getLatestMessageIndex() );
		AgentID sender = in.getSenderAID();
		String reason = new String();
		Object content = (String) in.getContentObject();

		try
		{
			if( content instanceof String )
			{
				AgentManagementException ame = new AgentManagementException( (String)content );

				if( ame.getException() != null )
				{
					reason = ame.getException();
				}
			}
			else if( content instanceof AgentManagementException )
			{
				reason = ( ( AgentManagementException) content ).getException();
			}
		}
		catch( Throwable t )
		{
		}
		
		// Indicate we're done and pass-back results
		done( new Object[] { sender, reason } );
	}
	

	/**
	 * Deal with failure messages
	 */
	public void handleInform( Conversation conv )
	{
		ACL in = conv.getACL( conv.getLatestMessageIndex() );
		AgentID sender = in.getSenderAID();
		
		// Indicate we're done and pass-back results
		done( sender );
	}
	
	
//----------------------------------------------------------------------
// INNER-CLASSES
//----------------------------------------------------------------------	
	
//----------------------------------------------------------------------
// MAIN/TESTING METHOD
//----------------------------------------------------------------------
	
}
