//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved.
//
//******************************************************************************

package fipaos.platform.ams;

//----------------------------------------------------------------------

import fipaos.agent.*;
import fipaos.agent.conversation.*;
import fipaos.agent.task.*;
import fipaos.mts.*;
import fipaos.ont.fipa.fipaman.*;
import fipaos.ont.fipa.*;
import fipaos.skill.db.*;
import fipaos.util.*;
import fipaos.ont.BASETOKENS;
import fipaos.agent.profile.*;

import java.util.*;
import java.io.*;

//----------------------------------------------------------------------

/**
 * TODO: Write a description of the class
 * <br>
 * <br>
 * $Revision: 1.8 $ $Date: 2001/05/24 18:07:38 $ $Author: alantreadway $
 */
public class AMSTask extends Task
{
	
	//----------------------------------------------------------------------
	// CONSTANTS
	//----------------------------------------------------------------------
	
	
	
	
	//----------------------------------------------------------------------
	// STATIC VARIABLES
	//----------------------------------------------------------------------
	
	/**
	 * Task constants
	 */
	private static final String IDLE 				= "IDLE";
	private static final String BUSY				= "BUSY";
	
	/**
	 * Message flags
	 */
	private static final String NO_MSG_SENT				= "no-messages-sent";
	private static final String AGREE_MSG				= "agree-message-sent";
	private static final String INFORM_MSG				= "inform-message-sent";
	private static final String REFUSE_MSG				= "refuse-message-sent";
	private static final String FAILURE_MSG				= "failure-message-sent";
	private static final String NOT_UND_MSG				= "not-understood-message-sent";
	
	//----------------------------------------------------------------------
	// INSTANCE VARIABLES
	//----------------------------------------------------------------------
	
	/**
	 * State of the AMS
	 */
	
	private FlagHandler _state;
	
	/**
	 * Original ACL message containing the request
	 */
	protected ACL _acl;
	
	/**
	 * Reference to the DF's Database
	 */
	protected Database _ams_db;
	
	/**
	 * Reference to the Conversation we are answering
	 */
	protected Conversation _conv;
	
	/**
	 * Reference to the AMA that this Task is attempting to carry out
	 */
	protected AgentManagementAction _ama;
	
	//----------------------------------------------------------------------
	// CONSTRUCTOR(S)
	//----------------------------------------------------------------------
	
	/** Initialise DFTask
	 * @param conv Conversation associated with this Task
	 * @param ama AgentManagementAction contained within the given Conversation
	 * @param db Database associated with the DF
	 */
	public AMSTask(Conversation conv,AgentManagementAction ama,Database db)
	{
		// Save references
		_conv = conv;
		_ama = ama;
		_ams_db = db;
		_acl = _conv.getACL( 0 );
		
		// Set up state
		_state = new FlagHandler();
		_state.addFlag( IDLE, BUSY );
		_state.addFlag( BUSY, IDLE );
		_state.addFlag( NO_MSG_SENT );
		_state.addFlag( AGREE_MSG, NO_MSG_SENT );
		_state.addFlag( INFORM_MSG, AGREE_MSG );
		_state.addFlag( REFUSE_MSG, NO_MSG_SENT );
		_state.addFlag( FAILURE_MSG, NO_MSG_SENT );
		_state.addFlag( NOT_UND_MSG, NO_MSG_SENT );
		
		_state.setFlag( IDLE );
	}
	
	//----------------------------------------------------------------------
	// PROTECTED METHODS
	//----------------------------------------------------------------------
	
	
	/** Indicates if the Agent the AMSAgentDescription relates to is registered
	 * @param amsad AMSAgentDescription
	 * @return True if the Agent is registered with this AMS
	 */
	protected boolean isRegistered( AMSAgentDescription amsad )
	{
		// Get the AgentID of the Agent in the DFAD
		AgentID agent = amsad.getAgentID();
		
		// Check if Agent is registered
		return isRegistered( agent );
	}
	
	
	/** Indicates if the given Agent is already registered
	 * @param aid Agent ID
	 * @return True if the Agent is registered
	 */
	protected boolean isRegistered( AgentID aid )
	{
		return ( getRegistered( aid ) != null );
	}
	
	
	/** Gets the AMSAgentDescription associated with the Agent
	 * @param aid Agent ID
	 * @return The AMSAgentDescription for the given Agent, or null
	 */
	protected AMSAgentDescription getRegistered( AgentID aid )
	{
		AMSAgentDescription amsad = null;
		
		try
		{
			// Check if it is contained within the database
			
			// Start transaction
			_ams_db.startTransaction( true );
			
			// Get AMSAD for given Agent
			amsad = (AMSAgentDescription) _ams_db.findObject( aid.getName() );
			
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "Exception when getting registered agent: ", t, this );
		}
		finally
		{
			try
			{
				// End transaction
				_ams_db.endTransaction();
			}
			catch( Throwable t )
			{
			}
		}
		
		return amsad;
	}
	
	
	/** Adds the given AMSAgentDescription into the AMS Database
	 * @param amsad AMSAgentDescription
	 * @return True if the operation was successful
	 */
	protected boolean register( AMSAgentDescription amsad )
	{
		boolean done = false;
		
		try
		{
			// Check if it is contained within the database
			
			// Start transaction
			_ams_db.startTransaction( false );
			
			// Get AMSAD for given Agent
			_ams_db.addObject( amsad );
			
			done = true;
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "Exception when registering agent: ", t, this );
		}
		finally
		{
			try
			{
				// End transaction
				_ams_db.endTransaction();
			}
			catch( Throwable t )
			{
			}
		}
		
		return done;
	}
	
	
	/** Removes the given AMSAgentDescription from the AMS's Databas
	 * @param amsad AMSAgentDescription
	 * @return True if it was removed OK
	 */
	protected boolean deregister( AMSAgentDescription amsad )
	{
		boolean done = false;
		
		try
		{
			// Check if it is contained within the database
			
			// Start transaction
			_ams_db.startTransaction( false );
			
			// Get DFAD for given Agent
			_ams_db.removeObject( amsad );
			
			done = true;
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "Exception when deregistering agent: ", t, this );
		}
		finally
		{
			try
			{
				// End transaction
				_ams_db.endTransaction();
			}
			catch( Throwable t )
			{
			}
		}
		
		return done;
	}
	
	
	/** Send an agree message back to the originator of the request for this Task
	 * @return True if everything went OK
	 */
	protected boolean sendAgree()
	{
		boolean done = false;
		
		try
		{
			// Get latest version of the Conversation
			Conversation conv = _owner.getCurrentConversation( _conv.getConversationID() );
			
			// Extract performatives
			List perfs = conv.getNextPerformatives();
			
			// Send agree ONLY IF an agree is allowed to be sent next
			if (perfs == null || perfs.contains( FIPACONSTANTS.AGREE ))
			{
				ACL acl = (ACL) _acl.clone();
				
				// Setup message
				acl.setPerformative( FIPACONSTANTS.AGREE );
				acl.setReceiverAID( acl.getSenderAID() );
				acl.setSenderAID( _owner.getAID() );
				acl.setInReplyTo( acl.getReplyWith() );
				acl.setReplyWith( null );
				
				// Attempt to send the agree
				forward( acl );
				
				// Update the AMS state
				_state.setFlag( AGREE_MSG );
				
				done = true;
			}
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "An exception occured whilst sending an agree: ", t,
			this, DIAGNOSTICS.LEVEL_MAX );
		}
		
		return done;
	}
	
	
	/** Sends a refuse message back to the originator of the request for this Task
	 * @param reason Failure reason
	 * @return True if everything went OK
	 */
	protected boolean sendRefuse( String reason )
	{
		boolean done = false;
		
		try
		{
			// Get original ACL
			ACL acl = (ACL) _acl.clone();
			
			// Create AgentManagementException
			AgentManagementException ame = new AgentManagementException( _ama, reason );
			
			// Setup message
			acl.setPerformative( FIPACONSTANTS.REFUSE );
			acl.setReceiverAID( acl.getSenderAID() );
			acl.setSenderAID( _owner.getAID() );
			acl.setInReplyTo( acl.getReplyWith() );
			acl.setReplyWith( null );
			acl.setContentObject( ame );
			
			// Attempt to send the refuse
			forward( acl );
			
			// Update the AMS state
			_state.setFlag( REFUSE_MSG );
			
			done = true;
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "An exception occured whilst sending a refuse: ", t,
			this, DIAGNOSTICS.LEVEL_MAX );
		}
		
		return done;
	}
	
	
	/** Sends a failure back to the originator of the request for this Task
	 * @param reason reason for failure
	 * @return Returns True if everything went OK
	 */
	protected boolean sendFailure( String reason )
	{
		boolean done = false;
		
		try
		{
			// Get original ACL
			ACL acl = (ACL) _acl.clone();
			
			// Create AgentManagementException
			AgentManagementException ame = new AgentManagementException( _ama, reason );
			
			// Setup message
			acl.setPerformative( FIPACONSTANTS.FAILURE );
			acl.setReceiverAID( acl.getSenderAID() );
			acl.setSenderAID( _owner.getAID() );
			acl.setInReplyTo( acl.getReplyWith() );
			acl.setReplyWith( null );
			acl.setContentObject( ame );
			
			// Attempt to send the refuse
			forward( acl );
			
			// Update the AMS state
			_state.setFlag( FAILURE_MSG );
			
			done = true;
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "An exception occured whilst sending a failure: ", t,
			this, DIAGNOSTICS.LEVEL_MAX );
		}
		
		return done;
	}
	
	
	/** Sends an inform (with associated "done" term in content) back to originator
	 * of request for this Task
	 * @return boolean whether successful or not
	 */
	protected boolean sendInformDone()
	{
		boolean done = false;
		
		try
		{
			// Get original ACL
			ACL acl = (ACL) _acl.clone();
			
			// Setup message
			acl.setPerformative( FIPACONSTANTS.INFORM );
			acl.setReceiverAID( acl.getSenderAID() );
			acl.setSenderAID( _owner.getAID() );
			acl.setInReplyTo( acl.getReplyWith() );
			acl.setReplyWith( null );
			acl.setContentObject( "(done " + _ama.toString() + ")" );
			
			// Attempt to send the refuse
			forward( acl );
			
			// Update the AMS state
			_state.setFlag( INFORM_MSG );
			
			done = true;
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "An exception occured whilst sending an inform(done): ", t,
			this, DIAGNOSTICS.LEVEL_MAX );
		}
		
		return done;
	}
	
	/** This method would be called when this df would like to send a result message
	 * @param results The set of results to propogate back
	 * @return boolean whether successful or not
	 */
	protected boolean sendInformResult( Set results )
	{
		
		boolean done = false;
		
		try
		{
			// Get original ACL
			ACL acl = (ACL) _acl.clone();
			
			// Create result object
			AgentManagementResult amr = new AgentManagementResult();
			amr.setAgentManagementAction( _ama );
			amr.setResultSet( results );
			
			// Setup message
			acl.setPerformative( FIPACONSTANTS.INFORM );
			acl.setReceiverAID( acl.getSenderAID() );
			acl.setSenderAID( _owner.getAID() );
			acl.setInReplyTo( acl.getReplyWith() );
			acl.setReplyWith( null );
			acl.setContentObject( amr );
			
			DIAGNOSTICS.println( "Sending inform with results...", this );
			
			forward(acl);
			_state.setFlag( INFORM_MSG );
			
			DIAGNOSTICS.println( "Sent inform with results :-)", this );
			
			done = true;
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "An exception occured whilst sending an inform(result): ", t,
			this, DIAGNOSTICS.LEVEL_MAX );
		}
		
		return done;
	}
	
	/** This method would be called when this ams wants to send a platform profile
	 * @return boolean whether it is successful */
	
	protected boolean sendPlatformProfile()
	{
		boolean done = false;
		
		try
		{
			// Create result set
			Set results=new TreeSet( fipaos.util.Comparators.OBJECT_COMPARE );
			
			// Get platform profile
			PlatformProfile platformProfile=_owner.getPlatformProfile();
			
			// Get ACC profile
			ACCProfile acc_profile = (ACCProfile) XMLDataBinding.unmarshallFile(
			new File( platformProfile.getProfileDirectory(), "acc.profile" ) );
			
			// Create AP-Transport-Description
			APTDescription aptd = new APTDescription();
			
			// Create MTPDescription for external transports
			List l = acc_profile.getExternalAddresss();
			Iterator i = l.iterator();
			while( i.hasNext() )
			{
				try
				{
					// Get next address
					ExternalAddress ext = (ExternalAddress) i.next();
					
					// Convert to URL
					URL url = new URL( ext.getAddress() );
					
					// Get profile name for MTP
					String profile = TransportProvider.getDefaultMTPProfileName( url.getProtocol() );
					
					// Is it valid?
					if( profile != null )
					{
						// Create MTPD for external transport
						MTPDescription mtpd = new MTPDescription();
						mtpd.addAddress( url );
						mtpd.setMTPName( profile );
						aptd.addMTPD( mtpd );
					}
				}
				catch( Throwable t )
				{
				}
			}
			
			// Generate APDescription object
			APDescription ap_desc=new APDescription();
			ap_desc.setName(platformProfile.getHAPName());
			ap_desc.setDynamic(new Boolean(platformProfile.getDynamic()));
			ap_desc.setMobility(new Boolean(platformProfile.getMobility()));
			ap_desc.setTransportDescription( aptd );
			
			results.add(ap_desc);
			
			// Get original ACL
			ACL acl = (ACL) _acl.clone();
			
			// Create result object
			AgentManagementResult amr = new AgentManagementResult();
			amr.setAgentManagementAction( _ama );
			amr.setResultSet( results );
			
			// Setup message
			acl.setPerformative( FIPACONSTANTS.INFORM );
			acl.setReceiverAID( acl.getSenderAID() );
			acl.setSenderAID( _owner.getAID() );
			acl.setInReplyTo( acl.getReplyWith() );
			acl.setReplyWith( null );
			acl.setContentObject( amr );
			
			// Attempt to send the platform profile
			forward( acl );
			
			// Update the AMS state
			_state.setFlag( INFORM_MSG );
			
			done = true;
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "An exception occured whilst sending an inform(done): ", t,
			this, DIAGNOSTICS.LEVEL_MAX );
		}
		
		return done;
	}
	
	/**
	 * Clears all message flags. Use before handling a new message.
	 */
	
	protected void clearMessageFlags()
	{
		
		_state.clearFlag( AGREE_MSG );
		_state.clearFlag( INFORM_MSG );
		_state.clearFlag( FAILURE_MSG );
		_state.clearFlag( REFUSE_MSG );
		_state.clearFlag( NOT_UND_MSG );
		_state.setFlag( NO_MSG_SENT );
		
	}
	
} // End class