//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.platform.ams;

//----------------------------------------------------------------------

import fipaos.agent.*;
import fipaos.agent.conversation.*;
import fipaos.agent.task.*;
import fipaos.ont.fipa.fipaman.*;
import fipaos.ont.fipa.*;
import fipaos.util.*;
import fipaos.skill.db.*;
import fipaos.platform.df.*;
import fipaos.ont.BASETOKENS;

//----------------------------------------------------------------------

/**
 * TODO: Write a description of the class
 * <br>
 * <br>
 */
public class DeregisterTask extends AMSTask
{

//----------------------------------------------------------------------
// CONSTRUCTOR(S)
//----------------------------------------------------------------------

	/** This is the constructor for the AMS Deregister task.
	 * @param conv Conversation
	 * @param ama Agent Management Action
	 * @param db Database
	 */
	public DeregisterTask( Conversation conv, AgentManagementAction ama, Database db )
	{
		super(conv,ama,db);
	}


//----------------------------------------------------------------------
// PUBLIC METHODS
//----------------------------------------------------------------------	
	
	/** This is the initial method called by the task.
	 */
	public void startTask()
	{
		clearMessageFlags();
		
		// Get the DFAD object from the AMA object
		AMSAgentDescription amsad = _ama.getAMSAgentDescription();
		
		DIAGNOSTICS.println( "Sender = " + _acl.getSenderAID() );
		DIAGNOSTICS.println( "Agent = " + amsad.getAgentID() );
		
		// Check if the Agent is permitted to register
		if( !_acl.getSenderAID().equals( amsad.getAgentID() ) &&
				!_acl.getSenderAID().getName().startsWith( FIPACONSTANTS.AMS + '@' ) )
		{
			
			sendRefuse( FIPAMANCONSTANTS.UNAUTHORISED );
			done();
			return;

		}
		else if ( ! amsad.canDeregister() ) 
		{
			// Reply with a refuse - not all mandatory fields
			sendRefuse( FIPAMANCONSTANTS.MANDATORY_ATTRIBUTE_MISSING );
			done();
			return;
		}

		// Send agree
		sendAgree();
		
		
		
		// Check if the Agent isn't already registered
		if( !isRegistered( amsad ) )
		{
			
			sendFailure( FIPAMANCONSTANTS.AGENT_NOT_REGISTERED );
			done();
			return;
			
		}
		
		
		// Attempt to register Agent
		if( !deregister( amsad ) )
		{
			
			sendFailure( FIPAMANCONSTANTS.INTERNAL_ERROR );
			done();
			return;
			
		}
		
		
		// Send inform back :-)
		sendInformDone();
		
		done();
	}
		

//----------------------------------------------------------------------
// MAIN/TESTING METHOD
//----------------------------------------------------------------------
  

} // End class

	
