//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.platform.ams;

//----------------------------------------------------------------------

import fipaos.agent.*;
import fipaos.agent.conversation.*;
import fipaos.agent.task.*;
import fipaos.ont.fipa.fipaman.*;
import fipaos.ont.fipa.*;
import fipaos.util.*;
import fipaos.skill.db.*;
import fipaos.platform.df.*;
import fipaos.ont.BASETOKENS;

//----------------------------------------------------------------------

/**
 * <br>
 * <br>
 */
public class ModifyTask extends AMSTask
{
     
//----------------------------------------------------------------------
// CONSTANTS
//----------------------------------------------------------------------




//----------------------------------------------------------------------
// STATIC VARIABLES
//----------------------------------------------------------------------
	


	
//----------------------------------------------------------------------
// INSTANCE VARIABLES
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// CONSTRUCTOR(S)
//----------------------------------------------------------------------

	/** This is the constructor for the ModifyTask.
	 * @param conv Conversation
	 * @param ama Agent Management Action
	 * @param db Database
	 */
	public ModifyTask( Conversation conv, AgentManagementAction ama, Database db )
	{
		super(conv,ama,db);
	}

//----------------------------------------------------------------------
// PRIVATE METHODS
//----------------------------------------------------------------------




//----------------------------------------------------------------------
// PROTECTED METHODS
//----------------------------------------------------------------------




//----------------------------------------------------------------------
// PUBLIC METHODS
//----------------------------------------------------------------------	
	
	/** This is the initial method to be invoked automatically.
	 */
	public void startTask()
	{
		clearMessageFlags();
		
		// Get the DFAD object from the AMA object
		AMSAgentDescription amsad = _ama.getAMSAgentDescription();
		
		// Check if the Agent is permitted to register
		if( !_acl.getSenderAID().equals( amsad.getAgentID() ) &&
				!_acl.getSenderAID().getName().startsWith( FIPACONSTANTS.AMS + '@' ) )
		{
			
			sendRefuse( FIPAMANCONSTANTS.UNAUTHORISED );
			done();
			return;

		}
		else if ( ! amsad.canModify() ) 
		{
			// Reply with a refuse - not all mandatory fields
			sendRefuse( FIPAMANCONSTANTS.UNAUTHORISED );
			done();
			return;
		}
		
		// Send agree
		sendAgree();
		
		
		// Check if the Agent isn't already registered
		if( !isRegistered( amsad ) )
		{
			
			sendFailure( FIPAMANCONSTANTS.AGENT_NOT_REGISTERED );
			done();
			return;
			
		}
		
		
		// Attempt to register Agent
		if( ! register( amsad ) )
		{
			
			sendFailure( FIPAMANCONSTANTS.INTERNAL_ERROR );
			done();
			return;
			
		}
		
		
		// Send inform back :-)
		sendInformDone();
		
		done();
	}
	
//----------------------------------------------------------------------
// MAIN/TESTING METHOD
//----------------------------------------------------------------------
  

} // End class

