//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.platform.df;

import fipaos.platform.DirectoryFacilitator;
import fipaos.ont.fipa.fipaman.*;
import fipaos.util.*;

import java.util.*;
import javax.swing.*;

/**
 * Temporary GUI to allow cross-DF registration
 */
public class DFCrossPlatformRegistrationFrame extends javax.swing.JFrame
{
	private DirectoryFacilitator _parent;
	private List _registered = new LinkedList();
	
	public DFCrossPlatformRegistrationFrame( DirectoryFacilitator df )
	{
		this();
		_parent = df;
		setTitle( "DF Cross Registration GUI (" + df.getAID().getName() + ")" );
	}
	
	/** Creates new form DFCrossPlatformRegistrationFrame */
	private DFCrossPlatformRegistrationFrame()
	{
		initComponents ();
		pack ();
		// Resize and reposition window so it appears in the center
		// of the screen, and is not too large
		java.awt.Toolkit tk = java.awt.Toolkit.getDefaultToolkit();
		int width = getSize().width;
		int height = getSize().height;
		
		if( width < 320 )
		{
			width = 320;
		}
		
		if( height < 240 )
		{
			height = 240;
		}
		
		setSize( width, height );
		width = tk.getScreenSize().width - width;
		height = tk.getScreenSize().height - height;
		setLocation( width/2, height/2 );
	}
	
		/** This method is called from within the constructor to
		 * initialize the form.
		 * WARNING: Do NOT modify this code. The content of this method is
		 * always regenerated by the FormEditor.
		 */
	private void initComponents()//GEN-BEGIN:initComponents
	{
		jPanel1 = new javax.swing.JPanel();
		jList1 = new javax.swing.JList();
		jLabel1 = new javax.swing.JLabel();
		jTextField1 = new javax.swing.JTextField();
		_register = new javax.swing.JButton();
		_add = new javax.swing.JButton();
		jLabel2 = new javax.swing.JLabel();
		getContentPane().setLayout(new java.awt.GridBagLayout());
		java.awt.GridBagConstraints gridBagConstraints1;
		setIconImage(new ImageIcon( ClassLoader.getSystemResource( "images/logo.gif" ) ).getImage());
		addWindowListener(new java.awt.event.WindowAdapter()
		{
			public void windowClosing(java.awt.event.WindowEvent evt)
			{
				exitForm(evt);
			}
		}
		);
		
		jPanel1.setLayout(new java.awt.BorderLayout());
		jPanel1.setBorder(new javax.swing.border.TitledBorder("Remote DF's Succesfully Registered With"));
		
		jPanel1.add(jList1, java.awt.BorderLayout.CENTER);
  		
  		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 0;
		gridBagConstraints1.gridy = 0;
		gridBagConstraints1.gridwidth = 0;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
		gridBagConstraints1.weightx = 1.0;
		gridBagConstraints1.weighty = 1.0;
		getContentPane().add(jPanel1, gridBagConstraints1);
		
		
		jLabel1.setText("Remote DF Name");
		jLabel1.setForeground(java.awt.Color.black);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.insets = new java.awt.Insets(3, 3, 3, 3);
		getContentPane().add(jLabel1, gridBagConstraints1);
		
		
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridwidth = 0;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints1.insets = new java.awt.Insets(3, 3, 3, 3);
		gridBagConstraints1.weightx = 1.0;
		getContentPane().add(jTextField1, gridBagConstraints1);
		
		
		_register.setLabel("Send Registration to DF");
		_register.setText("Register this DF with Remote DF");
		_register.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				jButton1ActionPerformed(evt);
			}
		}
		);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 1;
		gridBagConstraints1.gridy = 2;
		gridBagConstraints1.insets = new java.awt.Insets(3, 3, 3, 3);
		gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
		getContentPane().add(_register, gridBagConstraints1);
		
		
		_add.setLabel("Add DF");
		_add.setText("Add Remote DF to directory");
		_add.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				_addActionPerformed(evt);
			}
		}
		);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 2;
		gridBagConstraints1.gridy = 2;
		gridBagConstraints1.gridwidth = 0;
		gridBagConstraints1.insets = new java.awt.Insets(3, 3, 3, 3);
		gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
		getContentPane().add(_add, gridBagConstraints1);
		
		
		jLabel2.setBorder(new javax.swing.border.SoftBevelBorder(1));
		jLabel2.setText(" ");
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridwidth = 0;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints1.weightx = 1.0;
		getContentPane().add(jLabel2, gridBagConstraints1);
		
	}//GEN-END:initComponents

  private void _addActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event__addActionPerformed
		// Check that we've been given a valid AID-type name
		AgentID df = validateAID();
		if( df != null )
		{
			status( "Registering \"" + df.getName() + "\" with this DF..." );
			_parent.registerRemoteDF( df );
			status( "Registered \"" + df.getName() + "\" with this DF OK" );
		}
  }//GEN-LAST:event__addActionPerformed
		
  private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
		// Check that we've been given a valid AID-type name
		final AgentID df = validateAID();
		if( df != null )
		{
			Thread t = new Thread( new Runnable()
			{
				public void run()
				{
					try
					{
						status( "Registering with \"" + df.getName() + "\"..." );
						_parent.registerWithRemoteDF( df );
						status( "Registered with \"" + df.getName() + "\" OK" );
						_registered.add( df.getName() );
						jList1.setListData( _registered.toArray() );
					}
					catch( DFRegistrationException e )
					{
						status( "Registration with \"" + df.getName() + "\" failed: " + e.getExceptionReason() );
					}
					catch( Throwable e )
					{
						status( "Registration with \"" + df.getName() + "\" failed: " + e );
						DIAGNOSTICS.println( "Registration with \"" + df.getName() + "\" failed", e,
							DIAGNOSTICS.LEVEL_MAX );
					}
				}
			} );
			t.start();
		}
  }//GEN-LAST:event_jButton1ActionPerformed
	
	private void status( String s )
	{
		jLabel2.setText( s );
		jLabel2.repaint();
	}
	
	private AgentID validateAID()
	{
		String text = jTextField1.getText().trim();
		AgentID aid = null;
		
		status( "Validating agent name..." );
		
		try
		{
			AgentID aid2 = new AgentID();
			aid2.setName( text );
			aid = new AgentID( aid2.toString() );
			
			status( "Validated agent name OK" );
		}
		catch( Throwable t )
		{
			status( "Validation of agent name failed" );
			JOptionPane.showMessageDialog( this, "Please enter a valid agent name (e.g. of the form \"agent@hap\"" );
		}
		
		return aid;
	}
	
		/** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm

			setVisible( false );

    }//GEN-LAST:event_exitForm
		
		/**
		 * @param args the command line arguments
		 */
		public static void main (String args[])
		{
			new DFCrossPlatformRegistrationFrame();
		}
		
		
		// Variables declaration - do not modify//GEN-BEGIN:variables
		private javax.swing.JPanel jPanel1;
		private javax.swing.JList jList1;
		private javax.swing.JLabel jLabel1;
		private javax.swing.JTextField jTextField1;
		private javax.swing.JButton _register;
		private javax.swing.JButton _add;
		private javax.swing.JLabel jLabel2;
		// End of variables declaration//GEN-END:variables
		
}