//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.platform.df;

import fipaos.agent.task.*;
import fipaos.agent.*;
import fipaos.agent.conversation.*;
import fipaos.ont.fipa.*;
import fipaos.ont.fipa.fipaman.*;
import fipaos.ont.*;

public class DFDeregistrationTask extends Task 
{
   
//----------------------------------------------------------------------
// INSTANCE VARIABLES
//----------------------------------------------------------------------
	
	/**
	 * AID of DF to deregister from
	 */
	private AgentID _df;

	/**
	 * DF description of agent to deregister
	 */
	private DFAgentDescription _dfad;	
	
//----------------------------------------------------------------------
// CONSTRUCTOR(S)
//----------------------------------------------------------------------

	/**
	 * Create a Task to deregister with the DF
	 */
	public DFDeregistrationTask()
	{
		
	}
		
//----------------------------------------------------------------------
// PUBLIC METHODS
//----------------------------------------------------------------------	

	/**
	 * Invoked when task starts
	 */
	public void startTask()
	{

		_df = _owner.getLocalDFAID();
		
		// Get AID of Agent to deregister
		AgentID id = _owner.getAID();
		
		_dfad = new DFAgentDescription();
		_dfad.setAgentName( id );
		
		// Create Conversation for interaction with the AMS
		ACL to_df = getNewConversation( FIPACONSTANTS.FIPA_REQUEST );

		// Set-up message parameters
		to_df.setReceiverAID( _df );
		to_df.setOntology( FIPAMANCONSTANTS.FIPA_AGENT_MANAGEMENT );
		to_df.setLanguage( FIPACONSTANTS.SL0 );
		to_df.setPerformative( FIPACONSTANTS.REQUEST );

		// Create AgentManagementAction
		AgentManagementAction ama = new AgentManagementAction();
		ama.setAction( FIPAMANCONSTANTS.DEREGISTER );
		ama.setAgentID( _df );
		ama.setDFAgentDescription( _dfad );

		to_df.setContentObject( ama );

		// Send message
		forward( to_df );
	}
	
	
	/**
	 * Deal with not-understood messages
	 */
	public void handleNotUnderstood( Conversation conv )
	{
		ACL in = conv.getACL( conv.getLatestMessageIndex() );
		AgentID sender = in.getSenderAID();
		
		// Indicate we're done and pass-back results
		done( new Object[] { sender, FIPACONSTANTS.NOT_UNDERSTOOD } );
	}
	
	
	/**
	 * Deal with refuse messages
	 */
	public void handleRefuse( Conversation conv )
	{
		ACL in = conv.getACL( conv.getLatestMessageIndex() );
		AgentID sender = in.getSenderAID();
		String reason = new String();
		Object content = in.getContentObject();
		
		try
		{
			if( content instanceof String )
			{
				AgentManagementException ame = new AgentManagementException( (String)content );

				if( ame.getException() != null )
				{
					reason = ame.getException();
				}
			}
			else if( content instanceof AgentManagementException )
			{
				reason = ( ( AgentManagementException) content ).getException();
			}
		}
		catch( Throwable t )
		{
		}
		
		// Indicate we're done and pass-back results
		done( new Object[] { sender, reason } );
	}
	
	
	/**
	 * Deal with agree messages
	 */
	public void handleAgree( Conversation conv )
	{
		
		// Do nothing
		
	}
	
	
	/**
	 * Deal with failure messages
	 */
	public void handleFailure( Conversation conv )
	{
		ACL in = conv.getACL( conv.getLatestMessageIndex() );
		AgentID sender = in.getSenderAID();
		String reason = new String();
		Object content = in.getContentObject();
		
		try
		{
			if( content instanceof String )
			{
				AgentManagementException ame = new AgentManagementException( (String)content );

				if( ame.getException() != null )
				{
					reason = ame.getException();
				}
			}
			else if( content instanceof AgentManagementException )
			{
				reason = ( ( AgentManagementException) content ).getException();
			}
		}
		catch( Throwable t )
		{
		}
		
		// Indicate we're done and pass-back results
		done( new Object[] { sender, reason } );
	}
	

	/**
	 * Deal with failure messages
	 */
	public void handleInform( Conversation conv )
	{
		ACL in = conv.getACL( conv.getLatestMessageIndex() );
		AgentID sender = in.getSenderAID();
		
		// Indicate we're done and pass-back results
		done( sender );
	}
	
	
//----------------------------------------------------------------------
// INNER-CLASSES
//----------------------------------------------------------------------	
	
//----------------------------------------------------------------------
// MAIN/TESTING METHOD
//----------------------------------------------------------------------
	
}
