//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.platform.df;

//----------------------------------------------------------------------

import fipaos.agent.*;
import fipaos.agent.conversation.*;
import fipaos.agent.task.*;
import fipaos.ont.fipa.fipaman.*;
import fipaos.ont.fipa.*;
import fipaos.skill.db.*;
import fipaos.util.*;
import fipaos.platform.*;
import fipaos.platform.df.*;
import fipaos.ont.BASETOKENS;

import java.util.*;

//----------------------------------------------------------------------

/**
 * TODO: Write a description of the class
 * <br>
 * <br>
 * $Revision: 1.9 $ $Date: 2001/04/04 22:48:19 $ $Author: alantreadway $
 */
public class DFTask extends Task
{
     
//----------------------------------------------------------------------
// CONSTANTS
//----------------------------------------------------------------------




//----------------------------------------------------------------------
// STATIC VARIABLES
//----------------------------------------------------------------------
	


	
//----------------------------------------------------------------------
// INSTANCE VARIABLES
//----------------------------------------------------------------------

	/**
	 * Original ACL message containing the request
	 */
	protected ACL _acl;
	
	/**
	 * Reference to the DF's Database
	 */
	protected Database _df_db;

	/**
	 * Reference to the Conversation we are answering
	 */
	protected Conversation _conv;

	/**
	 * Reference to the AMA that this Task is attempting to carry out
	 */
	protected AgentManagementAction _ama;
	
//----------------------------------------------------------------------
// CONSTRUCTOR(S)
//----------------------------------------------------------------------

	/** Initialise DFTask
	 * @param conv Conversation associated with this Task
	 * @param ama AgentManagementAction contained within the given Conversation
	 * @param db Database associated with the DF
	 */
	public DFTask( Conversation conv, AgentManagementAction ama, Database db )
	{
		// Save references
		_conv = conv;
		_ama = ama;
		_df_db = db;
		_acl = _conv.getACL( 0 );
	}

	
//----------------------------------------------------------------------
// PROTECTED METHODS
//----------------------------------------------------------------------

	
	/** Indicates if the Agent the DFAgentDescription relates to is registered
	 * @param dfad
	 * @return True if the Agent is registered with this DF
	 */
	protected boolean isRegistered( DFAgentDescription dfad )
	{
		// Get the AgentID of the Agent in the DFAD
		AgentID agent = dfad.getAgentID();
		
		// Check if Agent is registered
		return isRegistered( agent );
	}
	
	
	/** Indicates if the given Agent is already registered
	 * @param aid
	 * @return True if the Agent is registered
	 */
	protected boolean isRegistered( AgentID aid )
	{
		return ( getRegistered( aid ) != null );
	}

	
	/** Gets the DFAgentDescription associated with the Agent
	 * @param aid
	 * @return The DFAgentDescription for the given Agent, or null
	 */
	protected DFAgentDescription getRegistered( AgentID aid )
	{
		DFAgentDescription dfad = null;
		
		try
		{
			// Check if it is contained within the database
			
			// Start transaction
			_df_db.startTransaction( true );
			
			// Get DFAD for given Agent
			dfad = (DFAgentDescription) _df_db.findObject( aid.getName() );
			
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "Exception when getting registered agent: ", t, this );
		}
		finally
		{
			try
			{
				// End transaction
				_df_db.endTransaction();				
			}
			catch( Throwable t )
			{
			}
		}
		
		return dfad;
	}
	
	
	/** Adds the given DFAgentDescription into the DF Database
	 * @param dfad
	 * @return True if the operation was successful
	 */
	protected boolean register( DFAgentDescription dfad )
	{
		boolean done = false;
		
		try
		{
			// Check if it is contained within the database
			
			// Start transaction
			_df_db.startTransaction( false );
			
			// Get DFAD for given Agent
			_df_db.addObject( dfad );
			
			done = true;
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "Exception when registering agent: ", t, this );
		}
		finally
		{
			try
			{
				// End transaction
				_df_db.endTransaction();				
			}
			catch( Throwable t )
			{
			}
		}
		
		return done;
	}
	
	
	/** Removes the given DFAgentDescription from the DF's Databas
	 * @param dfad
	 * @return True if it was removed OK
	 */
	protected boolean deregister( DFAgentDescription dfad )
	{
		boolean done = false;
		
		try
		{
			// Check if it is contained within the database
			
			// Start transaction
			_df_db.startTransaction( false );
			
			// Get DFAD for given Agent
			_df_db.removeObject( dfad );
			
			done = true;
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "Exception when deregistering agent: ", t, this );
		}
		finally
		{
			try
			{
				// End transaction
				_df_db.endTransaction();				
			}
			catch( Throwable t )
			{
			}
		}
		
		return done;
	}
	
	
	/** Send an agree message back to the originator of the request for this Task
	 * @return True if everything went OK
	 */
	protected boolean sendAgree()
	{
		boolean done = false;
		
		try
		{
			ACL acl = (ACL) _acl.clone();

			// Setup message
			acl.setPerformative( FIPACONSTANTS.AGREE );
			acl.setReceiverAID( acl.getSenderAID() );
			acl.setSenderAID( _owner.getAID() );
			acl.setInReplyTo( acl.getReplyWith() );
			acl.setReplyWith( null );

			// Attempt to send the agree
			forward( acl );
			
			done = true;
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "An exception occured whilst sending an agree: ", t,
																									this, DIAGNOSTICS.LEVEL_MAX );
		}
		
		return done;
	}
	
	
	/** Sends a refuse message back to the originator of the request for this Task
	 * @param reason Failure reason
	 * @return True if everything went OK
	 */
	protected boolean sendRefuse( String reason )
	{
		boolean done = false;
		
		try
		{
			// Get original ACL
			ACL acl = (ACL) _acl.clone();

			// Create AgentManagementException
			AgentManagementException ame = new AgentManagementException( _ama, reason );
			
			// Setup message
			acl.setPerformative( FIPACONSTANTS.REFUSE );
			acl.setReceiverAID( acl.getSenderAID() );
			acl.setSenderAID( _owner.getAID() );
			acl.setInReplyTo( acl.getReplyWith() );
			acl.setReplyWith( null );
			acl.setContentObject( ame );

			// Attempt to send the refuse
			forward( acl );
			
			done = true;
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "An exception occured whilst sending a refuse: ", t,
																									this, DIAGNOSTICS.LEVEL_MAX );
		}
		
		return done;
	}
	
	
	/** Sends a failure back to the originator of the request for this Task
	 * @param reason
	 * @return Returns True if everything went OK
	 */
	protected boolean sendFailure( String reason )
	{
		boolean done = false;
		
		try
		{
			// Get original ACL
			ACL acl = (ACL) _acl.clone();

			// Create AgentManagementException
			AgentManagementException ame = new AgentManagementException( _ama, reason );
			
			// Setup message
			acl.setPerformative( FIPACONSTANTS.FAILURE );
			acl.setReceiverAID( acl.getSenderAID() );
			acl.setSenderAID( _owner.getAID() );
			acl.setInReplyTo( acl.getReplyWith() );
			acl.setReplyWith( null );
			acl.setContentObject( ame );

			// Attempt to send the refuse
			forward( acl );
			
			done = true;
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "An exception occured whilst sending a failure: ", t,
																									this, DIAGNOSTICS.LEVEL_MAX );
		}
		
		return done;
	}
	
	
	/** Sends an inform (with associated "done" term in content) back to originator
	 * of request for this Task
	 * @return
	 */
	protected boolean sendInformDone()
	{
		boolean done = false;
		
		try
		{
			// Get original ACL
			ACL acl = (ACL) _acl.clone();

			// Setup message
			acl.setPerformative( FIPACONSTANTS.INFORM );
			acl.setReceiverAID( acl.getSenderAID() );
			acl.setSenderAID( _owner.getAID() );
			acl.setInReplyTo( acl.getReplyWith() );
			acl.setReplyWith( null );
			acl.setContentObject( "(done " + _ama.toString() + ")" );

			// Attempt to send the refuse
			forward( acl );
			
			done = true;
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "An exception occured whilst sending an inform(done): ", t,
																									this, DIAGNOSTICS.LEVEL_MAX );
		}
		
		// Send messages to appropriate subscribed Agents
		((DirectoryFacilitator) _owner)._subs_manager.DFUpdate( _ama );
		
		return done;
	}
	
	/**
	 * This method would be called when this df would like to send a result message
	 *
	 * @param results The set of results to propogate back
	 */
	protected boolean sendInformResult( Set results )
	{

			boolean done = false;
		
			try
			{
				// Get original ACL
				ACL acl = (ACL) _acl.clone();
				
				// Create result object
				AgentManagementResult amr = new AgentManagementResult();
				amr.setAgentManagementAction( _ama );
				amr.setResultSet( results );

				// Setup message
				acl.setPerformative( FIPACONSTANTS.INFORM );
				acl.setReceiverAIDs( Arrays.asList( new Object[] { acl.getSenderAID() } ) );
				acl.setSenderAID( _owner.getAID() );
				acl.setInReplyTo( acl.getReplyWith() );
				acl.setReplyWith( null );
				acl.setContentObject( amr );

				DIAGNOSTICS.println( "Sending inform with results...", this );
				
				forward(acl);
				
				DIAGNOSTICS.println( "Sent inform with results :-)", this );
				
				done = true;
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "An exception occured whilst sending an inform(result): ", t,
																									this, DIAGNOSTICS.LEVEL_MAX );
		}
		
		return done;
	} 
} // End class