//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.platform.df;

import fipaos.agent.task.*;
import fipaos.agent.conversation.*;
import fipaos.ont.fipa.fipaman.*;
import fipaos.ont.fipa.*;
import fipaos.util.*;

import java.util.*;

/** This Task manages SubscriptionHandlerTask's within a DF
 */
public class SubscriptionManagerTask extends Task
{
	private List _active_subscriptions;
	
/** Default constructor
 */
	public SubscriptionManagerTask()
	{
		_active_subscriptions = new LinkedList();
	}
	
	/** Invoked when the Task starts - doesn't do anything special
	 */
	public void startTask()
	{
		// Do nothing
	}
	
/** Invoked when a SubscriptionHandlerTask completes (i.e. the subscription is cancelled).
 * This causes the Task in question to be removed from the list of known SubscriptionHandlerTask's
 * @param t The Task that has finished
 */
	public void doneSubscriptionHandlerTask( Task t )
	{
		// Sub-task completed, remove it from the list of active
		// SubscriptionHandlerTask's
		_active_subscriptions.remove( t );
	}
	
/** Invoked when a SubscriptionHandlerTask fails for some reason - this causes
 * the Task in question to be removed from the list of known SubscriptionHandlerTask's
 * @param t Task that has failed
 * @param th Exception associated with failure
 */
	public void errorSubscriptionHandlerTask( Task t, Throwable th )
	{
		// Sub-task failed, remove it from the list of active
		// SubscriptionHandlerTask's
		_active_subscriptions.remove( t );
		
		DIAGNOSTICS.println( "SubscriptionHandlerTask failed", th, this, DIAGNOSTICS.LEVEL_MAX );
	}
	
/** Causes a SubscriptionHandlerTask to be instantiated to deal with the subscription
 * request from another Agent
 * @param conv Conversation associated with the subscription
 */
	public void newSubscriber( Conversation conv )
	{
		// Create a new SubscriptionHandlerTask to deal with subscriber
		SubscriptionHandlerTask sht = new SubscriptionHandlerTask( conv );
		newTask( sht );
		
		// Add SubscriptionHandlerTask to our list
		_active_subscriptions.add( sht );
	}
	
/** Invoked whenever an update to the DF's database has SUCCESSFULLY completed, causing
 * notification messages to be sent to subscribed Agents
 * @param ama The AgentManagementAction associated with the event
 */
	public void DFUpdate( AgentManagementAction ama )
	{
		// Propogate update to all known SubscriptionHandlerTask's
		Iterator i = _active_subscriptions.iterator();
		
		// Iterate through tasks
		while( i.hasNext() )
		{
			// Get next
			SubscriptionHandlerTask sht = (SubscriptionHandlerTask) i.next();
			
			// Propogate details
			sht.DFUpdate( ama );
		}
	}
}
