//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.platform.dfgui;

import fipaos.agent.task.*;
import fipaos.ont.fipa.fipaman.*;
import fipaos.util.*;

import java.util.*;

/** This Agent aims to highlight how easy it is to implement a DFGUI using the
 * DFGUIAgent base class
 */
public class DFGUITestAgent extends DFGUIAgent
{
/** Default Agent constructor for use with AgentLoader
 * @param name Agent name
 * @param platform Platform profile location
 * @param owner Ownership of Agent
 */	
	public DFGUITestAgent( String name, String platform, String owner )
	{
		super( name, platform, owner );
		
		// Indicate that messages can start arriving
		startPushing();
		
		// Start monitoring the DF
		DIAGNOSTICS.println( "Listing entries in local DF", this, DIAGNOSTICS.LEVEL_MAX );
		
		// Get list of Agents in local DF - ensures we are listening to this DF :)
		Map agents = listAgents( getLocalDFAID() );
		DIAGNOSTICS.println( "Found the following agents: \n" + agents, this, DIAGNOSTICS.LEVEL_MAX );
	}
	
/** Invoked when an Agent has been registered with a DF that has previously
 * been searched
 * @param df DF associated with this event
 * @param df_desc DF description of the entry that has been updated
 */
	public void registered( AgentID df, DFAgentDescription df_desc )
	{
		// Get "list" of Agents with the DF
		Map agents = listAgents( df );
		
		DIAGNOSTICS.println( "Received registration details from " + df.getName() + ":" +
		df_desc + "\nCurrent DF contents = " + agents, this, DIAGNOSTICS.LEVEL_MAX );
	}
	
/** Invoked when an Agent has been deregistered with a DF that has previously
 * been searched
 * @param df DF associated with this event
 * @param df_desc DF description of the entry that has been updated
 */
	public void deregistered( AgentID df, DFAgentDescription df_desc)
	{
		// Get "list" of Agents with the DF
		Map agents = listAgents( df );
		
		DIAGNOSTICS.println( "Received deregistration details from " + df.getName() + ":" +
		df_desc + "\nCurrent DF contents = " + agents, this, DIAGNOSTICS.LEVEL_MAX );
	}
	
/** Invoked when an Agent has been modified with a DF that has previously
 * been searched
 * @param df DF associated with this event
 * @param df_desc DF description of the entry that has been updated
 */
	public void modified( AgentID df, DFAgentDescription df_desc)
	{
		// Get "list" of Agents with the DF
		Map agents = listAgents( df );
		
		DIAGNOSTICS.println( "Received modification details from " + df.getName() + ":" +
		df_desc + "\nCurrent DF contents = " + agents, this, DIAGNOSTICS.LEVEL_MAX );
	}
}