//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.platform.dfgui.swing;

import javax.swing.*;
import java.awt.Dimension;
import java.util.*;

import fipaos.util.DIAGNOSTICS;
import fipaos.ont.fipa.fipaman.DFAgentDescription;

/**
 * An editor that can be used to view and modigy FIPA's df-agent-descriptions.
 *
 */
public class DFAgentDescriptionEditor extends javax.swing.JFrame
{
	
		/**
		 * Creates new form DFAgenrtDescriptionEditor.
		 *
		 * @param selected_agent name of the selected agent
		 * @param desc df-agent-description for the selected agent
		 * @param read_only indicates if the GUI has been opened for editing or viewing
		 */
	public DFAgentDescriptionEditor(String selected_agent,DFAgentDescription desc,boolean read_only)
	{
		try
		{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}
		catch(Throwable e)
		{
			DIAGNOSTICS.println("Exception", e, this, DIAGNOSTICS.LEVEL_MIN );
		}
		initComponents ();
		pack ();		

		// Resize and reposition window so it appears in the center
		// of the screen, and is not too large
		java.awt.Toolkit tk = java.awt.Toolkit.getDefaultToolkit();
		int width = getSize().width;
		int height = getSize().height;
		width = 500;
		height = 700;
		if( tk.getScreenSize().width < width )
		{
			width = tk.getScreenSize().width;
		}
		if( tk.getScreenSize().height < height )
		{
			height = tk.getScreenSize().height;
		}
		
		setSize( width, height );
		width = tk.getScreenSize().width - width;
		height = tk.getScreenSize().height - height;
		setLocation( width/2, height/2 );
		
		if (read_only)
		{
			//disable all text fields and areas
			_tf_aid.setEnabled(false);
			//disable all other buttons except OK and cancel
			_b_edit_address.setEnabled(false);
			_b_remove_address.setEnabled(false);
			_b_add_address.setEnabled(false);
			_b_edit_protocol.setEnabled(false);
			_b_add_protocol.setEnabled(false);
			_b_remove_protocol.setEnabled(false);
			_b_remove_ontology.setEnabled(false);
			_b_edit_ontology.setEnabled(false);
			_b_add_ontology.setEnabled(false);
			_b_add_language.setEnabled(false);
			_b_remove_language.setEnabled(false);
			_b_edit_language.setEnabled(false);
			_b_add_service.setEnabled(false);
			_b_edit_service.setEnabled(false);
			_b_remove_service.setEnabled(false);
		}
		_tf_aid.setText(desc.getAgentName());
		_addresses = new Vector(desc.getAgentID().getAddresses());
		_address_list.setListData(_addresses.toArray());
		_protocols = (desc.getProtocols() != null) ? desc.getProtocols().toArray() : new Object[]
		{};
		_protocol_list.setListData(_protocols);
		
		_ontologies = (desc.getOntologies() != null) ? desc.getOntologies().toArray() : new Object[]
		{};
		_ontology_list.setListData(_ontologies);
		
		_languages = (desc.getLanguages() != null) ? desc.getLanguages().toArray() : new Object[]
		{};
		_language_list.setListData(_languages);
		
		_services = (desc.getAgentServices() != null) ? desc.getAgentServices().toArray() : new Object[]
		{};
		for (int i=0; i<_services.length; i++)
		{
			_services[i] = properServicesLayout(_services[i].toString());
		}
		_service_list.setListData(_services);
		
		_string_text.setText(properStringifiedLayout(desc.toString()));
		
		_read_only = read_only;
		setVisible(true);
	}
	
		/** This method is called from within the constructor to
		 * initialize the form.
		 * WARNING: Do NOT modify this code. The content of this method is
		 * always regenerated by the FormEditor.
		 */
	private void initComponents()//GEN-BEGIN:initComponents
	{
		_tabbed_pane = new javax.swing.JTabbedPane();
		_agent_description_pane = new javax.swing.JPanel();
		_aid_panel = new javax.swing.JPanel();
		_b_edit_address = new javax.swing.JButton();
		_b_remove_address = new javax.swing.JButton();
		_b_add_address = new javax.swing.JButton();
		_tf_aid = new javax.swing.JTextField();
		_l_aid_addresses = new javax.swing.JLabel();
		_l_aid = new javax.swing.JLabel();
		_aid_scrollpane = new javax.swing.JScrollPane();
		_address_list = new javax.swing.JList();
		_protocol_panel = new javax.swing.JPanel();
		_b_remove_protocol = new javax.swing.JButton();
		_b_add_protocol = new javax.swing.JButton();
		_b_edit_protocol = new javax.swing.JButton();
		_protocol_scollpane = new javax.swing.JScrollPane();
		_protocol_list = new javax.swing.JList();
		_ontology_panel = new javax.swing.JPanel();
		_b_remove_ontology = new javax.swing.JButton();
		_b_edit_ontology = new javax.swing.JButton();
		_b_add_ontology = new javax.swing.JButton();
		_ontology_scrollpane = new javax.swing.JScrollPane();
		_ontology_list = new javax.swing.JList();
		_language_panel = new javax.swing.JPanel();
		_b_add_language = new javax.swing.JButton();
		_b_remove_language = new javax.swing.JButton();
		_b_edit_language = new javax.swing.JButton();
		_language_scrollpane = new javax.swing.JScrollPane();
		_language_list = new javax.swing.JList();
		_b_OK_agent_description = new javax.swing.JButton();
		_b_cancel_agent_description = new javax.swing.JButton();
		_service_pane = new javax.swing.JPanel();
		_service_scrollpane = new javax.swing.JScrollPane();
		_service_list = new javax.swing.JList();
		_b_add_service = new javax.swing.JButton();
		_b_edit_service = new javax.swing.JButton();
		_b_remove_service = new javax.swing.JButton();
		_b_OK_service = new javax.swing.JButton();
		_b_cancel_service = new javax.swing.JButton();
		_full_description = new javax.swing.JPanel();
		_b_OK_string = new javax.swing.JButton();
		_b_cancel_string = new javax.swing.JButton();
		_string_scollpane = new javax.swing.JScrollPane();
		_string_text = new javax.swing.JTextArea();
		_l_string = new javax.swing.JLabel();
		getContentPane().setLayout(new java.awt.GridBagLayout());
		java.awt.GridBagConstraints gridBagConstraints1;
		setIconImage(new ImageIcon( getClass().getResource(IMAGE_DIRECTORY+FIPA_GIF)).getImage());
		setTitle("DF Agent Description");
		setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
		addWindowListener(new java.awt.event.WindowAdapter()
		{
			public void windowClosing(java.awt.event.WindowEvent evt)
			{
				exitForm(evt);
			}
		}
		);
		
		_tabbed_pane.setName("DFAgentDescription");
		
		_agent_description_pane.setLayout(new java.awt.GridBagLayout());
  		java.awt.GridBagConstraints gridBagConstraints2;
  		_agent_description_pane.setName("Agent Description");
  		
  		_aid_panel.setLayout(new java.awt.GridBagLayout());
    		java.awt.GridBagConstraints gridBagConstraints3;
    		_aid_panel.setBorder(new javax.swing.border.TitledBorder("Agent Identifier (AID)"));
    		
    		_b_edit_address.setPreferredSize(new java.awt.Dimension(140, 27));
      		_b_edit_address.setMaximumSize(new java.awt.Dimension(140, 27));
      		_b_edit_address.setText("Edit address");
      		_b_edit_address.setMinimumSize(new java.awt.Dimension(140, 27));
      		_b_edit_address.addActionListener(new java.awt.event.ActionListener()
      		{
      			public void actionPerformed(java.awt.event.ActionEvent evt)
      			{
      				editAddress(evt);
      			}
      		}
      		);
      		gridBagConstraints3 = new java.awt.GridBagConstraints();
      		gridBagConstraints3.gridx = 2;
      		gridBagConstraints3.gridy = 3;
      		gridBagConstraints3.insets = new java.awt.Insets(2, 2, 2, 2);
      		_aid_panel.add(_b_edit_address, gridBagConstraints3);
      		
      		
    		_b_remove_address.setPreferredSize(new java.awt.Dimension(140, 27));
      		_b_remove_address.setMaximumSize(new java.awt.Dimension(140, 27));
      		_b_remove_address.setText("Remove address");
      		_b_remove_address.setMinimumSize(new java.awt.Dimension(140, 27));
      		_b_remove_address.addActionListener(new java.awt.event.ActionListener()
      		{
      			public void actionPerformed(java.awt.event.ActionEvent evt)
      			{
      				removeAddress(evt);
      			}
      		}
      		);
      		gridBagConstraints3 = new java.awt.GridBagConstraints();
      		gridBagConstraints3.gridx = 3;
      		gridBagConstraints3.gridy = 3;
      		gridBagConstraints3.insets = new java.awt.Insets(2, 2, 2, 2);
      		gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
      		gridBagConstraints3.weightx = 0.5;
      		_aid_panel.add(_b_remove_address, gridBagConstraints3);
      		
      		
    		_b_add_address.setPreferredSize(new java.awt.Dimension(140, 27));
      		_b_add_address.setMaximumSize(new java.awt.Dimension(140, 27));
      		_b_add_address.setText("Add address");
      		_b_add_address.setMinimumSize(new java.awt.Dimension(140, 27));
      		_b_add_address.addActionListener(new java.awt.event.ActionListener()
      		{
      			public void actionPerformed(java.awt.event.ActionEvent evt)
      			{
      				addAddress(evt);
      			}
      		}
      		);
      		gridBagConstraints3 = new java.awt.GridBagConstraints();
      		gridBagConstraints3.gridx = 0;
      		gridBagConstraints3.gridy = 3;
      		gridBagConstraints3.gridwidth = 2;
      		gridBagConstraints3.insets = new java.awt.Insets(2, 2, 2, 2);
      		gridBagConstraints3.anchor = java.awt.GridBagConstraints.EAST;
      		gridBagConstraints3.weightx = 0.5;
      		_aid_panel.add(_b_add_address, gridBagConstraints3);
      		
      		
    		_tf_aid.setEditable(false);
      		_tf_aid.setColumns(15);
      		gridBagConstraints3 = new java.awt.GridBagConstraints();
      		gridBagConstraints3.gridx = 1;
      		gridBagConstraints3.gridy = 0;
      		gridBagConstraints3.gridwidth = 0;
      		gridBagConstraints3.fill = java.awt.GridBagConstraints.HORIZONTAL;
      		gridBagConstraints3.insets = new java.awt.Insets(2, 2, 2, 2);
      		gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
      		_aid_panel.add(_tf_aid, gridBagConstraints3);
      		
      		
    		_l_aid_addresses.setText("Sequence of addresses:");
      		_l_aid_addresses.setForeground(java.awt.Color.black);
      		gridBagConstraints3 = new java.awt.GridBagConstraints();
      		gridBagConstraints3.gridwidth = 0;
      		gridBagConstraints3.insets = new java.awt.Insets(0, 3, 0, 0);
      		gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
      		_aid_panel.add(_l_aid_addresses, gridBagConstraints3);
      		
      		
    		_l_aid.setText("Agent name:");
      		_l_aid.setForeground(java.awt.Color.black);
      		gridBagConstraints3 = new java.awt.GridBagConstraints();
      		gridBagConstraints3.gridx = 0;
      		gridBagConstraints3.gridy = 0;
      		gridBagConstraints3.insets = new java.awt.Insets(0, 3, 0, 5);
      		gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
      		_aid_panel.add(_l_aid, gridBagConstraints3);
      		
      		
    		
      		_address_list.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        		_aid_scrollpane.setViewportView(_address_list);
        		
        		gridBagConstraints3 = new java.awt.GridBagConstraints();
      		gridBagConstraints3.gridx = 0;
      		gridBagConstraints3.gridy = 2;
      		gridBagConstraints3.gridwidth = 4;
      		gridBagConstraints3.fill = java.awt.GridBagConstraints.BOTH;
      		gridBagConstraints3.insets = new java.awt.Insets(2, 2, 2, 2);
      		gridBagConstraints3.weightx = 1.0;
      		gridBagConstraints3.weighty = 1.0;
      		_aid_panel.add(_aid_scrollpane, gridBagConstraints3);
      		
      		gridBagConstraints2 = new java.awt.GridBagConstraints();
    		gridBagConstraints2.gridwidth = 2;
    		gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
    		gridBagConstraints2.weightx = 1.0;
    		gridBagConstraints2.weighty = 1.0;
    		_agent_description_pane.add(_aid_panel, gridBagConstraints2);
    		
    		
  		_protocol_panel.setLayout(new java.awt.GridBagLayout());
    		java.awt.GridBagConstraints gridBagConstraints4;
    		_protocol_panel.setBorder(new javax.swing.border.TitledBorder("Protocols"));
    		
    		_b_remove_protocol.setPreferredSize(new java.awt.Dimension(140, 27));
      		_b_remove_protocol.setMaximumSize(new java.awt.Dimension(140, 27));
      		_b_remove_protocol.setText("Remove protocol");
      		_b_remove_protocol.setMinimumSize(new java.awt.Dimension(140, 27));
      		gridBagConstraints4 = new java.awt.GridBagConstraints();
      		gridBagConstraints4.gridx = 2;
      		gridBagConstraints4.gridy = 1;
      		gridBagConstraints4.insets = new java.awt.Insets(2, 2, 2, 2);
      		gridBagConstraints4.anchor = java.awt.GridBagConstraints.WEST;
      		gridBagConstraints4.weightx = 0.5;
      		_protocol_panel.add(_b_remove_protocol, gridBagConstraints4);
      		
      		
    		_b_add_protocol.setPreferredSize(new java.awt.Dimension(140, 27));
      		_b_add_protocol.setMaximumSize(new java.awt.Dimension(140, 27));
      		_b_add_protocol.setText("Add protocol");
      		_b_add_protocol.setMinimumSize(new java.awt.Dimension(140, 27));
      		gridBagConstraints4 = new java.awt.GridBagConstraints();
      		gridBagConstraints4.gridx = 0;
      		gridBagConstraints4.gridy = 1;
      		gridBagConstraints4.insets = new java.awt.Insets(2, 2, 2, 2);
      		gridBagConstraints4.anchor = java.awt.GridBagConstraints.EAST;
      		gridBagConstraints4.weightx = 0.5;
      		_protocol_panel.add(_b_add_protocol, gridBagConstraints4);
      		
      		
    		_b_edit_protocol.setPreferredSize(new java.awt.Dimension(140, 27));
      		_b_edit_protocol.setMaximumSize(new java.awt.Dimension(140, 27));
      		_b_edit_protocol.setText("Edit protocols");
      		_b_edit_protocol.setMinimumSize(new java.awt.Dimension(140, 27));
      		gridBagConstraints4 = new java.awt.GridBagConstraints();
      		gridBagConstraints4.gridx = 1;
      		gridBagConstraints4.gridy = 1;
      		gridBagConstraints4.insets = new java.awt.Insets(2, 2, 2, 2);
      		_protocol_panel.add(_b_edit_protocol, gridBagConstraints4);
      		
      		
    		
      		_protocol_scollpane.setViewportView(_protocol_list);
        		
        		gridBagConstraints4 = new java.awt.GridBagConstraints();
      		gridBagConstraints4.gridwidth = 3;
      		gridBagConstraints4.fill = java.awt.GridBagConstraints.BOTH;
      		gridBagConstraints4.insets = new java.awt.Insets(2, 2, 2, 2);
      		gridBagConstraints4.weightx = 1.0;
      		gridBagConstraints4.weighty = 1.0;
      		_protocol_panel.add(_protocol_scollpane, gridBagConstraints4);
      		
      		gridBagConstraints2 = new java.awt.GridBagConstraints();
    		gridBagConstraints2.gridx = 0;
    		gridBagConstraints2.gridy = 1;
    		gridBagConstraints2.gridwidth = 2;
    		gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
    		gridBagConstraints2.weightx = 1.0;
    		gridBagConstraints2.weighty = 1.0;
    		_agent_description_pane.add(_protocol_panel, gridBagConstraints2);
    		
    		
  		_ontology_panel.setLayout(new java.awt.GridBagLayout());
    		java.awt.GridBagConstraints gridBagConstraints5;
    		_ontology_panel.setBorder(new javax.swing.border.TitledBorder("Ontologies"));
    		
    		_b_remove_ontology.setPreferredSize(new java.awt.Dimension(140, 27));
      		_b_remove_ontology.setMaximumSize(new java.awt.Dimension(140, 27));
      		_b_remove_ontology.setText("Remove ontology");
      		_b_remove_ontology.setMinimumSize(new java.awt.Dimension(140, 27));
      		gridBagConstraints5 = new java.awt.GridBagConstraints();
      		gridBagConstraints5.gridx = 2;
      		gridBagConstraints5.gridy = 1;
      		gridBagConstraints5.insets = new java.awt.Insets(2, 2, 2, 2);
      		gridBagConstraints5.anchor = java.awt.GridBagConstraints.WEST;
      		gridBagConstraints5.weightx = 0.5;
      		_ontology_panel.add(_b_remove_ontology, gridBagConstraints5);
      		
      		
    		_b_edit_ontology.setPreferredSize(new java.awt.Dimension(140, 27));
      		_b_edit_ontology.setMaximumSize(new java.awt.Dimension(140, 27));
      		_b_edit_ontology.setText("Edit ontology");
      		_b_edit_ontology.setMinimumSize(new java.awt.Dimension(140, 27));
      		gridBagConstraints5 = new java.awt.GridBagConstraints();
      		gridBagConstraints5.gridx = 1;
      		gridBagConstraints5.gridy = 1;
      		gridBagConstraints5.insets = new java.awt.Insets(2, 2, 2, 2);
      		_ontology_panel.add(_b_edit_ontology, gridBagConstraints5);
      		
      		
    		_b_add_ontology.setPreferredSize(new java.awt.Dimension(140, 27));
      		_b_add_ontology.setMaximumSize(new java.awt.Dimension(140, 27));
      		_b_add_ontology.setText("Add ontology");
      		_b_add_ontology.setMinimumSize(new java.awt.Dimension(140, 27));
      		gridBagConstraints5 = new java.awt.GridBagConstraints();
      		gridBagConstraints5.gridx = 0;
      		gridBagConstraints5.gridy = 1;
      		gridBagConstraints5.insets = new java.awt.Insets(2, 2, 2, 2);
      		gridBagConstraints5.anchor = java.awt.GridBagConstraints.EAST;
      		gridBagConstraints5.weightx = 0.5;
      		_ontology_panel.add(_b_add_ontology, gridBagConstraints5);
      		
      		
    		
      		_ontology_list.setToolTipText("Ontologies");
        		_ontology_list.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        		_ontology_scrollpane.setViewportView(_ontology_list);
        		
        		gridBagConstraints5 = new java.awt.GridBagConstraints();
      		gridBagConstraints5.gridx = 0;
      		gridBagConstraints5.gridy = 0;
      		gridBagConstraints5.gridwidth = 3;
      		gridBagConstraints5.fill = java.awt.GridBagConstraints.BOTH;
      		gridBagConstraints5.insets = new java.awt.Insets(2, 2, 2, 2);
      		gridBagConstraints5.weightx = 1.0;
      		gridBagConstraints5.weighty = 1.0;
      		_ontology_panel.add(_ontology_scrollpane, gridBagConstraints5);
      		
      		gridBagConstraints2 = new java.awt.GridBagConstraints();
    		gridBagConstraints2.gridx = 0;
    		gridBagConstraints2.gridy = 2;
    		gridBagConstraints2.gridwidth = 2;
    		gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
    		gridBagConstraints2.weightx = 1.0;
    		gridBagConstraints2.weighty = 1.0;
    		_agent_description_pane.add(_ontology_panel, gridBagConstraints2);
    		
    		
  		_language_panel.setLayout(new java.awt.GridBagLayout());
    		java.awt.GridBagConstraints gridBagConstraints6;
    		_language_panel.setBorder(new javax.swing.border.TitledBorder("Languages"));
    		
    		_b_add_language.setPreferredSize(new java.awt.Dimension(140, 27));
      		_b_add_language.setMaximumSize(new java.awt.Dimension(140, 27));
      		_b_add_language.setText("Add language");
      		_b_add_language.setMinimumSize(new java.awt.Dimension(140, 27));
      		gridBagConstraints6 = new java.awt.GridBagConstraints();
      		gridBagConstraints6.gridx = 0;
      		gridBagConstraints6.gridy = 1;
      		gridBagConstraints6.insets = new java.awt.Insets(2, 2, 2, 2);
      		gridBagConstraints6.anchor = java.awt.GridBagConstraints.EAST;
      		gridBagConstraints6.weightx = 0.5;
      		_language_panel.add(_b_add_language, gridBagConstraints6);
      		
      		
    		_b_remove_language.setPreferredSize(new java.awt.Dimension(140, 27));
      		_b_remove_language.setMaximumSize(new java.awt.Dimension(140, 27));
      		_b_remove_language.setText("Remove language");
      		_b_remove_language.setMinimumSize(new java.awt.Dimension(140, 27));
      		gridBagConstraints6 = new java.awt.GridBagConstraints();
      		gridBagConstraints6.gridx = 2;
      		gridBagConstraints6.gridy = 1;
      		gridBagConstraints6.insets = new java.awt.Insets(2, 2, 2, 2);
      		gridBagConstraints6.anchor = java.awt.GridBagConstraints.WEST;
      		gridBagConstraints6.weightx = 0.5;
      		_language_panel.add(_b_remove_language, gridBagConstraints6);
      		
      		
    		_b_edit_language.setPreferredSize(new java.awt.Dimension(140, 27));
      		_b_edit_language.setMaximumSize(new java.awt.Dimension(140, 27));
      		_b_edit_language.setText("Edit language");
      		_b_edit_language.setMinimumSize(new java.awt.Dimension(140, 27));
      		gridBagConstraints6 = new java.awt.GridBagConstraints();
      		gridBagConstraints6.gridx = 1;
      		gridBagConstraints6.gridy = 1;
      		gridBagConstraints6.insets = new java.awt.Insets(2, 2, 2, 2);
      		_language_panel.add(_b_edit_language, gridBagConstraints6);
      		
      		
    		
      		_language_list.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        		_language_scrollpane.setViewportView(_language_list);
        		
        		gridBagConstraints6 = new java.awt.GridBagConstraints();
      		gridBagConstraints6.gridwidth = 3;
      		gridBagConstraints6.fill = java.awt.GridBagConstraints.BOTH;
      		gridBagConstraints6.insets = new java.awt.Insets(2, 2, 2, 2);
      		gridBagConstraints6.weightx = 1.0;
      		gridBagConstraints6.weighty = 1.0;
      		_language_panel.add(_language_scrollpane, gridBagConstraints6);
      		
      		gridBagConstraints2 = new java.awt.GridBagConstraints();
    		gridBagConstraints2.gridx = 0;
    		gridBagConstraints2.gridy = 3;
    		gridBagConstraints2.gridwidth = 2;
    		gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
    		gridBagConstraints2.weightx = 1.0;
    		gridBagConstraints2.weighty = 1.0;
    		_agent_description_pane.add(_language_panel, gridBagConstraints2);
    		
    		
  		_b_OK_agent_description.setPreferredSize(new java.awt.Dimension(67, 27));
    		_b_OK_agent_description.setText("OK");
    		_b_OK_agent_description.addActionListener(new java.awt.event.ActionListener()
    		{
    			public void actionPerformed(java.awt.event.ActionEvent evt)
    			{
    				pressedOK(evt);
    			}
    		}
    		);
    		gridBagConstraints2 = new java.awt.GridBagConstraints();
    		gridBagConstraints2.gridx = 0;
    		gridBagConstraints2.gridy = 4;
    		gridBagConstraints2.insets = new java.awt.Insets(5, 3, 5, 3);
    		gridBagConstraints2.anchor = java.awt.GridBagConstraints.EAST;
    		gridBagConstraints2.weightx = 0.5;
    		_agent_description_pane.add(_b_OK_agent_description, gridBagConstraints2);
    		
    		
  		_b_cancel_agent_description.setText("Cancel");
    		_b_cancel_agent_description.addActionListener(new java.awt.event.ActionListener()
    		{
    			public void actionPerformed(java.awt.event.ActionEvent evt)
    			{
    				pressedCancel(evt);
    			}
    		}
    		);
    		gridBagConstraints2 = new java.awt.GridBagConstraints();
    		gridBagConstraints2.gridx = 1;
    		gridBagConstraints2.gridy = 4;
    		gridBagConstraints2.insets = new java.awt.Insets(5, 3, 5, 3);
    		gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
    		gridBagConstraints2.weightx = 0.5;
    		_agent_description_pane.add(_b_cancel_agent_description, gridBagConstraints2);
    		
    		_tabbed_pane.addTab("Agent Description", _agent_description_pane);
  		
  		
		_service_pane.setLayout(new java.awt.GridBagLayout());
  		java.awt.GridBagConstraints gridBagConstraints7;
  		
  		
    		_service_scrollpane.setViewportView(_service_list);
      		
      		gridBagConstraints7 = new java.awt.GridBagConstraints();
    		gridBagConstraints7.gridwidth = 0;
    		gridBagConstraints7.fill = java.awt.GridBagConstraints.BOTH;
    		gridBagConstraints7.insets = new java.awt.Insets(2, 2, 2, 2);
    		gridBagConstraints7.weightx = 1.0;
    		gridBagConstraints7.weighty = 1.0;
    		_service_pane.add(_service_scrollpane, gridBagConstraints7);
    		
    		
  		_b_add_service.setPreferredSize(new java.awt.Dimension(113, 27));
    		_b_add_service.setText("Add service");
    		gridBagConstraints7 = new java.awt.GridBagConstraints();
    		gridBagConstraints7.insets = new java.awt.Insets(2, 2, 2, 2);
    		gridBagConstraints7.anchor = java.awt.GridBagConstraints.EAST;
    		gridBagConstraints7.weightx = 0.5;
    		_service_pane.add(_b_add_service, gridBagConstraints7);
    		
    		
  		_b_edit_service.setPreferredSize(new java.awt.Dimension(113, 27));
    		_b_edit_service.setText("Edit service");
    		gridBagConstraints7 = new java.awt.GridBagConstraints();
    		gridBagConstraints7.gridwidth = 2;
    		_service_pane.add(_b_edit_service, gridBagConstraints7);
    		
    		
  		_b_remove_service.setText("Remove service");
    		gridBagConstraints7 = new java.awt.GridBagConstraints();
    		gridBagConstraints7.gridx = 3;
    		gridBagConstraints7.gridy = 1;
    		gridBagConstraints7.insets = new java.awt.Insets(2, 2, 2, 2);
    		gridBagConstraints7.anchor = java.awt.GridBagConstraints.WEST;
    		gridBagConstraints7.weightx = 0.5;
    		_service_pane.add(_b_remove_service, gridBagConstraints7);
    		
    		
  		_b_OK_service.setPreferredSize(new java.awt.Dimension(67, 27));
    		_b_OK_service.setMaximumSize(new java.awt.Dimension(67, 27));
    		_b_OK_service.setText("OK");
    		_b_OK_service.setMinimumSize(new java.awt.Dimension(67, 27));
    		_b_OK_service.addActionListener(new java.awt.event.ActionListener()
    		{
    			public void actionPerformed(java.awt.event.ActionEvent evt)
    			{
    				pressedOK(evt);
    			}
    		}
    		);
    		gridBagConstraints7 = new java.awt.GridBagConstraints();
    		gridBagConstraints7.gridx = 1;
    		gridBagConstraints7.gridy = 2;
    		gridBagConstraints7.insets = new java.awt.Insets(5, 3, 5, 3);
    		_service_pane.add(_b_OK_service, gridBagConstraints7);
    		
    		
  		_b_cancel_service.setText("Cancel");
    		_b_cancel_service.addActionListener(new java.awt.event.ActionListener()
    		{
    			public void actionPerformed(java.awt.event.ActionEvent evt)
    			{
    				pressedCancel(evt);
    			}
    		}
    		);
    		gridBagConstraints7 = new java.awt.GridBagConstraints();
    		gridBagConstraints7.gridx = 2;
    		gridBagConstraints7.gridy = 2;
    		gridBagConstraints7.insets = new java.awt.Insets(5, 3, 5, 3);
    		_service_pane.add(_b_cancel_service, gridBagConstraints7);
    		
    		_tabbed_pane.addTab("Services", _service_pane);
  		
  		
		_full_description.setLayout(new java.awt.GridBagLayout());
  		java.awt.GridBagConstraints gridBagConstraints8;
  		
  		_b_OK_string.setPreferredSize(new java.awt.Dimension(67, 27));
    		_b_OK_string.setMaximumSize(new java.awt.Dimension(67, 27));
    		_b_OK_string.setText("OK");
    		_b_OK_string.setMinimumSize(new java.awt.Dimension(67, 27));
    		_b_OK_string.addActionListener(new java.awt.event.ActionListener()
    		{
    			public void actionPerformed(java.awt.event.ActionEvent evt)
    			{
    				pressedOK(evt);
    			}
    		}
    		);
    		gridBagConstraints8 = new java.awt.GridBagConstraints();
    		gridBagConstraints8.gridx = 0;
    		gridBagConstraints8.gridy = 2;
    		gridBagConstraints8.insets = new java.awt.Insets(5, 3, 5, 3);
    		gridBagConstraints8.anchor = java.awt.GridBagConstraints.EAST;
    		gridBagConstraints8.weightx = 0.5;
    		_full_description.add(_b_OK_string, gridBagConstraints8);
    		
    		
  		_b_cancel_string.setPreferredSize(new java.awt.Dimension(67, 27));
    		_b_cancel_string.setText("Cancel");
    		_b_cancel_string.addActionListener(new java.awt.event.ActionListener()
    		{
    			public void actionPerformed(java.awt.event.ActionEvent evt)
    			{
    				pressedCancel(evt);
    			}
    		}
    		);
    		gridBagConstraints8 = new java.awt.GridBagConstraints();
    		gridBagConstraints8.gridx = 1;
    		gridBagConstraints8.gridy = 2;
    		gridBagConstraints8.insets = new java.awt.Insets(5, 3, 5, 3);
    		gridBagConstraints8.anchor = java.awt.GridBagConstraints.WEST;
    		gridBagConstraints8.weightx = 0.5;
    		_full_description.add(_b_cancel_string, gridBagConstraints8);
    		
    		
  		
    		_string_text.setEditable(false);
      		_string_scollpane.setViewportView(_string_text);
      		
      		gridBagConstraints8 = new java.awt.GridBagConstraints();
    		gridBagConstraints8.gridx = 0;
    		gridBagConstraints8.gridy = 1;
    		gridBagConstraints8.gridwidth = 2;
    		gridBagConstraints8.fill = java.awt.GridBagConstraints.BOTH;
    		gridBagConstraints8.insets = new java.awt.Insets(0, 3, 0, 3);
    		gridBagConstraints8.weightx = 1.0;
    		gridBagConstraints8.weighty = 1.0;
    		_full_description.add(_string_scollpane, gridBagConstraints8);
    		
    		
  		_l_string.setText("Stringified DFAgentDescription:");
    		_l_string.setForeground(java.awt.Color.black);
    		gridBagConstraints8 = new java.awt.GridBagConstraints();
    		gridBagConstraints8.gridx = 0;
    		gridBagConstraints8.gridy = 0;
    		gridBagConstraints8.gridwidth = 0;
    		gridBagConstraints8.insets = new java.awt.Insets(3, 3, 3, 3);
    		gridBagConstraints8.anchor = java.awt.GridBagConstraints.WEST;
    		_full_description.add(_l_string, gridBagConstraints8);
    		
    		_tabbed_pane.addTab("Stringified Description", _full_description);
  		
  		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
		gridBagConstraints1.insets = new java.awt.Insets(2, 2, 2, 2);
		gridBagConstraints1.weightx = 1.0;
		gridBagConstraints1.weighty = 1.0;
		getContentPane().add(_tabbed_pane, gridBagConstraints1);
		
	}//GEN-END:initComponents
	
/**
 * Action event listener envoked when address will be removed from the list.
 * @param evt action event envoked
 */
  private void removeAddress(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_removeAddress
		int remove = JOptionPane.showConfirmDialog(this,
		"Remove value '" + _address_list.getSelectedValue() + "' from the address list?",
		"Remove",
		JOptionPane.YES_NO_OPTION,
		JOptionPane.QUESTION_MESSAGE);
		if (remove == JOptionPane.YES_OPTION)
		{
			_addresses.removeElement(_address_list.getSelectedValue());
		}
  }//GEN-LAST:event_removeAddress
	
/**
 * Action event listener envoked when address from the list will be edited.
 * @param evt action event envoked
 */
  private void editAddress(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editAddress
		String edit_value = (String) JOptionPane.showInputDialog(this,
		"Edit value:",
		"Edit",
		JOptionPane.QUESTION_MESSAGE,
		null,
		null,
		_address_list.getSelectedValue().toString());
		_addresses.setElementAt(edit_value, _address_list.getSelectedIndex());
  }//GEN-LAST:event_editAddress
	
/**
 * Action event listener envoked when address will be added to the list.
 * @param evt action event envoked
 */
  private void addAddress(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addAddress
		String new_value = (String) JOptionPane.showInputDialog(this,
		"Add a new value:",
		"Add",
		JOptionPane.QUESTION_MESSAGE,
		null,
		null,
		null);
		_addresses.addElement(new_value);
		_address_list.setListData(_addresses.toArray());
  }//GEN-LAST:event_addAddress
	
/**
 * Action event listener envoked when cancel button is pressed. Disposes the
 * window.
 * @param evt action event envoked
 */
  private void pressedCancel(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_pressedCancel
		dispose();
  }//GEN-LAST:event_pressedCancel
	
/**
 * Action event listener envoked when OK button is pressed. Checks whether the
 * GUI has been used for viewing or modifying the contents.
 * @param evt action event envoked
 */
  private void pressedOK(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_pressedOK
		if (_read_only)
		{
			dispose();
		}
		else
		{
			
		}
  }//GEN-LAST:event_pressedOK
	
		/**
		 * Exit the Application
		 * @param evt action event envoked
		 */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
			dispose();
			//System.exit (0);
    }//GEN-LAST:event_exitForm
		
		// Variables declaration - do not modify//GEN-BEGIN:variables
		private javax.swing.JTabbedPane _tabbed_pane;
		private javax.swing.JPanel _agent_description_pane;
		private javax.swing.JPanel _aid_panel;
		private javax.swing.JButton _b_edit_address;
		private javax.swing.JButton _b_remove_address;
		private javax.swing.JButton _b_add_address;
		private javax.swing.JTextField _tf_aid;
		private javax.swing.JLabel _l_aid_addresses;
		private javax.swing.JLabel _l_aid;
		private javax.swing.JScrollPane _aid_scrollpane;
		private javax.swing.JList _address_list;
		private javax.swing.JPanel _protocol_panel;
		private javax.swing.JButton _b_remove_protocol;
		private javax.swing.JButton _b_add_protocol;
		private javax.swing.JButton _b_edit_protocol;
		private javax.swing.JScrollPane _protocol_scollpane;
		private javax.swing.JList _protocol_list;
		private javax.swing.JPanel _ontology_panel;
		private javax.swing.JButton _b_remove_ontology;
		private javax.swing.JButton _b_edit_ontology;
		private javax.swing.JButton _b_add_ontology;
		private javax.swing.JScrollPane _ontology_scrollpane;
		private javax.swing.JList _ontology_list;
		private javax.swing.JPanel _language_panel;
		private javax.swing.JButton _b_add_language;
		private javax.swing.JButton _b_remove_language;
		private javax.swing.JButton _b_edit_language;
		private javax.swing.JScrollPane _language_scrollpane;
		private javax.swing.JList _language_list;
		private javax.swing.JButton _b_OK_agent_description;
		private javax.swing.JButton _b_cancel_agent_description;
		private javax.swing.JPanel _service_pane;
		private javax.swing.JScrollPane _service_scrollpane;
		private javax.swing.JList _service_list;
		private javax.swing.JButton _b_add_service;
		private javax.swing.JButton _b_edit_service;
		private javax.swing.JButton _b_remove_service;
		private javax.swing.JButton _b_OK_service;
		private javax.swing.JButton _b_cancel_service;
		private javax.swing.JPanel _full_description;
		private javax.swing.JButton _b_OK_string;
		private javax.swing.JButton _b_cancel_string;
		private javax.swing.JScrollPane _string_scollpane;
		private javax.swing.JTextArea _string_text;
		private javax.swing.JLabel _l_string;
		// End of variables declaration//GEN-END:variables
		
		private Vector _addresses;
		private Object[] _protocols;
		private Object[] _languages;
		private Object[] _services;
		private Object[] _ontologies;
		private static final String IMAGE_DIRECTORY = "/images/";
		private static final String FIPA_GIF = "logo.gif";
		
		private boolean _read_only = true;
		
/**
 * Used to get the minimum size for the frame.
 * @return minumum size for the frame as a Dimension
 */
		public Dimension getMinimumSize()
		{
			return new Dimension(450,425);
		}
		
/**
 * Removes line breaks from the agent-service item.
 * @param original_string the original string
 * @return string with no line breaks
 */
		private String properServicesLayout(String original_string)
		{
			return original_string.replace('\n', ' ');
		}
		
/**
 * This method should make a proper layout of an SL string. This needs some
 * more thought.
 * @param original_string original string with random white spaces
 * @return newly laid out version
 */
		private String properStringifiedLayout(String original_string)
		{
			original_string = properServicesLayout(original_string);
			final StringBuffer original = new StringBuffer(original_string);
			StringBuffer modified = new StringBuffer();
			
			modified.append(original.charAt(0));
			for (int i=1; i<original.length(); i++)
			{
				if (original.charAt(i) != '(')
				{
					modified.append(original.charAt(i));
				}
				else
				{
					modified.append('\n');
					modified.append(original.charAt(i));
				}
			}
			
			return modified.toString();
		}
		
}