//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.platform.dfgui.swing;

import fipaos.util.*;
import fipaos.ont.fipa.fipaman.*;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.Color;
import java.awt.Dimension;

import java.util.*;

/**
 * This is a DF GUI, implemented with Java's Swing classes. It gives the user
 * a view of what agents are registered with the DF, and what kind of services
 * they offer - in the future releases there is also functionality to modify
 * this code.
 */
public class DFGUIFrame extends JFrame implements KeyListener
{
	
/**
 * The constructor for the DFGUI frame. Starts up the GUI and
 * loads the agent details for the default DF.
 * @param parent the SwingDFGUIAgent that created the GUI
 * @param df the default DF
 */
	public DFGUIFrame(SwingDFGUIAgent parent, AgentID df)
	{
		_parent = parent;
		
		String df_name = df.getName();
		
		try
		{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}
		catch(Throwable e)
		{
			DIAGNOSTICS.println("Exception", e, this, DIAGNOSTICS.LEVEL_MIN );
		}
		
		initComponents();
		super.pack();
		
		// Resize and reposition window so it appears in the center
		// of the screen, and is not too large
		java.awt.Toolkit tk = java.awt.Toolkit.getDefaultToolkit();
		int width = getSize().width;
		int height = getSize().height;
		
		if( width < 320 )
		{
			width = 320;
		}
		
		if( height < 240 )
		{
			height = 240;
		}
		
		setSize( width, height );
		width = tk.getScreenSize().width - width;
		height = tk.getScreenSize().height - height;
		setLocation( width/2, height/2 );
		
		super.setVisible(true);
		
		addAgentToTree(df_name, df_name);
		listAgents(df);
	}
	
		/**
		 * This method is called from within the constructor to
		 * initialize the form.
		 * WARNING: Do NOT modify this code. The content of this method is
		 * always regenerated by the FormEditor.
		 */
	private void initComponents()//GEN-BEGIN:initComponents
	{
		_menu_bar = new javax.swing.JMenuBar();
		_menu_file = new javax.swing.JMenu();
		_m_add_df = new javax.swing.JMenuItem();
		_m_remove_df = new javax.swing.JMenuItem();
		_m_separator1 = new javax.swing.JSeparator();
		_m_refresh = new javax.swing.JMenuItem();
		_m_separator3 = new javax.swing.JSeparator();
		_m_exit = new javax.swing.JMenuItem();
		_menu_df_functionality = new javax.swing.JMenu();
		_m_deregister = new javax.swing.JMenuItem();
		_m_modify = new javax.swing.JMenuItem();
		_m_separator2 = new javax.swing.JSeparator();
		_m_register_df = new javax.swing.JMenuItem();
		_menu_view = new javax.swing.JMenu();
		_m_view = new javax.swing.JMenuItem();
		_agent_popup_menu = new javax.swing.JPopupMenu();
		_pm_deregister = new javax.swing.JMenuItem();
		_pm_modify = new javax.swing.JMenuItem();
		_pm_separator1 = new javax.swing.JSeparator();
		_pm_view = new javax.swing.JMenuItem();
		_pm_separator2 = new javax.swing.JSeparator();
		_pm_remove_df = new javax.swing.JMenuItem();
		_pm_register_with_df = new javax.swing.JMenuItem();
		_pm_federated_search = new javax.swing.JMenuItem();
		_FIPAOS_popup_menu = new javax.swing.JPopupMenu();
		_pm_add_df = new javax.swing.JMenuItem();
		_toolbar = new javax.swing.JToolBar();
		_b_deregister = new javax.swing.JButton();
		_b_modify = new javax.swing.JButton();
		_b_separator = new javax.swing.JSeparator();
		_b_add_df = new javax.swing.JButton();
		_b_remove_df = new javax.swing.JButton();
		_agent_scrollpane = new javax.swing.JScrollPane();
		_popup_listener = new DFGUIFrame.DFGUIPopupListener();
		_tree_model = new DFGUIFrame.DFGUIModel();
		_agent_tree = new JTree(_tree_model);
		_agent_tree.putClientProperty("JTree.lineStyle", "Angled");
		_agent_tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		_agent_tree.addMouseListener(_popup_listener);
		_agent_tree.addTreeSelectionListener(new DFGUIFrame.DFGUITreeSelectionListener());
		
		_status_bar = new javax.swing.JLabel();
		_menu_bar.setName("");
		
		_menu_file.setText("File");
  		_menu_file.addFocusListener(new java.awt.event.FocusAdapter()
  		{
  			public void focusGained(java.awt.event.FocusEvent evt)
  			{
  				fileMenuOpened(evt);
  			}
  		}
  		);
  		
  		_m_add_df.setText("Add new DF");
    		_m_add_df.addActionListener(new java.awt.event.ActionListener()
    		{
    			public void actionPerformed(java.awt.event.ActionEvent evt)
    			{
    				addDF(evt);
    			}
    		}
    		);
    		_menu_file.add(_m_add_df);
    		
  		_m_remove_df.setText("Remove DF");
    		_m_remove_df.addActionListener(new java.awt.event.ActionListener()
    		{
    			public void actionPerformed(java.awt.event.ActionEvent evt)
    			{
    				removeDF(evt);
    			}
    		}
    		);
    		_menu_file.add(_m_remove_df);
    		
  		_menu_file.add(_m_separator1);
    		
  		_m_refresh.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F5, 0));
    		_m_refresh.setText("Refresh");
    		_m_refresh.addActionListener(new java.awt.event.ActionListener()
    		{
    			public void actionPerformed(java.awt.event.ActionEvent evt)
    			{
    				refresh(evt);
    			}
    		}
    		);
    		_menu_file.add(_m_refresh);
    		
  		_menu_file.add(_m_separator3);
    		
  		_m_exit.setText("Exit");
    		_m_exit.addActionListener(new java.awt.event.ActionListener()
    		{
    			public void actionPerformed(java.awt.event.ActionEvent evt)
    			{
    				exit(evt);
    			}
    		}
    		);
    		_menu_file.add(_m_exit);
    		_menu_bar.add(_menu_file);
  		
		_menu_df_functionality.setText("DF Functionality");
  		_menu_df_functionality.addFocusListener(new java.awt.event.FocusAdapter()
  		{
  			public void focusGained(java.awt.event.FocusEvent evt)
  			{
  				dfMenuOpened(evt);
  			}
  		}
  		);
  		
  		_m_deregister.setText("Deregister");
    		_m_deregister.addActionListener(new java.awt.event.ActionListener()
    		{
    			public void actionPerformed(java.awt.event.ActionEvent evt)
    			{
    				deregister(evt);
    			}
    		}
    		);
    		_menu_df_functionality.add(_m_deregister);
    		
  		_m_modify.setText("Modify");
    		_m_modify.addActionListener(new java.awt.event.ActionListener()
    		{
    			public void actionPerformed(java.awt.event.ActionEvent evt)
    			{
    				modify(evt);
    			}
    		}
    		);
    		_menu_df_functionality.add(_m_modify);
    		
  		_menu_df_functionality.add(_m_separator2);
    		
  		_m_register_df.setText("Register DF with another DF");
    		_m_register_df.addActionListener(new java.awt.event.ActionListener()
    		{
    			public void actionPerformed(java.awt.event.ActionEvent evt)
    			{
    				registerDFWithDF(evt);
    			}
    		}
    		);
    		_menu_df_functionality.add(_m_register_df);
    		_menu_bar.add(_menu_df_functionality);
  		
		_menu_view.setText("View");
  		_menu_view.addFocusListener(new java.awt.event.FocusAdapter()
  		{
  			public void focusGained(java.awt.event.FocusEvent evt)
  			{
  				viewMenuOpened(evt);
  			}
  		}
  		);
  		
  		_m_view.setText("View agent");
    		_m_view.addActionListener(new java.awt.event.ActionListener()
    		{
    			public void actionPerformed(java.awt.event.ActionEvent evt)
    			{
    				view(evt);
    			}
    		}
    		);
    		_menu_view.add(_m_view);
    		_menu_bar.add(_menu_view);
  		
		_pm_deregister.setText("Deregister");
  		_pm_deregister.addActionListener(new java.awt.event.ActionListener()
  		{
  			public void actionPerformed(java.awt.event.ActionEvent evt)
  			{
  				deregister(evt);
  			}
  		}
  		);
  		_agent_popup_menu.add(_pm_deregister);
  		
		_pm_modify.setText("Modify");
  		_pm_modify.addActionListener(new java.awt.event.ActionListener()
  		{
  			public void actionPerformed(java.awt.event.ActionEvent evt)
  			{
  				modify(evt);
  			}
  		}
  		);
  		_agent_popup_menu.add(_pm_modify);
  		
		_agent_popup_menu.add(_pm_separator1);
  		
		_pm_view.setText("View");
  		_pm_view.addActionListener(new java.awt.event.ActionListener()
  		{
  			public void actionPerformed(java.awt.event.ActionEvent evt)
  			{
  				view(evt);
  			}
  		}
  		);
  		_agent_popup_menu.add(_pm_view);
  		
		_agent_popup_menu.add(_pm_separator2);
  		
		_pm_remove_df.setText("Remove DF");
  		_pm_remove_df.addActionListener(new java.awt.event.ActionListener()
  		{
  			public void actionPerformed(java.awt.event.ActionEvent evt)
  			{
  				removeDF(evt);
  			}
  		}
  		);
  		_agent_popup_menu.add(_pm_remove_df);
  		
		_pm_register_with_df.setText("Register with a DF");
  		_pm_register_with_df.addActionListener(new java.awt.event.ActionListener()
  		{
  			public void actionPerformed(java.awt.event.ActionEvent evt)
  			{
  				registerDFWithDF(evt);
  			}
  		}
  		);
  		_agent_popup_menu.add(_pm_register_with_df);
  		
		_pm_federated_search.setText("Perform Federated Search");
  		_pm_federated_search.addActionListener(new java.awt.event.ActionListener()
  		{
  			public void actionPerformed(java.awt.event.ActionEvent evt)
  			{
  				jMenuItem1ActionPerformed(evt);
  			}
  		}
  		);
  		_agent_popup_menu.add(_pm_federated_search);
  		
		_pm_add_df.setText("Add new DF");
  		_pm_add_df.addActionListener(new java.awt.event.ActionListener()
  		{
  			public void actionPerformed(java.awt.event.ActionEvent evt)
  			{
  				addDF(evt);
  			}
  		}
  		);
  		_FIPAOS_popup_menu.add(_pm_add_df);
  		getContentPane().setLayout(new java.awt.GridBagLayout());
		java.awt.GridBagConstraints gridBagConstraints1;
		setIconImage(new ImageIcon( ClassLoader.getSystemResource( "images/logo.gif" ) ).getImage());
		setTitle("DF Management GUI");
		setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
		addWindowListener(new java.awt.event.WindowAdapter()
		{
			public void windowClosing(java.awt.event.WindowEvent evt)
			{
				exitForm(evt);
			}
		}
		);
		
		
		_b_deregister.setIcon(new ImageIcon( ClassLoader.getSystemResource( "images/dfgui/deregister.gif" ) ));
  		_b_deregister.setToolTipText("Deregister");
  		_b_deregister.setBorder(new javax.swing.border.BevelBorder(0));
  		_b_deregister.setMargin(new java.awt.Insets(0, 0, 0, 0));
  		_b_deregister.addActionListener(new java.awt.event.ActionListener()
  		{
  			public void actionPerformed(java.awt.event.ActionEvent evt)
  			{
  				deregister(evt);
  			}
  		}
  		);
  		_toolbar.add(_b_deregister);
  		
  		
		_b_modify.setIcon(new ImageIcon( ClassLoader.getSystemResource( "images/dfgui/modify.gif" ) ));
  		_b_modify.setToolTipText("Modify");
  		_b_modify.setBorder(new javax.swing.border.BevelBorder(0));
  		_b_modify.setMargin(new java.awt.Insets(0, 0, 0, 0));
  		_b_modify.addActionListener(new java.awt.event.ActionListener()
  		{
  			public void actionPerformed(java.awt.event.ActionEvent evt)
  			{
  				modify(evt);
  			}
  		}
  		);
  		_toolbar.add(_b_modify);
  		
  		
		_b_separator.setPreferredSize(new java.awt.Dimension(10, 0));
  		_b_separator.setOrientation(javax.swing.SwingConstants.VERTICAL);
  		_toolbar.add(_b_separator);
  		
  		
		_b_add_df.setIcon(new ImageIcon( ClassLoader.getSystemResource( "images/dfgui/df.gif" ) ));
  		_b_add_df.setToolTipText("New DF");
  		_b_add_df.setBorder(new javax.swing.border.BevelBorder(0));
  		_b_add_df.setMargin(new java.awt.Insets(0, 0, 0, 0));
  		_b_add_df.addActionListener(new java.awt.event.ActionListener()
  		{
  			public void actionPerformed(java.awt.event.ActionEvent evt)
  			{
  				addDF(evt);
  			}
  		}
  		);
  		_toolbar.add(_b_add_df);
  		
  		
		_b_remove_df.setIcon(new ImageIcon( ClassLoader.getSystemResource( "images/dfgui/remove_df.jpg" ) ));
  		_b_remove_df.setToolTipText("Remove DF");
  		_b_remove_df.setBorder(new javax.swing.border.BevelBorder(0));
  		_b_remove_df.setMargin(new java.awt.Insets(0, 0, 0, 0));
  		_b_remove_df.addActionListener(new java.awt.event.ActionListener()
  		{
  			public void actionPerformed(java.awt.event.ActionEvent evt)
  			{
  				removeDF(evt);
  			}
  		}
  		);
  		_toolbar.add(_b_remove_df);
  		
  		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.insets = new java.awt.Insets(1, 3, 0, 3);
		gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
		gridBagConstraints1.weightx = 1.0;
		getContentPane().add(_toolbar, gridBagConstraints1);
		
		
		_agent_scrollpane.setBorder(new javax.swing.border.BevelBorder(1));
		
		_agent_tree.setPreferredSize(new java.awt.Dimension(400, 100));
  		_agent_tree.setBorder(new javax.swing.border.BevelBorder(1));
  		_agent_tree.setRowHeight(25);
  		_agent_tree.setMaximumSize(new java.awt.Dimension(500, 100));
  		_agent_tree.setVisibleRowCount(15);
  		_agent_tree.setCellRenderer(new DFGUIFrame.DFGUIRenderer());
  		_agent_tree.setMinimumSize(new java.awt.Dimension(400, 100));
  		_agent_scrollpane.setViewportView(_agent_tree);
  		
  		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 0;
		gridBagConstraints1.gridy = 1;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
		gridBagConstraints1.insets = new java.awt.Insets(3, 3, 3, 3);
		gridBagConstraints1.weightx = 1.0;
		gridBagConstraints1.weighty = 1.0;
		getContentPane().add(_agent_scrollpane, gridBagConstraints1);
		
		
		_status_bar.setBorder(new javax.swing.border.BevelBorder(1));
		_status_bar.setText(" ");
		_status_bar.setToolTipText("Status of the latest succesful command");
		_status_bar.setForeground(java.awt.Color.black);
		_status_bar.setFont(new java.awt.Font ("Dialog", 0, 10));
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 0;
		gridBagConstraints1.gridy = 2;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
		gridBagConstraints1.insets = new java.awt.Insets(1, 1, 1, 1);
		getContentPane().add(_status_bar, gridBagConstraints1);
		
		setJMenuBar(_menu_bar);
		
	}//GEN-END:initComponents

  private void jMenuItem1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem1ActionPerformed

		final JFrame dfgui = this;
		
		// Popup window to query search depth
		Thread t = new Thread( new Runnable()
		{
			public void run()
			{
				String result = JOptionPane.showInputDialog( "Depth for federated search" );
				
				if( result != null )
				{
					try
					{
						int depth = Integer.parseInt( result );
						
						String selected_agent = _agent_tree.getSelectionPath().getLastPathComponent().toString();
						AgentID df_aid = new AgentID();
						df_aid.setName(selected_agent);
						
						DIAGNOSTICS.println( "About to search " + df_aid.getName() + "....", this, DIAGNOSTICS.LEVEL_2 );
						
						Map m = _parent.federatedSearch( df_aid, depth );
						
						if( m != null )
						{
							DIAGNOSTICS.println( "Got results from " + df_aid.getName() + "....", this, DIAGNOSTICS.LEVEL_2 );
							new DFAgentDescriptionMapDialog( dfgui, m, false );
						}
						else
						{
							DIAGNOSTICS.println( "Didn't get results from " + df_aid.getName() + "!", this, DIAGNOSTICS.LEVEL_3 );							
						}
					}
					catch( Throwable x )
					{
					}
				}
			}
		} );
		t.start();
  }//GEN-LAST:event_jMenuItem1ActionPerformed

/**
 * Action event listener envoked when DF needs to be registered with another DF.
 * @param evt action event envoked
 */
  private void registerDFWithDF(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_registerDFWithDF
		DIAGNOSTICS.println("Register DF", DIAGNOSTICS.LEVEL_3);
		notifyError("Registering a DF with another DF is not supported in " +
		"DF GUI in FIPA-OS 2.0.0. \nThis feature will be available in a future release.");
		/*
		RegisterWithDFFrame register = new RegisterWithDFFrame(this, _agent_tree.getSelectionPath().getLastPathComponent().toString());
		register.show();
		 */
  }//GEN-LAST:event_registerDFWithDF
	
/**
 * Action event listener envoked when refresh is needed.
 * @param evt action event envoked
 */
  private void refresh(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_refresh
		DIAGNOSTICS.println("Refresh", DIAGNOSTICS.LEVEL_3);
		Iterator df_iterator = _dfs.keySet().iterator();
		while (df_iterator.hasNext())
		{
			String df_name = (String) df_iterator.next();
			Map registered_agents = _parent.listAgentsRequest(df_name);
			addAgentToTree(df_name, df_name);
			if (registered_agents != null)
			{
				Iterator iterator = registered_agents.values().iterator();
				while (iterator.hasNext())
				{
					String new_agent = ((DFAgentDescription) iterator.next()).getAgentID().getName();
					addAgentToTree(new_agent, df_name);
				}
			}
		}
  }//GEN-LAST:event_refresh
	
/**
 * Action event listener envoked when View menu is opened. It's used to
 * disable menus not available for current selection.
 * @param evt focus event envoked
 */
  private void viewMenuOpened(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_viewMenuOpened
		DIAGNOSTICS.println("View menu opened", DIAGNOSTICS.LEVEL_MIN);
		int count;
		try
		{
			count = _agent_tree.getSelectionPath().getPathCount();
		}
		catch (Throwable t)
		{
			count = 0;
		}
		
		if (count < 3)
		{
			_m_view.setEnabled(false);
		}
		else
		{
			_m_view.setEnabled(true);
		}
  }//GEN-LAST:event_viewMenuOpened
	
/**
 * Action event listener envoked when the DF Functions menu is opened. It is
 * used to disable the menus not allowed for the current selection.
 * @param evt focus event envoked
 */
  private void dfMenuOpened(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_dfMenuOpened
		DIAGNOSTICS.println("DF menu opened", DIAGNOSTICS.LEVEL_MIN);
		int count;
		try
		{
			count = _agent_tree.getSelectionPath().getPathCount();
		}
		catch (Throwable t)
		{
			count = 0;
		}
		
		if (count == 3)
		{
			//agent
			_m_deregister.setEnabled(true);
			_m_modify.setEnabled(true);
			_m_register_df.setEnabled(false);
		}
		else if (count == 2)
		{
			//DF
			_m_deregister.setEnabled(false);
			_m_modify.setEnabled(false);
			_m_register_df.setEnabled(true);
		}
		else
		{
			//FIPA-OS
			_m_deregister.setEnabled(false);
			_m_modify.setEnabled(false);
			_m_register_df.setEnabled(false);
		}
  }//GEN-LAST:event_dfMenuOpened
	
/**
 * Action event listener envoked when File menu is opened. It is used to
 * disable menus that are not available for the current selection.
 * @param evt focus event envoked
 */
  private void fileMenuOpened(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_fileMenuOpened
		DIAGNOSTICS.println("File menu opened", DIAGNOSTICS.LEVEL_MIN);
		int count;
		try
		{
			count = _agent_tree.getSelectionPath().getPathCount();
		}
		catch (Throwable t)
		{
			count = 0;
		}
		
		if (count == 2)
		{
			//DF
			_m_add_df.setEnabled(false);
			_m_remove_df.setEnabled(true);
		}
		else if (count == 1)
		{
			//FIPA-OS
			_m_add_df.setEnabled(true);
			_m_remove_df.setEnabled(false);
		}
		else
		{
			//agent
			_m_add_df.setEnabled(false);
			_m_remove_df.setEnabled(false);
		}
		
  }//GEN-LAST:event_fileMenuOpened
	
/**
 * Action event listener envoked when new DF needs to be added to list of
 * monitored agents.
 * @param evt action event envoked
 */
  private void addDF(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addDF
		DIAGNOSTICS.println("Add DF", DIAGNOSTICS.LEVEL_3);
		notifyError("Monitoring of a DF in another platform is not supported in " +
		"DF GUI in FIPA-OS 2.0.0. \nThis feature will be available in a future release. "+
		"However, you will be able to manually refresh the contents \nby selecting " +
		"File -> Refresh.");
		RegisterWithDFFrame register = new RegisterWithDFFrame(this);
		register.show();
  }//GEN-LAST:event_addDF
	
/**
 * Action event listener envoked when certain DF will no longer be monitored.
 * @param evt action event envoked
 */
  private void removeDF(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_removeDF
		DIAGNOSTICS.println("Remove DF", DIAGNOSTICS.LEVEL_3);
		
		String selected_agent = _agent_tree.getSelectionPath().getLastPathComponent().toString();
		_parent.dropDF(selected_agent);
		removeAgentFromTree(selected_agent);
  }//GEN-LAST:event_removeDF
	
/**
 * Action event listener envoked when selected agent services will be viewed.
 * @param evt action event envoked
 */
  private void view(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_view
		DIAGNOSTICS.println("View", DIAGNOSTICS.LEVEL_3);
		String df_of_selected_agent = _agent_tree.getSelectionPath().getParentPath().getLastPathComponent().toString();
		String selected_agent = _agent_tree.getSelectionPath().getLastPathComponent().toString();
		AgentID df_aid = new AgentID();
		df_aid.setName(df_of_selected_agent);
		DFAgentDescription description = null;
		
		Map registered_agents = (TreeMap) _parent.listAgentsRequest(df_aid);
		if (registered_agents != null)
		{
			Iterator iterator = registered_agents.values().iterator();
			while (iterator.hasNext())
			{
				DFAgentDescription temp = (DFAgentDescription) iterator.next();
				if (temp.getAgentID().getName().equals(selected_agent))
				{
					description = temp;
				}
			}
		}
		if ( description != null)
		{
			DFAgentDescriptionEditor editor = new DFAgentDescriptionEditor(selected_agent, description, true);
		}
		else
		{
			notifyError("Can't display details for '" + selected_agent + "' - the agent details not found.");
		}
  }//GEN-LAST:event_view
	
/**
 * Closes the window and shuts down the agent.
 * @param evt action event envoked
 */
  private void exit (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exit

		setVisible( false );
		dispose();
		_parent.shutdown();
		
  }//GEN-LAST:event_exit
	
/**
 * Action event listener envoked when selected agent will be modified.
 * @param evt action event envoked
 */
  private void modify (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_modify
		DIAGNOSTICS.println("Modify", DIAGNOSTICS.LEVEL_3);
		notifyError("Modifying an agent is not supported in " +
		"DF GUI in FIPA-OS 2.0.0. \nThis feature will be available in a future release.");
		/*
		try
		{
			String df_of_selected_agent = _agent_tree.getSelectionPath().getParentPath().getLastPathComponent().toString();
			String selected_agent = _agent_tree.getSelectionPath().getLastPathComponent().toString();
			AgentID df_aid = new AgentID();
			df_aid.setName(df_of_selected_agent);
			DFAgentDescription description = null;
		 
			Map registered_agents = (TreeMap) _parent.listAgentsRequest(df_aid);
			if (registered_agents != null)
			{
				Iterator iterator = registered_agents.values().iterator();
				while (iterator.hasNext())
				{
					DFAgentDescription temp = (DFAgentDescription) iterator.next();
					if (temp.getAgentID().getName().equals(selected_agent))
					{
						description = temp;
					}
				}
			}
			if ( description != null)
			{
				DFAgentDescriptionEditor editor = new DFAgentDescriptionEditor(selected_agent, description, false);
				notifyMessage("Modifying " + selected_agent + " was successful.");
			}
			else
			{
				notifyError("Modifying '" + selected_agent + "' was not successful.");
			}
		}
		catch (Throwable t)
		{
			notifyError("Nothing selected.");
		}
		 */
  }//GEN-LAST:event_modify
	
/**
 * Action event listener envoked when selected agent will be deregistered.
 * @param evt action event envoked
 */
  private void deregister (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_deregister
		DIAGNOSTICS.println("Deregister", DIAGNOSTICS.LEVEL_3);
		
		DIAGNOSTICS.println("Modify", DIAGNOSTICS.LEVEL_3);
		notifyError("Deregistering an agent from a DF is not supported in " +
		"DF GUI in FIPA-OS 2.0.0. \nThis feature will be available in a future release.");
		
		/*
		String selected_agent = _agent_tree.getSelectionPath().getLastPathComponent().toString();
		if (_parent.deregisterRequest(selected_agent))
		{
			removeAgentFromTree(selected_agent);
			notifyMessage("Deregistering " + selected_agent + " was successful.");
		}
		else
		{
			notifyError("Deregistering '" + selected_agent + "' was not successful.");
		}
		 */
  }//GEN-LAST:event_deregister
	
		/**
		 * Exit the Application
		 * @param evt action event envoked
		 */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm

			setVisible( false );

    }//GEN-LAST:event_exitForm
		
/**
 * Adds an agent to the JTree to a specified DF. When adding
 * a DF, both <CODE>new_agent</CODE> and <CODE>df</CODE> have the
 * value of the DF's name. If <CODE>df</CODE> doesn't exist in the
 * tree it's added in with <CODE>new_agent</CODE> as it's child.
 *
 * @param new_agent agent to be added
 * @param df the DF the agent has registered for
 */
		private void addAgentToTree(String new_agent,String df)
		{
			//check if it's a new DF
			if (new_agent.equals(df))
			{
				if (!_dfs.containsKey(df))
				{
					_dfs.put(df, new LinkedList());
					notifyMessage("Added new DF: " + df);
				}
			}
			else //it's a new agent for an existing DF
			{
				if (_dfs.containsKey(df))
				{
					List temp = (LinkedList) _dfs.get(df);
					if (!temp.contains(new_agent))
					{
						temp.add(new_agent);
						notifyMessage("Added new agent " + new_agent + " for DF " + df);
					}
				}
				else //no DF was found, add the DF in as well
				{
					List list = new LinkedList();
					list.add(new_agent);
					_dfs.put(df, list);
					notifyMessage("Added new DF " + df + " and agent " + new_agent);
				}
			}
			_tree_model.notifyChanges();
			DIAGNOSTICS.println("_dfs " + _dfs, DIAGNOSTICS.LEVEL_MIN);
		}
		
/**
 * Main method for testing.
 * @param agrs command line parameters (not used)
 */
		public static void main(java.lang.String[] agrs)
		{
			SwingDFGUIAgent agent =
			new SwingDFGUIAgent("e:\\fipa-os\\profiles\\platform.profile", "dfgui", "TRINITY");
		}
		
/**
 * Called when agents are removed from the tree.
 * @param agent_name name of the agent to be removed
 */
		private void removeAgentFromTree(String agent_name)
		{
			if (_dfs.containsKey(agent_name))
			{
				_dfs.remove(agent_name);
				_tree_model.notifyChanges();
				notifyMessage("DF " + agent_name + " no longer monitored.");
			}
			else
			{
				Iterator iterator = _dfs.values().iterator();
				while (iterator.hasNext())
				{
					List list = (LinkedList) iterator.next();
					if (list.contains(agent_name))
					{
						list.remove(agent_name);
						_tree_model.notifyChanges();
						notifyMessage("Agent " + agent_name + " no longer registered.");
					}
				}
			}
		}
		
/**
 * Creates a popup window notifying an error.
 * @param error_message the error message
 */
		void notifyError(String error_message)
		{
			JOptionPane.showMessageDialog(this, error_message, "Error", JOptionPane.ERROR_MESSAGE);
		}
		
/**
 * Shows a system message in the task bar of the GUI window.
 * @param message the notify message
 */
		void notifyMessage(String message)
		{
			_status_bar.setText(message);
		}
		
		/**
		 * Invoked when a key has been typed.
		 * This event occurs when a key press is followed by a
		 * key release. Used to detect menu hot-keys.
		 *
		 * @param e the key event detected
		 */
		public void keyTyped(KeyEvent e)
		{
			if (e.getKeyCode() == KeyEvent.VK_F5)
			{
				refresh(new ActionEvent(e, 0, "F5 pressed"));
			}
		}
		
		/**
		 * Invoked when a key has been pressed. Not implemented.
		 * @param e the key event detected
		 */
		public void keyPressed(KeyEvent e)
		{}
		
		/**
		 * Invoked when a key has been released. Not implemented.
		 * @param e the key event detected
		 */
		public void keyReleased(KeyEvent e)
		{}
		
/**
 * This method is envoked by the RegisterWithDFFrame when user wants to add in
 * new DF to be monitored.
 * @param df_aid AID of the new DF
 */
		void returnAID(AgentID df_aid)
		{
			DIAGNOSTICS.println("Add DF: " + df_aid);
			addAgentToTree(df_aid.getName(),df_aid.getName());
			listAgents(df_aid);
		}
		
/**
 * Lists the agents on a certain DF and displays them in the JTree.
 * @param df_aid AID of the DF
 */
		private void listAgents(final AgentID df_aid)
		{
			Thread t = new Thread( new Runnable()
			{
				public void run()
				{
					String df_name = df_aid.getName();
					
					Map registered_agents = (TreeMap) _parent.listAgentsRequest(df_aid);
					
					if (registered_agents != null)
					{
						Iterator iterator = registered_agents.values().iterator();
						while (iterator.hasNext())
						{
							String new_agent = ((DFAgentDescription) iterator.next()).getAgentID().getName();
							addAgentToTree(new_agent, df_name);
						}
					}
					else
					{
						notifyMessage("No agents registered on " + df_name);
					}
				}
			});
			t.start();
		}
		
/**
 * This method is envoked by the RegisterWithDFFrame when user wants to register
 * the selected DF with another DF.
 * @param new_df new DF the selected DF will be registered with
 * @param df the old DF
 */
		void returnRegister(AgentID new_df,String df)
		{
			DIAGNOSTICS.println("Register DF " + df + " with DF " + new_df);
			if (_parent.registerDFRequest(new_df, df))
			{
				notifyMessage("Send a register message to DF " + df + " for DF " + new_df.getName() + ".");
			}
			else
			{
				notifyError("Registering of DF '" + df + "' with DF '" + new_df.getName() + "' failed.");
			}
		}
		
/**
 * This method is envoked by the parent agent when a change has happened with
 * one of the monitored DFs.
 */
		void updateTree()
		{
			refresh(new ActionEvent(_parent, 0, "Refresh"));
		}
		
/**
 * This method is called when agent has deregistered
 * from an df.
 * @param df AgentID of the DF
 * @param agent AgentID of the agent deregistered
 */		
		void agentDeregistered(AgentID df,AgentID agent)
		{
			if(_dfs.containsKey(df.getName()))
			{
				List temp = (LinkedList) _dfs.get(df.getName());
				if(temp.contains(agent.getName()))
				{
					temp.remove(agent.getName());
					updateTree();
				}
			}
		}
		
		// Variables declaration - do not modify//GEN-BEGIN:variables
		private javax.swing.JMenuBar _menu_bar;
		private javax.swing.JMenu _menu_file;
		private javax.swing.JMenuItem _m_add_df;
		private javax.swing.JMenuItem _m_remove_df;
		private javax.swing.JSeparator _m_separator1;
		private javax.swing.JMenuItem _m_refresh;
		private javax.swing.JSeparator _m_separator3;
		private javax.swing.JMenuItem _m_exit;
		private javax.swing.JMenu _menu_df_functionality;
		private javax.swing.JMenuItem _m_deregister;
		private javax.swing.JMenuItem _m_modify;
		private javax.swing.JSeparator _m_separator2;
		private javax.swing.JMenuItem _m_register_df;
		private javax.swing.JMenu _menu_view;
		private javax.swing.JMenuItem _m_view;
		private javax.swing.JPopupMenu _agent_popup_menu;
		private javax.swing.JMenuItem _pm_deregister;
		private javax.swing.JMenuItem _pm_modify;
		private javax.swing.JSeparator _pm_separator1;
		private javax.swing.JMenuItem _pm_view;
		private javax.swing.JSeparator _pm_separator2;
		private javax.swing.JMenuItem _pm_remove_df;
		private javax.swing.JMenuItem _pm_register_with_df;
		private javax.swing.JMenuItem _pm_federated_search;
		private javax.swing.JPopupMenu _FIPAOS_popup_menu;
		private javax.swing.JMenuItem _pm_add_df;
		private javax.swing.JToolBar _toolbar;
		private javax.swing.JButton _b_deregister;
		private javax.swing.JButton _b_modify;
		private javax.swing.JSeparator _b_separator;
		private javax.swing.JButton _b_add_df;
		private javax.swing.JButton _b_remove_df;
		private javax.swing.JScrollPane _agent_scrollpane;
		private javax.swing.JTree _agent_tree;
		private javax.swing.JLabel _status_bar;
		// End of variables declaration//GEN-END:variables
		
		private Map _dfs = new TreeMap( fipaos.util.Comparators.OBJECT_COMPARE );
		private SwingDFGUIAgent _parent;
		private DFGUIModel _tree_model;
		private MouseListener _popup_listener;
		private static final String DF_IMAGE =  "dfgui/df.gif";
		private static final String AGENT_IMAGE =  "dfgui/Agent.gif";
		private static final String IMAGE_DIRECTORY = "images/";
		private static final String FIPA_GIF = "logo.gif";
		
/**
 * Internal class for rendering, mainly choosing the icons.
 * @see javax.swing.tree.DefaultTreeCellRenderer
 */
		
		class DFGUIRenderer extends DefaultTreeCellRenderer
		{
			
/**
 * Constructor that sets the selection colour to customised one.
 */
			public DFGUIRenderer()
			{
				super();
				setBackgroundSelectionColor(Color.lightGray);
				setBorderSelectionColor(Color.white);
			}
			
/**
 * This method returns the Component with correct icon
 * and text to represent the tree node.
 *
 * @param tree the JTree this render is attached to
 * @param value text in the label
 * @param sel is the component selected
 * @param expanded is the node expanded
 * @param leaf is the node leaf
 * @param row row number for this component
 * @param hasFocus focus
 * @return the rendered component, in this case a JLabel (subclass of component)
 */
			public java.awt.Component getTreeCellRendererComponent(JTree tree,Object value,boolean sel,boolean expanded,boolean leaf,int row,boolean hasFocus)
			{
				String text = tree.convertValueToText(value, sel, expanded, leaf, row, hasFocus);
				int count;
				if ((row >= 0) && (tree.getPathForRow(row) != null))
				{
					count = tree.getPathForRow(row).getPathCount();
				}
				else
				{
					count = 0;
				}
				
				//this.hasFocus = hasFocus;
				//setComponentOrientation(tree.getComponentOrientation());
				selected = sel;
				
				if (text.startsWith("{"))
				{
					setText("FIPA-OS");
					setIcon(getIcon(count));
				}
				else
				{
					setText(text);
					if (!tree.isEnabled())
					{
						setEnabled(false);
						if (leaf)
						{
							setDisabledIcon(getIcon(count));
						}
						else if (expanded)
						{
							setDisabledIcon(getIcon(count));
						}
						else
						{
							setDisabledIcon(getIcon(count));
						}
					}
					else
					{
						setEnabled(true);
						if (leaf)
						{
							setIcon(getIcon(count));
						}
						else if (expanded)
						{
							setIcon(getIcon(count));
						}
						else
						{
							setIcon(getIcon(count));
						}
					}
				}
				if(sel)
				{
					setForeground(getTextSelectionColor());
				}
				else
				{
					setForeground(getTextNonSelectionColor());
				}
				
				return this;
			}
			
/**
 * Returns an ImageIcon corresponding to the path count of certain tree item.
 * @param path_count path count for the tree item: 1 for FIPA-OS (root),
 * 2 for DFs and 3 for agents - other numbers shouldn't
 * possible, but null is returned for those
 * @return appropriate ImageIcon
 */
			private ImageIcon getIcon(int path_count)
			{
				switch (path_count)
				{
					case 1:		return new ImageIcon( ClassLoader.getSystemResource( IMAGE_DIRECTORY + FIPA_GIF ) );
					case 2: 	return new ImageIcon( ClassLoader.getSystemResource( IMAGE_DIRECTORY + DF_IMAGE ) );
					case 3: 	return new ImageIcon( ClassLoader.getSystemResource( IMAGE_DIRECTORY + AGENT_IMAGE ) );
					default: 	return null;
				}
			}
			
			/**
			 * If the preferredSize has been set to a non-null value
			 * just returns it.  If the UI delegate's getPreferredSize()
			 * method returns a non null value then return that; otherwise
			 * defer to the component's layout manager.
			 *
			 * @return the value of the preferredSize property
			 * @see #setPreferredSize
			 */
			public Dimension getPreferredSize()
			{
				return new Dimension(500, 25);
			}
			
		}
		
/**
 * The Tree model used with the DF GUI, which is used to show the real
 * tree structure in the GUI.
 */
		class DFGUIModel implements TreeModel
		{
			private List treeModelListeners = new LinkedList();
			
/**
 * Remove a listener.
 * @param listener listener to be removed
 */
			public void removeTreeModelListener(TreeModelListener listener)
			{
				if (treeModelListeners.contains(listener))
				{
					treeModelListeners.remove(listener);
					DIAGNOSTICS.println("listener removed", DIAGNOSTICS.LEVEL_MIN);
				}
			}
			
/**
 * Add a listener to this model
 * @param listener the listener to be added
 */
			public void addTreeModelListener(TreeModelListener listener)
			{
				treeModelListeners.add(listener);
				DIAGNOSTICS.println("listener added", DIAGNOSTICS.LEVEL_MIN);
			}
			
/**
 * Notify all listeners of the changes.
 */
			public void notifyChanges()
			{
				Iterator iterator = treeModelListeners.iterator();
				while (iterator.hasNext())
				{
					TreeModelListener listener = (TreeModelListener) iterator.next();
					listener.treeStructureChanged(new TreeModelEvent(this, new Object[]
					{_dfs}));
					DIAGNOSTICS.println("node added", DIAGNOSTICS.LEVEL_MIN);
				}
			}
			
/**
 * Checks if the p1 is a leaf node.
 * @param p1 the node object
 * @return boolean, the result of the check
 */
			public boolean isLeaf(java.lang.Object p1)
			{
				if (p1 == _dfs) //is root
				{
					DIAGNOSTICS.println("is leaf (" + p1 + ") false", DIAGNOSTICS.LEVEL_MIN);
					return false;
				}
				else
				{
					if (_dfs.containsKey( p1 )) //is a DF
					{
						List l = (LinkedList) _dfs.get(p1);
						if (l.isEmpty()) //has no agents
						{
							DIAGNOSTICS.println("is leaf (" + p1 + ") true", DIAGNOSTICS.LEVEL_MIN);
							return true;
						}
						else //has agents
						{
							DIAGNOSTICS.println("is leaf (" + p1 + ") false", DIAGNOSTICS.LEVEL_MIN);
							return false;
						}
					}
					else //must be an agent
					{
						DIAGNOSTICS.println("is leaf (" + p1 + ") true", DIAGNOSTICS.LEVEL_MIN);
						return true;
					}
				}
			}
			
/**
 * Returns the root node
 * @return the root node
 */
			public java.lang.Object getRoot()
			{
				DIAGNOSTICS.println("get root " + _dfs, DIAGNOSTICS.LEVEL_MIN);
				return _dfs;
			}
			
/**
 * Value for the changed path (not implemented).
 * @param p1 tree changed
 * @param p2 changed object
 */
			public void valueForPathChanged(javax.swing.tree.TreePath p1,java.lang.Object p2)
			{
				DIAGNOSTICS.println("value for path changed", DIAGNOSTICS.LEVEL_MIN);
			}
			
/**
 * Returns the child (index given) of the given object.
 * @param p1 parent for the object
 * @param p2 index of the child
 * @return child
 */
			public Object getChild(Object p1,int p2)
			{
				if( _dfs.containsKey(p1) ) //if p1 is a DF
				{
					List kids = (LinkedList) _dfs.get(p1);
					DIAGNOSTICS.println("get child (DF) " + kids.get(p2), DIAGNOSTICS.LEVEL_MIN);
					return kids.get(p2);
				}
				else if( p1 == _dfs ) //if p1 is root
				{
					List l = new LinkedList( _dfs.keySet() );
					Collections.sort( l, Comparators.STRING_COMPARE );
					DIAGNOSTICS.println("get child (root) " + l.get(p2), DIAGNOSTICS.LEVEL_MIN);
					return l.get(p2);
				}
				else
				{
					DIAGNOSTICS.println("get child (null)", DIAGNOSTICS.LEVEL_MIN);
					return null;
				}
			}
			
/**
 * How many children given object has.
 * @param p1 parent
 * @return child count
 */
			public int getChildCount(Object p1)
			{
				if( p1 == _dfs )
				{
					DIAGNOSTICS.println("get child count (root) " + _dfs.keySet().size(), DIAGNOSTICS.LEVEL_MIN);
					return _dfs.keySet().size();
				}
				else if( _dfs.containsKey( p1 ) )
				{
					DIAGNOSTICS.println("get child count (DF) " + ((List) _dfs.get(p1)).size(), DIAGNOSTICS.LEVEL_MIN);
					return ((List) _dfs.get(p1)).size();
				}
				else
				{
					DIAGNOSTICS.println("get child count (leaf)", DIAGNOSTICS.LEVEL_MIN);
					return 0;
				}
			}
			
/**
 * Gives the index of the given child.
 * @param p1 parent
 * @param p2 child
 * @return the index
 */
			public int getIndexOfChild(Object p1,Object p2)
			{
				if( p1 == _dfs ) //parent is the root
				{
					List l = new LinkedList( _dfs.keySet() );
					Collections.sort( l, Comparators.STRING_COMPARE );
					DIAGNOSTICS.println("index of child (root) " + l.indexOf(p2), DIAGNOSTICS.LEVEL_MIN);
					return l.indexOf(p2);
				}
				else if( _dfs.containsKey( p1 ) ) //parent is DF
				{
					DIAGNOSTICS.println("index of child (DF) " + ((List) _dfs.get(p1)).indexOf(p2), DIAGNOSTICS.LEVEL_MIN);
					return ((List) _dfs.get(p1)).indexOf(p2);
				}
				else //shouldn't happen
				{
					DIAGNOSTICS.println("shouldn't happen (-1)", DIAGNOSTICS.LEVEL_MIN);
					return -1;
				}
			}
		}
		
/**
 * Mouse adapter used to detect popup events and show the appropriate popup menus.
 */
		class DFGUIPopupListener extends MouseAdapter
		{
			
/**
 * This method is called when mouse press is detected.
 * @param e mouse event envoked
 */
			public void mousePressed(MouseEvent e)
			{
				showPopupMenu(e);
			}
			
/**
 * This method is called when mouse is released.
 * @param e mouse event envoked
 */
			public void mouseReleased(MouseEvent e)
			{
				showPopupMenu(e);
			}
			
/**
 * Checks the selected item on the tree, the mouse event and selects the
 * appropriate popup menu to show, if any.
 * @param e the mouse event
 */
			private void showPopupMenu(MouseEvent e)
			{
				DIAGNOSTICS.println("MouseEvent is " + e, DIAGNOSTICS.LEVEL_MIN);
				int count;
				try
				{
					count = _agent_tree.getPathForLocation(e.getX(), e.getY()).getPathCount();
					_agent_tree.setSelectionPath(_agent_tree.getPathForLocation(e.getX(), e.getY()));
				}
				catch (Throwable t)
				{
					count = 0;
				}
				
				DIAGNOSTICS.println("count " + count, DIAGNOSTICS.LEVEL_MIN);
				if((count > 1) && e.isPopupTrigger()) //not FIPA-OS
				{
					//TreePath selPath = _agent_tree.getPathForLocation(e.getX(), e.getY());
					//DIAGNOSTICS.println("path " + selPath, DIAGNOSTICS.LEVEL_MIN);
					if (count == 2) //DF
					{
						_pm_deregister.setEnabled(false);
						_pm_modify.setEnabled(false);
						_pm_view.setEnabled(false);
						_pm_remove_df.setEnabled(true);
						_pm_register_with_df.setEnabled(true);
						_pm_federated_search.setEnabled(true);
					}
					else //agent
					{
						_pm_deregister.setEnabled(true);
						_pm_modify.setEnabled(true);
						_pm_view.setEnabled(true);
						_pm_remove_df.setEnabled(false);
						_pm_register_with_df.setEnabled(false);
						_pm_federated_search.setEnabled(false);
					}
					_agent_popup_menu.show(e.getComponent(), e.getX(), e.getY());
				}
				else if ((count == 1) && e.isPopupTrigger()) //FIPA-OS
				{
					_FIPAOS_popup_menu.show(e.getComponent(), e.getX(), e.getY());
				}
			}
		}
		
/**
 * TreeSelectionListener that is used to disable buttons that are not usable
 * with selected tree items.
 */
		class DFGUITreeSelectionListener implements TreeSelectionListener
		{
			
			/**
			 * Called whenever the value of the selection changes.
			 * Used to update what buttons can be used at what time.
			 *
			 * @param e the event that characterizes the change.
			 */
			public void valueChanged(TreeSelectionEvent e)
			{
				int count;
				try
				{
					count = _agent_tree.getSelectionPath().getPathCount();
				}
				catch (Throwable t)
				{
					count = 0;
				}
				switch (count)
				{
					//FIPA-OS
					case 1:
						_b_deregister.setEnabled(false);
						_b_modify.setEnabled(false);
						_b_add_df.setEnabled(true);
						_b_remove_df.setEnabled(false);
						break;
						//DF
					case 2:
						_b_deregister.setEnabled(false);
						_b_modify.setEnabled(false);
						_b_add_df.setEnabled(false);
						_b_remove_df.setEnabled(true);
						break;
						//agent
					case 3:
						_b_deregister.setEnabled(true);
						_b_modify.setEnabled(true);
						_b_add_df.setEnabled(false);
						_b_remove_df.setEnabled(false);
						break;
						default:
							_b_deregister.setEnabled(false);
							_b_modify.setEnabled(false);
							_b_add_df.setEnabled(false);
							_b_remove_df.setEnabled(false);
							break;
				}
			}
		}
		
}