//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.platform.dfgui.swing;

import java.io.*;
import java.util.*;
import javax.swing.*;

import fipaos.util.*;
import fipaos.parser.*;
import fipaos.ont.fipa.fipaman.*;

/**
 * This GUI lets the user put in (either type or load from file) an agent ID,
 * which is validated. The GUI is used for both registering a selected DF with
 * another DF (input) and for adding another DF to the list of monitored DFs
 * (input).
 *
 */
public class RegisterWithDFFrame extends javax.swing.JFrame
{
	
		/**
		 * Creates new form RegisterWithDFFrame. Used for adding a new DF to the list
		 * of monitored DFs.
		 * @param parent the DFGUIFrame that created this frame
		 */
	public RegisterWithDFFrame(DFGUIFrame parent)
	{
		_current_dir = new File( "." );
		_parent = parent;
		try
		{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}
		catch(Throwable e)
		{
			DIAGNOSTICS.println("Exception", e, this, DIAGNOSTICS.LEVEL_MIN );
		}
		initComponents ();
		pack ();
		_l_describe.setText("Add new DF:");
		_b_register.setText("Add");
		setTitle("Add new DF");
	}
	
		/**
		 * Creates new form RegisterWithDFFrame. Used to register a DF with another DF.
		 * @param parent the DFGUIFrame that created this frame
		 * @param df DF the needs to be registered
		 */
	public RegisterWithDFFrame(DFGUIFrame parent, String df)
	{
		_current_dir = new File( "." );
		_parent = parent;
		_df = df;
		
		try
		{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}
		catch(Throwable e)
		{
			DIAGNOSTICS.println("Exception", e, this, DIAGNOSTICS.LEVEL_MIN );
		}
		initComponents ();
		pack ();
		_l_describe.setText("Register " + df + " with DF:");
	}
		/** This method is called from within the constructor to
		 * initialize the form.
		 * WARNING: Do NOT modify this code. The content of this method is
		 * always regenerated by the FormEditor.
		 */
	private void initComponents()//GEN-BEGIN:initComponents
	{
		_l_df_name = new javax.swing.JLabel();
		_df_name = new javax.swing.JTextField();
		_l_df_addresses = new javax.swing.JLabel();
		_df_addresses = new javax.swing.JTextField();
		_b_load_from_file = new javax.swing.JButton();
		_b_register = new javax.swing.JButton();
		_b_cancel = new javax.swing.JButton();
		_l_describe = new javax.swing.JLabel();
		getContentPane().setLayout(new java.awt.GridBagLayout());
		java.awt.GridBagConstraints gridBagConstraints1;
		setIconImage(new ImageIcon(getClass().getResource(IMAGE_DIRECTORY+FIPA_GIF)).getImage());
		setTitle("Register with another DF");
		setResizable(false);
		addWindowListener(new java.awt.event.WindowAdapter()
		{
			public void windowClosing(java.awt.event.WindowEvent evt)
			{
				exitForm(evt);
			}
		}
		);
		
		_l_df_name.setText("DF name:");
		_l_df_name.setForeground(java.awt.Color.black);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 0;
		gridBagConstraints1.gridy = 1;
		gridBagConstraints1.insets = new java.awt.Insets(3, 3, 0, 3);
		getContentPane().add(_l_df_name, gridBagConstraints1);
		
		
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 1;
		gridBagConstraints1.gridy = 1;
		gridBagConstraints1.gridwidth = 3;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints1.insets = new java.awt.Insets(3, 0, 2, 3);
		gridBagConstraints1.weightx = 0.5;
		getContentPane().add(_df_name, gridBagConstraints1);
		
		
		_l_df_addresses.setText("Addresses:");
		_l_df_addresses.setForeground(java.awt.Color.black);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 0;
		gridBagConstraints1.gridy = 2;
		gridBagConstraints1.insets = new java.awt.Insets(0, 3, 0, 3);
		getContentPane().add(_l_df_addresses, gridBagConstraints1);
		
		
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 1;
		gridBagConstraints1.gridy = 2;
		gridBagConstraints1.gridwidth = 3;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints1.insets = new java.awt.Insets(0, 0, 0, 3);
		gridBagConstraints1.weightx = 0.5;
		getContentPane().add(_df_addresses, gridBagConstraints1);
		
		
		_b_load_from_file.setBorder(new javax.swing.border.BevelBorder(0));
		_b_load_from_file.setText("Load from file...");
		_b_load_from_file.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				loadFromFile(evt);
			}
		}
		);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 3;
		gridBagConstraints1.gridy = 3;
		gridBagConstraints1.ipadx = 10;
		gridBagConstraints1.insets = new java.awt.Insets(3, 3, 3, 3);
		gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
		getContentPane().add(_b_load_from_file, gridBagConstraints1);
		
		
		_b_register.setBorder(new javax.swing.border.BevelBorder(0));
		_b_register.setSelected(true);
		_b_register.setText("Register");
		_b_register.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				register(evt);
			}
		}
		);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 1;
		gridBagConstraints1.gridy = 4;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints1.ipadx = 10;
		gridBagConstraints1.insets = new java.awt.Insets(3, 3, 3, 3);
		gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
		gridBagConstraints1.weightx = 0.5;
		getContentPane().add(_b_register, gridBagConstraints1);
		
		
		_b_cancel.setBorder(new javax.swing.border.BevelBorder(0));
		_b_cancel.setText("Cancel");
		_b_cancel.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				cancel(evt);
			}
		}
		);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 2;
		gridBagConstraints1.gridy = 4;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints1.ipadx = 10;
		gridBagConstraints1.insets = new java.awt.Insets(3, 3, 3, 3);
		gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
		gridBagConstraints1.weightx = 0.5;
		getContentPane().add(_b_cancel, gridBagConstraints1);
		
		
		_l_describe.setText(" ");
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridwidth = 0;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints1.insets = new java.awt.Insets(0, 3, 0, 0);
		gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
		getContentPane().add(_l_describe, gridBagConstraints1);
		
	}//GEN-END:initComponents
	
/**
 * Event listener which is envoked when register button is pressed. It checks
 * whether DF is null - if it is, this is a monitor request, and otherwise
 * it's a register request. Appropriate methods are envoked in the parent frame.
 *
 * @param evt action event envoked
 */
  private void register(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_register
		AgentID df_aid = null;
		String df_string = parseAID(_df_name.getText(), _df_addresses.getText());
		try
		{
			df_aid = new AgentID(df_string);
		}
		catch (ParserException pe)
		{
			JOptionPane.showMessageDialog(this, "'" + df_string + "' is not a valid AgentID.", "Parsing Error", JOptionPane.ERROR_MESSAGE);
		}
		DIAGNOSTICS.println("DF aid: " + df_aid, DIAGNOSTICS.LEVEL_3);
		if( df_aid != null )
		{
			if (_df == null)
			{
				_parent.returnAID(df_aid);
			}
			else
			{
				_parent.returnRegister(df_aid, _df);
			}
		}
		this.dispose();
  }//GEN-LAST:event_register
	
/**
 * Event listener which is envoked when cancel button is pressed.
 *
 * @param evt action event envoked
 */
  private void cancel(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancel
		this.dispose();
  }//GEN-LAST:event_cancel
	
/**
 * Event listener which is envoked when user wants to load an AID from a file.
 *
 * @param evt action event envoked
 */
  private void loadFromFile(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadFromFile
		// Create an open file dialog
		JFileChooser open = new JFileChooser( _current_dir );
		int result = open.showOpenDialog( this );
		AgentID df_aid = null;
		
		// OK pressed?
		if( result == JFileChooser.APPROVE_OPTION )
		{
			// Load file & parse it
			String doc = DocumentRetriever.getDocFromFile( open.getSelectedFile().getPath() );
			
			// Save selected directory
			_current_dir = new File( open.getSelectedFile().getParent() );
			
			// Check if we got doc
			if( doc == null )
			{
				DIAGNOSTICS.println( "Some error occured reading from file, aborted.", DIAGNOSTICS.LEVEL_3);
				return;
			}
			else
			{
				try
				{
					df_aid = new AgentID(doc);
					_df_name.setText(df_aid.getName());
					String addresses = "";
					Iterator iterator =	df_aid.getAddresses().iterator();
					while (iterator.hasNext())
					{
						addresses = addresses + " "  + iterator.next();
					}
					_df_addresses.setText(addresses);
				}
				catch (ParserException pe)
				{
					JOptionPane.showMessageDialog(this, "'" + doc + "' is not a valid AgentID.", "Parsing Error", JOptionPane.ERROR_MESSAGE);
				}
			}
			
		}
  }//GEN-LAST:event_loadFromFile
	
		/**
		 * Exit the Application.
		 * @param evt action event envoked
		 */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
			dispose();
    }//GEN-LAST:event_exitForm
		
/**
 * Parses a string into an aid.
 * @param name name of the agent
 * @param addresses addresses of an agent
 * @return the aid produced
 */
		private String parseAID(String name,String addresses)
		{
			StringBuffer aid = new StringBuffer("(agent-identifier :name ");
			aid.append(name);
			aid.append(" :addresses (sequence ");
			aid.append(addresses);
			aid.append("))");
			return aid.toString();
		}
		
		// Variables declaration - do not modify//GEN-BEGIN:variables
		private javax.swing.JLabel _l_df_name;
		private javax.swing.JTextField _df_name;
		private javax.swing.JLabel _l_df_addresses;
		private javax.swing.JTextField _df_addresses;
		private javax.swing.JButton _b_load_from_file;
		private javax.swing.JButton _b_register;
		private javax.swing.JButton _b_cancel;
		private javax.swing.JLabel _l_describe;
		// End of variables declaration//GEN-END:variables
		
		private File _current_dir;
		private DFGUIFrame _parent;
		private String _df = null;
		private static final String IMAGE_DIRECTORY = "/images/";
		private static final String FIPA_GIF = "logo.gif";
		
}
