//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files ("the
//     Programs") are Copyright (c) 1999 - 2000 Nortel Networks Corporation, 
//     8200 Dixie Road, Suite 100, Brampton, Ontario, Canada L6R 5P6.  
//     All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out in
//     the Nortel Networks FIPA-OS Public License, a copy of which can be found
//     in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version can
//     also be found at http://fipa-os.sourceforge.net/. By downloading the
//     files containing the Programs you accept the terms and  conditions of the
//     Public License. You do not have to accept these terms and conditions, but
//     unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.skill.constraint.ccl.variable;

//----------------------------------------------------------------------

import fipaos.ont.fipa.FIPACONSTANTS;
import fipaos.ont.BASETOKENS;
import fipaos.ont.fipa.fipaman.FIPAMANCONSTANTS;

import java.util.*;

//----------------------------------------------------------------------

/**
 * 
 * <br> This class describes the object Index-pair as described in the document 
 * <br> "CCL Language Specification v2.01" section 4.2.4.6, p17.
 * <br>
 * <br> Indices are used in relations to reference the individual fields in tuples.
 * <br> Given two variables with tuple valued variables, the index-pair indicates
 * <br> a field in the first and a field in the second which are somehow related.
 * <br>
 * <br> This type is not described in section 4.2.2 "Variable related items"
 * <br> but in section 4.2.4 "Ontology Related Items and Terminals".
 * <br> However all other terms in this section are simple types so this object
 * <br> has been put into the "variables" package for convenience.
 *
 */
 
public class IndexPair
{
     
//----------------------------------------------------------------------
// CONSTANTS
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// STATIC VARIABLES
//----------------------------------------------------------------------
		
//----------------------------------------------------------------------
// INSTANCE VARIABLES
//----------------------------------------------------------------------

   /** 
   * First value in the IndexPair to be stored. 
   */
   private int _first_index = 0;
   
   /** 
   *Second value in the IndexPair to be stored. 
   */
   private int _second_index = 0;

//----------------------------------------------------------------------
// CONSTRUCTOR
//----------------------------------------------------------------------
 
   /**
   * <br> Constructs an Empty IndexPair.
   */
   public IndexPair()
	 {
	 }
	 
	 /**
	 * <br> Constructs an IndexPair using the Values specified.
	 */
	 public IndexPair(int first_value, int second_value)
	 {
	    _first_index = new Integer(first_value).intValue();
	    _second_index = new Integer(second_value).intValue(); 
	 }
	 
	 /**
	 * <br> Constructs a new IndexPair and populates it given the information provided 
	 * <br> in the String.
	 * <br>
	 * <br> The String to be used should take one of the following forms:
	 * <br> {(2, 2)}
	 * <br>  (2, 2)
	 */
	 public IndexPair(String str)
	 {
	 	  // System.out.println("string 'str' is: " + str);
	 	
	 	  // Trim off the brackets
	 	  if(str.indexOf("(") != -1)
	 	  {
	 	     str = str.substring(str.indexOf("(") + 1, str.indexOf(")"));	
	 	  	 // System.out.println("substring is: " + str);
	 	  	 
	 	  	 StringTokenizer st = new StringTokenizer(str, ",");
	 	  
	 	     // allocate the first Integer of the index
		     String next = st.nextToken();  
		     next = next.trim();  
		     _first_index = new Integer(next).intValue();
		     
		     // allocate the second Integer of the index
		     next = st.nextToken();  
		     next = next.trim();  
		     _second_index = new Integer(next).intValue();
		         
	 	  }
	 	  else
	 	  {
	 	     System.out.println();
	 	     System.out.println("Error: String representing IndexPair does not have the correct form.");
	 	     System.out.println();
	 	  }
	 	  
	 } // IndexPair(String str)

//----------------------------------------------------------------------
// PRIVATE METHODS
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// PUBLIC METHODS
//----------------------------------------------------------------------

   /**
   * <br> Returns the first value contained in the IndexPair object.
   */
   public int getFirstValue()
	 {
	 	  return new Integer(_first_index).intValue();
	 }	
	 
	 /**
	 * <br> Sets the value of the first value held in the IndexPair object.
	 */ 
	 public void setFirstValue(int value)
	 {
	 	  _first_index = new Integer(value).intValue();
	 }
	 
	 /**
	 * <br> Returns the second value contained in the IndexPair object.
	 */
	 public int getSecondValue()
	 {
	    return new Integer(_second_index).intValue();
	 }
	 
	 /**
	 * <br> Sets the value of the second value held in the IndexPair object.
	 */
	 public void setSecondValue(int value)
	 {
	 	  _second_index = new Integer(value).intValue();
	 }
	 
	 /**
	 * <br> Returns an identical copy of the IndexPair object.
	 */
	 public IndexPair copy()
	 {
	 	  IndexPair copy = new IndexPair(_first_index, _second_index);
	 	  return copy; 	
	 } 
	 
	 /**
	 * <br> Returns a String representation of the IndexPair object.
	 */
	 public String toString()
	 {
	    String value = "{ " + _first_index + ", " + _second_index + " }";
	    return value;	             
	 	
	 } // toString()
	 
	
//----------------------------------------------------------------------
// MAIN/TESTING METHOD
//----------------------------------------------------------------------

   /**
   * <br> Used to test the class.
   */
   public static void main(String args[])
	 {
	    System.out.println();
	    System.out.println("Testing IndexPair class");
	    System.out.println();
	    
	    System.out.println("Constructing empty IndexPair class");	
	    
	    IndexPair test = new IndexPair();
	    
	    System.out.println("Adding values...");
	    
	    test.setFirstValue(1);
	    test.setSecondValue(2);
	    
	    System.out.println();
	    System.out.println("Values contained in IndexPair are:");
	    System.out.println("Index one: " + test.getFirstValue());
	    System.out.println("Index two: " + test.getSecondValue());
	    System.out.println();
	    
	    System.out.println("Contents of IndexPair:");
	    System.out.println(test);
	    System.out.println();
	    
	    System.out.println("Constructing IndexPair using Values");
	    
	    test = new IndexPair(2, 3);
	    
	    System.out.println();
	    System.out.println("Values contained in IndexPair are:");
	    System.out.println("Index one: " + test.getFirstValue());
	    System.out.println("Index two: " + test.getSecondValue());
	    System.out.println();
	    
	    System.out.println("Contents of IndexPair:");
	    System.out.println(test);
	    System.out.println();
	    
	    System.out.println("Copying IndexPair object");
	    
	    IndexPair test1 = test.copy();
	    
	    System.out.println("Contents of new copy of IndexPair:");
	    System.out.println(test);
	    System.out.println();
	    
	    String string1 = "{(2, 2)}";
	    String string2 = "  (  3 , 3  )  ";
	    
	    System.out.println();
	    System.out.println("Constructing a new IndexPair using the String: " + string1);
	    
	    IndexPair test2 = new IndexPair(string1);
	    
	    System.out.println("Values held in IndexPair: " + test2);
	    System.out.println();
	    
	    System.out.println();
	    System.out.println("Constructing a new IndexPair using the String: " + string2);
	    
	    test2 = new IndexPair(string2);
	    
	    System.out.println("Values held in IndexPair: " + test2);
	    System.out.println();
	    
	 	
	 	
	 } // main
  
	
} // IndexPair

	
