//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files ("the
//     Programs") are Copyright (c) 1999 - 2000 Nortel Networks Corporation, 
//     8200 Dixie Road, Suite 100, Brampton, Ontario, Canada L6R 5P6.  
//     All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out in
//     the Nortel Networks FIPA-OS Public License, a copy of which can be found
//     in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version can
//     also be found at http://fipa-os.sourceforge.net/. By downloading the
//     files containing the Programs you accept the terms and  conditions of the
//     Public License. You do not have to accept these terms and conditions, but
//     unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.skill.constraint.ccl.variable;

//----------------------------------------------------------------------

import fipaos.ont.fipa.FIPACONSTANTS;
import fipaos.ont.BASETOKENS;
import fipaos.ont.fipa.fipaman.FIPAMANCONSTANTS;

import java.util.*;

//----------------------------------------------------------------------

/**
 * 
 * <br> This class handles the creation and parsing of a List or Set.
 * <br> It converts between a Vector and a String in the following formats:
 * <br> 
 * <br> { blue, red, green}
 * <br> {(car, blue), (boat, red)}
 * <br> 
 * <br> It converts these into a set of elements based on the strings 
 * <br> separated by commas.
 * <br>
 * <br> For instance the first List above becomes the Vector [ blue, red, green]
 * <br>
 *
 */
 
public class List
{
     
//----------------------------------------------------------------------
// CONSTANTS
//----------------------------------------------------------------------

   public static final String OPEN_BRACKET	 = "(";
   public static final String CLOSE_BRACKET	 = ")";

//----------------------------------------------------------------------
// STATIC VARIABLES
//----------------------------------------------------------------------
		
//----------------------------------------------------------------------
// INSTANCE VARIABLES
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// CONSTRUCTOR
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// PRIVATE METHODS
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// PUBLIC METHODS
//----------------------------------------------------------------------

   /**
   * <br> Converts a String in a List format to a Vector of its elements.
   * <br> { blue, red, green} -> Vector [ blue, red, green]
   */
   public static Vector parseList(String str)
	 {
	 	  Vector value = new Vector();
	 	  
	 	  // braces "()" are present. Implies a List of tuples as illustrated below
	 	  // {(car, blue), (house, green)}
	 	  if(str.indexOf("(") != -1)
	 	  {
	 	     int start = str.indexOf(OPEN_BRACKET);
	 	     int finish = str.indexOf(CLOSE_BRACKET) + 1;
	 	     int end = str.lastIndexOf(CLOSE_BRACKET) + 1;
	 	     
	 	     // while there is another open bracket
	 	     while(( (str.indexOf(OPEN_BRACKET, start)) != -1) && (start != -1))	
	 	  	 {
	 	  	 	  String next = str.substring(start, finish);
	 	  	 	  next = next.trim();
	 	  	 	  value.addElement(next);
	 	  	 	  
	 	  	 	  // System.out.println("next: " + next);
	 	  	 	
	 	  	 	  // start just after the end of the last bracket	 	  	 	  
	 	  	 	  // the value of "start" will be "-1" if there are no more open brackets.
	 	  	 	  start = str.indexOf(OPEN_BRACKET, finish);
	 	  	 	  finish = str.indexOf(CLOSE_BRACKET, finish) +1;
	 	  	 	  
	 	  	 } // while
	 	  	
	 	  } // if
	 	  else
	 	  {
	 	  	 // List represents an ordinary list: {car, bike, boat}
	 	  	
		 	   // trim off curly brackets if present
		 	   if(str.indexOf("{") != -1)
		 	   {
		 	      str = str.substring(str.indexOf("{") + 1, str.indexOf("}"));
		 	   }
		 	
		 	   // split up String using commas as delimiters.
		     StringTokenizer st = new StringTokenizer(str, ",");
		 	  
	       while (st.hasMoreTokens()) 
	       {
	          String next = st.nextToken();  
	          next = next.trim();  
	          value.addElement(new String(next));
	       }
	       
	 	  } // else 
      
      return value; 	
	 	
	 } // parseList(String)
	 	
	 	
	 /**
	 * <br> Converts a Vector into a List represented as a String.
	 * <br> Vector [ blue, red, green] -> { blue, red, green}
	 */
	 public static String toString(Vector v)
	 {
	 	  StringBuffer strbuff = new StringBuffer();
	 	  
	 	  for(Enumeration e = v.elements(); e.hasMoreElements(); )
	 	  {
	 	     strbuff.append((String) e.nextElement());
	 	     
	 	     if(e.hasMoreElements())
	 	  	 {
	 	  	    strbuff.append(", ");
	 	  	 }
	 	  }
	 	  
	 	  return "{ " + strbuff + " } ";
	 	
	 } // toString(Vector v)
	 	
	
//----------------------------------------------------------------------
// MAIN/TESTING METHOD
//----------------------------------------------------------------------

   /**
   * <br>
   */
   public static void main(String args[])
	 {
	    System.out.println();
	    System.out.println("Testing List class");
	    
	    System.out.println("Creating new Vector of values");
	    Vector v = new Vector();
	    v.addElement("blue");
	    v.addElement("red");
	    v.addElement("green");
	    
	    System.out.println("Converting vector to a String List representation");
	    System.out.println("List: " +  toString(v));
	    System.out.println();
	    
	    String str = "{car, bike, boat}";
	    System.out.println("Testing creating a Vector with String: " + str);
	    System.out.println("Values contained in Vector: " + parseList(str));
	    System.out.println();
	    
	    str = "{(car, blue), (boat, red)}";
	    System.out.println("Testing creating a Vector with String: " + str);
	    System.out.println("Values contained in Vector: " + parseList(str));
	    System.out.println();
	    
	    str = "{(car, blue, 1000), (boat, red, 2000), (house, purple, 7000)}";
	    System.out.println("Testing creating a Vector with String: " + str);
	    System.out.println("Values contained in Vector: " + parseList(str));
	    System.out.println();
      System.out.println();
	
	 	
	 } // main
  
	
} // List

	
