//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files ("the
//     Programs") are Copyright (c) 1999 - 2000 Nortel Networks Corporation, 
//     8200 Dixie Road, Suite 100, Brampton, Ontario, Canada L6R 5P6.  
//     All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out in
//     the Nortel Networks FIPA-OS Public License, a copy of which can be found
//     in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version can
//     also be found at http://fipa-os.sourceforge.net/. By downloading the
//     files containing the Programs you accept the terms and  conditions of the
//     Public License. You do not have to accept these terms and conditions, but
//     unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.skill.constraint.ccl.variable;

//----------------------------------------------------------------------

import fipaos.ont.fipa.FIPACONSTANTS;
import fipaos.ont.BASETOKENS;
import fipaos.ont.fipa.fipaman.FIPAMANCONSTANTS;

import java.util.*;

//----------------------------------------------------------------------

/**
 * 
 * <br> A tuple specifies sets of allowed combinations.
 * <br> An example of a Tuple range would be:
 * <br> (Savoy, London)
 * <br>
 * <br> The values held in a Tuple may only be Strings.
 * <br> Hence integers must be represented as follows: "1".
 *
 */
public class Tuple
{
	
     
//----------------------------------------------------------------------
// CONSTANTS
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// STATIC VARIABLES
//----------------------------------------------------------------------
		
//----------------------------------------------------------------------
// INSTANCE VARIABLES
//----------------------------------------------------------------------

   /**
   * <br> Vector to hold the values in this tuple.
   * <br> This Vector should only ever contain String objects.
   */
   private Vector _values = null;

//----------------------------------------------------------------------
// CONSTRUCTOR
//----------------------------------------------------------------------

   /**
   *  Constructs an empty Tuple.
   */
   public Tuple()
	 {
	    _values = new Vector();
	 }
	 
	 /**
	 * <br> Constructs a new Tuple and populates it with the values passed
	 * <br> in the String. This should be a String representation of a tuple.
	 * <br> This String may take either of the following forms:
	 * <br> 
	 * <br> (house, car, boat)
	 * <br>  house, car, boat
	 * <br>
	 * <br> Hence brackets may or may not be present.
	 */
	 public Tuple(String str)
	 {
	 	  // System.out.println("str: " + str);
	 	
	 	  _values = new Vector();
	 	  
	 	  // Trim off the brackets
	 	  if(str.indexOf("(") != -1)
	 	  {
	 	     str = str.substring(str.indexOf("(") + 1, str.indexOf(")"));	
	 	  	 // System.out.println("substring is: " + str);
	 	  }
	 	  
	 	  StringTokenizer st = new StringTokenizer(str, ",");
	 	  
      while (st.hasMoreTokens()) 
      {
         String next = st.nextToken();  
         next = next.trim();  
         _values.addElement(new String(next));
      }
      
      // System.out.println("Values: " + _values);
	 	
	 } // Tuple(String str)
	 	
	 	
	 /**
	 * <br> Constructs a new Tuple and populates it with the values passed
	 * <br> in the Vector.
	 * <br> The Vector must contain String objects.
	 */
	 public Tuple(Vector v)
	 {
	 	  Vector copy = new Vector();
	 	
	    for(Enumeration e = v.elements(); e.hasMoreElements(); )
	 	  {
	 	  	 String new_string = new String((String)e.nextElement()); 
	 	     copy.addElement(new_string);
	 	  }	
	 	  
	 	  _values = copy;
	 	
	 } // Tuple(Vector v)
	 	
	 	
	 /**
	 * <br> Adds another Value to the tuple.
	 */
	 public void addValue(String str)
	 {
	    _values.addElement(new String(str));
	 }
	 

//----------------------------------------------------------------------
// PRIVATE METHODS
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// PUBLIC METHODS
//----------------------------------------------------------------------

   /**
   *  Returns an exact copy of this Tuple object.
   */	
   public Tuple copy()
	 {
	    return new Tuple(_values); 	
	 } 
	 	
	 /**
	 *  Returns a String representation of this Tuple.
	 */
	 public String toString()
	 {
	 	  StringBuffer stbuff = new StringBuffer();
	 	  
	 	  for(Enumeration e = _values.elements(); e.hasMoreElements(); )
	 	  {
	 	     stbuff.append((String) e.nextElement());
	 	     
	 	     if(e.hasMoreElements())
	 	  	 {
	 	  	 	  stbuff.append(", ");
	 	  	 }
	 	  }
	 	  
	 	  String value = "( " + stbuff + " ) ";
	 	
	 	  return value;
	 	  
	 } // toString()
	 	
	 	
	 /**
	 *  Returns an Enumeration of the elements held in this Tuple.
	 */
	 public Enumeration elements()
	 {
	    Vector v = new Vector();
	    
	    for(Enumeration e = _values.elements(); e.hasMoreElements(); )
	 	  {
	 	     v.addElement(new String((String) e.nextElement()));  
	 	  }	
	 	  
	 	  return v.elements();
	 	  
	 } // elements() 
	
//----------------------------------------------------------------------
// MAIN/TESTING METHOD
//----------------------------------------------------------------------

   /**
   *  Used to test the class.
   */
   public static void main(String args[])
	 {
	 	  Tuple test = new Tuple("(moo, cow)");
	 	  Tuple test1 = new Tuple(" (moo, cow) ");
	 	  Tuple test2 = new Tuple(" moo, cow ");
	 	  Tuple test3 = new Tuple(" (  moo   , cow  ) ");
	 	  
	 	  System.out.println();
	 	  System.out.println("Values held in test3");
	 	  System.out.println(test3);
	 	  
	 	  System.out.println();
	 	  System.out.println("Values held in copy of test3");
	 	  System.out.println(test3.copy());
	 	  
	 	  System.out.println("Adding a value to test3");
	 	  test3.addValue("horse");
	 	  
	 	  System.out.println();
	 	  System.out.println("Values held in test3");
	 	  System.out.println(test3);
	 	  
	 	  Vector v = new Vector();
	 	  
	 	  v.addElement(new String("blue"));
	 	  v.addElement(new String("red"));
	 	  
	 	  System.out.println();
	 	  System.out.println("Creating new Tuple using Vector");
	 	  Tuple test4 = new Tuple(v);
	 	  
	 	  System.out.println("Values held in test4:");
	 	  System.out.println(test4);
	 	  
	 	  System.out.println();
	 	  System.out.println("Creating an Enumeration of values in test4");
	 	  System.out.println("Printing out values retrieved:");
	 	  
	 	  for(Enumeration e = test4.elements(); e.hasMoreElements(); )
	 	  {
	 	     System.out.println("value: " + e.nextElement());
	 	  }
	 	  
	 	  System.out.println();
	 	  System.out.println("Creating new Tuple, test5");
	 	  Tuple test5 = new Tuple();
	 	  
	 	  test5.addValue("tree");
	 	  test5.addValue("bush");
	 	  
	 	  System.out.println("Values held in test5: ");
	 	  System.out.println(test5);
	 	  
	 	
	 } // main
  
	
} // Tuple

	
