//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.skill.db;

//----------------------------------------------------------------------

import fipaos.agent.profile.*;
import fipaos.agent.conversation.content.*;
import fipaos.util.*;
import fipaos.agent.profile.*;
import java.lang.reflect.*;

/**
 * This class provides a static method which will provide a
 * objects which implement the Database interface.
 *
 */
public class DatabaseFactory
{
     
//----------------------------------------------------------------------
// CONSTANTS
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// STATIC VARIABLES
//----------------------------------------------------------------------
		
//----------------------------------------------------------------------
// INSTANCE VARIABLES
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// CONSTRUCTOR
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// PUBLIC METHODS
//----------------------------------------------------------------------
	
	/** Method to dynamically load a database object which represents the named
	 * database, of the type specified by the given profile
	 * @return A Database object which will allow the named database to be modified
	 * @param name The name of the database to be accessed
	 * @param db_profile A profile describing properties of the database to be created
	 */
	public static Database getDatabase( String name, DatabaseProfile db_profile )
	{
		// Constructor signature definition, and parameter definition
		Class sig[] = { String.class, DatabaseProfile.class };
		
		Object params[] = { name, db_profile };
		
		try
		{			
			// Get a Class object for the required class
			Class db_class;
			
			// get the class name from the profile type field
			String class_name = db_profile.getDatabaseType();
			
			try
			{
				// Try the class_name as it is 
				db_class = Class.forName( class_name );
			}
			catch ( ClassNotFoundException e )
			{
				// Otherwise try making the assumption that the name is just the class name
				db_class = Class.forName( "fipaos.skill.db." + class_name.toLowerCase() + "." + class_name );	
			}

			// Check that the class given implements fipaos.skill.db.Database
			if( !fipaos.skill.db.Database.class.isAssignableFrom( db_class ) )
			{
				return null;
			}
				
			try
			{
				// Get the required constructor
				Constructor constructor = db_class.getConstructor( sig );
				
				// Create an instance of the Database object given the
				// database name, and return it
				return (Database) constructor.newInstance( params );
			}
			catch ( NoSuchMethodException e )
			{
				// If the default constructor is not defined, just create a new
				// instance of the object
				return (Database) db_class.newInstance();
			}
		}
		catch ( InvocationTargetException e )
		{
			DIAGNOSTICS.println( "Error when creating database", e.getTargetException(), DIAGNOSTICS.LEVEL_MAX );
		}
		catch ( Throwable t )
		{
			DIAGNOSTICS.println( "Error when creating database", t, DIAGNOSTICS.LEVEL_MAX );
		}
		
		// If we reach here, then something must of gone wrong!  Return null.
		return null;
	}
		
//----------------------------------------------------------------------
// PRIVATE METHODS
//----------------------------------------------------------------------

}
