//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files ("the
//     Programs") are Copyright (c) 1999 - 2000 Nortel Networks Corporation, 
//     8200 Dixie Road, Suite 100, Brampton, Ontario, Canada L6R 5P6.  
//     All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out in
//     the Nortel Networks FIPA-OS Public License, a copy of which can be found
//     in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version can
//     also be found at http://fipa-os.sourceforge.net/. By downloading the
//     files containing the Programs you accept the terms and  conditions of the
//     Public License. You do not have to accept these terms and conditions, but
//     unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

	/** 
	 * JessAgent provides an agent interface to Sandia National Laboratories' 
	 * Java Expert System Shell (JESS). This agent is not used as stand-alone, 
	 * but use it by extending it. The extended agent (MyAgent extends JessAgent)
	 * can then access the JessAgent's inferencing capabilities.
	 * 
	 */

package fipaos.skill.jess;

import java.util.Vector;
import java.util.Map;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.Set;
import java.util.Iterator;
import java.util.ListIterator;
import java.io.ByteArrayOutputStream;
import java.io.PrintWriter;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.File;

import fipaos.agent.FIPAOSAgent;

import fipaos.util.DIAGNOSTICS;

import fipaos.skill.jess.JESSAGENTCONSTANTS;

import jess.*;


public class JessAgent extends FIPAOSAgent
{
//----------------------------------------------------------------------
// STATIC VARIABLES
//----------------------------------------------------------------------	
      
//----------------------------------------------------------------------
// GLOBAL VARIABLES
//----------------------------------------------------------------------
	
//----------------------------------------------------------------------
// INSTANCE VARIABLES
//----------------------------------------------------------------------	

	private int DEBUG = DIAGNOSTICS.LEVEL_MAX;

	/** 
	 * inference engine
	 */	
	private Rete						_rete;

//----------------------------------------------------------------------
// CONSTRUCTOR
//----------------------------------------------------------------------	
	public JessAgent(String platform_profile, String name, String owner)
	{
		super(platform_profile, name, owner);
	 
		// set up jess	
		// Create a Jess engine (tested with jess60a2)
		_rete = new Rete();
		
		//remove jess default routers
		_rete.removeOutputRouter("WSTDOUT");
		_rete.removeOutputRouter("t");
		
		//load initalise KB - jess needs it
		try
		{
			_rete.executeCommand("(batch scriptlib.clp)");
		}
		catch (JessException je)
		{
			DIAGNOSTICS.println("Can't locate scriptlib.clp", this, DIAGNOSTICS.LEVEL_MAX);
			je.printStackTrace();
		}
	
		// Inner class that implements jess packages interface
	  Package pack = new Package();
	  pack.add(_rete);
	}	
	

//----------------------------------------------------------------------
// PUBLIC METHODS
//----------------------------------------------------------------------	

//----------------------------------------------------------------------
// public METHODS (JESS METHODS)
//----------------------------------------------------------------------
	/**
	* Resets the knowlege base.
	* Always call this function before a run of the knowlege base.
	*/
	protected void reset()
	{
		try
		{
			_rete.reset();
		}
		catch( JessException je)
		{
			DIAGNOSTICS.println("Engine failed to RESET: " + je, this, DIAGNOSTICS.LEVEL_MAX);
		}
 		catch (Throwable t)
 		{
 			DIAGNOSTICS.println( "Engine failed to RESET: " + t, this, DIAGNOSTICS.LEVEL_MAX);
 		}
		
	}
//----------------------------------------------------------------------
	
	/**
	* Clears the knowlege base from all facts and rules.
	*/
	protected void clearKB()
	{
		try
		{
			_rete.clear();
		}
		catch( JessException je)
		{
			DIAGNOSTICS.println("Engine failed to CLEAR KB: " + je, this, DIAGNOSTICS.LEVEL_MAX);
		}
 		catch (Throwable t)
 		{
 			DIAGNOSTICS.println( "Engine failed to CLEAR KB: " + t, this, DIAGNOSTICS.LEVEL_MAX);
 		}
		
	}
//----------------------------------------------------------------------
 	
	/**
	 * Runs the engine of knowlege base.
	 *
	 * @return     the ouput obtained from the engine as a result of running the engine
	 */
	protected String runEngine()
	{
	
		int val = 0;
		StringBuffer jess_output = new StringBuffer();
		
		try
		{
			val = _rete.run();
			DIAGNOSTICS.println("run value " + val, this, DEBUG);
		}
		catch (JessException je)
		{
			DIAGNOSTICS.println("ERROR In runEngine method: " + je, this, DIAGNOSTICS.LEVEL_MAX);
		}
 		catch (Throwable t)
 		{
 			DIAGNOSTICS.println( "ERROR In runEngine method: " + t, this, DIAGNOSTICS.LEVEL_MAX);
 		}
		 
		 return jess_output.toString();		
	}

	
	//----------------------------------------------------------------------

	/**
	 * List all the activations in Rete.
	 *
	 * @return     enumeration of the activations
	 */
	protected Enumeration listActivations()
	{
		Enumeration activation_list =	_rete.listActivations();	
		return activation_list;		
		
	}	
	
	//----------------------------------------------------------------------

	/**
	 * This method adds a template to the current Rete.
	 *
	 * @param template the template to be added
	 * @return was the operation successful    
	 */
 
	 protected synchronized boolean addTemplate(Deftemplate template)
	 {
	 		try
	 		{
			 	_rete.addDeftemplate(template);
	 		}
	 		catch (JessException je)
	 		{
	 			DIAGNOSTICS.println( "Adding the deftemplate was not successful: " + je, this, DIAGNOSTICS.LEVEL_4);
	 			return false;
	 		}
	 		catch (Throwable t)
	 		{
	 			DIAGNOSTICS.println( "Adding the deftemplate was not successful: " + t, this, DIAGNOSTICS.LEVEL_4);
	 			return false;
	 		}
	 		
			return true;
	 }
	

	/**
	 * This method adds deffacts to the Rete.
	 *
	 * @param   facts  the deffacts to be added
	 * @return     was the operation successful
	 */
	 protected synchronized boolean addFacts(Deffacts facts)
	 {
	 		try
	 		{
			 	_rete.addDeffacts(facts);
	 		}
	 		catch (JessException je)
	 		{
	 			DIAGNOSTICS.println( "Adding the deffacts was not successful: " + je, this, DIAGNOSTICS.LEVEL_4);
	 			return false;
	 		}
	 		catch (Throwable t)
	 		{
	 			DIAGNOSTICS.println( "Adding the deffacts was not successful: " + t, this, DIAGNOSTICS.LEVEL_4);
	 			return false;
	 		}
	 		
			return true;
	 }
	


	/**
	 * Asserts a fact to the Rete.
	 *
	 * @param   fact  fact to be asserted
	 * @return     was the operation successful
	 */
	 protected synchronized boolean assert(Fact fact)
	 {
	 		try
	 		{
			 	_rete.assert(fact);
	 		}
	 		catch (JessException je)
	 		{
	 			DIAGNOSTICS.println( "Asserting of '" + fact + "' (jess.Fact) was not successful: " + je, this, DIAGNOSTICS.LEVEL_4);
	 			return false;
	 		}
	 		catch (Throwable t)
	 		{
	 			DIAGNOSTICS.println( "Asserting of '" + fact + "' (jess.Fact) was not successful: " + t, this, DIAGNOSTICS.LEVEL_4);
	 			return false;
	 		}
	 		
			return true;
	 }
	


	/**
	 * Asserts a fact as a string to the Rete.
	 *
	 * @param   fact  fact to be asserted
	 * @return     was the operation successful
	 */
	 protected synchronized boolean assertString(String fact)
	 {
	 		try
	 		{
			 	_rete.assertString(fact);
	 		}
	 		catch (JessException je)
	 		{
	 			DIAGNOSTICS.println( "Asserting of '" + fact + "' (String) was not successful: " + je, this, DIAGNOSTICS.LEVEL_4);
	 			return false;
	 		}
	 		catch (Throwable t)
	 		{
	 			DIAGNOSTICS.println( "Asserting of '" + fact + "' (String) was not successful: " + t, this, DIAGNOSTICS.LEVEL_4);
	 			return false;
	 		}
	 		
			return true;
	 }
	

	/**
	 * Execute a command in JESS.
	 *
	 * @param   content  command to be executed
	 * @return     the value this run produces
	 * @exception   JessException  if something goes wrong with JESS
	 */
	protected synchronized Value engineExecuteCommand(String content) throws JessException
	{		
		Value result = null;
		String command = "(" + content + ")";
		
		DIAGNOSTICS.println("the command given to Jess is " + command, this, DIAGNOSTICS.LEVEL_3);
		
		// the return value from executing the command	
		result = _rete.executeCommand( command );

		DIAGNOSTICS.println("Jess result = " + result, this, DIAGNOSTICS.LEVEL_2);

		return result;			
	}	
	

	/**
	 * This gets the current global context of the current Rete.
	 *
	 * @return the context    
	 */
 
	 protected synchronized Context getGlobalContext()
	 {
			return _rete.getGlobalContext();
	 }
	
//----------------------------------------------------------------------
// PRIVATE METHODS (STRING MANIPULATION USED WITH LISPLIKE SYNTAXES)
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// INNER CLASSES
//----------------------------------------------------------------------
 
 private class Package implements jess.Userpackage
    {
      // The name method returns the name by which the function will appear in Jess code.
       public void add(Rete engine) 
      	{ 
      	
      			//jess50b1
      			String [] packages =  {"jess.StringFunctions",
										"jess.PredFunctions",
										"jess.MultiFunctions",
										"jess.MiscFunctions",
										"jess.MathFunctions",
										"jess.ReflectFunctions",
										"jess.ViewFunctions" };		
										
															
      	
      			// load all packages
 	
				  	for(int i = 0; i < packages.length; i++)
				  	{
				  		try 
				  		{
				  			engine.addUserpackage((Userpackage) Class.forName(packages[i]).newInstance());
				  		}
				  		catch(Throwable t) 
				  		{
				  			DIAGNOSTICS.println("package not present " + t, this, DIAGNOSTICS.LEVEL_2);
				  		}
				  	}      	
      	 }
    }
}	
