//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files ("the
//     Programs") are Copyright (c) 1999 - 2000 Nortel Networks Corporation, 
//     8200 Dixie Road, Suite 100, Brampton, Ontario, Canada L6R 5P6.  
//     All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out in
//     the Nortel Networks FIPA-OS Public License, a copy of which can be found
//     in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version can
//     also be found at http://fipa-os.sourceforge.net/. By downloading the
//     files containing the Programs you accept the terms and  conditions of the
//     Public License. You do not have to accept these terms and conditions, but
//     unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.skill.wrapper;

//----------------------------------------------------------------------

/**
 * This class is used as a central repository for all tokens used by
 * FIPA wrapper and ARB agents.
 * Agents wishing to use any wrapper token constants should use this file
 * and should not define constants locally.
 * <br>The standard naming for these constants is as follows:
 * <p><ul>
 * <li>Constants names are capitalised versions of the strings they represent
 * <li>Space ( ) and dash (-) characters are replaced by underscores (_)
 * <li>Colon (:) characters are omitted from constants names
 * <li>Constants do not include any whitespace at the start or then end of the string
 *     unless expicitly mentioned in the constants name.
 * </ul>
 * <br>
 * <br>
 */
public class WRAPPERCONSTANTS
{
     
//----------------------------------------------------------------------
// CONSTANTS
//----------------------------------------------------------------------

	// Agent type
	public static final String FIPA_WRAPPER						= "fipa-wrapper";
	public static final String FIPA_ARB								= "fipa-arb";
	
	// ARB commands
	public static final String REGISTER_SOFTWARE			= "register-software";
	public static final String DE_REGISTER_SOFTWARE		= "de-register-software";
	public static final String DEREGISTER_SOFTWARE		= DE_REGISTER_SOFTWARE;
	public static final String MODIFY_DESCRIPTION			= "modify-description";
	public static final String REGISTERED							= "registered";
	public static final String MEMBER									= "member";
	
	// Wrapper commands
	public static final String INIT										= "init";
	public static final String INVOKE									= "invoke";	
	public static final String CLOSE									= "close";
	public static final String STORE									= "store";
	public static final String RESTORE								= "restore";
	public static final String SOFTWARE_SUBSCRIBE			= "software-subscribe";
	public static final String SOFTWARE_UNSUBSCRIBE		= "software-unsubscribe";
	public static final String SUSPEND								= "suspend";
	public static final String RESUME									= "resume";
	public static final String ACHIEVE								= "achieve";
	public static final String PARAMETER							= "parameter";
	public static final String SUBSCRIBED							= "subscribed";
	public static final String OPERATION							= "operation";
	public static final String EXCEEDED								= "exceeded";
	
	// Service description stuff
	public static final String SERVICE_DESCRIPTION		= ":service-description";
	public static final String SERVICE_TYPE						= ":service-type";
	public static final String SERVICE_ONTOLOGY				= ":service-ontology";
	public static final String FIXED_PROPERTIES				= ":fixed-properties";
	public static final String NEGOTIABLE_PROPERTIES	= ":negotiable-properties";
	public static final String COMMUNICATION_PROPERTIES	= ":communication-properties";
	public static final String NET_PROTOCOL						= ":net-protocol";
	public static final String ADDRESS								= ":address";
	public static final String MESSAGE_BODY_FORMAT		= ":message-body-format";
	public static final String MESSAGE_BODY_ENCODING	= ":message-body-encoding";
	public static final String SERVICE_INSTANCE_ID		= ":service-instance-id";
	public static final String STATE_ID								= ":state-id";
	public static final String SERVICE_NAME						= ":service-name";
	public static final String EVENT_NAME							= ":event-name";
	
	// Error messages and other stuff
	public static final String NOT_VALID_DESCRIPTION	= "not-valid-description";
	public static final String SERVICE_NAME_IN_USE		= "service-name-in-use";
	public static final String UNAUTHORISED_REQUEST		= "unauthorised-request";
	public static final String NOT_VALID_SERVICE_NAME	= "not-valid-service-name";
	public static final String UNAUTHORISED						= "unauthorised";
	public static final String MAXIMUM_NUMBER_OF_INSTANCES_EXCEEDED = "maximum-number-of-instances-exceeded";
	public static final String SERVICE_IN_USE					= "service-in-use";
	public static final String SERVICE_UNREACHABLE		= "service-unreachable";
	public static final String NOT_VALID_SERVICE_INSTANCE_ID = "not-valid-service-instance-id";
	public static final String NOT_ENOUGH_RESOURCES		= "not-enough-resources";
	public static final String NOT_VALID_STATE_ID			= "not-valid-state-id";
	public static final String SERVICE_SUSPENDED			= "service-suspended";
	public static final String UNKNOWN_EVENT_NAME			= "unknown-event-name";
	public static final String SERVICE_ALREADY_SUSPENDED = "service-already-suspended";
	public static final String SERVICE_NOT_SUSPENDED	= "service-not-suspended";
	public static final String NOT_VALID_PREDICATE		= "not-valid-predicate";
	public static final String NOT_VALID_OPERATION		= "not-valid-operation";
	public static final String NOT_STORABLE						= "not-storable";
	public static final String NOT_SUSPENDABLE				= "not-suspendable";
	public static final String NOT_RESUMABLE					= "not-resumable";
	
	
} // End WRAPPERCONSTANTS

	
