//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.test.agent;

import java.util.*;

import junit.framework.*;

import fipaos.agent.FIPAOSAgent;
import fipaos.agent.task.Task;
import fipaos.agent.conversation.*;
import fipaos.ont.fipa.ACL;
import fipaos.ont.fipa.fipaman.*;

/**
 * Test-harness for MicroFIPAOSAgent messaging
 */
public class AgentMessagingTest extends TestCase
{
	private final static int MSG_COUNT = 100;
	
	private TestAgent _agent1,_agent2;
	
	/**
	 * Default constructor
	 */
	public AgentMessagingTest(String name)
	{
		super( name );
	}
	
	/**
	 * Setup the tests
	 */
	protected void setUp()
	{
		_agent1 = new TestAgent( "agent1" );
		_agent2 = new TestAgent( "agent2" );
	}
	
	/**
	 * Shutdown the tests
	 */
	protected void tearDown()
	{
		if( _agent1 != null )
		{
			_agent1.shutdown();
			_agent1 = null;
		}
		if( _agent2 != null )
		{
			_agent2.shutdown();
			_agent2 = null;
		}
	}
	
	/**
	 * Overrides method in parent to enable running of our tests
	 */
	protected void runTest() throws Throwable
	{
		messageTest1();
	}
	
	/**
	 * Checks that messages are propogated to/from two agents correctly
	 */
	private void messageTest1() throws Throwable
	{
		// Send 1000 messages from one Agent to the other & vice-versa
		
		// Reset counters
		_agent1._rcvd = 0;
		_agent2._rcvd = 0;
		
		// Start sending messages
		for( int i = 0; i < MSG_COUNT; i++ )
		{
			_agent1.sendMessage( "agent2@" + AgentTestFramework.getPlatformProfile().getHAPName() );
			_agent2.sendMessage( "agent1@" + AgentTestFramework.getPlatformProfile().getHAPName() );
		}
		
		// Wait for a while - if no progress is being made indicate a failure
		int last_reading1 = 0;
		int last_reading2 = 0;
		
		do
		{
			last_reading1 = _agent1._rcvd;
			last_reading2 = _agent2._rcvd;
		
			// Sleep for 1 second
			Thread.sleep( 1000 );
		}
		while( last_reading1 < _agent1._rcvd || last_reading2 < _agent2._rcvd );
		
		assert( "Agent1 received " + _agent1._rcvd + "/" + MSG_COUNT, _agent1._rcvd == MSG_COUNT );
		assert( "Agent1 received " + _agent2._rcvd + "/" + MSG_COUNT, _agent2._rcvd == MSG_COUNT );
	}
	
	/**
	 * Simple agent to send messages & count the number received
	 */
	public class TestAgent extends FIPAOSAgent
	{
		public int _rcvd = 0;
		
		public TestAgent( String name )
		{
			super( AgentTestFramework.getPlatformProfile(), AgentTestFramework.getAgentProfile(),
							name, "MicroFIPAOS_Test", true );
			setListenerTask( new IdleTask() );
		}
		
		public void sendMessage( String target )
		{
			AgentID aid = new AgentID();
			aid.setName( target );
			
			ACL acl = new ACL();
			acl.setSenderAID( getAID() );
			acl.setReceiverAID( aid );
			acl.setPerformative( "inform" );
			acl.setConversationID( getAID().getName() );
			forward( acl );
		}
		
		public class IdleTask extends fipaos.agent.task.Task
		{
			public void handleInform( fipaos.agent.conversation.Conversation conv )
			{
				_rcvd++;
			}
		}
	}
}
