//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.test.agent;

import fipaos.mts.*;
import fipaos.util.*;
import fipaos.ont.fipa.fipaman.AgentID;

import java.util.*;

/**
 * MTP implementation which allows an Agent to be tested at the ACL level of
 * abstraction
 */
public class TestingMTP extends fipaos.mts.InternalMTPBase
{
	/**
	 * NS Map ;)
	 */
	public static Map __ns = new TreeMap( fipaos.util.Comparators.STRING_COMPARE );
	
	/**
	 * Dummy object for observer to use
	 */
	public static Object __dummy = new Object();
	
	/**
	 * URL for tester
	 */
	public static URL __dummy_url = new URL( "test://localhost/test-harness" );
	
	static
	{
		__ns.put( "test-harness", __dummy );
		__ns.put( __dummy_url, __dummy );
	}
	
	/**
	 * Agent name for this
	 */
	private String _agent_name;
	
	/**
	 * NS URL for the agent
	 */
	private URL _this_url;
	
	/**
	 * Indicates if this is shutdown
	 */
	private boolean _shutdown;
	
	/**
	 * Incoming queue for the tester
	 */
	private static Queue __incoming = new FIFOQueue();
	
	public TestingMTP( String name, URL ns ) throws Exception
	{
		super(name,ns);
		_agent_name = name;
		_this_url = (URL) ns.clone();
		_this_url.setTarget( name );
	}
	
	protected void bind() throws fipaos.mts.InitialisationException
	{
		__ns.put( _this_url, this );
		__ns.put( _name, this );
	}
	
	public java.util.List getAddresses()
	{
		java.util.List l = new java.util.LinkedList();
		l.add( _this_url );
		return l;
	}
	
	public java.util.List getProtocols()
	{
		return new LinkedList( Arrays.asList( new Object[] { "test" } ) );
	}
	
	protected java.lang.Object lookup(fipaos.util.URL uRL) throws fipaos.mts.UnknownTargetException
	{
		if( __ns.containsKey( uRL ) )
		{
			return __ns.get( uRL );
		}
		else
		{
			throw new fipaos.mts.UnknownTargetException( "Target agent unknown by NS" );
		}
	}
	
	protected java.lang.Object lookup(java.lang.String str) throws fipaos.mts.UnknownTargetException
	{
		if( __ns.containsKey( str ) )
		{
			return __ns.get( str );
		}
		else
		{
			throw new fipaos.mts.UnknownTargetException( "Target agent unknown by NS" );
		}
	}
	
	protected void send(java.lang.Object obj,fipaos.mts.Message message) throws fipaos.mts.UnknownTargetException, fipaos.mts.UnknownProtocolException, fipaos.mts.UnableToSendException
	{
		// Check if this is being monitored - if so queue message
		if( obj != __dummy )
		{
			// Type cast
			TestingMTP mtp = (TestingMTP) obj;

			if( !mtp._shutdown )
			{
				mtp.incomingMessage( message );
			}
		}
		else
		{
			__incoming.addItem( message );
		}
	}
	
	public void shutdown()
	{
		_shutdown = true;
		unbind();
	}
	
	protected void unbind()
	{
		__ns.remove( _this_url );
		__ns.remove( _name );
	}
	
	private void incomingMessage( fipaos.mts.Message message )
	{
		super.handleIncoming( message );
	}
	
//------------------------------------------------------------------------------
// STATIC METHODS
//------------------------------------------------------------------------------
	
	/**
	 * Send the given message to the given Agent - please ensure to set AID in ACL and Envelope appropriately
	 */
	public static boolean sendMessage( String name, Message msg )
	{
		TestingMTP target = (TestingMTP) __ns.get( name );
		
		if( target != null )
		{
			return sendMessage( target, msg );
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Send the given message to the given Agent - please ensure to set AID in ACL and Envelope appropriately
	 */
	public static boolean sendMessage( URL url, Message msg )
	{
		TestingMTP target = (TestingMTP) __ns.get( url );
		
		if( target != null )
		{
			return sendMessage( target, msg );
		}
		else
		{
			return false;
		}		
	}
	
	/**
	 * Send the given message to the given Agent (bypasses-incoming queue if Agent
	 * is being monitored)
	 */
	private static boolean sendMessage( TestingMTP mtp, Message msg )
	{
		mtp.handleIncoming( msg );
		return true;
	}
	
	/**
	 * Get the next incoming message from the given Agent
	 */
	public static Message getNextIncomingMessage()
	{
		return (Message) __incoming.getItem();
	}
	
	/**
	 * Get the AID to be used by tester
	 */
	public static AgentID getAID()
	{
		AgentID aid = new AgentID();
		aid.setName( "tester" );
		aid.setAddresses( Arrays.asList( new Object[] { __dummy_url } ) );
		return aid;
	}
}
