//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.test.agent.conversation.standard;

import java.util.*;

import junit.framework.*;

import fipaos.agent.conversation.standard.*;
import fipaos.agent.conversation.standard.protocol.*;
import fipaos.agent.profile.*;
import fipaos.ont.fipa.fipaman.*;
import fipaos.ont.fipa.*;
import fipaos.mts.*;

/**
 * Test-harness for ConversationImpl
 */
public class ConversationTest extends TestCase
{
	private AgentID _agent1,_agent2;
	
	/**
	 * Default constructor
	 */
	public ConversationTest(String name)
	{
		super( name );
	}
	
	/**
	 * Setup the tests
	 */
	protected void setUp()
	{
		_agent1 = new AgentID();
		_agent1.setName( "agent1" );
		_agent2 = new AgentID();
		_agent2.setName( "agent2" );
	}
	
	/**
	 * Overrides method in parent to enable running of our tests
	 */
	protected void runTest() throws Throwable
	{
		regressionTest1();
	}
	
	/**
	 * Checks that clone() maintains ProtocolTree structure
	 */
	private void regressionTest1() throws Throwable
	{
		// Set-up FIPA-Request protocol mappings
		ProtocolProfile pp = new ProtocolProfileImpl();
		ProtocolMapping pm = new ProtocolMappingImpl();
		pm.setProtocolName( FIPACONSTANTS.FIPA_REQUEST );
		pm.setProtocolClass( FIPARequest.class.getName() );
		pp.addProtocolMapping( pm );
		ConversationManagerImpl.setProtocolProfile( pp );
		
		// Create conversation for FIPA-request protocol, and populate with
		// messages - then clone() and see what performatives should come next
		ConversationImpl conv = ConversationImpl.getConversation( FIPACONSTANTS.FIPA_REQUEST, _agent2 );
		
		// Create Request message, and add to Conversation
		ACL acl = new ACL();
		acl.setPerformative( FIPACONSTANTS.REQUEST );
		acl.setSenderAID( _agent1 );
		acl.setReceiverAID( _agent2 );
		acl.setConversationID( conv.getConversationID() );
		acl.setProtocol( FIPACONSTANTS.FIPA_REQUEST );
		conv.addMessage( new Message( acl ) );
		
		// List performatives
		List l = conv.getNextPerformatives();
		
		// Check it contain what we expect
		List expected = Arrays.asList( new Object[] { "refuse", "not-understood", "agree" } );
		assert( "Conversation.getNextPerformatives() doesn't contain all expected performatives: " + l,
						l.containsAll( expected ) );
		assert( "Conversation.getNextPerformatives() contains more than the expected performatives:" + l,
						expected.containsAll( l ) );
		
		
		
		// Duplicate Conversation and try again
		conv = (ConversationImpl) conv.clone();
		
		// List performatives
		l = conv.getNextPerformatives();
		
		// Check it contain what we expect
		expected = Arrays.asList( new Object[] { "refuse", "not-understood", "agree" } );
		assert( "Cloned Conversation.getNextPerformatives() doesn't contain all expected performatives: " + l,
						l.containsAll( expected ) );
		assert( "Cloned Conversation.getNextPerformatives() contains more than the expected performatives:" + l,
						expected.containsAll( l ) );
		
	}
}
