//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.test.mts.service;

import java.util.*;

import junit.framework.*;

import fipaos.parser.acl.bitefficient.Parser;
import fipaos.ont.fipa.*;
import fipaos.ont.fipa.fipaman.*;
import fipaos.mts.*;
import fipaos.mts.service.*;

/**
 * Test-harness for fipaos.mts.service.ParserService
 */
public class ParserServiceTest extends TestCase
{
	public static final String ACL_STRING = "(inform :sender (agent-identifier :name someAgent@here.com :addresses (sequence fipaos-rmi://192.168.0.1/someAgent ) ) :receiver ( set (agent-identifier :name otherAgent@here.com :addresses (sequence fipaos-rmi://192.168.0.1/otherAgent ) ) ) :reply-to ( set (agent-identifier :name someAgent@here.com :addresses (sequence fipaos-rmi://192.168.0.1/someAgent ) ) ) :content some-content :reply-with reply-with-value :in-reply-to in-reply-to-value :conversation-id conversation-id-value )";
	public static final int[] ACL_INT =
	{250, 16,  8,  2,  2, 16,115,111,
	109,101, 65,103,101,110,116, 64,
	104,101,114,101, 46, 99,111,109,
	0,  2, 16,102,105,112, 97,111,
	115, 45,114,109,105, 58, 47, 47,
	49, 57, 50, 46, 49, 54, 56, 46,
	48, 46, 49, 47,115,111,109,101,
	65,103,101,110,116,  0,  1,  1,
	3,  2, 16,111,116,104,101,114,
	65,103,101,110,116, 64,104,101,
	114,101, 46, 99,111,109,  0,  2,
	16,102,105,112, 97,111,115, 45,
	114,109,105, 58, 47, 47, 49, 57,
	50, 46, 49, 54, 56, 46, 48, 46,
	49, 47,111,116,104,101,114, 65,
	103,101,110,116,  0,  1,  1,  1,
	13, 16, 99,111,110,118,101,114,
	115, 97,116,105,111,110, 45,105,
	100, 45,118, 97,108,117,101,  0,
	8,  2, 16,115,111,109,101, 65,
	103,101,110,116, 64,104,101,114,
	101, 46, 99,111,109,  0,  2, 16,
	102,105,112, 97,111,115, 45,114,
	109,105, 58, 47, 47, 49, 57, 50,
	46, 49, 54, 56, 46, 48, 46, 49,
	47,115,111,109,101, 65,103,101,
	110,116,  0,  1,  1,  1,  4, 16,
	115,111,109,101, 45, 99,111,110,
	116,101,110,116,  0,  7, 16,105,
	110, 45,114,101,112,108,121, 45,
	116,111, 45,118, 97,108,117,101,
	0,  5, 16,114,101,112,108,121,
	45,119,105,116,104, 45,118, 97,
	108,117,101,  0,  1, 13, 10, };
	
	private static final byte[] ACL_BYTE;
	
	static
	{
		// Conver int[] to byte[]
		ACL_BYTE = new byte[ ACL_INT.length ];
		for( int i = 0; i < ACL_INT.length; i++ )
		{
			int next = ACL_INT[ i ];
			
			if( next >= 129 )
			{
				ACL_BYTE[ i ] = (byte) ( next - 256 );
			}
			else
			{
				ACL_BYTE[ i ] = (byte) next;
			}
		}
	}
	
	private ParserService _ps;
	private CollectorService _before,_after;
	
	/**
	 * Default constructor
	 */
	public ParserServiceTest(String name)
	{
		super( name );
	}
	
	/**
	 * Setup the tests
	 */
	protected void setUp()
	{
		// Create new ParserService
		_ps = new ParserService();
		
		// Append our dummy services
		_before = new CollectorService();
		_after = new CollectorService();
		_ps.initialise( (PreParserService) _before, (PreParserService) null );		
		_ps.initialise( (PostParserService) null, (PostParserService) _after );
	}
	
	/**
	 * Overrides method in parent to enable running of our tests
	 */
	protected void runTest() throws Throwable
	{
		testGoodOutgoingMessages();
		testGoodIncomingMessages();
		
		// TODO test failure cases
//		testBadOutgoingMessage();
//		testBadIncomingMessage();		
	}
	
	/**
	 * Executed when comparison with existing examples should be performed
	 */
	private void testGoodOutgoingMessages() throws Throwable
	{
		// Put test messages through, then pull them out
		ACL acl = new ACL( ACL_STRING );
		Message msg1 = new Message( acl );
		Message msg2 = new Message( acl );
		msg2.getEnvelope().setACLRepresentation( ACL.FIPA00069 );
		
		// Push messages
		_ps.outgoing( msg1 );
		_ps.outgoing( msg2 );
		
		// Check if messages were converted OK
		Object out1[] = _after.getNextEnvelopeAndByteArray();
		Object out2[] = _after.getNextEnvelopeAndByteArray();
		
		assert( "No output from ParserService!", out1[1] != null );
		ACL acl1 = new ACL( new String( (byte[]) out1[1] ) );
		assert( "Standard ACL encoding hasn't occured as expected",
						acl1.toString().equals( acl.toString() ) );
		
		assert( "No output from ParserService!", out2[1] != null );
		ACL acl2 = fipaos.parser.acl.bitefficient.Parser.parse( (byte[]) out2[1] );
		assert( "BitEfficient ACL wasn't deparsed correctly", acl2 != null );
		assert( "BitEfficient ACL encoding hasn't occured as expected",
						acl2.toString().equals( acl.toString() ) );
	}
	
	/**
	 * Executed when comparison with existing examples should be performed
	 */
	private void testGoodIncomingMessages() throws Throwable
	{
		// Put test messages through, then pull them out
		ACL acl = new ACL( ACL_STRING );
		byte acl1[] = acl.toString().getBytes();
		byte acl2[] = fipaos.parser.acl.bitefficient.Parser.deparse( acl );
		Envelope env1 = new Envelope();
		Envelope env2 = new Envelope();
		env2.setACLRepresentation( ACL.FIPA00069 );
		
		// Push messages
		_ps.incoming( env1, acl1 );
		_ps.incoming( env2, acl2 );
		
		// Check if messages were converted OK
		Message msg1 = _before.getNextMessage();
		Message msg2 = _before.getNextMessage();
		
		assert( "Standard ACL decoding hasn't occured as expected",
		( msg1.getACL().toString().equals( acl.toString() ) ) );
		
		assert( "BitEfficient ACL decoding hasn't occured as expected",
		( msg2.getACL().toString().equals( acl.toString() ) ) );
	}	
}