//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.test.ont.fipa.fipaman;

import java.util.*;

import junit.framework.*;

import fipaos.ont.fipa.fipaman.*;

/**
 * Test-harness for fipaos.ont.fipa.fipaman.AgentManagementAction and its dependants
 */
public class AgentManagementActionTest extends TestCase
{
	public static final String JADE_CONTENT[] = new String[] {
		"(action (agent-identifier :name df@IBM10308:1099/JADE :addresses (sequence ) :resolvers (sequence ) ) (register (df-agent-description :name (agent-identifier :name df@cmddata :addresses (sequence fipaos-rmi://cmddata:3000/df ) :resolvers (sequence ) ) :services (set (service-description :name fipa-df :type fipa-df :ontologies (set ) :languages (set ) :protocols (set ) :properties (set ) ) ) :protocols (set ) :ontologies (set ) :languages (set ) ) ) ) ",
		"(action (agent-identifier :name df@cmddata :addresses (sequence fipaos-rmi://cmddata:3000/df fipaos-rmi://cmddata:3000/acc IOR:000000000000001149444c3a464950412f4d54533a312e30000000000000000100000000000000300001000000000008636d6464617461000700000000000018afabcafe00000002f6c4c59c000000080000000000000000 corbaname://cmddata:4000/acc ) :resolvers (sequence ) ) (search (df-agent-description :services (set ) :protocols (set ) :ontologies (set ) :languages (set ) ) (search-constraints :max-depth 0 :max-results 9 ) ) )"
	};
	
	/**
	 * Default constructor
	 */
	public AgentManagementActionTest(String name)
	{
		super( name );
	}
	
	/**
	 * Setup the tests
	 */
	protected void setUp()
	{
	}
	
	/**
	 * Overrides method in parent to enable running of our tests
	 */
	protected void runTest() throws Throwable
	{
		testOldTests();
		testRegression1();
		testRegression2();
	}
	
	/**
	 * Original tests from AgentManagementAction
	 */
	private void testOldTests() throws Throwable
	{
		String action="(action (agent-identifier :name CameraProxy@foo.com :addresses (sequence iiop://foo.com/acc)) (register (ams-agent-description :name (agent-identifier :name CameraProxy@foo.com :addresses (sequence iiop://foo.com/acc)) :state active )))";
		AgentManagementAction ama;
		ama=new AgentManagementAction(action);

		assert( "Expected action isn't register",
						ama.getAction().equals( "register" ) );
		assert( "Expected AMSAgentDescription not found", 
						ama.getAMSAgentDescription() != null );
		assert( "Requestor not that expected",
						ama.getAMSAgentDescription().getAgentID().getName().equals( "CameraProxy@foo.com" ) );
		assert( "Expected Agent state isn't \"active\"",
						ama.getAMSAgentDescription().getState().equals( "active" ) );
		assert( "Unexpectedly found DFAgentDescription!",
						ama.getDFAgentDescription() == null );
		assert( "Unexpectedly found SearchConstraints!",
						ama.getSearchConstraints() == null );

		
		action= "(action (agent-identifier :name df@phard1f0.europe.nortel.com ) "+
		"(register (df-agent-description :name (agent-identifier :name df2@phard1f0.europe.nortel.com "+
		":addresses (sequence fipaos-rmi://localhost:3000/df2 ) ) "+
		":services (set (service-description :type fipa-df )) " +
		"))) ";
		ama=new AgentManagementAction(action);

		assert( "Expected action type not found!",
						ama.getAction().equals( "register" ) );
		assert( "Unexpectedly found AMSAgentDescription",
						ama.getAMSAgentDescription() == null );
		assert( "Expected DFAgentDescription not found!",
						ama.getDFAgentDescription() != null );
		assert( "Requestor not that expected",
						ama.getDFAgentDescription().getAgentID().getName().equals( "df2@phard1f0.europe.nortel.com" ) );
		assert( "Unexpected SearchConstratins found",
						ama.getSearchConstraints() == null );

		
		action="(action (agent-identifier :name CameraProxy@foo.com :addresses (sequence iiop://foo.com/acc)) (search (df-agent-description :name (agent-identifier :name CameraProxy@foo.com :addresses (sequence iiop://foo.com/acc)) :ontology (set meeting-scheduler) :language (set FIPA-L0 KIF)) (search-constraints :max-depth 2)))";
		ama=new AgentManagementAction(action);

		assert( "Expected action type not found!",
						ama.getAction().equals( "search" ) );
		assert( "Unexpectedly found AMSAgentDescription",
						ama.getAMSAgentDescription() == null );
		assert( "Expected DFAgentDescription not found!",
						ama.getDFAgentDescription() != null );
		assert( "Requestor not that expected",
						ama.getDFAgentDescription().getAgentID().getName().equals( "CameraProxy@foo.com" ) );
		assert( "Expected SearchConstratins not found",
						ama.getSearchConstraints() != null );
		assert( "Expected max-depth value of 2 not found",
						ama.getSearchConstraints().getMaxDepth().intValue() == 2 );
	}

	/**
	 * Tests that :state is actually optional, and doesn't have any side-effects
	 */
	private void testRegression1() throws Throwable
	{
		// De-registration message without state info.
		String dereg = "(action (agent-identifier :name test1 ) (deregister (ams-agent-description :name (agent-identifier :name test2 )))))";
		AgentManagementAction ama = new AgentManagementAction(dereg);
		
		assert( "AMSAgentDescription is null!",
						ama.getAMSAgentDescription() != null );
		assert( "Expected AID in AMSAgentDescription is null",
						ama.getAMSAgentDescription().getAgentID() != null );
		assert( "Expected AID not found",
						ama.getAMSAgentDescription().getAgentID().getName().equals( "test2" ) );
	}
	
	/**
	 * Tests that JADE-style content can be parsed
	 */
	private void testRegression2() throws Throwable
	{
		// Iterate through list and extract messages to test
		for( int i = 0; i < JADE_CONTENT.length; i++ )
		{
			try
			{
				AgentManagementAction ama = new AgentManagementAction( JADE_CONTENT[i] );
			}
			catch( Throwable t )
			{
				assert( "JADE AgentManagement content " + ( i + 1 ) + " failed to be parsed", false );
			}
		}
	}
}