//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.test.ont.fipa.fipaman;

import java.util.*;

import junit.framework.*;

import fipaos.ont.fipa.fipaman.*;

/**
 * Test-harness for fipaos.ont.fipa.fipaman.Envelope package
 */
public class EnvelopeTest extends TestCase
{
	/**
	 * Default constructor
	 */
	public EnvelopeTest(String name)
	{
		super( name );
	}
	
	/**
	 * Setup the tests
	 */
	protected void setUp()
	{
	}
	
	/**
	 * Overrides method in parent to enable running of our tests
	 */
	protected void runTest() throws Throwable
	{
		testGetLatest();
		testUserSlots();
		testRegression1();
	}
	
	/**
	 * Tests that the latest value assigned to a slot is maintained, but previous
	 * values can be retrieved
	 */
	private void testGetLatest() throws Throwable
	{
		// Create an envelope, and add a few values to the same slot
		Envelope env = new Envelope();
		
		// Check if the value changes within a single ACC/MTS
		env.setACLRepresentation( "type1" );
		env.setACLRepresentation( "type2" );
		assert( "Slot value change failed (use of a single Envelope)",
							env.getACLRepresentation().equals( "type2" ) );
		
		// Check if the previous value remains between two ACC's/MTS's
		env.setACLRepresentation( "type1" );
		env.newSubEnvelope();
		env.setACLRepresentation( "type2" );
		assert( "Slot value not correctly changed after new sub-envelope created",
							env.getACLRepresentation().equals( "type2" ) );
		assert( "Old slot value not maintained after new sub-envelope created",
							env.getSubEnvelope( 0 ).getACLRepresentation().equals( "type1" ) );
	}
	
	/**
	 * Tests that users-slots are treated as expected
	 */
	private void testUserSlots() throws Throwable
	{
		// Check that user-slots are distinguished from predefined
		// slots OK
		Envelope env = new Envelope();
		String userslots[] = new String[] { "X-test-1", "X-test-2", "X-test-3" };
		
		for( int i = 0; i < userslots.length; i++ )
		{
			env.setSlotValue( userslots[ i ], "test-value" );
		}
		
		// Set some predefined slots
		env.setPayloadEncoding( "test-payload-enc" );
		env.setPayloadLength( 3 );
		
		// Check that getUserSlots() returns the correct user-slot names & values
		List us = Arrays.asList( userslots );
		Set slots = env.getUserSlots();
		assert( "User-slots expected not found", us.containsAll( slots ) );
		assert( "Extra user-slots found", slots.containsAll( us ) );
	}
	
	/**
	 * Tests that clone() hasn't decoupled the last _sub_envelopes entry with
	 * _current_envelope
	 */
	private void testRegression1() throws Throwable
	{
		// Create new Envelope, then clone, add a value and retrieve it
		Envelope env = new Envelope();
		Envelope env2 = (Envelope) env.clone();
		
		env2.setACLRepresentation( "test1" );
		
		assert( "Envelope._current_envelop is decoupled from last Envelope in " +
		        "Envelope._sub_envelopes!", env2.getACLRepresentation() != null
						&& env2.getACLRepresentation().equals( "test1" ) );
	}
}