//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis,
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.test.parser.acl.bitefficient;

import java.util.*;

import junit.framework.*;

import fipaos.parser.acl.bitefficient.Parser;
import fipaos.ont.fipa.*;

/**
 * Test-harness for fipaos.parser.acl.bitefficient package
 */
public class ParserTest extends TestCase
{
	public static final String ACL_STRING = "(inform :sender (agent-identifier :name someAgent@here.com :addresses (sequence fipaos-rmi://192.168.0.1/someAgent ) ) :receiver ( set (agent-identifier :name otherAgent@here.com :addresses (sequence fipaos-rmi://192.168.0.1/otherAgent ) ) ) :reply-to ( set (agent-identifier :name someAgent@here.com :addresses (sequence fipaos-rmi://192.168.0.1/someAgent ) ) ) :content \"some-content\" :reply-with reply-with-value :in-reply-to in-reply-to-value :conversation-id conversation-id-value )";
	public static final int[] ACL_INT =
	{250, 16,  8,  2,  2, 16,115,111,
	109,101, 65,103,101,110,116, 64,
	104,101,114,101, 46, 99,111,109,
	0,  2, 16,102,105,112, 97,111,
	115, 45,114,109,105, 58, 47, 47,
	49, 57, 50, 46, 49, 54, 56, 46,
	48, 46, 49, 47,115,111,109,101,
	65,103,101,110,116,  0,  1,  1,
	3,  2, 16,111,116,104,101,114,
	65,103,101,110,116, 64,104,101,
	114,101, 46, 99,111,109,  0,  2,
	16,102,105,112, 97,111,115, 45,
	114,109,105, 58, 47, 47, 49, 57,
	50, 46, 49, 54, 56, 46, 48, 46,
	49, 47,111,116,104,101,114, 65,
	103,101,110,116,  0,  1,  1,  1,
	13, 16, 99,111,110,118,101,114,
	115, 97,116,105,111,110, 45,105,
	100, 45,118, 97,108,117,101,  0,
	8,  2, 16,115,111,109,101, 65,
	103,101,110,116, 64,104,101,114,
	101, 46, 99,111,109,  0,  2, 16,
	102,105,112, 97,111,115, 45,114,
	109,105, 58, 47, 47, 49, 57, 50,
	46, 49, 54, 56, 46, 48, 46, 49,
	47,115,111,109,101, 65,103,101,
	110,116,  0,  1,  1,  1,  4, 16,
	115,111,109,101, 45, 99,111,110,
	116,101,110,116,  0,  7, 16,105,
	110, 45,114,101,112,108,121, 45,
	116,111, 45,118, 97,108,117,101,
	0,  5, 16,114,101,112,108,121,
	45,119,105,116,104, 45,118, 97,
	108,117,101,  0,  1, 13, 10 };
	
	private static final byte[] ACL_BYTE;
	
	static
	{
		// Conver int[] to byte[]
		ACL_BYTE = new byte[ ACL_INT.length ];
		for( int i = 0; i < ACL_INT.length; i++ )
		{
			int next = ACL_INT[ i ];
			
			if( next >= 129 )
			{
				ACL_BYTE[ i ] = (byte) ( next - 256 );
			}
			else
			{
				ACL_BYTE[ i ] = (byte) next;
			}
		}
	}
	
	private ACL _acl;
	
	/**
	 * Default constructor
	 */
	public ParserTest( String name )
	{
		super( name );
	}
	
	/**
	 * Setup the tests
	 */
	protected void setUp()
	{
	}
	
	/**
	 * Overrides method in parent to enable running of our tests
	 */
	protected void runTest() throws Throwable
	{
		testParseAndDeparse();
		testComparePredefinedMessage();
	}
	
	/**
	 * Executed when comparison with existing examples should be performed
	 */
	private void testComparePredefinedMessage() throws Throwable
	{
		ACL acl = new ACL( ACL_STRING );
		
		byte acl2[] = Parser.deparse( acl );
		
		assert( "Deparsed ACL doesn't match pre-produced Bit-Efficient ACL (length-wise)",
		acl2 == null || acl2.length != ACL_BYTE.length );
		
		for( int i = 0; i < acl2.length; i++ )
		{
			assert( "Deparsed ACL doesn't match pre-produced Bit-Efficient ACL (content-wise)",
			ACL_BYTE[i] == acl2[i] );
		}
		
		ACL acl3 = Parser.parse( ACL_BYTE );
		
		assert( "Parsed Bit-Efficient ACL doesn't match pre-produced Stringified ACL",
		acl3.toString().equals( ACL_STRING ) );
	}
	
	/**
	 * Executed when Parse then Deparse test should be performed
	 */
	private void testParseAndDeparse() throws Throwable
	{
		ACL acl = new ACL( ACL_STRING );
		
		byte acl2[] = Parser.deparse( acl );
		
		ACL acl3 = Parser.parse( acl2 );
		
		assert( "String->BitEff.->String isn't reflexive!", acl.toString().equals( acl3.toString() ) );
	}
}