//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.tool;

import fipaos.agent.*;
import fipaos.agent.conversation.*;
import fipaos.agent.profile.*;
import fipaos.agent.task.*;
import fipaos.ont.fipa.*;
import fipaos.mts.*;
import fipaos.util.*;

import javax.swing.*;
import java.io.*;

/**
 * Simple test agent for testing interoperability. It has a GUI that controls the actions.
 * <br>
 * <br>
 */
public class IOTestAgent extends FIPAOSAgent
{
	private VisualACLParser _visual = null;

	private IdleTask _idle = null;
	
	/**
	 * IOTestAgent constructor. Normal agent registering, calling VisualACLParser.
	 * Then starts to wait for the button to be pressed.
	 * @param name java.lang.String agent's name
	 * @param agent_owner java.lang.String agent's owner (usually the same)
	 * @param ior True if this agent should use agent@IOR:.. addressing, false otherwise
	 */
	public IOTestAgent( String platform_profile_location, String name, String agent_owner )
	{
		super( platform_profile_location, name, agent_owner );

		// Attempt to set the default L&F for the platform we're running on
		try
		{
			UIManager.setLookAndFeel ( UIManager.getSystemLookAndFeelClassName() );
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "Couldn't set the default Look & Feel for this OS", this,
			DIAGNOSTICS.LEVEL_3 );
		}

		// Setup listener task
		_idle = new IdleTask();
		setListenerTask( _idle );

		// Calculate the location of the examples directory
		PlatformProfile pp = getPlatformProfile();
		File file = new File( pp.getProfileDirectory() );
		
		if( !file.exists() )
		{
			file = new File( "." );
		}
		else
		{
			file = new File( file.getParent(), "examples" );
			
			if( !file.exists() )
			{
				file = new File( "." );
			}
		}
		
		
		// initialisation of the VisualACLParser		
		_visual = new VisualACLParser( this, file );
	}


	/**
	 * Send message on behalf of GUI
	 */ 
	public void send( Message msg )
	{
		// FOrward message via IdleTask
		_idle.send( msg );
	}
	
	
	/**
	 * Shutdown Agent & GUI
	 */
	public void shutdown()
	{
		// Dispose of GUI
		_visual.dispose();
		
		// Close Agent
		super.shutdown();
	}
	
	
	/**
	 * Invoked when GUI should be shown
	 */
	public void activate()
	{
		if( _visual != null )
		{
			_visual.setVisible( true );
		}
	}
	
	
	/**
	 * Inner-class to deal with incoming messages
	 */
	private class IdleTask extends Task
	{
		public void handleOther( Conversation conv )
		{
			// Pass updated conversation to GUI
			_visual.setACLMessage( conv.getACL( conv.getLatestMessageIndex() ) );
			_visual.println( "Received message" );
		}
		
		public void send( Message msg )
		{
			forward( msg );
		}
	}
}
