//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.tool.build;

import fipaos.tool.configurator.ScriptEditor;
import fipaos.tool.configurator.ScriptVars;
import java.io.*;
import java.util.*;

/**
 * Class to update the variables defined in a SetupFIPAOS script
 */
public class SetupFIPAOSConfigurator implements ScriptVars
{
	private static void update( File f, String version ) throws Exception
	{
		ScriptEditor se = new ScriptEditor( f, SETUPFIPAOS_VARS );
		Map m = se.getScriptVariables();
		m.put( SETUPFIPAOS_FIPAOS_VER, version );
		se.setScriptVariables( m );
		se.saveNewConfiguration();
	}
	
	public static void main( String args[] )
	{
		try
		{
			if( args == null || args.length != 2 )
			{
				System.err.println( "Usage: java " + SetupFIPAOSConfigurator.class.getName() +
														" <SetupFIPAOS-file> <version_string>" );
				System.exit( -10 );
			}
			else
			{
				File f = new File( args[0] );
				if( f.exists() )
				{
					update( f, args[1] );
				}
				else
				{
					System.err.println( "Couldn't locate file \"" + args[0] + "\"" );
					System.exit( -10 );
				}
			}
			
			System.exit( 0 );
		}
		catch( Throwable t )
		{
			System.err.println( "An unexpected error occured" );
			t.printStackTrace( System.err );
		}
	}
}
