//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.tool.configurator;

import java.awt.event.*;
import java.awt.*;
import javax.swing.*;

import java.lang.reflect.Method;

import java.util.*;

/*********************************************************
 *
 * DataBindingEditor
 *
 *********************************************************/
public class DataBindingEditor extends JDialog implements ActionListener
{
	
/** The object being edited by this DataBindingEditor
 */
	private Object _to_edit;
	
/** Vector of the get methods on the object <CODE>_to_edit</CODE>
 */
	private Vector _members;
/** Vector of the current values of the fields in the object _to_edit
 */
	private Vector _values;
	
/** Vector of JTextField objects for editing the data members of _to_edit that have primitive types
 */
	private Vector _jtf_editable;
	
/** Vector of JButton objects that will open up DataBindingEditor guis to edit complex children of the _to_edit object being edited
 */
	private Vector	_jb_children;
	
/** JButton to save the latest changes to <CODE>_to_edit</CODE>
 */
	private JButton _jb_save;
	
/** JButton to discard the latest changes to <CODE>_to_edit</CODE>
 */
	private JButton _jb_cancel;
	
	/** Constructor method
	 *
	 * @param parent The parent gui of this editor to block when this gui is made modal
	 * @param to_edit A Databinding compatible object (get/set pairs for each data member)
	 * @param action A String describing the action being performed (add / edit) to display in the editor title bar.
	 */
	public DataBindingEditor(Object parent, Object to_edit, String action)
	{
		
		super ((JFrame)parent, true);
		
		_to_edit = to_edit;
		
		_members = getMembers(_to_edit);
		
		GridBagLayout gridbag 		= new GridBagLayout();
		GridBagConstraints constraints 	= new GridBagConstraints();
		
		constraints.fill 		= GridBagConstraints.BOTH;
		constraints.gridwidth 		= GridBagConstraints.REMAINDER;
		constraints.ipadx 		= 25;
		
		JPanel jp_main = new JPanel(new FlowLayout(FlowLayout.CENTER));
		
		JPanel jp_sub	= new JPanel(gridbag);
		
		Method current_method;
		
		_jtf_editable = new Vector();
		_jb_children = new Vector();
		_values = new Vector();
		
		JTextField jtf_local;
		
		for (int i = 0; i < _members.size(); i++)
		{
			
			current_method = (Method)_members.get(i);
			
			JLabel jl_member_name = new JLabel(current_method.getName().substring(3));
			gridbag.setConstraints(jl_member_name, constraints);
			jp_sub.add(jl_member_name);
			
			Object value = getCurrentValue(current_method);
			
			_values.add(value);
			
			if (returnsSimpleType(current_method))
			{
				// create a textfield to edit this member locally
				if (value == null)
				{
					value = new String();
				}
				
				jtf_local = new JTextField(value.toString());
				_jtf_editable.add(jtf_local);
				gridbag.setConstraints(jtf_local, constraints);
				jp_sub.add(jtf_local);
				
			}
			else if (returnsListType(current_method))
			{
				
				
			}
			else // returns databinding object
			{
				// create a button to open a child editor
				
				
			}
			
			
		}
		
		String object_name = _to_edit.getClass().getName();
		
		if (object_name.indexOf('.') != -1)
		{
			object_name = object_name.substring(object_name.lastIndexOf('.') + 1);
		}
		
		if (object_name.endsWith("Impl"))
		{
			object_name = object_name.substring(0, object_name.length() - 4);
		}
		
		setTitle(action + " " +  object_name);
		setResizable(true);
		
		JPanel jp_buttons = new JPanel(new FlowLayout(FlowLayout.CENTER));
		gridbag.setConstraints(jp_buttons, constraints);
		
		_jb_save = new JButton("Save");
		_jb_save.addActionListener(this);
		gridbag.setConstraints(_jb_save, constraints);
		
		_jb_cancel = new JButton("Cancel");
		_jb_cancel.addActionListener(this);
		gridbag.setConstraints(_jb_cancel, constraints);
		
		jp_buttons.add(_jb_save);
		jp_buttons.add(_jb_cancel);
		
		jp_sub.add(jp_buttons);
		
		jp_main.add(jp_sub);
		
		getContentPane().add(jp_main);
		
		this.pack();
		
		this.centre();
		
		this.setVisible(true);
		
	} // end DataBindingEditor()
	
/** Retrieves the accessor methods of the given Object and returns them in a Vector
 * @param to_edit The Object to get the accessor methods for
 * @return A Vector containing the accessor methods
 */
	private Vector getMembers(Object to_edit)
	{
		
		Class objectClass = to_edit.getClass();
		
		Vector members = new Vector();
		
		Method[] methods = objectClass.getMethods();
		
		String method_name;
		
		for (int i = 0; i < methods.length; i++)
		{
			
			// Get accessor methods, but not the Object.getClass() method
			
			method_name = methods[i].getName();
			
			if ((method_name.startsWith("get")) && (!method_name.equals("getClass")))
			{
				// do not add methods of the superclass
				if (methods[i].getDeclaringClass().equals(objectClass))
				{
					members.add(methods[i]);
				}
				
			} // end if
			
		} // end for each method
		
		return members;
		
	} // end getMembers()
	
/** Tests whether the given method returns a simple Object that has no child objects
 * @param method The method to examine
 * @return true if this object has no children. false otherwise
 */
	private boolean returnsSimpleType(Method method)
	{
		
		String return_type = method.getReturnType().getName();
		
		if (return_type.startsWith("java.lang."))
		{
			return true;
		}
		else if
		(
		return_type.equals("int") ||
		return_type.equals("long") ||
		return_type.equals("float") ||
		return_type.equals("double") ||
		return_type.equals("short") ||
		return_type.equals("boolean") ||
		return_type.equals("byte")
		)
		{
			
			return true;
		}
		else
		{
			return false;
		}
		
	} // end returnsSimpleType()
	
/** Tests whether the given method returns a list of objects
 * @param method The method to examine
 * @return true if this object returns a list. false otherwise
 */
	private boolean returnsListType(Method method)
	{
		
		Class return_type = method.getReturnType();
		
		try
		{
			
			if ((Class.forName("java.util.List")).isAssignableFrom(return_type))
			{
				return true;
			}
			else
			{
				return false;
			}
			
		}
		catch (Throwable t)
		{
			t.printStackTrace();
			return false;
		}
		
	} // end returnsListType()
	
/** Returns the current value of one of the data members of <CODE>_to_edit</CODE>
 * @param method The accessor method to call
 * @return The current value
 */
	private Object getCurrentValue(Method method)
	{
		
		try
		{
			Object value = method.invoke(_to_edit, new Object[]
			{ });
			
			return value;
		}
		catch (Throwable t)
		{
			return null;
		}
		
	} // end getCurrentValue()
	
	
/** Handler method for gui events
 * @param event The event to be handled
 */
	public void actionPerformed(ActionEvent event)
	{
		
		Object source = event.getSource();
		
		if (source == _jb_save)
		{
			try
			{
				// create a new empty instance of the object
				Class object_class = _to_edit.getClass();
				
				//_to_edit = object_class.newInstance();
				
				Method accessor;
				Method mutator;
				String mutator_name;
				Class parameter_type;
				Object param_to_set;
				String string_value;
				
				for (int i = 0; i < _members.size(); i++)
				{
					accessor = (Method)_members.get(i);
					
					parameter_type = accessor.getReturnType();
					
					mutator_name = "set" + accessor.getName().substring(3);
					
					mutator = object_class.getMethod(mutator_name, new Class[]
					{parameter_type});
					
					if (returnsSimpleType(accessor))
					{
						string_value = ((JTextField)_jtf_editable.get(i)).getText();
						
						param_to_set = convertToParamType(string_value, parameter_type);
					}
					else
					{
						param_to_set = _values.get(i);
						
					}
					
					mutator.invoke(_to_edit, new Object[]
					{param_to_set});
					
				}
				
				this.dispose();
				
			}
			catch (Throwable t)
			{
				t.printStackTrace();
			}
			
		}
		else if (source == _jb_cancel)
		{
			
			this.dispose();
			
		}
		
	} // end actionPerformed()
	
	/*
	 * Creates an object of the given type from this string value
	 *@param value. The value to convert
	 *@param type. The class of object to convert value into
	 *@return object
	 */
	private Object convertToParamType(String value, Class class_type)
	{
		
		Object ob = null;
		String type = class_type.getName();
		
		if (type.equals("java.lang.String"))
		{
			ob = value;
		}
		else if ((type.equals("int")) || (type.equals("java.lang.Integer")))
		{
			ob = Integer.valueOf(value);
		}
		else if ((type.equals("long")) || (type.equals("java.lang.Long")))
		{
			ob = Long.valueOf(value);
		}
		else if ((type.equals("float")) || (type.equals("java.lang.Float")))
		{
			ob = Float.valueOf(value);
		}
		else if ((type.equals("double")) || (type.equals("java.lang.Double")))
		{
			ob = Double.valueOf(value);
		}
		else if ((type.equals("short")) || (type.equals("java.lang.Short")))
		{
			ob = Short.valueOf(value);
		}		
		else if ((type.equals("boolean")) || (type.equals("java.lang.Boolean")))
		{
			ob = Boolean.valueOf(value);
		}
		
		return ob;
		
	} // end convertToParamType();
	
	
/** Centres this swing object on the screen
 */
	private void centre()
	{
		
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = this.getSize();
		
		this.setLocation((screenSize.width  - frameSize.width) / 2,
		(screenSize.height - frameSize.height) / 2);
		
	}
	
} // end DataBindingEditor.java
