//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.tool.configurator;

import java.util.*;

/**
 * TwoWayMapping is a convenience class that allows bidirectional lookup between
 * unique pairs of values. It is essentially a hashtable that can be searched
 * using either keys or values. The semantics for this class only hold if no
 * object exists as both a key and a value (unless they are a unique pair).
 */

public class TwoWayMapping
{
	
	private Hashtable _forward;
	private Hashtable _backward;
	
	/**
	 * Constructs a new TwoWayMapping
	 */
	public TwoWayMapping()
	{
		_forward = new Hashtable();
		_backward = new Hashtable();	
	}
	
	/**
	 * Puts a pair of objects into the TwoWayMapping.
	 * The order of the objects should not matter.
	 *
	 *@param first. The first object in the pair.
	 *@param second. The second object in the pair.
	 */
	public void put(Object first, Object second)
	{
		
		_forward.put(first, second);
		_backward.put(second, first);	
		
	} // end put()
	
	/**
	 * Attempts to retrieve the "other half" of a pair that is mapped to
	 * the supplied object.
	 *
	 *@param key. One half of a pair.
	 *@return The matching object if one is found, null if not.
	 */
	public Object get(Object key)
	{
		Object result;
		
		result = _forward.get(key);
		
		if (result != null)
		{
			return result;
		}
		else
		{
			result = _backward.get(key);
			
			if (result != null)
			{
				return result;
			}
			else
			{
				return null;
			}
		}
		
	} // end get()	
	
} // end TwoWayMapping.java