//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.tool.loader;

import java.util.*;
import javax.swing.*;

public class LoaderDialog extends javax.swing.JDialog
{
	private boolean _start = false; 
	
	public LoaderDialog( java.awt.Frame parent, List agents, List classes )
	{
		this( parent, true );
		
		setupComboBox( agents, jComboBox1 );
		setupComboBox( classes, jComboBox2 );
		
		pack ();
		
		// Resize and reposition window so it appears in the center
		// of the screen, and is not too large
		java.awt.Toolkit tk = java.awt.Toolkit.getDefaultToolkit();
		int width = getSize().width;
		int height = getSize().height;
		
		setSize( width, height );
		width = tk.getScreenSize().width - width;
		height = tk.getScreenSize().height - height;
		setLocation( width/2, height/2 );
		
		setVisible( true );
	}
	
		/** Creates new form LoaderDialog */
	private LoaderDialog(java.awt.Frame parent,boolean modal)
	{
		super (parent, modal);
		initComponents ();
		pack ();
		
	}
	
	/**
	 * Get selected Agent Name, or null if cancel was pressed
	 */
	public String getAgentName()
	{
		if( _start )
		{
			return (String) jComboBox1.getSelectedItem();
		}
		
		return null;
	}
	
	/**
	 * Get selected Agent Name, or null if cancel was pressed
	 */
	public String getClassName()
	{
		if( _start )
		{
			return (String) jComboBox2.getSelectedItem();
		}
		
		return null;
	}
	
	private void setupComboBox( List values, JComboBox cb )
	{
		Set added = new HashSet();
		
		Iterator i = values.iterator();
		while( i.hasNext() )
		{
			Object o = i.next();
			if( !added.contains( o ) )
			{
				cb.addItem( o );
				added.add( o );
			}
		}
		
		if( values.size() > 0 )
		{
			cb.setSelectedItem( values.get( 0 ) );
		}
	}
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the FormEditor.
	 */
	private void initComponents()//GEN-BEGIN:initComponents
	{
		jLabel1 = new javax.swing.JLabel();
		jComboBox1 = new javax.swing.JComboBox();
		jLabel2 = new javax.swing.JLabel();
		jComboBox2 = new javax.swing.JComboBox();
		jPanel1 = new javax.swing.JPanel();
		_ok = new javax.swing.JButton();
		_cancel = new javax.swing.JButton();
		getContentPane().setLayout(new java.awt.GridBagLayout());
		java.awt.GridBagConstraints gridBagConstraints1;
		setTitle("Start Agent");
		addWindowListener(new java.awt.event.WindowAdapter()
		{
			public void windowClosing(java.awt.event.WindowEvent evt)
			{
				closeDialog(evt);
			}
		}
		);
		
		jLabel1.setText("Agent Name");
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.insets = new java.awt.Insets(2, 2, 2, 2);
		gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
		getContentPane().add(jLabel1, gridBagConstraints1);
		
		
		jComboBox1.setEditable(true);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridwidth = 0;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints1.insets = new java.awt.Insets(2, 2, 2, 2);
		gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
		getContentPane().add(jComboBox1, gridBagConstraints1);
		
		
		jLabel2.setText("Class Name");
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.insets = new java.awt.Insets(2, 2, 2, 2);
		gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
		getContentPane().add(jLabel2, gridBagConstraints1);
		
		
		jComboBox2.setEditable(true);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridwidth = 0;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints1.insets = new java.awt.Insets(2, 2, 2, 2);
		gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
		getContentPane().add(jComboBox2, gridBagConstraints1);
		
		
		jPanel1.setLayout(new java.awt.GridBagLayout());
		java.awt.GridBagConstraints gridBagConstraints2;
		
		_ok.setText("Start Agent");
  		_ok.addActionListener(new java.awt.event.ActionListener()
  		{
  			public void actionPerformed(java.awt.event.ActionEvent evt)
  			{
  				_okActionPerformed(evt);
  			}
  		}
  		);
  		gridBagConstraints2 = new java.awt.GridBagConstraints();
  		gridBagConstraints2.insets = new java.awt.Insets(3, 3, 3, 3);
  		gridBagConstraints2.anchor = java.awt.GridBagConstraints.EAST;
  		gridBagConstraints2.weightx = 1.0;
  		jPanel1.add(_ok, gridBagConstraints2);
  		
  		
		_cancel.setText("Cancel");
  		_cancel.addActionListener(new java.awt.event.ActionListener()
  		{
  			public void actionPerformed(java.awt.event.ActionEvent evt)
  			{
  				_cancelActionPerformed(evt);
  			}
  		}
  		);
  		gridBagConstraints2 = new java.awt.GridBagConstraints();
  		gridBagConstraints2.gridx = 1;
  		gridBagConstraints2.gridy = 0;
  		gridBagConstraints2.insets = new java.awt.Insets(3, 3, 3, 3);
  		gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
  		gridBagConstraints2.weightx = 1.0;
  		jPanel1.add(_cancel, gridBagConstraints2);
  		
  		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridwidth = 0;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
		getContentPane().add(jPanel1, gridBagConstraints1);
		
	}//GEN-END:initComponents

  private void _cancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event__cancelActionPerformed
		closeDialog( null );
  }//GEN-LAST:event__cancelActionPerformed

  private void _okActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event__okActionPerformed
		
		_start = true;
		closeDialog( null );
  }//GEN-LAST:event__okActionPerformed
		
		/** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
			setVisible (false);
			dispose ();
    }//GEN-LAST:event_closeDialog
		
		/**
		 * @param args the command line arguments
		 */
		public static void main (String args[])
		{
			new LoaderDialog (new javax.swing.JFrame (), true).show ();
		}
		
		
		// Variables declaration - do not modify//GEN-BEGIN:variables
		private javax.swing.JLabel jLabel1;
		private javax.swing.JComboBox jComboBox1;
		private javax.swing.JLabel jLabel2;
		private javax.swing.JComboBox jComboBox2;
		private javax.swing.JPanel jPanel1;
		private javax.swing.JButton _ok;
		private javax.swing.JButton _cancel;
		// End of variables declaration//GEN-END:variables
		
}
