//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved.
//
//******************************************************************************

package fipaos.tool.loader;

import fipaos.agent.profile.AgentDescription;
import fipaos.tool.loader.threadpoolmonitor.ThreadPoolMonitorFrame;
import fipaos.tool.loader.taskmanagermonitor.TaskManagerMonitorFrame;

import java.awt.event.*;
import java.util.*;
import javax.swing.*;

/**
 *
 * Gui for the visualisation of agents running within a virtual machine and
 * starting and stopping agents
 */
public class LoaderGUI extends javax.swing.JFrame
{
	private AgentLoader _loader;
	private List _started = new LinkedList();
	private List _ready = new LinkedList();
	private List _previous_agents = new LinkedList();
	private List _previous_classes = new LinkedList();
	private long _last_click;
	private List _last_sel = new LinkedList();
		
	
	/** Creates new form LoaderGUI1 */
	public LoaderGUI( AgentLoader loader )
	{
		// Save reference to loader
		_loader = loader;
		initComponents ();
		
		// Update Lists
		updateLists();
		
		// Update _previous_agents & _previous_classes
		Iterator i = _loader.getAgents().iterator();
		while( i.hasNext() )
		{
			String agent = (String) i.next();
			AgentDescription ad = _loader.getAgentDescription( agent );
			_previous_agents.add( agent );
			_previous_classes.add( ad.getClassName() );
		}
		
		// Start polling
		startPolling();

		pack();
		
		// Resize and reposition window so it appears in the center
		// of the screen, and is not too large
		java.awt.Toolkit tk = java.awt.Toolkit.getDefaultToolkit();
		int width = getSize().width;
		int height = getSize().height;
		
		setSize( width, height );
		width = tk.getScreenSize().width - width;
		height = tk.getScreenSize().height - height;
		setLocation( width/2, height/2 );

		setVisible( true );
	}
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the FormEditor.
	 */
	private void initComponents()//GEN-BEGIN:initComponents
	{
		jMenuBar1 = new javax.swing.JMenuBar();
		jMenu1 = new javax.swing.JMenu();
		jMenuItem1 = new javax.swing.JMenuItem();
		jMenuItem2 = new javax.swing.JMenuItem();
		jPanel1 = new javax.swing.JPanel();
		jScrollPane1 = new javax.swing.JScrollPane();
		_started_list = new javax.swing.JList();
		_start_button = new javax.swing.JButton();
		jPanel2 = new javax.swing.JPanel();
		jScrollPane2 = new javax.swing.JScrollPane();
		_ready_to_start = new javax.swing.JList();
		_start_other = new javax.swing.JButton();
		_shutdown_button = new javax.swing.JButton();
		
		jMenu1.setText("Tools");
  		
  		jMenuItem1.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_P, java.awt.event.InputEvent.CTRL_MASK));
    		jMenuItem1.setText("Thread Pool Monitor");
    		jMenuItem1.addActionListener(new java.awt.event.ActionListener()
    		{
    			public void actionPerformed(java.awt.event.ActionEvent evt)
    			{
    				jMenuItem1ActionPerformed(evt);
    			}
    		}
    		);
    		jMenu1.add(jMenuItem1);
    		
  		jMenuItem2.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_T, java.awt.event.InputEvent.CTRL_MASK));
    		jMenuItem2.setText("Task Manager Monitor");
    		jMenuItem2.addActionListener(new java.awt.event.ActionListener()
    		{
    			public void actionPerformed(java.awt.event.ActionEvent evt)
    			{
    				jMenuItem2ActionPerformed(evt);
    			}
    		}
    		);
    		jMenu1.add(jMenuItem2);
    		jMenuBar1.add(jMenu1);
  		getContentPane().setLayout(new java.awt.GridBagLayout());
		java.awt.GridBagConstraints gridBagConstraints1;
		setIconImage(new ImageIcon( ClassLoader.getSystemResource( "images/logo.gif" ) ).getImage());
		setTitle("FIPA-OS v" + System.getProperty( "fipaos.version" ) + " - Agent Loader");
		addWindowListener(new java.awt.event.WindowAdapter()
		{
			public void windowClosing(java.awt.event.WindowEvent evt)
			{
				exitForm(evt);
			}
		}
		);
		
		jPanel1.setLayout(new java.awt.BorderLayout());
		jPanel1.setBorder(new javax.swing.border.TitledBorder("Running Agents"));
		
		jScrollPane1.setPreferredSize(new java.awt.Dimension(150, 100));
  		
  		_started_list.addMouseListener(new java.awt.event.MouseAdapter()
    		{
    			public void mouseClicked(java.awt.event.MouseEvent evt)
    			{
    				startedListMouseClick(evt);
    			}
    		}
    		);
    		jScrollPane1.setViewportView(_started_list);
    		
    		jPanel1.add(jScrollPane1, java.awt.BorderLayout.CENTER);
  		
  		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 0;
		gridBagConstraints1.gridy = 0;
		gridBagConstraints1.gridheight = 0;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
		gridBagConstraints1.weightx = 1.0;
		gridBagConstraints1.weighty = 1.0;
		getContentPane().add(jPanel1, gridBagConstraints1);
		
		
		_start_button.setText("< Start");
		_start_button.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				_start_buttonActionPerformed(evt);
			}
		}
		);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 1;
		gridBagConstraints1.gridy = 0;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints1.insets = new java.awt.Insets(2, 2, 2, 2);
		gridBagConstraints1.anchor = java.awt.GridBagConstraints.SOUTH;
		gridBagConstraints1.weighty = 1.0;
		getContentPane().add(_start_button, gridBagConstraints1);
		
		
		jPanel2.setLayout(new java.awt.BorderLayout());
		jPanel2.setBorder(new javax.swing.border.TitledBorder("Known Agents"));
		
		jScrollPane2.setPreferredSize(new java.awt.Dimension(150, 100));
  		
  		jScrollPane2.setViewportView(_ready_to_start);
    		
    		jPanel2.add(jScrollPane2, java.awt.BorderLayout.CENTER);
  		
  		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 2;
		gridBagConstraints1.gridy = 0;
		gridBagConstraints1.gridwidth = 0;
		gridBagConstraints1.gridheight = 0;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
		gridBagConstraints1.weightx = 1.0;
		gridBagConstraints1.weighty = 1.0;
		getContentPane().add(jPanel2, gridBagConstraints1);
		
		
		_start_other.setText("Start other...");
		_start_other.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				_start_otherActionPerformed(evt);
			}
		}
		);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 1;
		gridBagConstraints1.gridy = 1;
		gridBagConstraints1.insets = new java.awt.Insets(2, 2, 2, 2);
		getContentPane().add(_start_other, gridBagConstraints1);
		
		
		_shutdown_button.setActionCommand("Shutdown");
		_shutdown_button.setText("Shutdown >");
		_shutdown_button.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				_shutdown_buttonActionPerformed(evt);
			}
		}
		);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 1;
		gridBagConstraints1.gridy = 2;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints1.insets = new java.awt.Insets(2, 2, 2, 2);
		gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTH;
		gridBagConstraints1.weighty = 1.0;
		getContentPane().add(_shutdown_button, gridBagConstraints1);
		
		setJMenuBar(jMenuBar1);
		
	}//GEN-END:initComponents

  private void jMenuItem2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem2ActionPerformed
		new TaskManagerMonitorFrame();
  }//GEN-LAST:event_jMenuItem2ActionPerformed

  private void jMenuItem1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem1ActionPerformed
		new ThreadPoolMonitorFrame();
  }//GEN-LAST:event_jMenuItem1ActionPerformed

  private void startedListMouseClick(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_startedListMouseClick

		long curr = System.currentTimeMillis();
		List curr_sel = Arrays.asList( _started_list.getSelectedValues() );

		if( _last_click == 0 )
		{
			_last_click = System.currentTimeMillis();
			_last_sel.clear();
			_last_sel.addAll( curr_sel );
		}
		else if( ( curr - _last_click ) > 500 )
		{
			_last_click = curr;
			_last_sel.clear();
			_last_sel.addAll( curr_sel );
		}
		else
		{
			if( curr_sel.containsAll( _last_sel ) && _last_sel.containsAll( curr_sel ) )
			{
				Iterator i = curr_sel.iterator();
				while( i.hasNext() )
				{
					_loader.activate( (String) i.next() );
				}
				_last_click = 0;
			}
		}
		
  }//GEN-LAST:event_startedListMouseClick

  private void _start_otherActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event__start_otherActionPerformed
		LoaderDialog ld = new LoaderDialog( this, _previous_agents, _previous_classes );
		String agent_name = ld.getAgentName();
		String class_name = ld.getClassName();
		
		if( agent_name != null )
		{
			_previous_agents.add( 0, agent_name );
			_previous_classes.add( 0, class_name );
			_loader.startAgent( agent_name, class_name );
		}
  }//GEN-LAST:event__start_otherActionPerformed

  private void _shutdown_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event__shutdown_buttonActionPerformed

		Object agents[] = null;
		
		synchronized( this )
		{
			// Get agent selected and start
			agents = _started_list.getSelectedValues();
		}
		
		if( agents == null || agents.length == 0 )
		{
			JOptionPane.showMessageDialog( this, "Please select an Agent to shutdown" );
			return;
		}

		// Start selected Agents
		for( int i = 0; i < agents.length; i++ )
		{
			_loader.shutdownAgent( (String) agents[ i ] );
			updateLists();
		}		
		
  }//GEN-LAST:event__shutdown_buttonActionPerformed

  private void _start_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event__start_buttonActionPerformed

		Object agents[] = null;
		
		synchronized( this )
		{
			// Get agent selected and start
			agents = _ready_to_start.getSelectedValues();
		}
		
		if( agents == null || agents.length == 0 )
		{
			JOptionPane.showMessageDialog( this, "Please select an Agent to start" );
			return;
		}

		// Start selected Agents
		for( int i = 0; i < agents.length; i++ )
		{
			_loader.startAgent( (String) agents[ i ], true );
			updateLists();
		}		
		
  }//GEN-LAST:event__start_buttonActionPerformed
	
  private void jButton1ActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
  }//GEN-LAST:event_jButton1ActionPerformed
	
  private void shutdownButton2ActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_shutdownButton2ActionPerformed
  }//GEN-LAST:event_shutdownButton2ActionPerformed
	
/** Exit the Application */
  private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
		
		Thread t = new Thread( new Runnable()
		{
			public void run()
			{
				_loader.shutdownAgents();
				System.exit( 0 );
			}
		} );
		t.start();

		dispose();
		
  }//GEN-LAST:event_exitForm
	
	
	/**
	 * Update currently displayed lists based upon info. from AgentLoader
	 */
	private void updateLists()
	{
		synchronized( this )
		{
			Object values1[] = _ready_to_start.getSelectedValues();
			Object values2[] = _started_list.getSelectedValues();
			
			// Get the lists
			List started = _loader.getAliveAgents();
			List available = _loader.getAgents();

			// Claculate list of Agents that can be started
			available.removeAll( started );
			
			// Update lists, if necessary
			if( !_started.containsAll( started ) || !started.containsAll( _started ) )
			{
				Vector v = new Vector( started );
				_started_list.setListData( v.toArray() );
				updateSelected( values2, _started_list, v );
				_started.clear();
				_started.addAll( started );
			}
			
			if( !_ready.containsAll( available ) || !available.containsAll( _ready ) )
			{
				Vector v = new Vector( available );
				_ready_to_start.setListData( v.toArray() );								
				updateSelected( values1, _ready_to_start, v );
				_ready.clear();
				_ready.addAll( available );
			}
		}
	}
	
	/**
	 * Updated selected entries in given list
	 */
	private void updateSelected( Object select[], JList list, List entries )
	{
		if( select == null || select.length == 0 )
		{
			return;
		}
		
		int next_index = 0;
		int selected_indexs[] = new int[ select.length ];
		
		// Iterate through select[], calculating indexes to add to int[]
		for( int i = 0; i < select.length; i++ )
		{
			int index = entries.indexOf( select[ i ] );
			if( index != -1 )
			{
				selected_indexs[ next_index++ ] = index;
			}
		}
		
		// Make selections if there is anything to select
		if( next_index != 0 )
		{
			// Copy indexes, and select in JList
			int sel[] = new int[ next_index ];
			System.arraycopy( selected_indexs, 0, sel, 0, next_index );
			list.setSelectedIndices( sel );
		}
		else
		{
			list.clearSelection();
		}
	}
	
	/**
	 * Method to initialise regular updates of lists
	 */
	private void startPolling()
	{
		Thread t = new Thread( new Runnable()
		{
			public void run()
			{
				while( true )
				{
					updateLists();
					
					try
					{
						Thread.sleep( 500 );
					}
					catch( Throwable x )
					{
					}
				}
			}
		} );
		t.start();
	}
	
	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JMenuBar jMenuBar1;
	private javax.swing.JMenu jMenu1;
	private javax.swing.JMenuItem jMenuItem1;
	private javax.swing.JMenuItem jMenuItem2;
	private javax.swing.JPanel jPanel1;
	private javax.swing.JScrollPane jScrollPane1;
	private javax.swing.JList _started_list;
	private javax.swing.JButton _start_button;
	private javax.swing.JPanel jPanel2;
	private javax.swing.JScrollPane jScrollPane2;
	private javax.swing.JList _ready_to_start;
	private javax.swing.JButton _start_other;
	private javax.swing.JButton _shutdown_button;
	// End of variables declaration//GEN-END:variables

}
