//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.tool.loader.threadpoolmonitor;

import java.util.*;
import javax.swing.*;
import fipaos.util.*;

/**
 * Class to monitor the ThreadPool's within a given VM
 */
public class ThreadPoolMonitorFrame extends javax.swing.JFrame
{
	/**
	 * Refresh rate for GUI
	 */
	private int _refresh = 1000;
	
	/**
	 * List of currently known ThreadPools
	 */
	private List _known_tps = new Vector();
	
	/**
	 * Indicates monitor should shutdown
	 */
	private boolean _shutdown;
	
	/**
	 * Last cumulative values for thread usage
	 */
	private long _last_active,_last_inactive;
	
	/** Creates new form ThreadPoolMonitorFrame */
	public ThreadPoolMonitorFrame()
	{
		initComponents ();
		pack ();
		
		// Refresh GUI
		refreshGUI();
		
		// Start monitor
		Thread t = new Thread( new Runnable()
		{
			public void run()
			{
				monitor();
			}
		});
		t.start();
		
		// Resize and reposition window so it appears in the center
		// of the screen, and is not too large
		java.awt.Toolkit tk = java.awt.Toolkit.getDefaultToolkit();
		int width = getSize().width;
		int height = getSize().height;
		
		setSize( width, height );
		width = tk.getScreenSize().width - width;
		height = tk.getScreenSize().height - height;
		setLocation( width/2, height/2 );

		// Show window
		setVisible( true );
	}
	
		/** This method is called from within the constructor to
		 * initialize the form.
		 * WARNING: Do NOT modify this code. The content of this method is
		 * always regenerated by the FormEditor.
		 */
	private void initComponents()//GEN-BEGIN:initComponents
	{
		jPanel2 = new javax.swing.JPanel();
		jSplitPane1 = new javax.swing.JSplitPane();
		jPanel1 = new javax.swing.JPanel();
		jScrollPane1 = new javax.swing.JScrollPane();
		_jlist = new javax.swing.JList();
		jPanel4 = new javax.swing.JPanel();
		jLabel3 = new javax.swing.JLabel();
		_count = new javax.swing.JLabel();
		jLabel5 = new javax.swing.JLabel();
		_active = new javax.swing.JLabel();
		jLabel2 = new javax.swing.JLabel();
		_time_active = new javax.swing.JLabel();
		jLabel6 = new javax.swing.JLabel();
		_time_inactive = new javax.swing.JLabel();
		jLabel7 = new javax.swing.JLabel();
		_pending = new javax.swing.JLabel();
		jPanel5 = new javax.swing.JPanel();
		_utilisation = new javax.swing.JProgressBar();
		jPanel6 = new javax.swing.JPanel();
		_rate = new javax.swing.JSlider();
		getContentPane().setLayout(new java.awt.GridBagLayout());
		java.awt.GridBagConstraints gridBagConstraints1;
		setIconImage(new ImageIcon( ClassLoader.getSystemResource( "images/logo.gif" ) ).getImage());
		setTitle("ThreadPool Monitor");
		addWindowListener(new java.awt.event.WindowAdapter()
		{
			public void windowClosing(java.awt.event.WindowEvent evt)
			{
				exitForm(evt);
			}
		}
		);
		
		jPanel2.setLayout(new java.awt.BorderLayout());
		jPanel2.setBorder(new javax.swing.border.TitledBorder("VM Thread Pools"));
		
		
  		jPanel1.setLayout(new java.awt.BorderLayout());
    		jPanel1.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(1, 1, 1, 1)));
    		
    		
      		_jlist.addMouseListener(new java.awt.event.MouseAdapter()
        		{
        			public void mouseClicked(java.awt.event.MouseEvent evt)
        			{
        				selectionChanged(evt);
        			}
        		}
        		);
        		jScrollPane1.setViewportView(_jlist);
        		
        		jPanel1.add(jScrollPane1, java.awt.BorderLayout.CENTER);
      		
      		jSplitPane1.setLeftComponent(jPanel1);
    		
    		
  		jPanel4.setLayout(new java.awt.GridBagLayout());
    		java.awt.GridBagConstraints gridBagConstraints2;
    		jPanel4.setBorder(new javax.swing.border.TitledBorder("Thread Pool Details"));
    		
    		jLabel3.setText("Thread Count:");
      		gridBagConstraints2 = new java.awt.GridBagConstraints();
      		gridBagConstraints2.insets = new java.awt.Insets(3, 3, 3, 3);
      		gridBagConstraints2.anchor = java.awt.GridBagConstraints.NORTHEAST;
      		jPanel4.add(jLabel3, gridBagConstraints2);
      		
      		
    		_count.setText("jLabel1");
      		_count.setForeground(java.awt.Color.black);
      		gridBagConstraints2 = new java.awt.GridBagConstraints();
      		gridBagConstraints2.gridwidth = 0;
      		gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
      		gridBagConstraints2.insets = new java.awt.Insets(3, 3, 3, 3);
      		gridBagConstraints2.anchor = java.awt.GridBagConstraints.NORTHWEST;
      		gridBagConstraints2.weightx = 1.0;
      		jPanel4.add(_count, gridBagConstraints2);
      		
      		
    		jLabel5.setText("Active Thread Count:");
      		gridBagConstraints2 = new java.awt.GridBagConstraints();
      		gridBagConstraints2.insets = new java.awt.Insets(3, 3, 3, 3);
      		gridBagConstraints2.anchor = java.awt.GridBagConstraints.NORTHEAST;
      		jPanel4.add(jLabel5, gridBagConstraints2);
      		
      		
    		_active.setText("jLabel1");
      		_active.setForeground(java.awt.Color.black);
      		gridBagConstraints2 = new java.awt.GridBagConstraints();
      		gridBagConstraints2.gridwidth = 0;
      		gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
      		gridBagConstraints2.insets = new java.awt.Insets(3, 3, 3, 3);
      		gridBagConstraints2.anchor = java.awt.GridBagConstraints.NORTHWEST;
      		gridBagConstraints2.weightx = 1.0;
      		jPanel4.add(_active, gridBagConstraints2);
      		
      		
    		jLabel2.setText("Total Time Active:");
      		gridBagConstraints2 = new java.awt.GridBagConstraints();
      		gridBagConstraints2.insets = new java.awt.Insets(3, 3, 3, 3);
      		gridBagConstraints2.anchor = java.awt.GridBagConstraints.NORTHEAST;
      		jPanel4.add(jLabel2, gridBagConstraints2);
      		
      		
    		_time_active.setText("jLabel1");
      		_time_active.setForeground(java.awt.Color.black);
      		gridBagConstraints2 = new java.awt.GridBagConstraints();
      		gridBagConstraints2.gridwidth = 0;
      		gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
      		gridBagConstraints2.insets = new java.awt.Insets(3, 3, 3, 3);
      		gridBagConstraints2.anchor = java.awt.GridBagConstraints.NORTHWEST;
      		gridBagConstraints2.weightx = 1.0;
      		jPanel4.add(_time_active, gridBagConstraints2);
      		
      		
    		jLabel6.setText("Total Time Inactive:");
      		gridBagConstraints2 = new java.awt.GridBagConstraints();
      		gridBagConstraints2.insets = new java.awt.Insets(3, 3, 3, 3);
      		gridBagConstraints2.anchor = java.awt.GridBagConstraints.NORTHEAST;
      		jPanel4.add(jLabel6, gridBagConstraints2);
      		
      		
    		_time_inactive.setText("jLabel1");
      		_time_inactive.setForeground(java.awt.Color.black);
      		gridBagConstraints2 = new java.awt.GridBagConstraints();
      		gridBagConstraints2.gridwidth = 0;
      		gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
      		gridBagConstraints2.insets = new java.awt.Insets(3, 3, 3, 3);
      		gridBagConstraints2.anchor = java.awt.GridBagConstraints.NORTHWEST;
      		gridBagConstraints2.weightx = 1.0;
      		jPanel4.add(_time_inactive, gridBagConstraints2);
      		
      		
    		jLabel7.setText("Pending Jobs:");
      		gridBagConstraints2 = new java.awt.GridBagConstraints();
      		gridBagConstraints2.insets = new java.awt.Insets(3, 3, 3, 3);
      		gridBagConstraints2.anchor = java.awt.GridBagConstraints.NORTHEAST;
      		gridBagConstraints2.weighty = 1.0;
      		jPanel4.add(jLabel7, gridBagConstraints2);
      		
      		
    		_pending.setText("jLabel1");
      		_pending.setForeground(java.awt.Color.black);
      		gridBagConstraints2 = new java.awt.GridBagConstraints();
      		gridBagConstraints2.gridwidth = 0;
      		gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
      		gridBagConstraints2.insets = new java.awt.Insets(3, 3, 3, 3);
      		gridBagConstraints2.anchor = java.awt.GridBagConstraints.NORTHWEST;
      		gridBagConstraints2.weightx = 1.0;
      		gridBagConstraints2.weighty = 1.0;
      		jPanel4.add(_pending, gridBagConstraints2);
      		
      		jSplitPane1.setRightComponent(jPanel4);
    		
    		jPanel2.add(jSplitPane1, java.awt.BorderLayout.CENTER);
  		
  		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridwidth = 0;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
		gridBagConstraints1.weightx = 1.0;
		gridBagConstraints1.weighty = 1.0;
		getContentPane().add(jPanel2, gridBagConstraints1);
		
		
		jPanel5.setLayout(new java.awt.BorderLayout());
		jPanel5.setBorder(new javax.swing.border.TitledBorder("Overall Thread Utilisation"));
		
		jPanel5.add(_utilisation, java.awt.BorderLayout.CENTER);
  		
  		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
		gridBagConstraints1.weightx = 1.0;
		getContentPane().add(jPanel5, gridBagConstraints1);
		
		
		jPanel6.setLayout(new java.awt.BorderLayout());
		jPanel6.setBorder(new javax.swing.border.TitledBorder("Refresh Period (ms)"));
		
		_rate.setMinorTickSpacing(250);
  		_rate.setPaintLabels(true);
  		_rate.setPaintTicks(true);
  		_rate.setMinimum(250);
  		_rate.setMajorTickSpacing(1000);
  		_rate.setMaximum(2250);
  		_rate.setValue(_refresh);
  		_rate.addChangeListener(new javax.swing.event.ChangeListener()
  		{
  			public void stateChanged(javax.swing.event.ChangeEvent evt)
  			{
  				refreshRateChanged(evt);
  			}
  		}
  		);
  		jPanel6.add(_rate, java.awt.BorderLayout.CENTER);
  		
  		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridwidth = 0;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
		gridBagConstraints1.weightx = 0.8;
		getContentPane().add(jPanel6, gridBagConstraints1);
		
	}//GEN-END:initComponents

  private void refreshRateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_refreshRateChanged
		_refresh = _rate.getValue();
  }//GEN-LAST:event_refreshRateChanged

  private void selectionChanged(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_selectionChanged
		refreshGUI();
  }//GEN-LAST:event_selectionChanged
	
		/** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
			_shutdown = true;
			dispose();
    }//GEN-LAST:event_exitForm
		
		/**
		 * @param args the command line arguments
		 */
		public static void main (String args[])
		{
			new ThreadPoolMonitorFrame ();
		}
		
		/**
		 * Refresh GUI
		 */
		private void refreshGUI()
		{
			// Get currently selected TP
			ThreadPool selected = (ThreadPool) _jlist.getSelectedValue();
			
			// Update the list of available TP's
			Set current = ThreadPool.getThreadPools();
			
			// What has been added
			Set added = new HashSet( current );
			added.removeAll( _known_tps );
			
			// What has been removed
			Set removed = new HashSet( _known_tps );
			removed.removeAll( current );
			
			// Update List appropriately
			_known_tps.removeAll( removed );
			_known_tps.addAll( added );
			
			// Check if previously selected value is still selected
			if( selected != null && removed.contains( selected ) )
			{
				selected = null;
			}
			
			// Update JList, but only if necessary
			if( removed.size() > 0 || added.size() > 0 )
			{
				_jlist.setListData( (Vector) _known_tps );

				// Re-select the appropriate TP
				_jlist.setSelectedValue( selected, false );
			}
			
			// Update the currently selected TP
			refreshGUI( selected );
			
			// Refresh usage graph
			refreshGUI( _known_tps );
		}
		
		/**
		 * Refresh TP details
		 */
		private void refreshGUI( ThreadPool tp )
		{
			if( tp == null )
			{
				_active.setText( "N/A" );
				_count.setText( "N/A" );
				_pending.setText( "N/A" );
				_time_active.setText( "N/A" );
				_time_inactive.setText( "N/A" );
			}
			else
			{
				_active.setText( "" + tp.getActiveThreadCount() );
				_count.setText( "" + tp.getThreadCount() );
				_pending.setText( "" + tp.getWaitingJobCount() );
				_time_active.setText( "" + tp.getTotalActiveTime() + " ms" );
				_time_inactive.setText( "" + tp.getTotalInactiveTime() + " ms" );
			}
		}
		
		
		/**
		 * Update utilisation graph
		 */
		private void refreshGUI( List l )
		{
			// Count number of threads
			long inactive = 0;
			long active = 0;
			Iterator i = l.iterator();
			while( i.hasNext() )
			{
				ThreadPool tp = (ThreadPool) i.next();
				inactive += tp.getTotalInactiveTime() + tp.getTotalActiveTime();
				active += tp.getTotalActiveTime();
			}
			
			// Update progress bar
			int usage = 0;
			if( _last_inactive != 0 && inactive != _last_inactive )
			{
				usage = (int) ( ( ( active - _last_active ) * 100 ) / ( inactive - _last_inactive ) );
			}
			_last_active = active;
			_last_inactive = inactive;
			
			_utilisation.setValue( usage );
			_utilisation.setString( usage + "%" );
			_utilisation.setToolTipText( usage + "%" );
		}
		
		
		public void monitor()
		{
			while( !_shutdown )
			{
				try
				{
					Thread.sleep( _refresh );
					refreshGUI();
				}
				catch( Throwable t )
				{
				}
			}
		}
		
		// Variables declaration - do not modify//GEN-BEGIN:variables
		private javax.swing.JPanel jPanel2;
		private javax.swing.JSplitPane jSplitPane1;
		private javax.swing.JPanel jPanel1;
		private javax.swing.JScrollPane jScrollPane1;
		private javax.swing.JList _jlist;
		private javax.swing.JPanel jPanel4;
		private javax.swing.JLabel jLabel3;
		private javax.swing.JLabel _count;
		private javax.swing.JLabel jLabel5;
		private javax.swing.JLabel _active;
		private javax.swing.JLabel jLabel2;
		private javax.swing.JLabel _time_active;
		private javax.swing.JLabel jLabel6;
		private javax.swing.JLabel _time_inactive;
		private javax.swing.JLabel jLabel7;
		private javax.swing.JLabel _pending;
		private javax.swing.JPanel jPanel5;
		private javax.swing.JProgressBar _utilisation;
		private javax.swing.JPanel jPanel6;
		private javax.swing.JSlider _rate;
		// End of variables declaration//GEN-END:variables
		
}
