//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis,
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************
package fipaos.tool.wizard;


/** Represents a definition of an internal or external transport in
 *  the FIPA-OS configuration Wizard
 */
public class TransportDefinition
{
	
/** The name of this transport
 */
	private String _name;
	
/** If this transport is an external transport.
 * NB: An internal transport is a FIPA-OS proprietary transport that has been optimised for use between FIPA-OS platforms
 * An external transport is a non-FIPA-OS specific transport that can communicate with non-FIPA-OS agent platforms
 */
	private boolean 	_external;
	
/** If this transport will be used by the agent in the current FIPA-OS configuration
 */
	private boolean 	_used_by_agent;
	
/** If this transport will be used by the acc in the current FIPA-OS configuration
 */
	private boolean 	_used_by_acc;	
	
/** The port that the naming service for this transport will run on
 */
	private int 		_ns_port;
	
/** The variable name in the startup scripts that will store the ns port value
 */
	private String 		_script_ns_port_var	= null;
	
/** The variable in the startup scripts that is used to enable starting of an
 *  external naming service program (such as an RMI registry or CORBA Naming Service)
 */
	private String 		_script_ns_enable_var 	= null;
	
    /** Creates new TransportDefinition */
	public TransportDefinition()
	{
	}
	
	public void setName(String name)
	{
		_name = name;
	}
	
	public void setExternal(boolean external)
	{
		_external = external;
	}
	
	public void setUsedByAgent(boolean used_by_agent)
	{
		_used_by_agent = used_by_agent;
	}
	
	public void setUsedByACC(boolean used_by_acc)
	{
		_used_by_acc = used_by_acc;
	}	
	
	public void setNSPort(int ns_port)
	{
		_ns_port = ns_port;
	}
	
	public void setScriptNSPortVar(String script_ns_port_var)
	{
		_script_ns_port_var = script_ns_port_var;
	}
	
	public void setScriptNSEnableVar(String script_ns_enable_var)
	{
		_script_ns_enable_var = script_ns_enable_var;
	}
	
	public String getName()
	{
		return _name;
	}
	
	public boolean getExternal()
	{
		return _external;
	}
	
	public boolean getUsedByAgent()
	{
		return _used_by_agent;
	}
	
	public boolean getUsedByACC()
	{
		return _used_by_acc;
	}	
	
	public int getNSPort()
	{
		return _ns_port;
	}
	
	public String getScriptNSPortVar()
	{
		return _script_ns_port_var;
	}
	
	public String getScriptNSEnableVar()
	{
		return _script_ns_enable_var;
	}
	
} // end TransportDefinition