//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis,
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.tool.wizard.panels;


import fipaos.tool.wizard.*;
import fipaos.agent.profile.*;
import fipaos.util.*;

import java.util.*;
import javax.swing.*;

public class NamingServicePorts extends javax.swing.JDialog
{
	private PlatformProfile _platform_profile;
	private AgentProfile _default_profile;
	private ACCProfile _acc_profile;
	private boolean _modified = false;
	private PlatformDetails _pd;
	
	/** Creates new form NamingServicePorts */
	public NamingServicePorts(java.awt.Frame parent, ACCProfile acc, PlatformProfile plat, AgentProfile agent, PlatformDetails pd )
	{
		super(parent, true);
		
		_platform_profile = plat;
		_default_profile = agent;
		_acc_profile = acc;
		_pd = pd;
		
		setupInitialValues();
		
		initComponents ();
		pack ();
		
		// Resize and reposition window so it appears in the center
		// of the screen, and is not too large
		java.awt.Toolkit tk = java.awt.Toolkit.getDefaultToolkit();
		int width = getSize().width;
		int height = getSize().height;
		
		setSize( width, height );
		width = tk.getScreenSize().width - width;
		height = tk.getScreenSize().height - height;
		setLocation( width/2, height/2 );
		
		setVisible( true );
	}
	
	public boolean isModified()
	{
		return _modified;
	}
	
	private void updateProfiles()
	{
		DIAGNOSTICS.println("Updating profiles", this, DIAGNOSTICS.LEVEL_2);
		
		// Update ACC profile appropriately
		List int_addr = new LinkedList();
		List ext_addr = new LinkedList();
		List agt_addr = new LinkedList();
		
		TransportDefinition transport_def;
		
		for( int i = 0; i < FIPAOSWizard.TRANSPORTS.length; i++ )
		{
			
			transport_def = FIPAOSWizard.TRANSPORTS[i];
			
			if( transport_def.getUsedByACC() == true )
			{
				if( transport_def.getExternal() == true )
				{
					ExternalAddress addr = new ExternalAddressImpl();
					URL url = new URL( "x://y:1/");
					url.setProtocol( transport_def.getName() );
					url.setHost( "somehost" );
					url.setPort( transport_def.getNSPort() );
					addr.setAddress( url.toString() );
					ext_addr.add( addr );
					
					DIAGNOSTICS.println("Added new ACC external transport:" + transport_def.getName(), this, DIAGNOSTICS.LEVEL_2);
				}
				else
				{
					InternalAddress addr = new InternalAddressImpl();
					URL url = new URL( "x://y:1/");
					url.setProtocol( transport_def.getName() );
					url.setHost( "somehost" );
					url.setPort( transport_def.getNSPort() );
					addr.setAddress( url.toString() );
					int_addr.add( addr );
					
					DIAGNOSTICS.println("Added new ACC internal transport:" + transport_def.getName(), this, DIAGNOSTICS.LEVEL_2);
				}
			}
			
			if( transport_def.getUsedByAgent() == true )
			{
				// if internal transport
				if( transport_def.getExternal() == false  )
				{
					InternalAddress addr = new InternalAddressImpl();
					URL url = new URL( "x://y:1/");
					url.setProtocol( transport_def.getName() );
					url.setHost( "somehost" );
					url.setPort( transport_def.getNSPort());
					addr.setAddress( url.toString() );
					agt_addr.add( addr );
					
					DIAGNOSTICS.println("Added new Agent internal transport:" + transport_def.getName(), this, DIAGNOSTICS.LEVEL_2);
				}
			}
		}
		
		_modified = true;
		
		// clear old data from profiles
		
		if (_acc_profile.getInternalAddresss() != null)
		{
			_acc_profile.getInternalAddresss().clear();
		}
		
		if (_acc_profile.getExternalAddresss() != null)
		{
			_acc_profile.getExternalAddresss().clear();
		}
		
		if (_default_profile.getInternalAddresss() != null)
		{
			_default_profile.getInternalAddresss().clear();
		}
		
		// now set new data into ACC + default profiles
		
		Iterator int_addr_iter = int_addr.iterator();
		
		while (int_addr_iter.hasNext())
		{
			_acc_profile.addInternalAddress((InternalAddress)int_addr_iter.next());
		}
		
		Iterator ext_addr_iter = ext_addr.iterator();
		
		while (ext_addr_iter.hasNext())
		{
			_acc_profile.addExternalAddress((ExternalAddress)ext_addr_iter.next());
		}
		
		Iterator agt_addr_iter = agt_addr.iterator();
		
		while (agt_addr_iter.hasNext())
		{
			_default_profile.addInternalAddress((InternalAddress)agt_addr_iter.next());
		}
	
	}
	
		/** This method is called from within the constructor to
		 * initialize the form.
		 * WARNING: Do NOT modify this code. The content of this method is
		 * always regenerated by the FormEditor.
		 */
	private void initComponents()//GEN-BEGIN:initComponents
	{
		jScrollPane1 = new javax.swing.JScrollPane();
		jTable1 = new JTable( new NSTableModel() );
		jTextArea1 = new javax.swing.JTextArea();
		jSeparator1 = new javax.swing.JSeparator();
		_done = new javax.swing.JButton();
		getContentPane().setLayout(new java.awt.GridBagLayout());
		java.awt.GridBagConstraints gridBagConstraints1;
		setModal(true);
		setTitle("Advanced NS Ports Config.");
		setResizable(false);
		addWindowListener(new java.awt.event.WindowAdapter()
		{
			public void windowClosing(java.awt.event.WindowEvent evt)
			{
				closeDialog(evt);
			}
		}
		);
		
		
		jTable1.setBorder(new javax.swing.border.BevelBorder(0));
  		jTable1.setCellSelectionEnabled(true);
  		jTable1.setPreferredScrollableViewportSize(new java.awt.Dimension(450, 150));
  		jScrollPane1.setViewportView(jTable1);
  		
  		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridwidth = 0;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
		gridBagConstraints1.weightx = 1.0;
		gridBagConstraints1.weighty = 1.0;
		getContentPane().add(jScrollPane1, gridBagConstraints1);
		
		
		jTextArea1.setWrapStyleWord(true);
		jTextArea1.setLineWrap(true);
		jTextArea1.setEditable(false);
		jTextArea1.setTabSize(4);
		jTextArea1.setFont(new java.awt.Font ("Dialog", 0, 12));
		jTextArea1.setText("Here you can change the Naming Service port associated with a particular protocol - these setting should be the same for all FIPA-OS installations which are part of the platform.  You can also specify which protocols the agents or ACC (if being started on this computer) should use\n\nNOTE: Currently Agents can only use the RMI transport - selecting other transports for use by Agents has undefined behaviour");
		jTextArea1.setAutoscrolls(false);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridwidth = 0;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints1.insets = new java.awt.Insets(5, 5, 5, 5);
		gridBagConstraints1.weightx = 1.0;
		getContentPane().add(jTextArea1, gridBagConstraints1);
		
		
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridwidth = 0;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints1.insets = new java.awt.Insets(5, 5, 5, 5);
		gridBagConstraints1.weightx = 1.0;
		getContentPane().add(jSeparator1, gridBagConstraints1);
		
		
		_done.setText("Done");
		_done.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				_doneActionPerformed(evt);
			}
		}
		);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.insets = new java.awt.Insets(5, 5, 5, 5);
		gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHEAST;
		getContentPane().add(_done, gridBagConstraints1);
		
	}//GEN-END:initComponents
	
  private void _doneActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event__doneActionPerformed
	  updateProfiles();
	  closeDialog( null );
  }//GEN-LAST:event__doneActionPerformed
  
		/** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
	    setVisible (false);
	    dispose ();
    }//GEN-LAST:event_closeDialog
    
    private class NSTableModel implements javax.swing.table.TableModel
    {
	    private final String[] COLUMN = new String[]
	    { "Used by ACC", "Used by Agents", "Protocol", "NS Port" };
	    private final Class[] COLUMN_CLASS = new Class[]
	    { Boolean.class, Boolean.class, String.class, Integer.class };
	    private final boolean[] EDITABLE = new boolean[]
	    { true, true, false, true };
	    private List _listeners = new LinkedList();
	    
	    public String getColumnName( int i )
	    {
		    return COLUMN[ i ];
	    }
	    
	    public Class getColumnClass( int i )
	    {
		    return COLUMN_CLASS[ i ];
	    }
	    
	    public boolean isCellEditable(int rowIndex, int columnIndex)
	    {
		    return EDITABLE[ columnIndex ];
	    }
	    
	    public void setValueAt(Object aValue, int row, int column)
	    {
		    switch( column )
		    {
			    case 0:
				    FIPAOSWizard.TRANSPORTS[row].setUsedByACC( ((Boolean)aValue).booleanValue());
				    break;
				    
			    case 1:
				    FIPAOSWizard.TRANSPORTS[row].setUsedByAgent(( (Boolean) aValue ).booleanValue() );
				    break;
				    
			    case 3:
						Integer int_value = null;
						
						if( aValue instanceof String )
						{
							int_value = new Integer( (String) aValue );
						}
						else
						{
							int_value = (Integer) aValue;
						}
						
						FIPAOSWizard.TRANSPORTS[row].setNSPort( int_value.intValue() );
				    break;
		    }
	    }
	    
	    public int getRowCount()
	    {
		    return FIPAOSWizard.TRANSPORTS.length;
	    }
	    
	    public int getColumnCount()
	    {
		    return COLUMN.length;
	    }
	    
	    public Object getValueAt(int row, int column)
	    {
		    switch( column )
		    {
			    case 0:
				    return new Boolean( FIPAOSWizard.TRANSPORTS[row].getUsedByACC() );
				    
			    case 1:
				    return new Boolean( FIPAOSWizard.TRANSPORTS[row].getUsedByAgent() );
				    
			    case 2:
				    return FIPAOSWizard.TRANSPORTS[ row ].getName();
				    
			    case 3:
				    return new Integer( FIPAOSWizard.TRANSPORTS[ row ].getNSPort() );
		    }
		    
		    return null;
	    }
	    
	    public void removeTableModelListener(javax.swing.event.TableModelListener p1)
	    {
		    _listeners.remove( p1 );
	    }
	    
	    public void addTableModelListener(javax.swing.event.TableModelListener p1)
	    {
		    _listeners.add( p1 );
	    }
    }
    
		// Variables declaration - do not modify//GEN-BEGIN:variables
		private javax.swing.JScrollPane jScrollPane1;
		private javax.swing.JTable jTable1;
		private javax.swing.JTextArea jTextArea1;
		private javax.swing.JSeparator jSeparator1;
		private javax.swing.JButton _done;
		// End of variables declaration//GEN-END:variables
		
	private void setupInitialValues()
	{
		// update the TransportDefinition object with the data in the profiles

		//-------------------------------------------------
		// Update TransportDefinition's that are internal
		// and used by ACC
		//-------------------------------------------------
		List int_addr = _acc_profile.getInternalAddresss();

		if (int_addr != null)
		{


			InternalAddress addr;
			TransportDefinition transport_def;
			String protocol_name;
			int port;

			for (int i = 0; i < int_addr.size(); i++)
			{

				addr = (InternalAddress)int_addr.get(i);

				protocol_name = (new fipaos.util.URL(addr.getAddress())).getProtocol();
				port = (new fipaos.util.URL(addr.getAddress())).getPort();

				for (int j = 0; j < FIPAOSWizard.TRANSPORTS.length; j++)
				{
					transport_def = FIPAOSWizard.TRANSPORTS[j];

					if (transport_def.getName().equals(protocol_name))
					{

						if (transport_def.getExternal() == false)
						{
							DIAGNOSTICS.println("Internal Protocol " + protocol_name + " is used by ACC", this, DIAGNOSTICS.LEVEL_2);

							transport_def.setUsedByACC(true);
							transport_def.setNSPort(port);
						}

					} // end if protocol matches transport name

				} // end for j

			} // end for i

		} // end if acc internal addresses != null

		//-------------------------------------------------
		// Update TransportDefinition's that are external
		// and used by ACC
		//-------------------------------------------------
		List ext_addr = _acc_profile.getExternalAddresss();

		if (ext_addr != null)
		{
			// set any TransportDefinition's that are used by ACC and are not external

			ExternalAddress addr;
			TransportDefinition transport_def;
			String protocol_name;
			int port;

			for (int i = 0; i < ext_addr.size(); i++)
			{

				addr = (ExternalAddress)ext_addr.get(i);

				protocol_name = (new fipaos.util.URL(addr.getAddress())).getProtocol();
				port = (new fipaos.util.URL(addr.getAddress())).getPort();

				for (int j = 0; j < FIPAOSWizard.TRANSPORTS.length; j++)
				{
					transport_def = FIPAOSWizard.TRANSPORTS[j];

					if (transport_def.getName().equals(protocol_name))
					{

						if (transport_def.getExternal() == true)
						{
							DIAGNOSTICS.println("External Protocol " + protocol_name + " is used by ACC", this, DIAGNOSTICS.LEVEL_2);

							transport_def.setUsedByACC(true);
							transport_def.setNSPort(port);
						}

					} // end if protocol matches transport name

				} // end for j

			} // end for i

		} // end if acc external addresses != null

		//-------------------------------------------------
		// Update TransportDefinition's that are internal
		// and used by the agents
		//-------------------------------------------------
		List agt_addr = _default_profile.getInternalAddresss();

		if (agt_addr != null)
		{


			InternalAddress addr;
			TransportDefinition transport_def;
			String protocol_name;
			int port;

			for (int i = 0; i < agt_addr.size(); i++)
			{

				addr = (InternalAddress)agt_addr.get(i);

				protocol_name = (new fipaos.util.URL(addr.getAddress())).getProtocol();
				port = (new fipaos.util.URL(addr.getAddress())).getPort();

				for (int j = 0; j < FIPAOSWizard.TRANSPORTS.length; j++)
				{
					transport_def = FIPAOSWizard.TRANSPORTS[j];

					if (transport_def.getName().equals(protocol_name))
					{

						if (transport_def.getExternal() == false)
						{
							DIAGNOSTICS.println("Internal Protocol " + protocol_name + " is used by Agents", this, DIAGNOSTICS.LEVEL_2);

							transport_def.setUsedByAgent(true);
							transport_def.setNSPort(port);
						}

					} // end if protocol matches transport name

				} // end for j

			} // end for i

		} // end if agent internal addresses != null

	} // end setupInitialValues()
		
}
