//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved.
//
//******************************************************************************

package fipaos.util.objecteditor;

import fipaos.util.*;

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import java.lang.reflect.*;

/**
 * This is a panel to allow editing of complex types (i.e. non-primitive & non-collection)
 * <p>It uses the ObjectEditorComponentFactory to generate panels for each of the
 * properties of the complex type to enable it to be edited
 */
public class ComplexEditorPanel extends javax.swing.JPanel implements ObjectEditorComponent
{
	private Map _components = new HashMap();
	private Object _editing;
	private boolean _immutable = false;
	
	public ComplexEditorPanel( Object o )
	{
		// Attempt to duplicate object
		_editing = ( new DynamicClass( o ) ).duplicate();
		if( _editing == null )
		{
			_editing = o;
		}
		
		// Initialise GUI components
		initComponents ();
		
		// Check if we're dealing with an immutable object
		DynamicClass dc = new DynamicClass( _editing );
		_immutable = ( dc.getProperties() == null );
		
		if( !_immutable )
		{
			// Iterate through properties for the given object
			int row = 0;
			Iterator i = dc.getProperties().iterator();
			while( i.hasNext() )
			{
				DynamicClass.Property prop = (DynamicClass.Property) i.next();

				// Create appropriate JLabel & Component
				JLabel label = new JLabel( prop.getName() );
				Component comp = (Component) ObjectEditorComponentFactory.getComponent( prop );

				// Add to Map of components
				_components.put( prop, comp );

				// Add to GUI
				addComponent( row++, label, comp );
			}
		}
		else
		{
			// Create single GUI component
			JLabel label = new JLabel( "Value" );
			Component comp = (Component) ObjectEditorComponentFactory.getComponent( o.toString() );
			
			// Save reference to components
			_components.put( "value", comp );
			
			// Add to GUI
			addComponent( 0, label, comp );
		}
	}
	
	public ComplexEditorPanel( DynamicClass.Property p ) throws InstantiationException, IllegalAccessException
	{
		// Create an instance of the appropriate type if the property
		// is set to null for editing purposes
		this( ( p.getValue() == null ? p.getType().newInstance() : p.getValue() ) );
	}
	
	private void addComponent( int row, Component label, Component editor )
	{
		GridBagConstraints gbc = new java.awt.GridBagConstraints();
		gbc.insets = new java.awt.Insets(3, 3, 3, 3);
		gbc.anchor = java.awt.GridBagConstraints.EAST;
		gbc.fill = java.awt.GridBagConstraints.NONE;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.weightx = 0.0;
		gbc.weighty = 0.0;
		gbc.gridx = 0;
		gbc.gridy = row;
		add( label, gbc );
		
		gbc = new java.awt.GridBagConstraints();
		gbc.insets = new java.awt.Insets(3, 3, 3, 3);
		gbc.anchor = java.awt.GridBagConstraints.WEST;
		gbc.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.weightx = 1.0;
		gbc.weighty = 0.0;
		gbc.gridx = 1;
		gbc.gridy = row;
		add( editor, gbc );
	}
	
		/** This method is called from within the constructor to
		 * initialize the form.
		 * WARNING: Do NOT modify this code. The content of this method is
		 * always regenerated by the FormEditor.
		 */
	private void initComponents()//GEN-BEGIN:initComponents
	{
		setLayout(new java.awt.GridBagLayout());
		java.awt.GridBagConstraints gridBagConstraints1;
	}//GEN-END:initComponents
	
	public java.lang.Object getValue()
	{
		// Are we dealing with an immutable object?
		if( _immutable )
		{
			// Get value to assign to immutable object
			Object value = ((ObjectEditorComponent) _components.get( "value" )).getValue();
			
			// Create new instance of immutable object
			try
			{
				Class arg_types[] = new Class[] { String.class };
				Object args[] = new Object[] { value };
				DynamicClass dc = new DynamicClass( _editing.getClass().getName(), arg_types, args );
				_editing = dc.getObject();
			}
			catch( DynamicClassException e )
			{
				DIAGNOSTICS.println( "Exception caught from inside class - maybe a parsing problem", e.getThrowable(), DIAGNOSTICS.LEVEL_MAX );
			}
			catch( Throwable t )
			{
				DIAGNOSTICS.println( "Unexception caught", t, DIAGNOSTICS.LEVEL_MAX );
			}
		}
		else
		{
			// Iterate through the properties and set values
			Iterator i = _components.keySet().iterator();
			while( i.hasNext() )
			{
				DynamicClass.Property prop = (DynamicClass.Property) i.next();
				ObjectEditorComponent comp = (ObjectEditorComponent) _components.get( prop );
				prop.setValue( comp.getValue() );
			}
		}
		
		return _editing;
	}
	
	public static void main( String args[] )
	{
		ObjectEditorComponent comp = ObjectEditorComponentFactory.getDialog( null, new fipaos.ont.fipa.ACL() );
		System.out.println( comp.getValue() );
	}
	
	// Variables declaration - do not modify//GEN-BEGIN:variables
	// End of variables declaration//GEN-END:variables
}
