//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.util.objecteditor;

import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;

/**
 * Editor for a Map.  NOTE: Keys are expected to be String instances ONLY!
 */
public class MapEditorDialog extends javax.swing.JDialog
{
	/**
	 * Types of class that can be added
	 */
	private Class _types[];
	
	/**
	 * Names associated with types that can be added - can be null
	 */
	private String _type_names[];
	
	/**
	 * Map to be edited
	 */
	private Map _map;
	
	/**
	 * Map being edited
	 */
	private Map _edit_map = new HashMap();
	
	/**
	 * List of keys in Map
	 */
	private List _edit_keys = new LinkedList();
	
	/**
	 * Indicates if OK was pressed
	 */
	private boolean _ok = false;
	
	/**
	 * TableModel for Map this is editing
	 */
	private MapTableModel _table_model = new MapTableModel();
	
	/**
	 * Reference to this for inner-classes
	 */
	private JDialog _this = this;
	
	public MapEditorDialog(Map m,Class[] types,String names[])
	{
		super( (java.awt.Frame)null, true );
		
		_map = m;
		_edit_map.putAll( _map );
		_edit_keys.addAll( _edit_map.keySet() );
		_types = types;
		_type_names = names;
		
		initComponents ();
		updateTable();
		pack ();
		
		// Resize and reposition window so it appears in the center
		// of the screen, and is not too large
		java.awt.Toolkit tk = java.awt.Toolkit.getDefaultToolkit();
		int width = getSize().width;
		int height = getSize().height;
		
		if( width < 320 )
		{
			width = 320;
		}
		
		if( height < 240 )
		{
			height = 240;
		}
		
		setSize( width, height );
		width = tk.getScreenSize().width - width;
		height = tk.getScreenSize().height - height;
		setLocation( width/2, height/2 );
		
		setVisible(true);
		
		// If _ok, copy new contents of List into original List
		if( _ok )
		{
			_map.clear();
			_map.putAll( _edit_map );
		}		
	}
	
	public MapEditorDialog(Map m,Class[] types)
	{
		this(m,types,null);
	}
	
		/** This method is called from within the constructor to
		 * initialize the form.
		 * WARNING: Do NOT modify this code. The content of this method is
		 * always regenerated by the FormEditor.
		 */
	private void initComponents()//GEN-BEGIN:initComponents
	{
		jPopupMenu1 = new javax.swing.JPopupMenu();
		// Create additional items to add new items
		if( _types != null && _types.length > 0 )
		{
			for( int i = 0; i < _types.length; i++ )
			{
				JMenuItem jmi = null;
				if( _type_names != null && _type_names.length > i )
				{
					jmi = new JMenuItem( "Add " + _type_names[i] );
				}
				else
				{
					jmi = new JMenuItem( "Add " + _types[i].getName() );
				}
				jPopupMenu1.add( jmi );
				jmi.addActionListener( new AddItemListener( _types[ i ] ) );
			}
		}
		jMenuItem1 = new javax.swing.JMenuItem();
		jMenuItem2 = new javax.swing.JMenuItem();
		jScrollPane1 = new javax.swing.JScrollPane();
		jTable1 = new javax.swing.JTable();
		jButton1 = new javax.swing.JButton();
		jButton2 = new javax.swing.JButton();
		
		jMenuItem1.setText("Edit Item");
  		jMenuItem1.addActionListener(new java.awt.event.ActionListener()
  		{
  			public void actionPerformed(java.awt.event.ActionEvent evt)
  			{
  				jMenuItem1ActionPerformed(evt);
  			}
  		}
  		);
  		jPopupMenu1.add(jMenuItem1);
  		
		jMenuItem2.setText("Delete Item");
  		jMenuItem2.addActionListener(new java.awt.event.ActionListener()
  		{
  			public void actionPerformed(java.awt.event.ActionEvent evt)
  			{
  				jMenuItem2ActionPerformed(evt);
  			}
  		}
  		);
  		jPopupMenu1.add(jMenuItem2);
  		getContentPane().setLayout(new java.awt.GridBagLayout());
		java.awt.GridBagConstraints gridBagConstraints1;
		setTitle("Edit java.util.Map");
		addWindowListener(new java.awt.event.WindowAdapter()
		{
			public void windowClosing(java.awt.event.WindowEvent evt)
			{
				closeDialog(evt);
			}
		}
		);
		
		jScrollPane1.addMouseListener(new java.awt.event.MouseAdapter()
		{
			public void mouseClicked(java.awt.event.MouseEvent evt)
			{
				mouseClicked1(evt);
			}
		}
		);
		
		jTable1.setModel(_table_model);
  		jTable1.addMouseListener(new java.awt.event.MouseAdapter()
  		{
  			public void mouseClicked(java.awt.event.MouseEvent evt)
  			{
  				mouseClicked1(evt);
  			}
  		}
  		);
  		jScrollPane1.setViewportView(jTable1);
  		
  		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridwidth = 0;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
		gridBagConstraints1.insets = new java.awt.Insets(3, 3, 3, 3);
		gridBagConstraints1.weightx = 1.0;
		gridBagConstraints1.weighty = 1.0;
		getContentPane().add(jScrollPane1, gridBagConstraints1);
		
		
		jButton1.setText("OK");
		jButton1.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				jButton1ActionPerformed(evt);
			}
		}
		);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.insets = new java.awt.Insets(3, 3, 3, 3);
		gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
		gridBagConstraints1.weightx = 1.0;
		getContentPane().add(jButton1, gridBagConstraints1);
		
		
		jButton2.setText("Cancel");
		jButton2.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				jButton2ActionPerformed(evt);
			}
		}
		);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.insets = new java.awt.Insets(3, 3, 3, 3);
		gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
		gridBagConstraints1.weightx = 1.0;
		getContentPane().add(jButton2, gridBagConstraints1);
		
	}//GEN-END:initComponents

  private void mouseClicked1(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_mouseClicked1
		if( SwingUtilities.isRightMouseButton( evt ) )
		{
			setupPopupMenu( jTable1.getSelectedRow() >= 0 );
			jPopupMenu1.show( evt.getComponent(), evt.getX(), evt.getY() );
		}
  }//GEN-LAST:event_mouseClicked1
	
  private void jMenuItem2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem2ActionPerformed
		try
		{
			// Get key of object to remove
			Object key = _edit_keys.get( jTable1.getSelectedRow() );
			
			// Remove it and update GUI
			_edit_map.remove( key );
			updateTable();
		}
		catch( Throwable t )
		{
			// An exception occurred
			JOptionPane.showMessageDialog( _this, "An unexpected error occured whilst removing object" );
		}
  }//GEN-LAST:event_jMenuItem2ActionPerformed
	
  private void jMenuItem1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem1ActionPerformed
		try
		{
			// Get key of object to edit
			Object key = _edit_keys.get( jTable1.getSelectedRow() );
			
			// Get object to edit
			Object obj = _edit_map.get( key );
			
			// Construct a new Dialog to deal with creating this new type
			ObjectEditorComponent editor = ObjectEditorComponentFactory.getDialog( null, obj );
			
			// Add value to list (if necessary)
			if( editor.getValue() != null )
			{
				_edit_map.put( key, editor.getValue() );
				updateTable();
			}
		}
		catch( Throwable t )
		{
			// An exception occurred
			JOptionPane.showMessageDialog( _this, "An unexpected error occured whilst editing object" );
		}
  }//GEN-LAST:event_jMenuItem1ActionPerformed
	
  private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
		_ok = false;
		setVisible (false);
		dispose();
  }//GEN-LAST:event_jButton2ActionPerformed
	
  private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
		_ok = true;
		setVisible (false);
		dispose();
  }//GEN-LAST:event_jButton1ActionPerformed
	
		/** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
			setVisible (false);
			dispose ();
    }//GEN-LAST:event_closeDialog
		
		private void setupPopupMenu( boolean itemSelected )
		{
			jMenuItem1.setEnabled( itemSelected );
			jMenuItem2.setEnabled( itemSelected );
		}
		
		private void updateTable()
		{
			_edit_keys.clear();
			_edit_keys.addAll( _edit_map.keySet() );
			_table_model.notifyChange();
		}
		
		/**
		 * @param args the command line arguments
		 */
		public static void main (String args[])
		{
			Map l = new HashMap();
			new MapEditorDialog ( l, new Class[]
			{ String.class, Integer.class }, new String[] { "Name", "Age" } );
			System.out.println( l.toString() );
			System.exit( 0 );
		}
		
		
		/**
		 * Listener for ActionEvent's indicating that a new Object needs to be created
		 */
		private class AddItemListener implements ActionListener
		{
			private Class _type;
			
			public AddItemListener( Class type )
			{
				_type = type;
			}
			
			public void actionPerformed(ActionEvent e)
			{
				try
				{
					// Attempt to get the key for the new value
					String key = JOptionPane.showInputDialog( "Enter key to be associated with new value please" );
					
					if( key != null )
					{
						// Attempt to create an instance of the required type
						Object new_object = _type.newInstance();

						// Construct a new Dialog to deal with creating this new type
						ObjectEditorComponent editor = ObjectEditorComponentFactory.getDialog( null, new_object );

						// Add value to list (if necessary)
						if( editor.getValue() != null )
						{
							_edit_map.put( key, editor.getValue() );
							updateTable();
						}
					}
				}
				catch( Throwable t )
				{
					// An exception occurred
					JOptionPane.showMessageDialog( _this, "An unexpected error occured whilst creating new object" );
				}
			}
		}
		
		/**
		 * TableModel for the Map we're editing
		 */
		private class MapTableModel implements javax.swing.table.TableModel
		{
			private final String[] COLUMN = new String[]
			{ "Key", "Value" };
			private final Class[] COLUMN_CLASS = new Class[]
			{ String.class, String.class };
			private final boolean[] EDITABLE = new boolean[]
			{ true, false };
			private List _listeners = new LinkedList();
			
			public String getColumnName( int i )
			{
				return COLUMN[ i ];
			}
			
			public Class getColumnClass( int i )
			{
				return COLUMN_CLASS[ i ];
			}
			
			public boolean isCellEditable(int rowIndex, int columnIndex)
			{
				return EDITABLE[ columnIndex ];
			}
			
			public void setValueAt(Object aValue, int row, int column)
			{
				switch( column )
				{
					case 0:
						// Get original key & value
						Object orig_key = _edit_keys.get( row );
						Object orig_value = _edit_map.get( orig_key );
						
						// Remove key
						_edit_map.remove( orig_key );
						
						// Re-add new key and old value
						_edit_map.put( aValue, orig_value );
						
						// Update Table
						updateTable();
						break;
				}
			}
			
			public int getRowCount()
			{
				return _edit_keys.size();
			}
			
			public int getColumnCount()
			{
				return COLUMN.length;
			}
			
			public Object getValueAt(int row, int column)
			{
				switch( column )
				{
					case 0:
						return _edit_keys.get( row );
						
					case 1:
						return _edit_map.get( _edit_keys.get( row ) ).toString();
				}
				
				return null;
			}
			
			public void removeTableModelListener(javax.swing.event.TableModelListener p1)
			{
				_listeners.remove( p1 );
			}
			
			public void addTableModelListener(javax.swing.event.TableModelListener p1)
			{
				_listeners.add( p1 );
			}
			
			public void notifyChange()
			{
				Iterator i = _listeners.iterator();
				while( i.hasNext() )
				{
					TableModelListener tml = (TableModelListener) i.next();
					tml.tableChanged( new TableModelEvent( this ) );
				}
			}
		}
		
		
		// Variables declaration - do not modify//GEN-BEGIN:variables
		private javax.swing.JPopupMenu jPopupMenu1;
		private javax.swing.JMenuItem jMenuItem1;
		private javax.swing.JMenuItem jMenuItem2;
		private javax.swing.JScrollPane jScrollPane1;
		private javax.swing.JTable jTable1;
		private javax.swing.JButton jButton1;
		private javax.swing.JButton jButton2;
		// End of variables declaration//GEN-END:variables
		
}
