//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved.
//
//******************************************************************************

package fipaos.util.objecteditor;

import fipaos.util.*;

/**
 * Dialog that wraps an ObjectEditorComponentDialog
 */
public class ObjectEditorComponentDialog extends javax.swing.JDialog implements ObjectEditorComponent
{
	private ObjectEditorComponent _component;
	private String _name;
	private Object _result = null;
	
	private ObjectEditorComponentDialog( java.awt.Frame parent )
	{
		super( parent, true );
	}
	
	public ObjectEditorComponentDialog( java.awt.Frame parent, Object obj )
	{
		this( parent );
		
		// Get name of property
		_name = "Edit " + obj.getClass().getName();
		
		// Get component to deal with property
		_component = ObjectEditorComponentFactory.getComponent( obj );
		
		// Check its non-null
		if( _component != null )
		{
			// Initialise all GUI components
			initComponents ();
			
			// Add our own component to the dialog
			getContentPane().add((javax.swing.JComponent) _component, java.awt.BorderLayout.CENTER);
			
			// Pack and display
			pack ();
			
			// Resize and reposition window so it appears in the center
			// of the screen, and is not too large
			java.awt.Toolkit tk = java.awt.Toolkit.getDefaultToolkit();
			int width = getSize().width;
			int height = getSize().height;
			
			if( width < 320 )
			{
				width = 320;
			}
			
			width = tk.getScreenSize().width - width;
			height = tk.getScreenSize().height - height;
			setLocation( width/2, height/2 );
						
			setVisible( true );
		}
	}
	
	public ObjectEditorComponentDialog(java.awt.Frame parent, DynamicClass.Property property)
	{
		this( parent );
		
		// Get name of property
		_name = "Edit " + property.getName();
		
		// Get component to deal with property
		_component = ObjectEditorComponentFactory.getComponent( property );
		
		// Check its non-null
		if( _component != null )
		{
			// Initialise all GUI components
			initComponents ();
			
			// Add our own component to the dialog
			getContentPane().add((javax.swing.JComponent) _component, java.awt.BorderLayout.CENTER);
			
			// Pack and display
			pack ();
			
			// Resize and reposition window so it appears in the center
			// of the screen, and is not too large
			java.awt.Toolkit tk = java.awt.Toolkit.getDefaultToolkit();
			int width = getSize().width;
			int height = getSize().height;
			width = tk.getScreenSize().width - width;
			height = tk.getScreenSize().height - height;
			setLocation( width/2, height/2 );
			
			setVisible( true );
		}
	}
	
	/**
	 * Get the new value for the property being edited
	 */
	public java.lang.Object getValue()
	{
		return _result;
	}
	
		/** This method is called from within the constructor to
		 * initialize the form.
		 * WARNING: Do NOT modify this code. The content of this method is
		 * always regenerated by the FormEditor.
		 */
	private void initComponents()//GEN-BEGIN:initComponents
	{
		jPanel1 = new javax.swing.JPanel();
		jSeparator1 = new javax.swing.JSeparator();
		jButton1 = new javax.swing.JButton();
		jButton2 = new javax.swing.JButton();
		setTitle(_name);
		addWindowListener(new java.awt.event.WindowAdapter()
		{
			public void windowClosing(java.awt.event.WindowEvent evt)
			{
				closeDialog(evt);
			}
		}
		);
		
		jPanel1.setLayout(new java.awt.GridBagLayout());
		java.awt.GridBagConstraints gridBagConstraints1;
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
  		gridBagConstraints1.gridwidth = 0;
  		gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
  		gridBagConstraints1.insets = new java.awt.Insets(3, 3, 3, 3);
  		gridBagConstraints1.weightx = 1.0;
  		jPanel1.add(jSeparator1, gridBagConstraints1);
  		
  		
		jButton1.setText("OK");
  		jButton1.addActionListener(new java.awt.event.ActionListener()
  		{
  			public void actionPerformed(java.awt.event.ActionEvent evt)
  			{
  				jButton1ActionPerformed(evt);
  			}
  		}
  		);
  		gridBagConstraints1 = new java.awt.GridBagConstraints();
  		gridBagConstraints1.insets = new java.awt.Insets(3, 3, 3, 3);
  		gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
  		gridBagConstraints1.weightx = 1.0;
  		jPanel1.add(jButton1, gridBagConstraints1);
  		
  		
		jButton2.setText("Cancel");
  		jButton2.addActionListener(new java.awt.event.ActionListener()
  		{
  			public void actionPerformed(java.awt.event.ActionEvent evt)
  			{
  				jButton2ActionPerformed(evt);
  			}
  		}
  		);
  		gridBagConstraints1 = new java.awt.GridBagConstraints();
  		gridBagConstraints1.insets = new java.awt.Insets(3, 3, 3, 3);
  		gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
  		gridBagConstraints1.weightx = 1.0;
  		jPanel1.add(jButton2, gridBagConstraints1);
  		
  		
		getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);
		
	}//GEN-END:initComponents
	
  private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
		setVisible (false);
		dispose ();
  }//GEN-LAST:event_jButton2ActionPerformed
	
  private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
		_result = _component.getValue();
		setVisible (false);
		dispose ();
  }//GEN-LAST:event_jButton1ActionPerformed
	
		/** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
			setVisible (false);
			dispose ();
    }//GEN-LAST:event_closeDialog
		
		// Variables declaration - do not modify//GEN-BEGIN:variables
		private javax.swing.JPanel jPanel1;
		private javax.swing.JSeparator jSeparator1;
		private javax.swing.JButton jButton1;
		private javax.swing.JButton jButton2;
		// End of variables declaration//GEN-END:variables
		
}
