//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.util.objecteditor;

import java.awt.event.*;
import java.util.*;
import javax.swing.*;

/**
 * Editor for a Set
 */
public class SetEditorDialog extends javax.swing.JDialog
{
	/**
	 * Types of class that can be added
	 */
	private Class _types[];
	
	/**
	 * Set to be edited
	 */
	private Set _set;
	
	/**
	 * Set being edited
	 */
	private Set _edit_set = new HashSet();
	
	/**
	 * Indicates if OK was pressed
	 */
	private boolean _ok = false;
	
	/**
	 * Reference to this for inner-classes
	 */
	private JDialog _this = this;
	
	public SetEditorDialog(Set l,Class[] types)
	{
		super( (java.awt.Frame)null, true );
		
		_set = l;
		_edit_set.addAll( _set );
		_types = types;
		
		initComponents ();
		updateList();
		pack ();
		
		// Resize and reposition window so it appears in the center
		// of the screen, and is not too large
		java.awt.Toolkit tk = java.awt.Toolkit.getDefaultToolkit();
		int width = getSize().width;
		int height = getSize().height;
		
		if( width < 320 )
		{
			width = 320;
		}
		
		if( height < 240 )
		{
			height = 240;
		}
		
		setSize( width, height );
		width = tk.getScreenSize().width - width;
		height = tk.getScreenSize().height - height;
		setLocation( width/2, height/2 );
		
		setVisible(true);
		
		// If _ok, copy new contents of List into original List
		if( _ok )
		{
			_set.clear();
			_set.addAll( _edit_set );
		}
	}
	
		/** This method is called from within the constructor to
		 * initialize the form.
		 * WARNING: Do NOT modify this code. The content of this method is
		 * always regenerated by the FormEditor.
		 */
	private void initComponents()//GEN-BEGIN:initComponents
	{
		jPopupMenu1 = new javax.swing.JPopupMenu();
		// Create additional items to add new items
		if( _types != null && _types.length > 0 )
		{
			for( int i = 0; i < _types.length; i++ )
			{
				JMenuItem jmi = new JMenuItem( "Add " + _types[i].getName() );
				jPopupMenu1.add( jmi );
				jmi.addActionListener( new AddItemListener( _types[ i ] ) );
			}
		}
		jMenuItem1 = new javax.swing.JMenuItem();
		jMenuItem2 = new javax.swing.JMenuItem();
		jScrollPane1 = new javax.swing.JScrollPane();
		jList1 = new javax.swing.JList();
		jButton1 = new javax.swing.JButton();
		jButton2 = new javax.swing.JButton();
		
		jMenuItem1.setText("Edit Item");
  		jMenuItem1.addActionListener(new java.awt.event.ActionListener()
  		{
  			public void actionPerformed(java.awt.event.ActionEvent evt)
  			{
  				jMenuItem1ActionPerformed(evt);
  			}
  		}
  		);
  		jPopupMenu1.add(jMenuItem1);
  		
		jMenuItem2.setText("Delete Item");
  		jMenuItem2.addActionListener(new java.awt.event.ActionListener()
  		{
  			public void actionPerformed(java.awt.event.ActionEvent evt)
  			{
  				jMenuItem2ActionPerformed(evt);
  			}
  		}
  		);
  		jPopupMenu1.add(jMenuItem2);
  		getContentPane().setLayout(new java.awt.GridBagLayout());
		java.awt.GridBagConstraints gridBagConstraints1;
		setTitle("Edit java.util.Set");
		addWindowListener(new java.awt.event.WindowAdapter()
		{
			public void windowClosing(java.awt.event.WindowEvent evt)
			{
				closeDialog(evt);
			}
		}
		);
		
		
		jList1.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
  		jList1.addMouseListener(new java.awt.event.MouseAdapter()
  		{
  			public void mouseClicked(java.awt.event.MouseEvent evt)
  			{
  				mouseClicked1(evt);
  			}
  		}
  		);
  		jScrollPane1.setViewportView(jList1);
  		
  		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridwidth = 0;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
		gridBagConstraints1.insets = new java.awt.Insets(3, 3, 3, 3);
		gridBagConstraints1.weightx = 1.0;
		gridBagConstraints1.weighty = 1.0;
		getContentPane().add(jScrollPane1, gridBagConstraints1);
		
		
		jButton1.setText("OK");
		jButton1.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				jButton1ActionPerformed(evt);
			}
		}
		);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.insets = new java.awt.Insets(3, 3, 3, 3);
		gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
		gridBagConstraints1.weightx = 1.0;
		getContentPane().add(jButton1, gridBagConstraints1);
		
		
		jButton2.setText("Cancel");
		jButton2.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				jButton2ActionPerformed(evt);
			}
		}
		);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.insets = new java.awt.Insets(3, 3, 3, 3);
		gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
		gridBagConstraints1.weightx = 1.0;
		getContentPane().add(jButton2, gridBagConstraints1);
		
	}//GEN-END:initComponents

  private void jMenuItem2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem2ActionPerformed
		try
		{
			// Get object to remove
			Object obj = jList1.getSelectedValue();

			// Remove it and update GUI
			_edit_set.remove( obj );
			updateList();
		}
		catch( Throwable t )
		{
			// An exception occurred
			JOptionPane.showMessageDialog( _this, "An unexpected error occured whilst removing object" );
		}
  }//GEN-LAST:event_jMenuItem2ActionPerformed

  private void jMenuItem1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem1ActionPerformed
		try
		{
			// Get object to edit
			Object obj = jList1.getSelectedValue();

			// Construct a new Dialog to deal with creating this new type
			ObjectEditorComponent editor = ObjectEditorComponentFactory.getDialog( null, obj );

			// Add value to list (if necessary)
			if( editor.getValue() != null )
			{
				_edit_set.remove( obj );
				_edit_set.add( editor.getValue() );
				updateList();
			}
		}
		catch( Throwable t )
		{
			// An exception occurred
			JOptionPane.showMessageDialog( _this, "An unexpected error occured whilst editing object" );
		}
  }//GEN-LAST:event_jMenuItem1ActionPerformed
	
  private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
		_ok = false;
		setVisible (false);
		dispose();
  }//GEN-LAST:event_jButton2ActionPerformed
	
  private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
		_ok = true;
		setVisible (false);
		dispose();
  }//GEN-LAST:event_jButton1ActionPerformed
	
  private void mouseClicked1(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_mouseClicked1
		if( SwingUtilities.isRightMouseButton( evt ) )
		{
			int index = jList1.getSelectedIndex();
			setupPopupMenu( jList1.getSelectedValue() != null );
			jPopupMenu1.show( evt.getComponent(), evt.getX(), evt.getY() );
		}
  }//GEN-LAST:event_mouseClicked1
	
		/** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
			setVisible (false);
			dispose ();
    }//GEN-LAST:event_closeDialog
		
		private void setupPopupMenu( boolean itemSelected )
		{
			jMenuItem1.setEnabled( itemSelected );
			jMenuItem2.setEnabled( itemSelected );
		}
		
		private void updateList()
		{
			int i = jList1.getSelectedIndex();
			jList1.setListData( new Vector( _edit_set ) );
			jList1.setSelectedIndex( i );
		}
		
		/**
		 * @param args the command line arguments
		 */
		public static void main (String args[])
		{
			Set l = new HashSet();
			new SetEditorDialog ( l, new Class[]
			{ String.class, Integer.class } );
			System.out.println( l.toString() );
			System.exit( 0 );
		}
		
		
		/**
		 * Listener for ActionEvent's indicating that a new Object needs to be created
		 */
		private class AddItemListener implements ActionListener
		{
			private Class _type;
			
			public AddItemListener( Class type )
			{
				_type = type;
			}
			
			public void actionPerformed(ActionEvent e)
			{
				try
				{
					// Attempt to create an instance of the required type
					Object new_object = _type.newInstance();
					
					// Construct a new Dialog to deal with creating this new type
					ObjectEditorComponent editor = ObjectEditorComponentFactory.getDialog( null, new_object );
					
					// Add value to list (if necessary)
					if( editor.getValue() != null )
					{
						_edit_set.add( editor.getValue() );
						updateList();
					}
				}
				catch( Throwable t )
				{
					// An exception occurred
					JOptionPane.showMessageDialog( _this, "An unexpected error occured whilst creating new object" );
				}
			}
		}
		
		
		
		// Variables declaration - do not modify//GEN-BEGIN:variables
		private javax.swing.JPopupMenu jPopupMenu1;
		private javax.swing.JMenuItem jMenuItem1;
		private javax.swing.JMenuItem jMenuItem2;
		private javax.swing.JScrollPane jScrollPane1;
		private javax.swing.JList jList1;
		private javax.swing.JButton jButton1;
		private javax.swing.JButton jButton2;
		// End of variables declaration//GEN-END:variables
		
}
