//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.util.objecteditor.primitive;

import fipaos.util.*;
import fipaos.util.objecteditor.*;

import java.util.*;
import javax.swing.*;

/**
 * Panel that allows complex-types to be displayed as a simple string, but provides
 * a button to allow editing of the properties of the complex type
 */
public class ComplexPropertyEditorPanel extends fipaos.util.objecteditor.primitive.CollectionEditorPanel
{
	private DynamicClass.Property _property;
	private Object _value = null;
	
	public ComplexPropertyEditorPanel(DynamicClass.Property property)
	{
		super( property.getValue() );
		
		// Save Property object
		_property = property;
		
		// Get value of property
		_value = _property.getValue();
	}
	
	public java.lang.Object getValue()
	{
		return _value;
	}
	
	/**
	 * Sub classes should implement this method to display some dialog
	 * to allow modification of the collection this edits
	 */
	public void modifyValue()
	{
		try
		{
			if( _value == null )
			{
				_value = _property.getType().newInstance();
			}
			
			// Display appropriate Dialog & collect result
			_value = ObjectEditorComponentFactory.getDialog( null, _value ).getValue();
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "Unexception caught", t, DIAGNOSTICS.LEVEL_MAX );
			
			// An exception occurred
			JOptionPane.showMessageDialog( null, "An unexpected error occured whilst creating new object" );
		}
	}
}
