//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the 
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis, 
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.util.objecteditor.primitive;

import fipaos.util.*;
import fipaos.util.objecteditor.*;

import java.util.*;
import javax.swing.*;

/**
 * Panel that allows lists to be displayed as a simple string, but provides
 * a button to allow editing of the list
 */
public class ListEditorPanel extends fipaos.util.objecteditor.primitive.CollectionEditorPanel
{
	private DynamicClass.Property _property;
	private List _value = null;
	
	public ListEditorPanel( DynamicClass.Property property )
	{
		super( property.getValue() );
		
		// Save Property object
		_property = property;
		
		// Add content of property to our set
		if( _property.getValue() != null )
		{
			_value = new LinkedList();
			_value.addAll( (Collection) _property.getValue() );
		}
	}
	
	public java.lang.Object getValue()
	{
		return _value;
	}
	
	/**
	 * Sub classes should implement this method to display some dialog
	 * to allow modification of the collection this edits
	 */
	public void modifyValue()
	{
		// Check if hints are given
		if( _property.getHint() == null )
		{
			JOptionPane.showMessageDialog( this, "Property isn't designed to be used with appropriate editor GUI - can't edit" );
		}
		else
		{
			boolean previously_null = false;
			
			// Check value isn't null BEFORE editing
			if( _value == null )
			{
				_value = new LinkedList();
				previously_null = true;
			}
			
			// Display appropriate Dialog
			new ListEditorDialog( _value, 
									((DynamicClass.CollectionHint) _property.getHint())._types );
			
			// Check if null-value should be re-asserted
			if( previously_null && _value.isEmpty() )
			{
				_value = null;
			}
		}
	}
}
