//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files ("the
//     Programs") are Copyright (c) 1999 - 2000 Nortel Networks Corporation, 
//     8200 Dixie Road, Suite 100, Brampton, Ontario, Canada L6R 5P6.  
//     All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out in
//     the Nortel Networks FIPA-OS Public License, a copy of which can be found
//     in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version can
//     also be found at http://fipa-os.sourceforge.net/. By downloading the
//     files containing the Programs you accept the terms and  conditions of the
//     Public License. You do not have to accept these terms and conditions, but
//     unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.util.proxy;

//----------------------------------------------------------------------

// These are the import statements that are required for this class.
import java.util.*;
import java.net.*;
import java.io.*;
import java.lang.*;

  /**
   * description of class
   * <br>
   * <br>
   */

public class Proxy 
{
	//----------------------------------------------------------------------
	// CONSTANTS
	//----------------------------------------------------------------------

	//----------------------------------------------------------------------
	// STATIC VARIABLES
	//----------------------------------------------------------------------

	/** 
	 * This is a static variable to contain the location of HTTP proxy.
	 * This should be of the form "<host-name/ip-address>:<port-number>".
	 */
	private static String __proxy_host = null;
	
	/**
	 * These are the two vectors that are used to store the complete list of all 
	 * of the proxies and the list of used proxies respectively. */
	private static Vector __proxy_vector = new Vector();
	private static Vector __used_proxies = new Vector();
	
	private static String __auto_proxy_location = null;		
	
	//----------------------------------------------------------------------
	// INSTANCE VARIABLES
	//----------------------------------------------------------------------
	
	
	//----------------------------------------------------------------------
	// CONSTRUCTOR
	//----------------------------------------------------------------------

	/** This is the default Proxy class constructor. */
	public Proxy() {}
	
	
	//----------------------------------------------------------------------
	// PUBLIC METHODS
	//----------------------------------------------------------------------

	/** This method is one of two methods which is public. It is used to set the 
	 *  automatic configuration script. */
	public static void setAutoConfigScriptURL(String url)
	{
		__auto_proxy_location = url;
	}
	
	/** This is a test method that will fetch a specific document. */
	public static String fetchDocument(String test_url) throws NoContentException,
																		MovedException, BadRequestException, BadMachineException
	{
		if(__auto_proxy_location != null && __proxy_host == null)
		{
			fetchAutoConfigScript(__auto_proxy_location);
		}
		
		String test_doc = "";
		
   	try
 		{
 	  	URL the_test_doc_url = new URL(test_url);
 	  	HttpURLConnection test_doc_url = (HttpURLConnection)the_test_doc_url.openConnection();
			BufferedReader in = new BufferedReader(new InputStreamReader
																						(test_doc_url.getInputStream()));

			String next_line = in.readLine();
 			
			while(next_line != null)
 			{
 				test_doc += next_line + "\n";
 				next_line = in.readLine();
 			}
 			
 			fipaos.util.DIAGNOSTICS.println(test_doc);
 			
 			int response_code = test_doc_url.getResponseCode();
 			String response_status = test_doc_url.getResponseMessage();
 			final String protocol = "HTTP/1.0";  
 			
 			if(response_code == 204)
 			{
 				fipaos.util.DIAGNOSTICS.println(protocol + " " + response_code + " " + response_status);
 				throw new NoContentException("ERROR: NO CONTENT EXCEPTION");
 			}
 			else 
 			{
 				if((response_code == 300) || (response_code == 301) || (response_code == 302) || (response_code == 304))
 				{
 					fipaos.util.DIAGNOSTICS.println(protocol + " " + response_code + " " + response_status);
 					throw new MovedException("ERROR: MOVED EXCEPTION");
				}
 				else
 				{
 					if((response_code == 400) || (response_code == 401) || (response_code == 403) || (response_code == 404))
 					{
 						fipaos.util.DIAGNOSTICS.println(protocol + " " + response_code + " " + response_status);
 						throw new BadRequestException("ERROR: BAD REQUEST EXCEPTION");
 					}
 					else
 					{
 						if((response_code == 500) || (response_code == 501) || (response_code == 502) || (response_code == 503))
 						{
 							fipaos.util.DIAGNOSTICS.println(protocol + " " + response_code + " " + response_status);
 							throw new BadMachineException("ERROR: BAD MACHINE EXCEPTION");
						}
 					}
 				}
 			}
 	
			in.close();
			
			return test_doc;
 		}
 		catch(MalformedURLException a)
 		{
/* 			fipaos.util.DIAGNOSTICS.println("ERROR HAS OCCURED WITH LOADING THE test_doc DOCUMENT: "); 
 			fipaos.util.DIAGNOSTICS.println("MalformedURLException");			 			
 			a.printStackTrace();
*/
 			return null;
		}
		catch(FileNotFoundException b)
 		{
/* 			fipaos.util.DIAGNOSTICS.println("ERROR HAS OCCURED WITH LOADING THE test_doc DOCUMENT: "); 
 			fipaos.util.DIAGNOSTICS.println("FileNotFoundException");			 			
 		  b.printStackTrace();
*/
 			return null;
		}
		catch(UnknownHostException c)
 		{
/* 			fipaos.util.DIAGNOSTICS.println("ERROR HAS OCCURED WITH LOADING THE test_doc DOCUMENT: "); 
 			fipaos.util.DIAGNOSTICS.println("UnknownHostException");			 			
 			c.printStackTrace();
*/
 			if(whichProxy())
 			{
	 			return fetchDocument(test_url);
 			}
			else
 			{
				return null;
 			}
 		}
 		catch(ConnectException d)
 		{
/* 			fipaos.util.DIAGNOSTICS.println("ERROR HAS OCCURED WITH LOADING THE test_doc DOCUMENT: "); 
 			fipaos.util.DIAGNOSTICS.println("ConnectException");				 
 			fipaos.util.DIAGNOSTICS.println("Bad URL");			
 			d.printStackTrace();
*/ 			
 			return null;
 		}
 		catch(IOException e)
 		{
/* 			fipaos.util.DIAGNOSTICS.println("ERROR HAS OCCURED WITH LOADING THE test_doc DOCUMENT: "); 
			fipaos.util.DIAGNOSTICS.println("IOException"); 			
 			e.printStackTrace();
*/
 			return null;
 		}
	}

	/** This is the private constructor method that sets up the proxy properies. */
	public static void setProxyProperties(String proxy)
	{
		try
		{
			int index = (proxy != null ? proxy.indexOf(":") : 0);
			
			fipaos.util.DIAGNOSTICS.println("About to set proxy to " + proxy);
			
			Properties props = System.getProperties();
			
			if(proxy == null)
			{
				//props.put("http.useProxy", "false");
				fipaos.util.DIAGNOSTICS.println("proxy was null");				
			}
			else
			{		
				fipaos.util.DIAGNOSTICS.println("proxy not null, setting");	
				props.put("http.useProxy", "true");
    		props.put("http.proxyHost", proxy.substring(0, index));
    		fipaos.util.DIAGNOSTICS.println("http.proxyHost: " + proxy.substring(0, index));
    		props.put("http.proxyPort", proxy.substring(index + 1));
    		fipaos.util.DIAGNOSTICS.println("http.proxyPort: " + proxy.substring(index + 1));
			}
			
    	__proxy_host = proxy;
		}
		catch(StringIndexOutOfBoundsException a)
		{
			fipaos.util.DIAGNOSTICS.println("ERROR IN SETTING THE PROXY PROPERTIES: ");
			fipaos.util.DIAGNOSTICS.println("StringIndexOutOfBoundsException");
		}
		catch(Exception e)
		{
			fipaos.util.DIAGNOSTICS.println("ERROR IN SETTING THE PROXY PROPERTIES: ");
		}
	}
	
	
	//----------------------------------------------------------------------
	// PRIVATE METHODS
	//----------------------------------------------------------------------
	
	/** This is a public method that calls the setProxyAddresses method. */
	private static void fetchAutoConfigScript(String proxy_url)
	{
		//setProxyProperties(null);
		
		String proxy_doc = null;
		String protocol = "http://";
		
		int index1 = protocol.length();
		int index2 = proxy_url.indexOf(":", index1);
		int index3 = proxy_url.lastIndexOf("/");
		
		String machine = proxy_url.substring(index1, index2);
		String port = proxy_url.substring(index2 + 1, index3);
		String file_name = proxy_url.substring(index3 + 1);
		
		ProxySocket the_socket = new ProxySocket(machine, 8888);
		
		proxy_doc = the_socket.getProxyScript(file_name);
 		
 		fipaos.util.DIAGNOSTICS.println(proxy_doc);
 		initialProxy(proxy_doc);
 	}
		 		
	/** This is the getProxyAddresses method which finds all of the proxies and 
	 *  stores them in a vector. */
	private static Vector getProxyAddresses(String proxy_script)
	{	
		if(proxy_script == null)
		{
			return null;
		}
		
		StringTokenizer tokens = new StringTokenizer(proxy_script, "\"; \t\n", 
																								 false);
		
		fipaos.util.DIAGNOSTICS.println("ALL PROXIES:");
		
		Vector proxy_vector = new Vector();
		
		while(tokens.hasMoreTokens())
		{
			String token = tokens.nextToken();
			
			if(token.equals("PROXY"))
			{	
				String proxy_ip = tokens.nextToken();
				
				if(!proxy_vector.contains(proxy_ip))
				{
					proxy_vector.addElement(proxy_ip);
					fipaos.util.DIAGNOSTICS.println(proxy_ip);
				}
			}
		}
		
		return proxy_vector;
	}
	
	/** This method is called to select the first proxy IP to use and is only 
	 *  called once at the beginning. */
	private static void initialProxy(String proxy_script)
	{
		__proxy_vector = getProxyAddresses(proxy_script);
		setProxyProperties((String)__proxy_vector.firstElement());
	}
	
	/** This method is called to select which proxy IP to use depending on 
	 *  whether or not the current one is working or not. */
	private static boolean whichProxy()
	{
		if(__proxy_host != null)
		{
			__used_proxies.addElement(__proxy_host);
			__proxy_vector.removeElement((Object)__proxy_host);
			fipaos.util.DIAGNOSTICS.println("USED PROXIES: " + __used_proxies);	
			fipaos.util.DIAGNOSTICS.println("REMAINING PROXIES: " + __proxy_vector);
			System.out.print("CURRENT PROXY: ");
		}
		
		if(__proxy_vector.isEmpty())
		{
			fipaos.util.DIAGNOSTICS.println("none");
			return false;
		}
		else
		{
			fipaos.util.DIAGNOSTICS.println();
			fipaos.util.DIAGNOSTICS.println("***********************************************************");
			fipaos.util.DIAGNOSTICS.println("*************************** BEFORE ************************");
			fipaos.util.DIAGNOSTICS.println("***********************************************************");	
			fipaos.util.DIAGNOSTICS.println((String)__proxy_vector.firstElement());
			fipaos.util.DIAGNOSTICS.println("***********************************************************");
			fipaos.util.DIAGNOSTICS.println("**************************** AFTER ************************");
			fipaos.util.DIAGNOSTICS.println("***********************************************************");	                
			setProxyProperties((String)__proxy_vector.firstElement());
			return true;
		}
	}

	private int getIssue(String name)
	{
		String issue_string = name.substring(name.indexOf('_' ) + 1);
		int issue = java.lang.Integer.getInteger(issue_string).intValue();
		
		return issue;
	}
}
